/*** A_ADD_RESP.   Add a response or item to a conference.
/
/    ok = a_add_resp (resp, mask, wait);
/
/    Parameters:
/       int       ok;       (return status)
/       Resp      resp;     (object containing new response)
/       long      mask;     (P_ATTACH indicates attachment)
/       int       wait;     (synchronicity flag)
/
/    Purpose:
/       Add a new response (or, if the response number is 0, a new item).
/
/    How it works:
/       Adding a new item: caller fills in CNUM, INUM = -1, TEXT, TITLE,
/       AUTHOR, OWNER members of RESP.
/
/       Adding a new response: caller fills in CNUM, INUM, TEXT, and
/       AUTHOR, OWNER members of RESP.
/
/       A_add_resp() fills in INUM, RNUM, DATE, and TIME as it works.
/
/       A_add_resp calls chg_ditem(), chgxitem(), chg_dresp(), 
/       and chgxresp() as needed to do the dirty work.
/
/       To ensure that file attachment info doesn't exist for a non-existent
/       attachment, we use the following algorithm:
/          1) Lock all relevant files or bail-out
/          2) chg_dresp() to add an empty response
/          3) copy attached file to ATTACH/Cnnn/iiirrr
/          4) if copy fails, delete response
/          5) Change response to hold real text
/          6) Send response to 'Link neighbors
/          7) Send attachment to 'Link neighbors
/
/    Returns: A_OK       success
/             A_NOCONF   no such conference
/             A_NOWRITE  user not allowed to write in conf or to item
/             A_NOITEM   no such item
/             A_DBERR    unknown error
/
/    Error Conditions:
/  
/    Side effects: 
/
/    Related functions:
/
/    Called by:  UI
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  api/aaddresp.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR 12/03/92 14:44 New function. */
/*: CR 12/04/92 15:27 Make it compile! */
/*: CR 12/07/92  0:01 Set 'ready' flag. */
/*: CR 12/07/92 10:41 Use L(' ') in chxcatval(). */
/*: CL 12/11/92 15:08 Long chxlong arg. */
/*: CR  2/22/93 23:31 Add new args to a_cache() calls. */
/*: CR  2/28/93 21:44 Replace global 'tbuf' with a_cch_tbuf call. */
/*: JV  3/03/93 14:47 Add File Attachments. */
/*: JA  7/27/93 16:15 Change file locking order. */
/*: JA  7/28/93 09:15 Fix call to chg_dresp(). */
/*: CK  8/12/93 18:02 Copy attachment from attach->name, not ->db_name. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "api.h"
#include "unitwipe.h"
#include "done.h"
#include "xaction.h"

extern struct   master_template  master;
extern Textbuf  tbuf;

FUNCTION  a_add_resp (resp, mask, wait)
   Resp   resp;
   long   mask;
   int    wait;
{
   int     access, additem, chgresp, item, error, success, ok, response,
           oitem, chgsubj;
   Chix    cday, ctime, chn, frozen, itemchn;
   Chix   hdr, emptychx, getline();
   long    zero, systime();
   Textbuf tbuf, a_cch_tbuf();

   if (resp == (Resp) NULL  ||  resp->tag != T_RESP)      return (A_BADARG);
   if (resp->cnum <= 0)                                   return (A_NOCONF);

   /*** Make sure this user can write in this conference. */
   if (NOT a_cache (V_GET, resp->cnum, &access, &additem, &chgresp,
                           &chgsubj, nullchix))            return (A_NOCONF);
   if (access < 2)                                        return (A_NOWRITE);
   if (access < 3  &&  resp->rnum == 0  &&  NOT additem)  return (A_NOWRITE);

   chn    = chxalloc ( 4L, THIN, "a_add_re chn");
   cday   = chxalloc (20L, THIN, "a_add_re cday");
   ctime  = chxalloc (20L, THIN, "a_add_re ctime");
   frozen = chxalloc ( 4L, THIN, "a_add_re frozen");
   emptychx = chxalloc (4L, THIN, "a_add_re empty");

   sysdaytime (cday, ctime, 0, systime());
   chxcpy    (resp->date, cday);
   chxcatval (resp->date, THIN, L(' '));
   chxcat    (resp->date, ctime);

   /*** New Item. */
   if (resp->inum < 0) {
      if (NOT (mask & P_ATTACH)) {
         chg_ditem (XT_NEW, resp->cnum, 0, &item, chn,
                   resp->author, resp->owner, resp->text->data, resp->title,
                   cday, ctime, resp->attach, &error);
         if (error != 0)  DONE (A_DBERR);
         resp->inum = item;
         resp->rnum = 0;
      }
      else {     /* Add attachment and item.  See algorithm in header. */
         error = add_resp_attach (resp, cday, ctime);
         if (error) DONE (error);
      }

      if (is_distributed (resp->cnum)) {
         get_chn  (chn, XSNF, 0);
         ok = chgxitem (XT_NEW, resp->cnum, item, chn, resp->author,
                        resp->owner, resp->title, resp->text->data, cday,
                        ctime, nullchix, &error);
         if (NOT ok)  DONE (A_DBERR);
      }
   }

   /*** New response. */
   else {

      /*** Decide if this item is frozen. */
      tbuf = a_cch_tbuf (V_GET, XCRD, XCRF, resp->cnum, resp->inum, 0, 
                                nullchix);
      hdr  = getline (XCRD, XCRF, resp->cnum, nullchix, resp->inum, 0, 0, tbuf);
      if (hdr == nullchix)  DONE (A_NOITEM);
      chxtoken (frozen, nullchix, 3, hdr);
      zero = 0;
      if (chxlong (frozen, &zero) > 0)  DONE (A_NOWRITE);

      if (NOT (mask & P_ATTACH)) {
         /*** Add the response locally. */
         chg_dresp (XT_NEW, resp->cnum, resp->inum, 0, &response, chn,
                    resp->author, resp->owner, resp->text->data, cday, ctime, 
                    mask & P_ATTACH ? resp->attach : (Attachment) NULL, &error);
         if (error != 0)  DONE (A_DBERR);
         resp->rnum = response;
      }
      else {      /* Add attachemnt & response. See algorithm in header. */
         error = add_resp_attach (resp, cday, ctime);
         if (error) DONE (error);
      }
 
      /*** Send the response to CaucusLink neighbors. */
      if (is_distributed (resp->cnum)) {
         if (master.cnum != resp->cnum)  srvldmas (1, &master, resp->cnum);
         itemchn = chxalloc (40L, THIN, "aaddres itemchn");
         getoitem (&oitem, itemchn, resp->inum, &master);
         get_chn  (chn, XSNF, 0);
         ok = chgxresp (XT_NEW, resp->cnum, oitem, itemchn, resp->rnum, chn,
                        resp->author, resp->owner, resp->text->data, cday,
                        ctime, nullchix, &error);
         chxfree (itemchn);
         if (NOT ok)  DONE (A_DBERR);
      }
   }

   resp->ready = 1;
   DONE (A_OK);

done:
   a_cch_tbuf (V_CLR, XCRD, XCRF, resp->cnum, resp->inum, 0, nullchix);
   chxfree (chn);
   chxfree (cday);
   chxfree (ctime);
   chxfree (frozen);
   chxfree (emptychx);

   return  (success);
}

/*** ADD_RESP_ATTACH: Only used when adding an item or resp *and*
/    an attachment. */

add_resp_attach (resp, cday, ctime)
Chix cday, ctime;
Resp resp;
{
   char name[100];
   int  success=0, response, error;
   Chix chn, owner, ownerh;

   chn    = chxalloc ( 4L, THIN, "a_add_re chn");
   owner  = chxalloc (16L, THIN, "a_add_re owner");
   ownerh = chxalloc (16L, THIN, "a_add_re ownerh");

   /*** First check for existence of ATTACH/Cnnn. */
   if (NOT unit_lock  (XACD, READ, L(resp->cnum), 0L, 0L, 0L, nullchix))
      DONE (A_DBERR);

   if (NOT unit_check (XACD)) {
      unit_name (name, XACD, L(resp->cnum), 0L, 0L, 0L, nullchix);
      sysmkdir  (name, 1);
   }
   unit_unlk (XACD);

   /*** Lock source of attachment. */
   chxbreak (resp->owner, CQ("@"), owner, ownerh);
   if (NOT unit_lock (XIRE, READ,  0L, 0L, 0L, 0L, resp->attach->name))
      DONE (A_DBERR);

   /*** Add empty response locally. */
   chg_dresp (XT_NEW, resp->cnum, resp->inum, 0, &response, chn,
              resp->author, resp->owner, nullchix, cday, ctime, 
              (Attachment) NULL, &error);
   if (error != 0) {
      unit_unlk (XARF);
      unit_unlk (XIRE);
      DONE (A_DBERR);
   }
   resp->rnum = response;

   /*** Lock file where attachment will actually go. */
   if (NOT unit_lock (XARF, WRITE, L(resp->cnum), L(resp->inum),
                      L(response), 0L, nullchix)) {
      unit_unlk (XIRE);
      DONE (A_DBERR);
   }
         
   /*** Copy the attachment to local DB. */
   if (NOT unit_b_copy (XIRE, XARF)) {
      /* If copy failed, back out the empty response. */
      chg_dresp (XT_DELETE, resp->cnum, resp->inum, 0, &response, nullchix,
           nullchix, nullchix, nullchix, nullchix, nullchix, 
           (Attachment) NULL, &error);
      unit_unlk (XIRE);
      unit_kill (XARF);   /* Maybe some was copied over. */
      unit_unlk (XARF);
      DONE (A_DBERR);
   }
   unit_unlk   (XIRE);    /* Done with this. */

   /*** Add the real text locally. */
   chg_dresp (XT_CHANGE, resp->cnum, resp->inum, 0, &response, nullchix,
              nullchix, nullchix, resp->text->data, nullchix, nullchix,
              resp->attach, &error);

   if (error != 0) {
      unit_kill (XARF);
      unit_unlk (XARF);
      DONE (A_DBERR);
   }

   unit_unlk (XARF);

done:
   chxfree (chn);
   chxfree (owner);
   chxfree (ownerh);

   return (success);
}
