/*** GET_SETTING.   Decode a particular SET option.
/
/    ok = get_setting (optstr, which, t_value, n_value);
/
/    Parameters:
/       int     ok;       (success?)
/       Chix    optstr;   (string of options, from a_get_person)
/       int     which;    (number of desired option; 1 is 1st)
/       Chix    t_value;  (returned value, if option has text value)
/       int    *n_value   (returned value, if option has numeric value)
/
/    Purpose:
/       Decode a particular option from a user's option string (typically
/       taken from Person->setopts, as returned by a_get_person), and
/       return the string or numeric value.
/
/    How it works: the table lists the WHICH number, the name, and value
/    of each option.
/       1  debug         number (0=off)
/       2  screensize    number (of lines)
/       3  screenwidth   number (of single-width characters)
/       4  format        number (0=off, 1=on)
/       5  editor        text   (name of editor)
/       6  tab           number (tab turns into this many blanks)
/       7  page          number (ascii value of desired new page character)
/       8  print         number (0=>default printcode, 1=>user edited printcode)
/       9  verbose       number (0=>off, 1=>on)
/      10  dictionary    number (of dictionary to use)
/      11  eot           text   (of EOT string; ""=>2return, $endfile$=>1eof)
/      12  expiration    number (0=>off, 1=>on)
/      13  text_entry    number (0=>terminal, 1=>editor, 2=>wordwrap)
/      14  receipt_size  number (of lines in return receipt)
/      15  mytext        number (0=>later, 1=>now, 2=>never)
/      16  startmenu     text   (menu to start on conference entry; or "")
/      17  import_mail   number (0=>off, 1=>import, 2=>copy)
/      18  export_mail   number (0=>off, 1=>export, 2=>copy.  Not used.)
/
/    Returns: 1 on success.
/             0 on bad arguments.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: UI
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  api/getset.c
/ */

/*** Copyright (C) 1992 Camber-Roth.   All rights reserved. */

/*: CR 12/10/92 21:46 New function. */
/*: CR 12/13/92 11:25 Complete. */
/*: CR 12/13/92 22:40 Add comments. */
/*: CR  1/13/93 14:55 Make LPOS long. */

#include <stdio.h>
#include "chixuse.h"
#include "api.h"

#define ON_OFF_NUM(x)  (is_on ? x : (is_off ? 0 : numvalue))

FUNCTION  get_setting (optstr, which, t_value, n_value)
   Chix   optstr, t_value;
   int    which;
   int   *n_value;
{
   static Chix save_defaults = nullchix;
   Chix   word, temp;
   long   lpos;
   int    is_num, is_off, is_on, numvalue, chs;

   if (optstr == nullchix)            return (0);
   if (which  <  1   ||  which > 18)  return (0);

   word = chxalloc (16L, THIN, "get_set word");

   /*** Pluck out the string value of the WHICH'th option. */
   chxtoken (word, nullchix, which, optstr);

   /*** If OPTSTR does not have that option, use the default from the
   /    dictionary. */
   if (EMPTYCHX (word)) {
      temp = chxalloc (80L, THIN, "get_set tempchx");
      mdstr    (temp, "sys_Tset0", &save_defaults);
      chxtoken (word, nullchix, which, temp);
      chxfree  (temp);
   }

   jixreduce (word);
   lpos     = 0;
   numvalue = chxlong  (word, &lpos);
   is_num   = chxvalue (word,  lpos) == '\0';
   is_on    = chxeq    (word, CQ("on"));
   is_off   = chxeq    (word, CQ("off"));

   switch (which) {

      /* DEBUG */
      case ( 1): *n_value     = ON_OFF_NUM (-1);    break;

      /* SCREENSIZE */
      case ( 2): *n_value     = numvalue;           break;

      /* SCREENWIDTH */
      case ( 3): *n_value     = numvalue;           break;

      /* FORMAT */
      case ( 4): *n_value  = 1 - ON_OFF_NUM (1); break;

      /* EDITOR */
      case ( 5): chxcpy (t_value, word);        break;

      /* TAB */
      case ( 6): *n_value = numvalue;           break;

      /* PAGE */
      case ( 7): *n_value   = numvalue;           break;

      /* PRINT */
      case ( 8):
         chs = tablematch (CQ("default edit"), word);
         *n_value = (is_num ? numvalue : (chs >= 0 ? chs : 0));
         break;

      /* VERBOSE */
      case ( 9): *n_value   = ON_OFF_NUM (1);     break;

      /* DICTIONARY */
      case (10): *n_value      = numvalue;           break;

      /* EOT */
      case (11):
         chs = tablematch (CQ("1eof 2return"), word);
         chxcpy (t_value, (chs==1 ? chxquick("", 0 ) : 
                           (chs==0 ? chxquick("$endfile$", 1 ) : word)));
         break;

      /* EXPIRATION */
      case (12): *n_value   = ON_OFF_NUM(1);      break;

      /* TEXT_ENTRY */
      case (13):
         chs = tablematch (CQ("terminal editor wordwrap window"), word);
         *n_value = (is_num ? numvalue : (chs >= 0 ? chs : 0));
         break;

      /* RECEIPT_SIZE */
      case (14): *n_value = numvalue;         break;

      /* MYTEXT */
      case (15):
         chs = tablematch (CQ("later now never"), word);
         *n_value = (is_num ? numvalue : (chs >= 0 ? chs : 0));
         break;

      /* STARTMENU */
      case (16): chxcpy (t_value, (is_off ? CQ("") : word));  break;

      /* IMPORT_MAIL */
      case (17):
         chs = tablematch (CQ("off import copy"), word);
         *n_value = (is_num ? numvalue : (chs >= 0 ? chs : 0));
         break;

      /* EXPORT_MAIL */
      case (18):
         chs = tablematch (CQ("off export copy"), word);
         *n_value = (is_num ? numvalue : (chs >= 0 ? chs : 0));
         break;
   }

   chxfree (word);
   return  (1);
}
