
/*** CARVMARK.  Mark a file with a "CARVES" revision marker.
/
/    Invoked as:
/       carvmark file1 [file2...]
/ or    carvmark <list
/
/    Arguments:
/       fileN      file(s) to be marked
/       list       file containing list of files to be marked
/
/    Purpose:
/       Carvmark is a fundamental part of "CARVES", the CAmber-Roth
/       VErsion control System.  It is used to add a "carvmark"
/       to header comments in a file, to indicate that this file
/       has been revised.
/
/    How it works:
/       Carvmark tries to determine the type of file by looking for
/       indicators of the first columns of the first few lines:
/
/           Indicator     type of file
/              /*            C program
/              #             Unix shell script, or Caucus dictionary file
/              $!            Vax/VMS command file
/
/       It then scans for the end of the previous carvmarks (or the
/       beginning of "code", if it doesn't see any) and inserts a
/       "carvmark" comment with the users signature, date/time, and
/       a one-line comment.
/
/    Exit status: 0
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  carves/carvmark.c
/ */

/*: CR  3/01/89 15:56 Start carve-marking carve source code. */
/*: CR  7/05/89 15:44 Change VV #include's to pass other compilers. */
/*: CR 12/08/89 16:10 Use longer system type names, and "typesys.h". */
/*: CR 12/21/89 17:00 Take out defn of localtime() in UNIX/PRIMOS. */
/*: CR 08/21/89 13:00 Move all #define sys types to TYPESYS.H. */
/*: CR  9/07/92 21:11 Expand to handle scripts and dictionary files. */
/*: CR 10/27/92 13:58 Complain if no file; Unix set perms to 755. */
/*: CV 04/07/93 14:35 Remove 'long time()' in sysdate(). */
/*: CR  6/08/93 12:48 Add "$!:" marking for Vax/VMS command scripts. */

#include "typesys.h"

/*** Macros for readability. */
#define  FUNCTION
#define  NOT       !

/*** State table values for searching through input files. */
#define UNKNOWN     0
#define CODE_START  1
#define CODE_MARK   2
#define TEXT_START 11
#define TEXT_MARK  12
#define VAXN_START 21
#define VAXN_MARK  22
#define DONE       99

#include <stdio.h>
#include <ctype.h>

FUNCTION main (argc, argv)
   int   argc;
   char *argv[];
{
   char  sig[20], comment[80], name[100];
   FILE *fp, *fopen();
   int   i, blank;

   if (argc == 1) {
      printf ("Usage:  carvmark  C_source_code_files\n");
      printf ("   or:  carvmark  <file_containing_list_of_files\n");
      exit   (1);
   }

   printf   ("Mark initials: ");
   gets     (sig);
   strupper (sig);

   printf ("Enter brief comment: ");
   gets   (comment);

   if (*argv[1] == '<') {
      if ( (fp = fopen (argv[1]+1, "r")) == NULL) {
         printf ("Can't open file '%s'\n", argv[1]+1);
         exit   (1);
      }
      while (fgets (name, 100, fp) != NULL) {
         name [ strlen(name)-1 ] = '\0';
         if ( (blank = strindex (name, " ")) > 0)  name[blank] = '\0';
         markfile (name, sig, comment);
      }
      fclose (fp);
   }

   else for (i=1;   i<argc;   ++i)  markfile (argv[i], sig, comment);
}


FUNCTION  markfile (name, sig, comment)
   char   name[], sig[], comment[];
{
   FILE *in, *ot, *fopen();
   int   state, marked, pastit;
   char  line[200];

   if ( (in = fopen (name, "r")) == NULL) {
      fprintf (stderr, "Cannot open file '%s'.\n", name);
      return;
   }
   ot = fopen ("tempxxxx", "w");


   state = UNKNOWN;
   while (fgets (line, 150, in) != NULL) {

      if (state == UNKNOWN) {
         if (strindex (line, "/*") == 0)  state = CODE_START;
         if (strindex (line, "#" ) == 0)  state = TEXT_START;
         if (strindex (line, "$!") == 0)  state = VAXN_START;
      }

      if      (state==CODE_START  ||  state==CODE_MARK) {
         marked = strindex (line, "/*:") == 0;
         pastit = (line[0]=='#');
      }
      else if (state==TEXT_START  ||  state==TEXT_MARK) {
         marked = strindex (line, "#:") == 0;
         pastit = (line[0]==' '  ||  line[0]=='\0');
      }
      else if (state==VAXN_START  ||  state==VAXN_MARK) {
         marked = strindex (line, "$!:") == 0;
         pastit = (line[0]=='$'  &&  line[1]!='!');
      }

      if      (state == CODE_START) {
         if (marked)   state = CODE_MARK;
         if (pastit) { state = DONE;  writemark (ot, "/*", sig, comment); }
      }
      else if (state == CODE_MARK) {
         if (NOT marked  ||  pastit)
                     { state = DONE;  writemark (ot, "/*", sig, comment); }
      }
      else if (state == TEXT_START) {
         if (marked)   state = TEXT_MARK;
         if (pastit) { state = DONE;  writemark (ot, "#",  sig, comment); }
      }
      else if (state == TEXT_MARK) {
         if (NOT marked  ||  pastit)
                     { state = DONE;  writemark (ot, "#",  sig, comment); }
      }
      else if (state == VAXN_START) {
         if (marked)   state = VAXN_MARK;
         if (pastit) { state = DONE;  writemark (ot, "$!",  sig, comment); }
      }
      else if (state == VAXN_MARK) {
         if (NOT marked  ||  pastit)
                     { state = DONE;  writemark (ot, "$!",  sig, comment); }
      }

      fprintf (ot, "%s", line);
   }

   if (state==CODE_START  ||  state==CODE_MARK)
      writemark (ot, "/*", sig, comment);

   if (state==TEXT_START  ||  state==TEXT_MARK)
      writemark (ot, "#",  sig, comment);
   
   if (state==VAXN_START  ||  state==VAXN_MARK)
      writemark (ot, "$!",  sig, comment);
   
   fclose (ot);
   fclose (in);
   sysunlink (name);
   sysrename ("tempxxxx", name);
   printf ("Marked file '%s'\n", name);
}


FUNCTION  writemark (ot, comment, sig, line)
   FILE  *ot;
   char   sig[], comment[], line[];
{
   char   now[40];

   sysdate (now);
   fprintf (ot, "%s: %s %s %s", comment, sig, now, line);
   if (streq (comment, "/*"))  fprintf (ot, " */");
   fprintf (ot, "\n");
}
/*----------------------------------------------------------------*/


/*** SYSDATE.  Returns the date and time in the form: "mm/dd/yy hh:mm". */

FUNCTION  sysdate (str)
   char   str[];
{

/*--------------------------UNIX--------*/
#if UNIX | VAXVMS | PRIMOS | MSDOS
#include <time.h>
   long   t;
   struct tm  *now;

   t = time ( (long *) 0);
   now = localtime (&t);
   sprintf (str, "%2d/%02d/%d %2d:%02d", now->tm_mon+1, now->tm_mday,
            now->tm_year, now->tm_hour, now->tm_min);
#endif

/*---------------------------------------VM/CMS-------------------------*/
#if CVM
#include <time.h>
   struct tm *localtime(), *now;

   now = localtime();
   sprintf (str, "%2d/%02d/%d %2d:%02d", now->tm_mon+1, now->tm_mday,
            now->tm_year, now->tm_hour, now->tm_min);
#endif

   return (1);
}
/*----------------------------------------------------------------*/


/*** SYSUNLINK.  System-dependent "delete file" procedure. */

FUNCTION  sysunlink (file)
   char file[];
{

#if UNIX | MSDOS
   unlink (file);
#endif

#if VAXVMS | PRIMOS
   delete (file);
#endif

   return (1);
}
/*----------------------------------------------------------------*/


/*** SYSRENAME.  Renames file A to file B.  */

FUNCTION  sysrename (a, b)
   char   a[], b[];
{

#if VAXVMS
#include <ssdef>
#include <descrip>
   struct dsc$descriptor_s a_desc, b_desc;

   a_desc.dsc$w_length  = strlen (a);
   a_desc.dsc$a_pointer = a;
   a_desc.dsc$b_class   = DSC$K_CLASS_S;
   a_desc.dsc$b_dtype   = DSC$K_DTYPE_T;

   b_desc.dsc$w_length  = strlen (b);
   b_desc.dsc$a_pointer = b;
   b_desc.dsc$b_class   = DSC$K_CLASS_S;
   b_desc.dsc$b_dtype   = DSC$K_DTYPE_T;

   if (lib$rename_file (&a_desc, &b_desc) == SS$_NORMAL)   return (1);
   return (0);
#endif

#if UNIX
   link   (a, b);
   unlink (a);
   chmod  (b, 0755);
   return (1);
#endif

#if PRIMOS
   move   (a, b);
   return (1);
#endif

#if MSDOS
   return (rename (a, b));
#endif

#if CVM
   return (rename (a, b) == 0);
#endif
}
/*----------------------------------------------------------------*/


/*** STRUPPER.  Convert string S to all upper case letters. */

FUNCTION  strupper (s)
   char  *s;
{
   for (;   *s;   ++s)   if (islower (*s))  *s = toupper (*s);

   return (1);
}
/*----------------------------------------------------------------*/


/*** STRINDEX.  Return starting position in string S of string T. */

FUNCTION  strindex (s, t)
   char  s[], t[];
{
   register int  i, j, k;

   for (i=0;  s[i] != '\0';  i++) {
      for (j=i, k=0;  t[k] != '\0' && s[j]==t[k];  j++, k++) ;
      if (t[k] == '\0')  return (i);
   }
   return (-1);
}
/*----------------------------------------------------------------*/

/*** STREQ.   Return 1 if string S equals string T; 0 otherwise. */

FUNCTION  streq (s, t)
   char *s, *t;
{
   if (s == NULL  ||  t == NULL)  return (0);

   while (*s == *t)  {
      if (NOT *s)  return (1);
      ++s;
      ++t;
   }
   return (0);
}
