
/*** WORDWRAP.  Input text from user's terminal, with automatic word wrap.
/
/    wordwrap (file, length);
/
/    Parameters:
/       int   file;       (unit number to write text to)
/       int   length;     (width of screen, in single-width chars)
/
/    Purpose:
/       For all other systems, inputs text from the terminal, performing
/       automatic word-wrap and other neat functions.
/
/    How it works:
/       Wordwrap() assumes that FILE has been locked by the caller.
/       Wordwrap appends all of the input from the user's terminal
/       to the end of FILE.
/
/    Returns: 1 on success
/             0 otherwise
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:
/       Auto-indent may not work correctly.  However, since that
/       feature appeared undesireable, it has not been fully tested.
/
/    Notes:
/       If the Caucus option "ECHO OFF" has been set, wordwrap()
/       does not echo *anything* that the user types, nor does it display
/       the wrapping that it is doing.  The text is still wrapped internally
/       as it is input, however.
/
/    Home:  inout/wordwrap.c
/ */

/*** Copyright (C) 1990 by Camber-Roth.  All rights reserved. */

/*: CX  6/02/92 16:41 New function, from old systextinput(). */
/*: CX  6/17/92 19:02 Miscellaneous debugging. */
/*: CX  6/29/92 17:24 Replace unit_write (XWTX... with unit_term(). */
/*: CR 10/15/92 12:41 Use new version of syspluck(). */
/*: CR 10/21/92 11:39 Replace input_redirect() with copy_redirect(). */
/*: CR 12/10/92 15:17 syspluck 2nd arg is long. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CL 12/11/92 11:13 Long Chxcatsub args. */
/*: CL 12/11/92 13:46 Long Chxvalue N. */
/*: CV  4/07/93 15:05 Replace "ctype.h" with <ctype.h>. */
/*: CP  8/01/93 22:39 Attempt to handle ECHO OFF case. */
/*: CK  8/06/93 11:06 Make ECHO OFF not echo *anything*. */

#include <stdio.h>
#include <ctype.h>
#include "caucus.h"
#include "handicap.h"
#include "systype.h"
#include "sys.h"
#include "diction.h"
#include "units.h"

/*** Wordwrap() performs the following actions:
/       it turns off screen echoing and input processing;
/       it uses ss_inprom as a prompt, unless no1prompt is set;
/       it backspaces on ^H, DEL, or ESC [ D (left arrow)
/       it allows user to edit the session in the middle, 
/            just like textenter();
/       it deals with a hangup condition properly;
/       it slices, it dices, it julienne fries...  */

#define  ESC    '\033'   /* Escape key. */
#define  CTLH   '\010'   /* Control-H, backspace. */
#define  DEL    '\177'   /* Del key. */
#define  BELL   "\007"   /* String containing bell (^G). */

/*** Finish_line outputs a buffered line and prepares for next line. */
#define FINISH_LINE if (echo) PUT_PROMPT;  WRITE_BUF;  RESET_BUF;  DISPLAY_BUF
#define DISPLAY_BUF if (echo) unit_term (buf)
#define PUT_PROMPT  { sysputnl();   unit_term ( prompt); }
#define WRITE_BUF   chxcatval (buf, THIN, L('\n'));   unit_write(file, buf)
#define RESET_BUF   chxclear(buf);   INDENT;   wordbreak = nonblank = -1
#define INDENT      for (width=pos=0;  pos<nonblank;  ++pos) ADDBLANK
#define ADDBLANK    { chxcatval (buf, THIN, L(' '));   ++width; }

extern Userreg              thisuser;
extern struct flag_template flag;
extern struct termstate_t   term_in_caucus;
extern struct sysio_t       sysio;
extern union  null_t        null;
extern Chix                 ss_inprom, yesnotable, ss_inred;
extern int                  debug;

FUNCTION  wordwrap (file, length)
   int   file, length;
{
   static char autoindent = 0;
   int    i, nwipe, success, code, redirect, blanks, nonblank;
   int    lang, japanese, jstate, wordbreak, conv, wide, echo;
   long   width, pos;
   long   c, inred1;
   char   str[2], outseq[80];
   Chix   buf, prompt, yes, editor, temp, jix, answer, infile;

   ENTRY ("wordwrap", "");

   /*** Make sure we can append data to FILE, or it's all over! */
   if (NOT unit_append (file))  RETURN (0);

   /*** Tell terminal driver to stop echoing characters,
   /    and to stop pre-processing characters ('cooking' them). */
   echo = term_in_caucus.c_echo;
   systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, OFF);
   systurn (&term_in_caucus, COOK_INPUT,            OFF);

   /*** Keep a flag handy to tell if we might have japanese chars. */
   lang     = unit_control (XKEY, UNIT_GET);
   japanese = (lang==LANG_EUC  ||  lang==LANG_SJIS);

   /*** PROMPT points to the prompt the user wants, if any. */
   prompt = chxalloc (40L, THIN, "wordwrap prompt");
   if (NOT flag.no1prompt)  chxcpy (prompt, ss_inprom);
   if (echo) unit_term ( prompt);

   /*** Miscellaneous allocations. */
   yes    = chxalloc (20L, THIN, "wordwrap yes");
   chxtoken (yes, null.chx, 2, yesnotable);
   editor = chxalloc ( 40L, THIN, "wordwrap editor");
   jix    = chxalloc (  4L, WIDE, "wordwrap j");
   temp   = chxalloc (100L, THIN, "wordwrap temp");
   inred1 = chxvalue (ss_inred, 0L);
   answer = chxalloc ( 20L, THIN, "wordwrap answer");
   infile = chxalloc ( 40L, THIN, "wordwrap infile");

   /*** BUF contains the line of text being assembled.  POS is the
   /    the position number in BUF of the character we are about
   /    to add.  WIDTH is the current width of BUF, in single-width
   /    character units.
   /
   /    WORDBREAK is the position of the last (most recent) char
   /    at which we could end the line in case of word-wrap.
   /    NONBLANK is the position of the first non-blank character
   /    in BUF; it is used to implement auto-indent. */
   buf       = chxalloc (100L, THIN, "wordwrap buf");
   pos       =  0;
   width     =  0;
   wordbreak = -1;
   nonblank  = -1;


   /*** Loop, reading 1 character at a time.  If sysget1key() fails or a
   /    break/hangup occurs, we're finished. */
   success  = 1;
   redirect = 0;
   jstate   = 0;
   while (1) {
      code = sysget1key (&c);
      if (sysio.state.eightbit <= 0) c = c & 0x7F;   /* Just 7 bits, please. */

      /*** Toss text on hangup. */
      if (code == -4)  { success = 0;   break; }  /* Complete failure. */

      /*** If an End-of-File is detected at the start of a line, then 
      /    we're finished entering text.  Otherwise, it produces a bad
      /    character (and will cause a beep). */
      if (code == -2) {
         if (pos==0)  break;
         sysputstr(BELL);
         continue;
      }

      /*** If user hit <CANCEL>, prompt user to confirm tossing text. */
      if (sysbrktest()  ||  code == -3) {
         unit_close (file);
         systurn (&term_in_caucus, COOK_INPUT,            ON);
         systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, ON);
         sysputnl();
         if (mdprompter ("ced_Pcantex", yesnotable, 2, "ced_Htexcan", 10, 
                            answer, yes) != 0)  {
            unit_kill (file);
            FAIL;
         }

         /*** If not cancelled, continue exactly where we left off,
         /    by repainting the line entered so far. */
         if (NOT unit_append (file))  FAIL;
         mdwrite (XWTX, "ced_Tcontype", null.md);
         systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, OFF);
         systurn (&term_in_caucus, COOK_INPUT,            OFF);
         unit_term ( prompt);
         unit_term ( buf);
         continue;
      }

      /*** At this point, we've handled all the special conditions.
      /    Now begin examining the character we got from sysget1key. */

      /*** Map DEL and left-arrow into backspace. */
      if (c == DEL  ||  c == 70102L)  c = CTLH;

      /*** Ignore other function keys. */
      if (c >= 70000L)  { sysputstr (BELL);   continue; }

      /*** Newline means either EOT, EDIT request, input redirection, 
      /    or simply end of line. */
      chxclear (temp);
      if (c=='\n'  ||  c=='\r')  {

         /*** EOT means we're done entering text!  (Note that this
         /    properly handles the "2return" EOT as well.) */
         if (chxeq (buf, thisuser->eot))  break;
   
         /*** Look for input redirection.  Must be just one word. */
         if (redirect  &&  chxindex (buf, 0L, ss_inred)  == 0 
                       &&  chxtoken (infile, null.chx, 2, buf) == -1) {
            syspluck (buf, 0L, infile);       /* Non-VM/CMS always succeeds. */
            sysputnl ();
            copy_redirect (file, infile);
            PUT_PROMPT;
            RESET_BUF;
            unit_term ( buf);
         }

         /*** Check to see if user wants to edit this text entry. */
         else if (iseditreq (buf, temp, thisuser->editor, editor)) {
            sysputnl();
            unit_close (file);
            systurn  (&term_in_caucus, COOK_INPUT,            ON);
            systurn  (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, ON);
            editfile (file, editor, 1 /*length*/);
            if (NOT unit_append (file)) FAIL;
            systurn  (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, OFF);
            systurn  (&term_in_caucus, COOK_INPUT,            OFF);
            mdwrite  (XWTX, "ced_Tcontype", null.md);
            PUT_PROMPT;
            RESET_BUF;
            unit_term ( buf);
         }
         else {
            FINISH_LINE;
            redirect = 0;
         }

         continue;   /* All '\n' cases back to top of loop. */
      }
   
      /*** If we may be reading multi-byte japanese characters, we need
      /    to use JSTATE to determine if we finished getting an entire
      /    character C. */
      if (japanese) {
         str[0] = (char) (c & 0xFF);    str[1] = 0;
         if (lang==LANG_EUC)   conv = jixofeuc  (jix, str, jstate, outseq);
         if (lang==LANG_SJIS)  conv = jixofsjis (jix, str, jstate, outseq);

         jstate = 0;
         switch (conv) {
            case (1):   jstate = 1;               continue;
            case (2):   sysputstr (BELL);         continue;
            case (0):   c = chxvalue (jix, 0L);    break;
         }
      }

      /*** The rest of the code handles actually doing something with
      /    the specific character value now in C. */

      /*** ^H anywhere but column zero backspaces and erases, either
      /    single width or double width depending on the char to be
      /    erased. */
      if (c==CTLH  &&  pos > 0) {
         if (japanese  &&  jixwidth (chxvalue (buf, L(pos-1))) == 2) {
            if (echo) sysputstr ("\b\b  \b\b");
            width -= 2;
         }
         else {
            if (echo) sysputstr ("\b \b");
            width -= 1;
         }

         /*** Remove the last char in BUF. */
         --pos;
         chxclear  (temp);
         chxcatsub (temp, buf, 0L, pos);
         chxcpy    (buf, temp);
         if (pos <= nonblank)  nonblank = -1;
      }
   
      /*** A tab that goes over the max line width starts a new line. */
      else if (c=='\t'  &&  width + thisuser->tablength - 
                           (width % thisuser->tablength) > length) {
         FINISH_LINE;
      }

      /*** Replace tab with enough blanks to get to next tab stop. */
      else if (c=='\t') {
         blanks = thisuser->tablength - (width % thisuser->tablength);
         for (i=0;   i<blanks;   ++i) {
            if (echo) sysputstr (" ");
            chxcatval (buf, THIN, L(' '));
            ++width;
            ++pos;
         }
      }

      /*** A space that goes over the max line width starts a new line. */
      else if (c==' '  &&  width+1 > length)  {
         FINISH_LINE;
      }
   
      /*** If a printing character, examine to see if this puts us over
      /    the line width limit... */ /* Needs sysclean help! */
      else if (japanese || (c < 256  &&  isprint(c))) {
   
         /*** Always save position of most recent word-break position,
         /    and mark position of *first* non-blank character. */
         if (c==' '  ||  c > 256)                     wordbreak = pos;
         if (c!=' '  &&  nonblank<0  &&  autoindent)  nonblank  = pos;
   
         /*** Check (and remember) for possible input redirection. */
         if (pos==0  &&  c==inred1)  redirect = 1;

         /*** Increment width of line as if we added this character. */
         wide   = jixwidth (c);
         width += wide;

         /*** If a very long word (longer than length - HALFW) takes us past
         /    the max line width, chop it in the middle and be damned. */
         if (width > length  &&  wordbreak < length/2)  { FINISH_LINE; }
   
         /*** Any kanji (1 or 2 width) character does character-wrap at
         /    the end of a line. */
         else if (width > length  &&  c > 256) {
            if (echo) PUT_PROMPT;
            WRITE_BUF;
            RESET_BUF;
/*          chxcatval  (buf, wide, c);  */
/*          unit_write (XWTX, buf);     */
/*          pos   = 0;                  */
            width = wide;
         }

         /*** If a reasonably-sized word takes us past the max line width,
         /    erase the part of it already on this line, write out the line
         /    to the file, and copy the old part onto the beginning of the
         /    next line. */
         else if (width > length) {
            nwipe = pos-wordbreak-1;
            if (echo) {
               for (i=0;  i<nwipe;   ++i)  sysputstr ("\b");
               for (i=0;  i<nwipe;   ++i)  sysputstr (" ");
            }
            chxcpy    (temp, buf);
            chxclear  (buf);
            chxcatsub (buf, temp, 0L, L(wordbreak+1));
            WRITE_BUF;
            chxclear (buf);
            INDENT;
            chxcatsub (buf, temp, L(wordbreak+1), ALLCHARS);
            if (echo) { PUT_PROMPT;    unit_term ( buf); }
            pos = width = chxlen (buf);  /* autoindent bug? */
         }
   
         /*** In all cases, take the current character, add it to the end
         /    of the line, and display it. */
         chxcatval  (buf, wide, c);
         ++pos;                 /* (width is set elsewhere) */
         chxclear  (jix);
         chxcatval (jix, wide, c);
         if (echo)  unit_term ( jix);
      }
   
      /*** All non-printing characters beep and do nothing. */
      else  sysputstr (BELL);
   }
   sysputnl ();

   /*** Restore the terminal the way it was. */
   systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, ON);
   systurn (&term_in_caucus, COOK_INPUT,            ON);

   unit_close (file);
   if (NOT success)  unit_kill (file);

done:
   chxfree (buf);
   chxfree (prompt);
   chxfree (yes);
   chxfree (editor);
   chxfree (jix);
   chxfree (temp);
   chxfree (answer);
   chxfree (infile);

   RETURN  (success);
}
