
/*** LOAD_ULIST.   Load a conference userlist into a ulist.
/
/    ok = load_ulist (u, cnum, ulist, errors);
/
/    Parameters:
/       int       ok;         (success?)
/       int       u;          (unit number containing userlist file)
/       int       cnum;       (conference number)
/       int      *errors;     (number of errors detected)
/       Chix     *ulist;      (put processed userlist here)
/
/    Purpose:
/       Load the conference userlist from the file U for conference
/       CNUM into ULIST.
/
/    How it works:
/       Load_ulist() scans the userlist file U, and parses both old-style
/       (":include") and new-style (":1.1 include") control lines.
/       Everything else is data, and is placed into ULIST according to
/       the scheme below.  Each line of data ("entry") is a userid, group
/       file, @host name, or comment.
/
/       Ulist index      Data
/       -----------      --------------------------
/         0              beginning comments
/         1              paragraph 1 entries given organizer status
/         2              paragraph 1 userids included
/         3              paragraph 1 entries with readonly access
/         4              paragraph 1 entries excluded
/         5              paragraph 2 entries given organizer status
/         ...            ...
/
/       If Load_ulist() can not understand a control line, it comments
/       it out (and preceeds it with a comment that the next line is
/       in error).  Load_ulist() sets ERROR to the number of such errors
/       encountered.
/
/       Note: load_ulist() assumes that ULIST is empty.  If ULIST is not
/       empty, the contents of file U are *added* *to* what is in ULIST.
/
/    Returns: 1 on success.
/             0 if conference does not exist, or other error.
/
/    Error Conditions
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: 
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  ulist/loadulis.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR 10/22/91 22:54 New function. */
/*: CR 10/31/91 14:19 Fix a variety of bugs. */
/*: CR 11/06/91 11:14 Decl strscan(). */
/*: CR 11/15/91 13:38 Add host to data entries missing "@host". */
/*: CR 11/18/91 11:40 Fix special case of index==0. */
/*: UL 12/11/91 18:41 Make unit_lock args 3-6 long.*/
/*: JX  5/19/92 20:52 Fix Chix (no Chix carve!). */
/*: JX  9/23/92 16:52 Fixify. */
/*: CR 12/07/92 15:56 Expand comments. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */

#include <stdio.h>
#include "caucus.h"

extern union null_t null;

FUNCTION  load_ulist (u, cnum, ulist, errors)
   int    u, cnum, *errors;
   Chix  *ulist;
{
   static short order[] = { 0, 1, 3, 2, 0 };
   static Chix inextab;
   int    high_para, this_para, high_subp, this_subp;
   int    index, type, max_para, a_para, a_subp;
   int    success;
   long   pos, sub;
   Chix   line, word, id, host, myhost;

   ENTRY ("load_ulist", "");

   line       = chxalloc (160L, THIN, "loadulis line");
   word       = chxalloc (100L, THIN, "loadulis word");
   id         = chxalloc (100L, THIN, "loadulis id");
   host       = chxalloc (100L, THIN, "loadulis host");
   myhost     = chxalloc (100L, THIN, "loadulis myhost");
   success    = 0;

   /*** Open the userlist file. */
   *errors = 0;
   if (NOT unit_lock (u, READ, L(cnum), 0L, 0L, 0L, null.chx))   FAIL;
   if (NOT unit_view (u))           { unit_unlk (u);   FAIL; }

   if (is_distributed (cnum))  getmyhost (myhost);
   mdtable  ("mai_Ainex", &inextab);

   /*** Each line read belongs to a para/subpara pair, e.g.
   /    "1.4".  HIGH_PARA and HIGH_SUBP are the highest para/subp #s
   /    seen so far.  THIS_PARA and THIS_SUBP are the current para/subp #s.
   /    INDEX maps THIS_PARA/THIS_SUBP into a slot in the ULIST. */

   max_para  = (MAXULIST/4) - 2;
   high_para = this_para = 0;
   high_subp = this_subp = 3;
   index     = 0;
   while (unit_read (u, line, 0L)) {

      /*** Add data lines to the current ULIST slot. */
      if ( chxvalue(line,0L) != ':') {
         chxtrim  (line);
         if ( chxvalue(line,0L) !='#'  &&  is_distributed (cnum)) {
            chxbreak (line, id, host, CQ("@"));
            if (EMPTYCHX (host))  {
               chxclear  (line);
               chxformat (line, CQ("%s@%s"), 0L, 0L, id, myhost);
            }
         }
         index = 4 * this_para + this_subp - 3;
         if (index == 0  &&  ulist[index] == null.chx)
            ulist[index] =  chxalloc (100L, THIN, "load_ulist ulist[]");
         chxcat (ulist[index], line);
         chxcat (ulist[index], NEWLINE);
         continue;
      }

      /*** Interpret old-style ":include", etc. control lines. */
      jixreduce (line);
      chxtoken (word, null.chx, 1, line);
      type = tablematch (inextab, word);
      if (type >= 0) {
         type   = order [type];
         if (high_subp >= type)    ++high_para;
         if (high_para >= max_para)  high_para = max_para - 1;
         this_para = high_para;
         this_subp = high_subp = type;
      }
      else {

         /*** Interpret new-style ":1.1 include" etc. control lines. */
         pos = 1;
         a_para = chxlong (word, &pos);
         if (a_para >= max_para)  a_para = max_para - 1;
      
         /*** If a bad control line, complain about it, but keep it in as
         /    a comment! */
         if (pos == 1  ||  chxvalue (word, pos) != '.') {
            chxcat (ulist[index], CQ("#--Error in next line:\n"));
            chxcat (ulist[index], line);
            chxcat (ulist[index], NEWLINE);
            ++*errors;
            continue;
         }
         sub = ++pos;
         a_subp = chxlong (word, &pos);
         if (pos == sub)  a_subp = 1;
         if (a_subp > 3)  a_subp = 3;
      
         /*** If "1.4 include", the "include" overrides the ".4". */
         chxtoken (word, null.chx, 2, line);
         if (NOT EMPTYCHX(word)) {
            chxconc (word, CQ(":"), word);
            type = tablematch (inextab, word);
            if (type >= 0)  a_subp = order [type];
         }
         
         this_para = a_para;
         this_subp = a_subp;

         if (this_para > high_para) 
            { high_para = this_para;   high_subp = this_subp; }
         if (this_subp > high_subp)    high_subp = this_subp;
      }

      /*** If needed, allocate a chix for this para/subpara. */
      index = 4 * this_para + this_subp - 3;
      if (ulist[index] == nullchix)
          ulist[index] =  chxalloc (100L, THIN, "load_ulist ulist[]");
   }

   unit_close (u);
   unit_unlk  (u);

   SUCCEED;
   
done:

   chxfree ( line );
   chxfree ( word );
   chxfree ( id );
   chxfree ( host );
   chxfree ( myhost );

   RETURN (success);
}
