
/*** CHXRESIZE.  Resize the data area of a chix.
/
/    chxresize (x, newtype, newmax);
/
/    Parameters:
/       Chix  x;
/       int   newtype;
/       long  newmax;
/
/    Purpose:
/       Resize the data area of chix X, to (at least) NEWMAX characters of
/       type NEWTYPE.  The old data is copied (and converted, if necessary)
/       into the new data area.
/
/       The new data is at least as large, in size and type, as the old
/       data area, regardless of the values of NEWTYPE and NEWMAX.
/
/    How it works:
/       Chxresize allocates a new data area with chxmem(), copies the
/       old data into it, resets X's data area pointer to point to the
/       new area, and frees the old data area.
/
/       Reasonable values are enforced for NEWTYPE and NEWMAX.
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions:  x is not a proper chix.
/                       NEWMAX is too big (> 32760)
/  
/    Side effects:      none
/
/    Related functions:
/
/    Called by:  chx...() functions only, never by application.
/
/    Operating system dependencies:
/       Uses sysfree() to free memory allocated by sysmem().
/
/    Known bugs:      none
/
/    Home:  chx/chxresiz.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:36 New function. */
/*: CR 10/07/91 17:38 Always expand by at least 1.5 old max. */
/*: CR  1/03/92 13:22 Limit resizing to ALLCHARS max. */
/*: CR 12/11/92 11:05 chxcapc last arg is long. */

#include <stdio.h>
#include "chixdef.h"

FUNCTION  chxresize (x, newtype, newmax)
   Chix   x;
   int    newtype;
   long   newmax;
{
   char  *p0, *p1;
   long   expand;

   ENTRY ("chxresize", "");

   /*** Input verification. */
   if (NOT chxcheck (x, "chxresize"))  RETURN (0);
   if (USE_THIN_CHIX)        newtype = THIN;
   if (newtype < THINNEST)   newtype = THINNEST;
   if (newtype > WIDEST)     newtype = WIDEST;
   if (newtype < x->type)    newtype = x->type;
   if (newmax  < 2)          newmax  = 2;

   /*** Always expand to at least 1.5 times old max len. */
   expand = (3L * x->maxlen) / 2L;
   if (newmax < expand)                newmax = expand;
   
   /*** P0 is the "old" character buffer, P1 the "new" buffer. */
   p0 = x->p;
   p1 = chxmem ( (newmax+1) * newtype, x->name);

   /*** Copy (and promote if necessary) the characters from P0 to P1. */
   chxcapc (p1, newtype, p0, x->type, -1L);

   x->maxlen = newmax;
   x->type   = newtype;
   x->p      = p1;
   sysfree (p0);

   /*** Warn user if this chix is resized too often. */
   if (++x->resize > RESIZE_LIM) {
      chxerror (CXE_RESIZE, "chxresize", x->name);
      x->resize = 0;
   }

   RETURN  (1);
}
