
/*** CHXSCAN.  Scan across parts of two chix strings until they are not equal.
/
/    n = chxscan (ap, atype, bp, btype, enda, endb);
/
/    Parameters:
/       long  n;       (number of characters scanned)
/       char *ap;      (pointer to first character in A chix)
/       int   atype;   (type of A chix)
/       char *bp;      (pointer to first character in B chix)
/       int   btype;   (type of B chix)
/       long *enda;    (long value of not-equal character in A)
/       long *endb;    (long value of not-equal character in B)
/
/    Purpose:
/       Chxscan() is a chix service function, used by higher-level
/       chix functions such as chxeq().  It compares two chix strings,
/       finds where they differ, and returns information about where
/       and how they differ.
/
/    How it works:
/       Chxscan() scans and compares the text of two chix, pointed to by AP 
/       and BP.  (It uses the types ATYPE and BTYPE to automatically promote
/       individual characters as needed so that they can be compared.)
/       It stops when it finds a difference (or at the end of the chix if
/       they are identical).
/
/       Because chxscan() is an internal chix function, it does not do
/       any validation of the input arguments.
/
/    Returns: N, the index of the differing character.  (i.e., if the 3rd
/       characters differ, chxscan() returns 2.)
/
/       Also puts the value of the N'th A character in ENDA as a long,
/       and  puts the value of the N'th B character in ENDB.
/
/    Error Conditions:  none
/  
/    Side effects:      none
/
/    Related functions:
/
/    Called by:  chx...() functions only, never by application.
/
/    Operating system dependencies:
/
/    Known bugs:
/       Limitation: requires that a character in a chix be no more than 
/       sizeof(long) bytes, typically 4.
/
/    Home:  chx/chxscan.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:36 New function. */
/*: CR 12/11/92 13:04 Returns long. */

#include <stdio.h>
#include "chixdef.h"

FUNCTION  long chxscan (ap, atype, bp, btype, enda, endb)
   char  *ap, *bp;
   int    atype, btype;
   long  *enda, *endb;
{
   long   a, b, n;
   int    i;

   ENTRY ("chxscan", "");

   for (n=0;   (1);   ++n) {
      a = *ap++ & 0xFF;
      for (i=1;   i<atype;   ++i)  { a = a << 8;   a += *ap++ & 0xFF; }
      b = *bp++ & 0xFF;
      for (i=1;   i<btype;   ++i)  { b = b << 8;   b += *bp++ & 0xFF; }

      if (a != b  ||  (a==0L  &&  b==0L))  break;
   }
   *enda = a;
   *endb = b;
   RETURN (n);
}
