/*** CHXTOKEN.  Get N'th token out of a line.
/
/    i = chxtoken (word, rest, n, line);
/
/    Parameters:
/       int   i;     (starting position of found token, or -1 if fail)
/       Chix  word;  (Put N'th token here)
/       Chix  rest;  (Put N'th token through end of line here)
/       int   n;     (Desired token.  First token is #1. )
/       Chix  line;  (Line of text being scanned)
/
/    Purpose:
/       Extract single words or "tokens" from a chix.  Tokens are
/       sequences of characters separated by any number of blanks.
/       Blanks are characters that have been assigned the SC_BLANK
/       property by chxspecial().
/
/    How it works:
/       Scan LINE for the N'th token.  If WORD is not nullchix, put
/       that token into WORD.  If REST is not nullchix, copy all of
/       line, from the N'th token on, into REST.
/
/       The first token in a line is token number 1.  If there is no
/       N'th token, WORD and REST are cleared.
/
/    Returns: -1 on failure
/             starting position of N'th token
/
/    Error Conditions: LINE not chix.  REST not nullchix and not chix,
/       WORD not nullchix and not chix.  Complains via chxerror().
/  
/    Side effects:
/
/    Related functions: chxspecial()
/
/    Called by:  application
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  chx/chxtoken.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:36 New function. */
/*: CR  7/28/91 15:48 chxalloc() first argument is long. */
/*: CR 10/18/91 14:22 Return starting position of token. */
/*: JV 10/15/92 11:02 Actually free BLANKS. */
/*: JV 10/16/92 09:24 Make BLANKS static, don't free it. */
/*: CL 12/11/92 11:13 Long Chxcatsub args. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */
/*: CR 12/17/92 12:19 Return -1 on bad args. */

#include <stdio.h>
#include "chixdef.h"
#include "done.h"

FUNCTION  chxtoken (word, rest, n, line)
   Chix   word, rest, line;
   int    n;
{
   static Chix blanks = nullchix;
   long   c, b;
   int    j, w, start, between, space, line_type, success;
   long   i;

   ENTRY ("chxtoken", "");

   /*** Input validation. */
   if (word != nullchix  &&  NOT chxcheck (word, "chxtoken"))  RETURN (-1);
   if (rest != nullchix  &&  NOT chxcheck (rest, "chxtoken"))  RETURN (-1);
   if (                      NOT chxcheck (line, "chxtoken"))  RETURN (-1);

   /*** Get the list of "blank" characters.  If it's empty, use the
   /    standard ascii blank, ' '. */
   if (blanks == nullchix) blanks = chxalloc (16L, THIN, "chxtoken blanks");
   chxspecial (SC_GET, SC_BLANK, blanks);
   if (chxvalue (blanks, 0L) == 0L)  chxcatval (blanks, THIN, (long) ' ');

   if (word != nullchix)  chxclear (word);
   if (rest != nullchix)  chxclear (rest);

   /*** BETWEEN means we are between tokens (in the white space).   START
   /    is the starting position of the N'th token.  W is the current word
   /    number we are on. */
   between =  1;
   start   = -1;
   w       =  0;

   /*** Scan characters C in LINE one at a time... */
   line_type = chxtype (line);
   for (i=0;   (c = chxvalue (line, i)) != 0L;   ++i) {

      /*** Is this character a SPACE? */
      for (space=j=0;   (b = chxvalue (blanks, L(j))) != 0L;   ++j) {
         if (c == b)  { space = 1;   break; }
      }

      if (space) {
         if (w >= n)  break;
         between = 1;
      }

      /*** Not a space... */
      else {
         if (between  &&  ++w == n)          start = i;
         if (w == n   &&  word != nullchix)  chxcatval (word, line_type, c);
         between = 0;
      }
   }

   if (start < 0)  DONE (-1);
   if (rest != nullchix) {
      chxclear  (rest);
      chxcatsub (rest, line, L(start), ALLCHARS);
   }

   DONE (start);

done:
   RETURN (success);
}
