/*** DOMENU.   Handle MENU processing.
/
/    domenu (command, cname, next)
/
/    Parameters:
/       Chix   command    (command line, e.g. "_callmenu Name v0")
/              cname      (current conference)
/              next       (returned name of 'next' conference to join)
/
/    Purpose:
/      DOMENU is called with the entire COMMAND line, and the name of
/      the current conference (CNAME).   It processes the menu specified
/      on the command line, and returns when the user has finished with
/      that menu.
/  
/    How it works:
/      Processing the menu means repeatedly executing the menu-start
/      macro, prompting the user for a choice, and executing the macro
/      that corresponds to each choice.  The macro(s) are executed by
/      calling menu_command(), which calls interpret(), which may
/      recursively call domenu().
/  
/    Returns:
/      DOMENU returns under three circumstances, with the return codes shown:
/  
/      0: interpret() saw a JOIN or STOP command.  The name of the next
/         conference is returned in NEXT.
/  
/      1: a RETURN was encountered that pops back one (and only one) level;
/         or, an invalid menu name was given on the command line.
/  
/      2: a RETURN was encountered that may pop back multiple levels.
/  
/    Error Conditions:
/  
/    Side Effects:
/  
/    Related to:
/  
/    Called by:
/  
/    Home: cmd2/domenu.c
/ */

/*** Copyright (C) 1989-1992 Camber-Roth.  All rights reserved. */

/*: CR  9/08/89 15:53 New function. */
/*: CR 11/14/89 18:44 Return (1) if dictionary changes! */
/*: CR 11/21/89 16:28 Use sysprompt() instead of sysread(). */
/*: JV  3/02/90 10:51 Moved definition of menu_level to globals.c */
/*: CR  2/01/90 10:21 If no command escaping, forbid it from menu prompt. */
/*: CR  3/21/90 17:08 Add 4th arg to sysprompt(). */
/*: CR  9/07/90 13:14 Process commands from "<file" via menu_command(). */
/*: CR 10/07/90 14:52 Expand line[] and word[] to 202 chars. */
/*: JV  4/07/91 12:00 Modified since last carvmark, reason unknown. */
/*: CR  6/11/91 12:41 Use mai_Ajoin instead of stopcoms for STOP. */
/*: CR  9/13/91 21:40 Sysbrkclear() before executing SM part. */
/*: JX  2/22/91 15:00 Chixify. */
/*: JX  5/26/92 17:23 Fixify. */
/*: CX  6/05/92 15:02 Replace sysprompt() with readprompt(). */
/*: CR  4/18/92 18:03 Remove "!" command escape checking. */
/*: CR  6/09/92 15:26 Add stuff for "_callmenu X v0 <file". */
/*: CR  6/22/92 17:58 Must unit_view XIMF after parse_input(). */
/*: CR  7/09/92 14:43 Allow for preloading from XIMF. */
/*: CI 10/06/92 17:22 Chix Integration. */
/*: TP 10/19/92 11:29 Replace unit_read() with input_control().  */
/*: CR 10/28/92 16:19 Update u_menu, u_keyb, u_comm each time around. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CL 12/10/92 17:06 Long ascofchx args. */
/*: CL 12/11/92 11:13 Long Chxcatsub args. */
/*: CR  2/08/93 14:53 chxfree s! */
/*: CN  3/17/93 16:25 readprompt()==0 => hangup! */
/*: CN  4/26/93 13:50 Always close XIMF at end of top-level menu. */
/*: CK  9/21/93 23:06 Add 'eot' arg to readprompt() call. */
/*: CP 11/23/93 13:25 Add debugging for DEB_MENU. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "inputcon.h"

extern struct screen_template used;
extern struct flag_template   flag;
extern union null_t  null;
extern Userreg       thisuser;
extern Chix          memvar[];
extern int           menu_level;
extern int           debug;

FUNCTION  domenu (command, cname, next)
   Chix   command, cname, next;
{
   static Chix stop;
   int    count, which, loop, dictionary;
   int    u_comm, u_keyb, u_menu;
   char   sm_name[40], ch_name[40], rp_name[40], mm_name[40],
          basechar[100], wordchar[100];
   Chix   sm_ptr, ch_ptr, rp_ptr, mm_ptr, sm_copy;
   Chix   s;
   Chix   format, vnumchx, mess, line, var, macro, word, base;
   int    action=1, preload;
   long   vnum, repeat;

   ENTRY ("domenu", "");

   s       = chxalloc (40L,  THIN, "domenu s");
   base    = chxalloc (40L,  THIN, "domenu base");
   word    = chxalloc (100L, THIN, "domenu word");
   mess    = chxalloc (100L, THIN, "domenu mess");
   line    = chxalloc (100L, THIN, "domenu line");
   macro   = chxalloc (100L, THIN, "domenu macro");
   sm_ptr  = chxalloc (100L, THIN, "domenu sm_ptr");
   sm_copy = chxalloc (100L, THIN, "domenu sm_copy");
   ch_ptr  = chxalloc (100L, THIN, "domenu ch_ptr");
   rp_ptr  = chxalloc (100L, THIN, "domenu rp_ptr");
   mm_ptr  = chxalloc (100L, THIN, "domenu mm_ptr");
   format  = chxalloc (100L, THIN, "domenu format");
   vnumchx = chxalloc (100L, THIN, "domenu vnumchx");

   /*** Get the base name of the menu. */
   dictionary = thisuser->dict;
   if (chxtoken (base, nullchix, 2, command) == -1) {
      mdwrite (XWER, "mnu_Enomenu", null.md);
      action = 1; goto done;
   }

   /*** Pre-load the next line from the menu input redirection file.
   /    If there is no next line, revert to keyboard input. */
   u_comm = input_control (IC_ISUNIT | IC_COMM, nullchix);
   u_keyb = input_control (IC_ISUNIT | IC_SESS, nullchix);
   u_menu = input_control (IC_ISUNIT | IC_MENU, nullchix);
   preload = 0;
   if (u_comm == u_keyb  &&  u_menu == XIMF) {
      if (input_control (IC_GETEOF | IC_MENU, line))  preload = 1;
      else u_menu = input_control (IC_ISUNIT | IC_MENU, nullchix);
   }

   /*** Test for menu input redirection. */
   if (u_menu == u_keyb) {
      if (parse_input (command, XIMF) == XIMF) {
         if (unit_view (XIMF))
                 input_control (IC_RED | IC_MENU | XIMF, nullchix);
         else    unit_unlk (XIMF);
         u_menu = input_control (IC_ISUNIT | IC_MENU, nullchix);
      }
   }

   /*** Get the memory variable index VNUM.  Make sure that variable
   /    has memory allocated to it.  VAR points to variable. */
   if (chxtoken (word, nullchix, 3, command) != -1) {
      chxcatsub (vnumchx, word, L(1), L(10));
      chxnum    (vnumchx, &vnum);
   }
   else  vnum = -1;

   if (vnum >  9)  vnum = -1;
   if (vnum >= 0  &&  memvar[vnum] == null.chx) {
      if ( (memvar[vnum] = chxalloc (100L, THIN, "memvar")) == null.chx)
         vnum = -1;
   }
   var = (vnum >= 0 ? memvar[vnum] : line);

   /*** Get the SM, CH, RP, and MM strings from the dictionary. */
   ascofchx (basechar, base, 0L, L(98));
   strconc  (sm_name, basechar, "SM");
   strconc  (ch_name, basechar, "CH");
   strconc  (rp_name, basechar, "RP");
   strconc  (mm_name, basechar, "MM");
   mdstr (sm_ptr, sm_name, null.md);
   mdstr (ch_ptr, ch_name, null.md);
   mdstr (rp_ptr, rp_name, null.md);
   mdstr (mm_ptr, mm_name, null.md);

   while (chxalter (sm_ptr, 0L, chxquick("\n", 0), chxquick(";", 1))) ;

   /*** Push the stack on the menu level depth. */
   ++menu_level;

   /*** Repeat the instructions for this menu until done. */
   repeat = count = 0;
   while (1) {

      /*** If we're reading input commands from a "<file", interpret
      /    the commands directly via menu_command().  If we've run out
      /    of text in the file, return to getting input from the user. */
      u_comm = input_control (IC_ISUNIT | IC_COMM, nullchix);
      u_keyb = input_control (IC_ISUNIT | IC_SESS, nullchix);
      if (u_comm != u_keyb) {
         if (input_control (IC_GETEOF | IC_COMM, line)) {
            if (menu_command (line, cname, next, base, menu_level, &action))
                 continue;
            else break;
         }
         else u_comm = input_control (IC_ISUNIT | IC_COMM, nullchix);
      }

      /*** Reading from the XIMF file instead of the keyboard?  Read
      /    the input *now*, before we decide whether to execute the
      /    SM macro.  That way, if the read fails, we reset to read
      /    from the keyboard, and know to execute the SM macro. */
      u_menu = input_control (IC_ISUNIT | IC_MENU, nullchix);
      if (u_menu == XIMF) {
         if (NOT preload  &&  NOT input_control (IC_GETEOF | IC_MENU, line))
            u_menu = input_control (IC_ISUNIT | IC_MENU, nullchix);
         preload = 0;
      }

      /*** On the first time around, or when the repeat count comes up,
      /    execute the SM macro.  Use the copy of SM, since interpret()
      /    trashes the command string. */
      if (++count >= repeat  &&  u_comm == u_keyb  &&  u_menu == u_keyb) {
         chxcpy (sm_copy, sm_ptr);
         sysbrkclear();
         if (NOT menu_command (sm_copy, cname, next, base,
                                        menu_level, &action))  break;
         count = 0;
      }
      if (dictionary != thisuser->dict) { action = 1;   break; }

      /*** Otherwise, read a line from the keyboard.  STOP if error occurs. */
      u_keyb = input_control (IC_ISUNIT | IC_SESS, nullchix);
      u_menu = input_control (IC_ISUNIT | IC_MENU, nullchix);
      if (u_menu == u_keyb) {
         used.lines = used.width = 0;
         sysbrkclear();
         if (NOT readprompt (IC_MENU, null.chx, null.chx, line, 0, 
                                                          thisuser->eot)) {
            mdtable  ("mai_Ajoin", &stop);
            chxtoken (next, null.chx, 1, stop);
            action = 0;
            break;
         }
      }
      if (debug & DEB_MENU) {
         unit_write (XWTX, CQ("Menu read: '"));
         unit_write (XWTX, line);
         unit_write (XWTX, CQ("'\n"));
      }
      chxtoken  (word, nullchix, 1, line);
      jixreduce (word);

      /*** Depending on what the user typed, select WHICH macro is going
      /    to be executed, and set the memory variable appropriately. */
      if      (sysbrktest())   { which = 1;  chxclear (var);  sysbrkclear(); }

      else if (EMPTYCHX(line))  { which = 2;  chxclear (var); }

      else if ( (which = tablematch (ch_ptr, word)) >= 0) {
         which += 4;
         if ( (chxtoken (word, s, 2, line)) != -1)  chxcpy   (var, s);
         else                                       chxclear (var);
      }

      else                     { which = 3;   chxcpy (var, line); }

      /*** Get WHICH macro from the dictionary, and process it! */
      chxtoken (word, nullchix, which, mm_ptr);
      ascofchx (wordchar, word, 0L, L(98));
      mdstr (macro, wordchar, null.md);
      while (chxalter (macro, 0L, chxquick("\n", 0), chxquick(";", 1))) ;
      loop = menu_command (macro, cname, next, base, menu_level, &action);

      if (NOT loop)                                     break;
      if (dictionary != thisuser->dict) { action = 1;   break; }

      /*** Get repeat factor. */
      chxtoken (word, null.chx, which, rp_ptr);
      chxnum   (word, &repeat);
   }

   --menu_level;

   if (menu_level == 0  &&
       input_control (IC_ISUNIT | IC_MENU, nullchix) == XIMF)
       input_control (IC_CLOSE  | IC_MENU, nullchix);

   /*** In order to 'break' out the loop, action must have been set
   /    by now. */
done:
   chxfree (base);
   chxfree (line);
   chxfree (mess);
   chxfree (word);
   chxfree (macro);
   chxfree (sm_ptr);
   chxfree (ch_ptr);
   chxfree (rp_ptr);
   chxfree (mm_ptr);
   chxfree (format);
   chxfree (vnumchx);
   chxfree (sm_copy);
   chxfree (s);
   RETURN  (action);
}
