/*** SEARCHPERSON.   Search a set of people for some words.
/
/    ok = searchperson (red, person, forlist, needed, confnum, brief);
/
/    Parameters:
/       int       ok;        (success?)
/       Redirect *red;       (redirection info; write results here)
/       Chix      person;    (partial name(s) of people to search)
/       Namelist  forlist;   (namelist of words to search for)
/       int       needed;    (negative # of words needed to make a match)
/       int       confnum;   (current conference number)
/       int       brief;     (brief modifier?  (names only?) )
/
/    Purpose:
/       Search the brief introduction of a set of people for some words.
/       Display either the names or the full intro of the people that match.
/
/    How it works:
/       Searches all the people who match PERSON.  (If PERSON is "*", 
/       searches everyone who is a member of conference CONFNUM.)
/
/       For each such person, search the intro text for the words in
/       the namelist FORLIST.  NEEDED is the (negative) number of words
/       in FORLIST that are needed to be found to consider this a match.
/       (NEEDED is usually -1, for an "or" search, or -size of FORLIST,
/       an "and" search.)
/
/       If BRIEF is true, only the names of the people who match are
/       displayed.  Otherwise, the string sch_Fperson controls what
/       information about the person is displayed.
/
/    Returns: 1 on success
/             0 if error opening conference membership file ("*" = all)
/
/    Error Conditions:
/  
/    Side effects: scribbles in text buffer.  Later (chix) versions
/       should probably have their own temporary area.
/
/    Related functions:
/
/    Called by:  search()
/
/    Operating system dependencies: None.
/
/    Known bugs:
/
/    Home:  cmd2/searchper.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR  8/02/92 20:14 New function. */
/*: CP  8/30/93 15:25 Replace 'out' arg with 'red'. */

#include <stdio.h>
#include "caucus.h"
#include "redirect.h"

extern Textbuf          tbuf;
extern union  null_t    null;

FUNCTION  searchperson (red, person, forlist, needed, confnum, brief)
   int       needed, confnum, brief;
   Redirect *red;
   Chix      person;
   struct    namlist_t *forlist;
{
   static Chix s_wildcard;
   Chix   wildcard, id;
   struct namlist_t *this, *namdir, *wholist;
   int    success;
   int    ok, i;
   struct namlist_t *nluserid(), *nlnode();

   ENTRY ("searchperson", "");

   wildcard = chxalloc (10L, THIN, "searchperson wildcard");
   id       = chxalloc (20L, THIN, "searchperson id");

   /*** WILDCARD is for matching PERSON "*". */
   mdstr (wildcard, "sch_Kwildname", &s_wildcard);

   /*** If a name(part) was supplied, search the namelist of people
   /    who could be PERSON.  Searchuser() does most of the actual
   /    searching work. */
   if (NOT chxeq (person, wildcard)) {
      wholist = nluserid (person, 0);
      for (this=wholist->next;   this!=null.nl;   this = this->next) {
         if (sysbrktest())  break;
         searchuser (red->unit, this->str, forlist, needed, brief);
      }
      nlfree (wholist);
   }

   /*** Otherwise, (PERSON "*"), search everyone in this conference.
   /    Load the directory for the membership file, then load the
   /    the individual membership namepart files, and scan the people
   /    mentioned in there, one at a time. */
   else {
      if (NOT unit_lock (XCMD, READ, L(confnum), 0L, 0L, 0L, nullchix)) FAIL;
      namdir = nlnode(4);
      ok = loadnmdir (XCMD, namdir);
      unit_unlk (XCMD);
      if (NOT ok)     { nlfree (namdir);        FAIL; }
   
      /*** Load each namepart file in turn, and search each user therein. */
      sysbrkclear();
      for (this=namdir->next;   this != null.nl;   this = this->next) {
         if (sysbrktest())    break;
         if (this->num <= 0)  continue;
         if (NOT unit_lock (XCMN, READ, L(confnum), L(this->num), 0L,
                                 0L, nullchix))     continue;
         if (NOT loadnpart (XCMN, this->num, (Textbuf) NULL))
                             { unit_unlk (XCMN); continue; }
         unit_unlk (XCMN);
   
         /*** Each line that has a userid is a person.  Search him/her. */
         for (i=0;   i<tbuf->l_used;   ++i) {
            if (sysbrktest())    break;
            if (chxtoken (id, nullchix, 2, tbuf->line[i]) == -1)  continue;
            searchuser (red->unit, id, forlist, needed, brief);
         }
      }
      nlfree (namdir);
   }
   SUCCEED;

done:
   chxfree (wildcard);
   chxfree (id);
   RETURN  (success);
}

/*** Search engine.  Search person ID. */

FUNCTION  searchuser (out, id, forlist, needed, brief)
   int    out, needed, brief;
   Chix   id;
   struct namlist_t *forlist;
{
   static Chix s_format;
   Chix        format, result, empty;
   int         success;
   Userreg     user;
   Userreg     make_userreg();
   Chix        person_format();

   ENTRY ("searchuser", "");

   format = chxalloc (100L, THIN, "searchuser format");
   empty  = chxalloc (100L, THIN, "searchuser empty");

   /*** Load the registration information about person ID.
   /    Searchintro() tells us if the words in FORLIST
   /    are actually found in ID's intro.  Display BRIEF
   /    or long results. */
   user = make_userreg();
   if (NOT loadreg (id, user, NULLSITE))         FAIL;

   if (NOT searchintro (user, forlist, needed))  FAIL;

   if (brief) { unit_write (out, user->name);    unit_write (out, CQ("\n")); }
   else {
      mdstr (format, "sch_Fperson", &s_format);
      result = person_format (format, user->name, user->id, empty,
                              user->phone, user->briefs);
      unit_write (out, result);
      chxfree (result);
   }
   SUCCEED;

done:
   chxfree (format);
   chxfree (empty);
   free_userreg (user);

   RETURN (success);
}
