/*** ALLOWED_ON.  Is a user ALLOWED_ON this conference?
/
/    ok = allowed_on (userid, groups, cnum);
/
/    Parameters:
/       int      ok;        (Success?)
/       Chix     userid;    (userid or "@host" host name)
/       Namelist groups;    (history of previous matches against GROUP files)
/       int      cnum;      (conference number)
/
/    Purpose:
/       Determine if a userid (or host) is allowed to join (subscribe to)
/       a conference, and if so, at what security level?
/
/    How it works:
/       Allowed_on() examines the XCUS file for conference CNUM to see
/       if USERID is included or excluded from the list of valid users.
/
/       If the XCUS file "<" includes any groups files, the history
/       of matches found in each groups file is recorded (and re-used,
/       where appropriate) in the namelist GROUPS.
/
/       Allowed_on() uses a caching mechanism that must be flushed under
/       certain circumstances.  To flush its cache, call allowed_on() with
/       CNUM = -1.
/
/       Two examples of places the cache must be flushed:
/       1) After the userlist is changed (cus_user())
/       2) The user is about to join a conf (caucus())
/
/       The static CONFNUM is initialized to -1.  Thereafter it is either
/       1) =-1: ULIST does not point to an allocated Chix;
/       2) > 0: ULIST *does* point  to  an allocated Chix;
/
/    Returns: 3 if USERID has organizer privileges
/             2 if USERID may join as a regular member
/             1 if USERID may join as a read-only member
/             0 if USERID may not join
/            -1 if conference CNUM does not exist
/            -2 on any other error
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  XFU, Caucus
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  main/allowedo.c
/ */

/*** Copyright (C) 1985-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:47 Source code master starting point */
/*: CR  4/13/89 11:47 Take wildcard character from dictionary. */
/*: CW  5/18/89 14:53 Add XW unit codes. */
/*: CR  9/15/89 16:46 Add ":readonly" class of users. */
/*: CR  7/13/90 14:10 Add ":organizer" class. */
/*: CR  7/17/90 11:46 Add group matching, via groupmatch() call. */
/*: CR  9/12/90 13:39 Fix interaction between :readonly & :organizer. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/25/91  1:09 Trim trailing blanks from entries in XCUS! */
/*: CR  6/20/91 16:19 Use equal_id() to parse host information. */
/*: CR  7/24/91 21:45 Remove unused variables. */
/*: CR 10/21/91 18:15 Add 3rd arg, change 1st arg & return codes. */
/*: CR 10/30/91 17:32 Use ulist's. */
/*: JX  3/04/92 11:20 Chixify. */
/*: DE  3/26/92 16:37 Patched chxindex w/ 0, */
/*: DE  5/26/92 15:19 Fix chixification */
/*: JX  8/10/92 10:54 Add cache. */
/*: JX 10/01/92 13:45 Protect upper-case for group file names. */
/*: CR 12/02/92 14:34 Use ()'s for confnum=-1. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern union  null_t    null;
extern Chix             ss_inred;

#define FLUSH -1

FUNCTION  allowed_on (userid, groups, cnum)
   Chix   userid;
   struct namlist_t  *groups;
   int                cnum;
{
   static int   confnum = (-1);
   static Chix  inextab;
   static Chix *ulist;
   Chix   wildcard, name, line, group_file;
   int    errors, index, subp, allowed, match, success;
   long   pos;
   Chix  *make_ulist();

   ENTRY ("allowed_on", "");

   line     = chxalloc (100L, THIN, "allowed line");
   name     = chxalloc (80L,  THIN, "allowed name");
   wildcard = chxalloc (20L,  THIN, "allowed wildcard");
   group_file =  chxalloc (40L,  THIN, "allowed group_file");

   if (cnum == FLUSH) {                     /* Flush cache. */
      if (confnum != -1) free_ulist (ulist);
      confnum = -1;
      SUCCEED;
   }

   /*** Allocate a ULIST to hold the XCUS userlist data. */
   if (confnum == -1) ulist = make_ulist();
   if (ulist == (Chix *) NULL)  DONE (-2);

   /*** Load the ULIST from the XCUS file. */
   if (confnum != cnum)
      if (NOT load_ulist (XCUS, cnum, ulist, &errors)) 
         DONE (-1);

   /*** Get the wildcard string for use in matching userids. */
   mdtable  ("mai_Ainex", &inextab);
   if (chxtoken (wildcard, null.chx, 1, inextab) == -1)
      chxcpy (wildcard, CQ("*"));

   /*** Scan each sub-paragraph in the ULIST. */
   allowed = 0;
   for (index=1;   index < MAXULIST;   ++index) {
      if (ulist[index] == nullchix)  continue;

      /*** For this sub-paragraph, scan all the data lines.  Is there
      /    a match? */
      subp = (index + 3) % 4;
      for (pos=0;   chxnextline (line, ulist[index], &pos);  ) {
         if (chxvalue (line, 0L) == L('#'))  continue;
         chxcpy (group_file, line);              /* Preserve upper-case. */
         jixreduce    (line);
         chxsimplify  (line);
         match = 0;
   
         /*** If "<file" redirection, see if the groups file contains 
         /    a match. */
         if (chxindex (group_file, 0L, ss_inred) == 0) {
            chxclear  (name);
            chxcatsub (name, group_file, chxlen(ss_inred), chxlen(group_file));
            match = groupmatch (name, groups, userid, wildcard);
         }
   
         /*** Otherwise, this is a userid.  Was it a MATCH?  Allow for a
         /    wildcard at the end, e.g. "roth" matches "ro*" and everyone
         /    matches "*". */
         else  match = equal_id (line, userid, wildcard);
   
         /*** If this was a MATCH, set ALLOWED according to category! */
         if (match) {
            allowed = 3 - subp;
            break;
         }
      }
   }
   success = allowed;

done:
   chxfree (line);
   chxfree (name);
   chxfree (wildcard);
   chxfree (group_file);

   RETURN (success);
}
