/*** MAKE_CONF.  Create the database structures that "are" a conference.
/
/    ok = make_conf (confname, orgname, listed, library, userlist, error);
/
/    Parameters:
/       Chix  confname;          Name of the conference. Must be unique.
/       Chix  orgname;           Name of the primary organizer.
/       int   listed;            Conf is Open=1, Closed=0.
/       int   library;           New Conf=0, New File Library=1
/       int   userlist;          Unit number containing userlist.
/       int  *error;             Place for storing error code.
/
/    Purpose:
/       Both caumnt_x and netmaint_x provide a method for the Caucus manager
/       to create a new conference or file library.  This function takes the
/       information they get/provide and create a new, empty conference or 
/       file library based on that information.
/
/    How it works:
/ 
/       Note: ORGNAME may be a null string, but cannot point to garbage.
/             MAKE_CONF assumes that oknewconf() has been called.
/
/       Format of info lines in XSCD:
/          A#001[#B[#C]]
/       where: A is the          "listed"  flag, o=open,  c=closed
/              B is the optional "local"   flag, L=local, T=true
/              C is the optional "library" flag, C=conference, L=library
/
/    Returns: 1 on success.
/             0 on failure.
/
/    Error Conditions:
/ 
/    Side effects:
/       MAKE_CONF deletes the file indicated by USERLIST.
/
/    Called by: costart(), do_conf().
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xmaint/makeconf.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: JV 11/06/91 09:24 New function. */
/*: CR 11/18/91 17:20 Add CNUM argument to modnames() call. */
/*: WC 11/29/91 21:14 Add 3rd arg to cnumber(). */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 long.*/
/*: JV  1/03/92 15:10 Declare strcpy. */
/*: CR  7/26/92 15:45 Use cst_Tchgstat to control conf chg perms. */
/*: JX  3/09/92 17:45 Chixify. */
/*: DE  5/26/92 13:04 Fixed inconsistant args */
/*: JX  9/22/92 14:48 Fixify. */
/*: CI 10/07/92 17:44 Chix Integration. */
/*: JV  5/24/93 19:11 Create XACD (Attachment directory). */
/*: CP  5/06/92 12:00 Add initialization of XCED/XCEN. */
/*: CP  5/06/92 12:41 Add 0L as 'item' arg to modnames() call. */
/*: CN  5/16/93 13:27 Add 'cnum' arg to initnmdir(). */
/*: CP  7/08/93 16:05 Integrate 2.5 changes. */
/*: JZ  9/28/93 14:52 Add support for file libraries. */
/*: JZ 10/18/93 10:48 Fix chxformat call. */

#include <stdio.h>
#include "unitcode.h"
#include "handicap.h"
#include "derr.h"
#include "conf.h"
#include "chixuse.h"
#include "namelist.h"
#include "null.h"
#include "done.h"

extern union null_t     null;
extern short            confnum;

FUNCTION  make_conf (confname, orgname, listed, library, unit, error)
   Chix  confname,  orgname;
   int   listed, library, unit, *error;
{
   char   name[160];
   Chix   str, old, new;
   struct namlist_t *add, *del, *nlnode();
   int    v, success;

   ENTRY ("make_conf", "");

   old = chxalloc (80L, THIN, "old");
   str = chxalloc (80L, THIN, "str");
   new = chxalloc (80L, THIN, "new");

   /*** Create the new conference name and number. */
   chxcpy (str, CQ("!confs"));
   confnum = cnumber (str, LOCALCONF, &v) + 1;
   if (confnum < 1) {
      *error = DBADDB;
      FAIL;
   }
   chxformat (old, CQ("!confs o#%03d"), L(confnum-1), 0L, nullchix, nullchix);
   chxformat (new, CQ("!confs o#%03d"), L(confnum), 0L, nullchix, nullchix);
   chxclear  (str);
   chxformat (str, chxquick("%s %s#%03d#L",0),  L(confnum), 0L, confname,
              (listed  ? chxquick("o",1) : chxquick("c",2)));
   chxformat (str, chxquick("#%s",0), 0L, 0L,
              (library ? chxquick("L",1) : chxquick("C",2)), nullchix);
   add = nlnode(1);
   nladd (add, new, 0);
   nladd (add, str, 0);
   del = nlnode(1);
   nladd (del, old, 0);
   modnames (XSCD, XSCN, nullchix, confnum, 0L, add, del, NOPUNCT);
   nlfree (add); nlfree (del);

   /*** First try and make the conference directory... */
   if (NOT sysmkconf (confnum)) {
      *error = DMKDIR;
      FAIL;
   }

   /*** And if that succeeds, confirm that we can write files in it.
   /    Our first attempt is the XCUS user list file. */
   if (NOT copyfile (unit,0,0,0,nullchix, XCUS,0,0,0,nullchix)) {
      *error = DMKDIR;
      FAIL;
   }
   if (unit_lock (unit, WRITE, 0L, 0L, 0L, 0L, nullchix)) {
       unit_kill (unit);
       unit_unlk (unit);
   }

   /*** The rest of the code assumes that since we could lock & write
   /    one file, we can do the rest OK.  Probably not the greatest
   /    idea, but it has worked historically. */

   /*** Write the text of a simple welcome to the conference. */
   unit_lock (XCGR, WRITE, L(confnum), 0L, 0L, 0L, nullchix);
   unit_make (XCGR);
   mdwrite   (XCGR, "cst_Twelcome", null.md);
   unit_write(XCGR, confname);
   unit_write(XCGR, CQ("'\n("));
   if (NOT EMPTYCHX (orgname)) unit_write (XCGR, orgname);
   mdwrite   (XCGR, "cst_Tendwelc", null.md);
   unit_close(XCGR);
   unit_unlk (XCGR);

   /*** Write the text of a simple jointext file--what a user sees when
   /    considering first joining the conference. */
   unit_lock (XCIN, WRITE, L(confnum), 0L, 0L, 0L, nullchix);
   unit_make (XCIN);
   mdwrite   (XCIN, "cst_Tjointext", null.md);
   unit_close(XCIN);
   unit_unlk (XCIN);

   /*** Write a MASTER file that implies "no items". */
   unit_lock (XCMA, WRITE, L(confnum), 0L, 0L, 0L, nullchix);
   unit_make (XCMA);
   if (EMPTYCHX (orgname)) unit_write (XCMA, CQ("---@---"));
   else                    unit_write (XCMA, orgname);
   unit_write (XCMA, CQ("\n"));
   unit_write (XCMA, CQ("0 "));
   mdwrite    (XCMA, "cst_Tchgstat", null.md);
   unit_write (XCMA, CQ("\n"));

   unit_close (XCMA);
   unit_unlk  (XCMA);
 
   /*** Create the conference membership directory. */
   initnmdir (XCMD, XCMN, confnum, nullchix, 0L);
 
   /*** Create the conference title partfile directory. */
   initnmdir (XCTD, XCTN, confnum, nullchix, 0L);
 
   /*** Create the conference author-item partfile directory. */
   initnmdir (XCAD, XCAN, confnum, nullchix, 0L);
 
   /*** Create the conference Subject namelist files. */
   initnmdir (XCND, XCNN, confnum, nullchix, 0L);
 
   /*** Create the conference Alphabetized subject files. */
   initnmdir (XCOD, XCON, confnum, nullchix, 0L);
 
   /*** Create the "time item entered" namelist file. */
   initnmdir (XCED, XCEN, confnum, nullchix, 0L);

   /*** Create the conference Subject text file. */
   unit_lock (XCSF, WRITE, L(confnum), 0L, 0L, 0L, nullchix);
   unit_make (XCSF);
   unit_write(XCSF, CQ("#0 0\n"));
   unit_close(XCSF);
   unit_unlk (XCSF);
 
   /*** Create the TrueConf file. */
   unit_lock  (XCTC, WRITE, L(confnum), 0L, 0L, 0L, nullchix);
   unit_make  (XCTC);
   unit_write (XCTC, confname);   unit_write (XCTC, CQ("\n"));
   unit_close (XCTC);  unit_unlk (XCTC);

   /*** Create the Attachments directory. */
   unit_name (name, XACD, L(confnum), 0L, 0L, 0L, nullchix);
   sysmkdir  (name, 1);
   SUCCEED;

done:
   chxfree (old);
   chxfree (str);
   chxfree (new);
   RETURN (success);
}
