
/*** MDWRITE.   Write a text string from the Master Dictionary to unit OUT.
/
/    ok = mdwrite (out, name, save);
/
/    Parameters:
/       int     ok;      (success?)
/       int     out;     (unit code to write to)
/       char   *name;    (name of string to write)
/       Chix   *save;    (place to save memory-resident strings)
/
/    Purpose:
/       Look in the current dictionary for a string called NAME.  Write
/       it to unit OUT.
/
/    How it works:
/       If the string is marked memory-resident in the Master Dictionary,
/       and SAVE is not NULL, and *SAVE is NULL, then:
/         (a) allocate a chix.
/         (b) set SAVE to point to that chix.
/         (c) copy the string into that chix.
/
/       This also means that on future calls to mdwrite(), if *SAVE is already
/       set to point to the memory-resident copy of the string, then it
/       is written directly to OUT w/o going to the dictionary.
/
/       (For memory-resident strings, mdwrite() also remembers which dictionary
/       the string came from.  If the user switches dictionaries, mdwrite() will
/       take note and load the string from the new dictionary.)
/
/    Returns: 1 on success, 0 if no string NAME in the dictionary.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  inout/mdwrite.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:42 Source code master starting point*/
/*: AM  2/07/89 10:27 Jcaucus changes*/
/*: CW  6/16/89 16:30 Add sysbrktest() test. */
/*: CR 10/20/89 15:17 Modify sysbrktest() to check if XWTY. */
/*: CR 10/22/89 19:50 Use 2 bytes to save dic number in string. */
/*: JV 11/03/89 12:59 Use MDBUFSIZE for limit on mdbuffer. */
/*: CR 11/22/89 15:09 Fix off-by-one error in use of mdbuffer[]. */
/*: CR  8/12/90 19:18 Cast sysmem() call. */
/*: JV  7/22/90 18:09 Cast arg to sysmem to shut lint up. */
/*: CX 10/20/91 17:42 Chixify. */
/*: CX  5/09/92 23:04 Change to use sysmdstr(). */
/*: CL 12/11/92 15:52 Long sysmem arg. */

#include <stdio.h>
#include "caucus.h"
#include "diction.h"

extern union  null_t    null;
extern Userreg          thisuser;

FUNCTION  mdwrite (out, name, save)
   int    out;
   char  *name;
   Chix  *save;
{
   int    status;
   Chix   temp;

   ENTRY ("mdwrite", "");

   if (name == null.str)  RETURN (1);

   /*** If the string is already memory-resident, write it out. */
   if (save != null.md  &&  *save != null.chx) {
      if ((*save)->dic == thisuser->dict) {
         unit_write (out, *save);
         RETURN (1);
      }
      chxfree (*save);
      *save = null.chx;
   }

   temp = chxalloc (2L, THIN, "mdwrite temp");
   
   /*** Position to the desired text string. */
   if (NOT sysmdstr (name, thisuser->dict, temp, &status)) {
      unit_write (out, CQ("(no string for '"));
      unit_write (out, CQ(name));
      unit_write (out, CQ("')\n"));
      chxfree (temp);
      RETURN  (0);
   }

   unit_write (out, temp);

   /*** If we can make this string memory resident, do so. */
   if ( (status & MDRES)  &&  (save != null.md)) {
      *save = temp;
      (*save)->dic = thisuser->dict;
   }
   else  chxfree (temp);

   RETURN (1);
}
