
/*** MESS_FORMAT.   Display (part of) a message according to a format string.
/
/    more = mess_format (out, mnum, id, line, fstr, subject, new);
/
/    Parameters:
/       int    more;        (Success / Are there more message parts?)
/       int    out;         (unit number for output)
/       int    mnum;        (message to be displayed)
/       Chix   id;          (userid of message owner)
/       int   *line;        (start display at this line number)
/       Chix   fstr;     (format string)
/       Chix   subject;     (message subject)
/       int    new;         (is this message "new"?)
/
/    Purpose:
/       Display (on unit OUT) part of message MNUM,
/       according to the "tailorable format string" in FORMSTR.
/
/    How it works:
/       Starting at line number LINE of message MNUM of user ID,
/       display on unit OUT the text of FORMSTR, interpreting any
/       special "&" directives as follows:
/          &a      the message part author's name
/          &d      today's date
/          &h      the author's hostname
/          &m      the message number
/          &s      the message subject
/          &t      the current time
/          &u      the author's userid
/          &w      "new message" indicator
/          &x      the message text
/          &z      column position
/
/       If FORMSTR contains a "&x" directive, mess_format() reads
/       the message text from LINE up through the end of the current
/       message part.  LINE is incremented in the process so that
/       the caller knows how far we got.
/
/    Returns: 1 on success.
/             0 if &x directive read last line of last part of message.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  mess/messform.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR  7/22/92 19:02 New function. */
/*: CR 10/08/92 13:06 Chixified. */
/*: CR 10/11/92 22:29 Remove CQ from buildline() call. */
/*: CR 12/10/92 15:13 chxalter arg long. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */
/*: CR 12/15/92 15:16 Be paranoid about buildline(...,-1,...). */

#include <stdio.h>
#include "caucus.h"

extern union  null_t     null;

FUNCTION  mess_format (out, mnum, id, line, fstr, subject, new)
   int    out, mnum, *line, new;
   Chix   id, fstr, subject;
{
   int    success, word, shift;
   long   v, length, lastwordpos, t, pos;
   Chix   author, date, time, header, userid, host, temp, how_s, how_d,
          indent, newmsg;
   Chix   ptr;
   Chix   getmline(), messheader();

   ENTRY ("messformat", "");

   sysbrkclear();

   author = chxalloc (60L, THIN, "mess_form author");
   date   = chxalloc (60L, THIN, "mess_form date");
   time   = chxalloc (60L, THIN, "mess_form time");
   header = chxalloc (60L, THIN, "mess_form header");
   userid = chxalloc (60L, THIN, "mess_form userid");
   host   = chxalloc (60L, THIN, "mess_form host");
   temp   = chxalloc (60L, THIN, "mess_form temp");
   how_s  = chxalloc (60L, THIN, "mess_form how_s");
   how_d  = chxalloc (60L, THIN, "mess_form how_d");
   indent = chxalloc (60L, THIN, "mess_form indent");
   newmsg = chxalloc (60L, THIN, "mess_form newmsg");

   /*** Make sure message part exists, and fetch the header. */
   if ( (ptr = getmline (id, mnum, *line, 1)) == null.chx)   FAIL;

   /*** Pluck the author, date, etc. from the header. */
   if (NOT parse_preface (ptr, author, date, time, header))  FAIL;

   /*** Look for "(userid@host)" in the last word in AUTHOR, and pluck
   /    them out as well. */
   for (word=1;   chxtoken (userid, nullchix,   word, author) != -1;   ++word) ;
   lastwordpos =  chxtoken (userid, nullchix, --word, author);
   if (lastwordpos == -1)  FAIL;
   if (chxinside (userid, chxquick("(",0), temp, chxquick(")",1) ) >= 0 ) {
      chxalter (author, lastwordpos, userid, CQ(""));
      chxbreak (temp, userid, host, CQ("@"));
      if (NOT EMPTYCHX (host)) chxconc (host, CQ("@"), host);
   }
   chxtrim (author);


   /*** Loop through the format string, processing all "&x" directives,
   /    and displaying all other text as is.  */
   buildline (out, null.chx, 0L, 0L, BLINIT);
   success = 1;
   for (pos=0;   v = chxvalue (fstr, L(pos));  ) {

      /*** If no more "&"s left, display to the end of the string. */
      if (  (t = chxindex (fstr, pos, CQ("&"))) < 0) 
         { buildline (out, fstr, pos, ALLCHARS, BLADD);   break; }

      /*** Treat escaped "&"s as ordinary text to be displayed. */
      else if (chxvalue (fstr, L(t+1)) == '&') {
         buildline (out, fstr, pos, t, BLADD);
         pos = t + 2;
         continue;
      }

      /*** Remaining code in loop processes "&x" forms.   Start by 
      /    displaying all text before the "&". */
      if (t > pos)  buildline (out, fstr, pos, t-1, BLADD);

      /*** Parse out the string length, including "-" => left-justify, and
      /    make a sprintf format string to use to display the info. */
      pos = t + 1;
      length = chxlong (fstr, &pos);
      if (pos == t+1) {
         length = 0;
         chxcpy    (how_s, CQ("%s"));
         chxcpy    (how_d, CQ("%d"));
      }
      else {
         chxclear  (how_s);
         chxclear  (how_d);
         chxformat (how_s, CQ("%%%ds"), length, 0L, null.chx, null.chx);
         chxformat (how_d, CQ("%%%dd"), length, 0L, null.chx, null.chx);
      }
      
      /*** Decide what to display according to "x" code in "&x". */
      chxclear (temp);
      v = chxvalue (fstr, L(pos));
      switch (v) {

         case ('a'):       /* Author. */
            chxformat (temp, how_s, 0L, 0L, author, null.chx);   break;

         case ('d'):       /* Date. */
            chxformat (temp, how_s, 0L, 0L, date, null.chx);   break;

         case ('m'):       /* Mess #. */
            chxformat (temp, how_d, L(mnum), 0L, null.chx, null.chx);   break;

         case ('t'):       /* Time.   */
            chxformat (temp, how_s, 0L, 0L, time, null.chx);   break;

         case ('u'):       /* (userid). */
            chxformat (temp, how_s, 0L, 0L, userid, null.chx);   break;

         case ('H'):
            chxformat (temp, how_s, 0L, 0L, messheader(header), null.chx);
            break;

         case ('h'):
            if (NOT EMPTYCHX(host))
               chxformat (temp, how_s, 0L, 0L, host, null.chx);
            break;

         case ('w'):                                            /* new msg? */
            if (NOT new)  { 
               chxformat (temp, how_s, 0L, 0L, CQ(" "), null.chx);
               break;
            }
            if (EMPTYCHX(newmsg))  mdstr (newmsg, "mes_Tnewmsg", null.md);
            chxformat (temp, how_s, 0L, 0L, newmsg, null.chx);
            break;

         case ('z'):                                            /* Tab pos. */
            buildline (out, temp, 0L, 0L, (int) length);
            break;

         case ('s'):
            chxformat (temp, how_s, 0L, 0L, subject, null.chx);   break;

         /*** Text of message part.  In this case only, the
         /    length field refers to the number of spaces to indent
         /    each line of text. */
         case ('x'):
            ++(*line);
            chxclear (indent);
            chxcpy   (temp, CQ("\n"));
            if (length > 0)
               chxformat (indent, how_s, 0L, 0L, CQ(" "), null.chx);
            while (1) {
               ptr = getmline (id, mnum, (*line)++, 1);
               if (ptr == null.chx)         { success = 0;   break; }
               if (chxeq (ptr, CQ("---")))  { success = 1;   break; }

               /*** Note that after an interrupt, we still have to
               /    count through the message part lines, even though
               /    we don't display them. */
               if (sysbrktest())  continue;

               if (length > 0)  buildline (out, indent, 0L, ALLCHARS, BLADD);
               shift = (chxvalue (ptr, 0L) == ' ' ? 1 : 0);
               buildline (out, ptr, L(shift), ALLCHARS, BLADD);
               buildline (out, temp,  0L,      1L,      BLADD);
            }
            chxclear (temp);
            break;

         default:   break;
      }

      /*** Enforce maximum length of info, and display it. */
      buildline (out, temp, 0L, (length>0 ? length-1 : ALLCHARS), BLADD);
      ++pos;
   }
   buildline (out, null.chx, 0L, -1L, BLFLUSH);

done:
   chxfree ( author );
   chxfree ( date );
   chxfree ( time );
   chxfree ( header );
   chxfree ( userid );
   chxfree ( host );
   chxfree ( temp );
   chxfree ( how_s );
   chxfree ( how_d );
   chxfree ( indent );
   chxfree ( newmsg );
   RETURN  (success);
}
