
/*** PREVMESS.   Get previous parts of a message. 
/
/    parts = prevmess (mymess, userid, allprev, rrsize, prevtext);
/
/    Parameters:
/       long   parts;       (number of message parts put in prevtext)
/       int    mymess;      (message number, if reply or forward)
/       Chix   userid;      (userid of message owner)
/       int    allprev;     (1==>include all prev parts, 0==>only last one)
/       int    rrsize;      (return receipt size, or -1)
/       Chix   prevtext;    (place to put text of previous parts)
/
/    Purpose:
/       When a Caucus user REPLYs to a message, some or all of the parts
/       of the message are picked up and sent along with the reply.
/       Similarly, when a message is FORWARDed, all of the parts are
/       picked up and sent along.
/
/       PREVMESS picks up all of the parts (ALLPREV==1) or just the last part
/       (ALLPREV==0) of message MYMESS, and places them in the chix PREVTEXT.
/
/    How it works:
/       USERID is the id of the person who owns the message being
/       forwarded or replied to.
/
/       If MYMESS is 0, prevmess() clears PREVTEXT and returns 0.
/
/       If RRSIZE >= 0, pick up only that many lines of the previous message
/       part(s).
/
/    Returns: the number of message parts placed in PREVTEXT.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  Caucus functions
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  mess/prevmess.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  7/05/91 18:14 New function. */
/*: CR  7/08/91 17:00 Add RRSIZE argument. */
/*: DE  3/30/92 14:05 Chixified */
/*: DE  4/21/92 14:50 Fixed inconsistant args */

#include <stdio.h>
#include "caucus.h"

extern union  null_t      null;

FUNCTION  prevmess (mymess, userid, allprev, rrsize, prevtext)
   int    mymess, allprev, rrsize;
   Chix   userid;
   Chix   prevtext;
{
   long   parts;
   int    skip, lnum, i, success;
   Chix   str;
   Chix   s;
   Chix   getmline();

   ENTRY ("prevmess", "");

   str      = chxalloc (200L, THIN, "prevmess str");
   success  = 0;

   chxclear (prevtext);
   if (mymess == 0)  FAIL;

   /*** If the user is replying to or forwarding a message MYMESS, open it,
   /    and skip over all but the last part if user doesn't want ALLPREV. */
   if ( (s = getmline (userid, mymess, 0, 1)) == NULL)  FAIL;
   chxtoken (str, null.chx, 3, s);
   chxnum   (str, &parts);

   /*** Figure how many parts to SKIP, total number of PARTS remaining, and
   /    then skip over said parts. */
   skip  = (allprev ? 0 : parts-1);
   parts = parts - skip;
   for (lnum=3, i=0;   i<skip;   ++i) {
      do    { s = getmline (userid, mymess, lnum++, 1); } 
      UNTIL ( s == NULL  ||  chxeq (s, CQ("---"))  ||  chxeq (s, CQ(" ---")));
   }

   /*** Put (non-skipped) parts of previous message into PREVTEXT. 
   /    If a return receipt size was specified, only take that many
   /    lines. */
   if (rrsize >= 0)  ++lnum;
   for (i=0;   (s = getmline (userid, mymess, lnum++, 1)) != NULL;   ++i) {
      if (rrsize >= 0  &&  i >= rrsize)  break;
      chxcat    (prevtext, s);
      chxcatval (prevtext, THIN, (long) '\n');
   }

   DONE( (int) parts);

 done:

   chxfree ( str );

   RETURN ( success );
}
