/*** CHK_SYSTEM:  ChekLink function to check the ParseBays and XSNF file.
/
/   ok = chk_system (confid, fix);
/
/   Parameters:
/      int   ok;     (0 if ok, 1 if at least one problem)
/      Chix  confid; (Caucus home dir)
/      char  fix;    (fix things?)
/
/   Purpose:
/      cheklink uses this to check the parts of the CaucusLink database
/      that aren't related to a particular host or shipping or receiving
/      method.
/
/   How it works:
/      Gets info from XSNF, checks it against the ParseBay and ReParseBay
/      files.
/ 
/   Returns:
/        0 if everything seems correct
/        1 if it detected at least one problem
/
/   Error conditions:
/
/   Side effects:
/      With FIX, can cause unexpected results if you're not careful.
/
/   Related functions:
/      cheklink_x, chk_receive, chk_ship
/
/   Called by:
/      cheklink_x
/
/   Operating system dependencies: none
/
/   Known bugs: 
/
/   Home: netmaint/chksyste.c
/*/

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JV 11/06/93 18:03 Create this. */
/*: CK 12/01/93 14:52 Make i long to agree with arg requirements. */

#include <stdio.h>
#include "handicap.h"
#include "chixuse.h"
#include "unitcode.h"
#include "xaction.h"
#include "xfp.h"

FUNCTION chk_system (confid, fix)
   Chix confid;
   char fix;
{
   char     tmpname[200];
   int      errors=0;
   long     i;
   Chix     out;
   HOSTPTR  me, make_host();
   Hinfoptr make_hinfo();

   out = chxalloc (20L, THIN, "chk_system out");

   me       = make_host();
   me->info = make_hinfo();

   unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, nullchix);
   get_neighbor (0, me, XSNF);
   unit_unlk (XSNF);

   /*** First just check validity of data in XSNF. */

   if (chxlen (me->chn) < 1 || chxlen (me->chn) > 60) {
      chxformat (out, CQ("CHN is probably bad: %s.\n"), 0L, 0L,
                 me->chn, nullchix);
      unit_write (XWTX, out);
      errors++;
   }

   if (me->lhnumber != 0)
      unit_write (XWTX, CQ("LHN in XSNF should be zero, but isn't"));

   if (chxlen (me->info->mgrid) < 1 || chxlen (me->info->mgrid) > 60) {
      chxclear  (out);
      chxformat (out, CQ("CHN is probably bad: %s.\n"), 0L, 0L,
                 me->info->mgrid, nullchix);
      unit_write (XWTX, out);
      errors++;
   }

   if (me->info->seqnum_from < 0 || me->info->seqnum_from > SEQ_MOD) {
      chxclear  (out);
      chxformat (out, CQ("XSNF: SEQ_FROM (%d) is out of bounds.\n"), 
                 me->info->seqnum_from, 0L, nullchix, nullchix);
      unit_write (XWTX, out);
      errors++;
   }

   if (me->info->ackseq < 0 || me->info->ackseq > SEQ_MOD) {
      chxclear  (out);
      chxformat (out, CQ("XSNF: SEQ_ACK (%d) is out of bounds.\n"),
                 me->info->ackseq, 0L, nullchix, nullchix);
      unit_write (XWTX, out);
      errors++;
   }

   if (me->info->seqnum_thru_low < 0 || me->info->seqnum_thru_low > SEQ_MOD) {
      chxclear  (out);
      chxformat (out, CQ("XSNF: SEQ_THRULO (%d) is out of bounds.\n"),
                 me->info->seqnum_thru_low, 0L, nullchix, nullchix);
      unit_write (XWTX, out);
      errors++;
   }

   if (me->info->seqnum_thru_hi < 0 || me->info->seqnum_thru_hi > SEQ_MOD) {
      chxclear  (out);
      chxformat (out, CQ("XSNF: SEQ_THRUHI (%d) is out of bounds.\n"),
                 me->info->seqnum_thru_hi, 0L, nullchix, nullchix);
      unit_write (XWTX, out);
      errors++;
   }


   /*** Second, match the meanings of the above numbers with files in the
   /    CaucusLink database. */


   /* Make sure all files which should be in XSPB are there. */
   for (i=me->info->seqnum_from; i < me->info->ackseq; i++) {
      unit_lock (XSPB, READ, i, 0L, 0L, 0L, nullchix);
      if (NOT unit_check (XSPB)) {
         chxclear  (out);
         chxformat (out,
     CQ("XSNF indicates that next ParseBay file to be processed is: %d,\n"),
                    i, 0L, nullchix, nullchix);
         unit_name (tmpname, XSPB, i, 0L, 0L, 0L, nullchix);
         stralter  (tmpname, ascquick(confid), "");
         stralter  (tmpname, "/", "");
         chxformat (out, CQ("but that file (%s) doesn't exist.\n"), 0L, 0L,
                    chxquick (tmpname, 1), nullchix);
         unit_write (XWTX, out);
         errors++;
      }
      unit_unlk (XSPB);
   }

   /* Next make sure that all files in XSPB should be there. */

   /* Now make sure that the boundary files for the XSRP are there. */
   for (i=me->info->seqnum_thru_low; i < me->info->seqnum_thru_hi; ) {
      if (i != me->info->seqnum_thru_low && i != me->info->seqnum_thru_hi)
         continue;     /* only check boundaries - others might not be there */
      unit_lock (XSRP, READ, me->info->seqnum_from, 0L, 0L, 0L, nullchix);
      if (NOT unit_check (XSRP)) {
         chxclear  (out);
         chxformat (out,
     CQ("XSNF indicates that next ReParseBay file to be processed is: %d,\n"),
                    i, 0L, nullchix, nullchix);
         unit_name (tmpname, XSPB, i, 0L, 0L, 0L, nullchix);
         stralter  (tmpname, ascquick(confid), "");
         stralter  (tmpname, "/", "");
         chxformat (out, CQ("but that file (%s) doesn't exist.\n"), 0L, 0L,
                    chxquick (tmpname, 1), nullchix);
         unit_write (XWTX, out);
         errors++;
      }
      unit_unlk (XSRP);
   }

   /* Finally make sure that all files in XSRP should be there. */

   return (errors);
}
