/* PUBLISH_CONF.  Find out which conf should be published, and to which 
/                 neighbors, and alter the db accordingly.
/
/  publish_conf ()
/
/  Parameters:
/
/  Purpose:
/    A conference is "unpublished" until the database is changed to allow
/    subscription requests.  This function gets the appropriate info from
/    the manager and then changes the database.
/
/    Only the "Original Host" for a conference publishes the conf.  Only
/    local confs can be published.
/
/  How it works:
/    Three changes must be made to the Conf DB:
/      1) Add the neighbors file (XCNB)
/      2) Add the hosts to which this should be published to the userlist
/         (XCUS).
/      3) Add to (or create) the true conference name file (XCTC).
/
/  Returns:
/    1: completed the publication
/    0: didn't complete the publication
/
/  Error conditions:
/    If the user aborts cleanly, 0 is returned but no action is taken.
/    If the process can't be completed due to an inconsistent db, the
/    mgr is warned of the problem.
/
/  Called by: do_conf
/
/  Home: netmaint/publishc.c
/
/  Dictionary strings:
/    net_Ppub_conf, net_Hpub_conf: ask mgr for true conf name.
/    net_Ppub_chn,  net_Hpub_chn:  ask mgr for subscribing CHNs.
/    net_Tnochn: that CHN doesn't exist.
/    net_Tpub_done: we're done.
*/

/*: JV 10/29/91 10:12 Creat (sic) this. */
/*: CR 11/18/91 17:47 Add CNUM arg to modnames() call. */
/*: WC 11/29/91 20:21 True/local entries via cus_dvisib; XCUS lock reduce. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: CR 12/26/91 11:43 Remove unused variables. */
/*: JX  5/15/92 11:53 Chixify. */

#include <stdio.h>
#include "handicap.h"
#include "conf.h"
#include "netmaint.h"
#include "xfp.h"
#include "unitcode.h"
#include "caucus.h"

extern union null_t null;

FUNCTION publish_conf ()
{
   int  result, any=0, cnum, is_local, existed, openval;
   int  success;
   HOSTPTR  me, make_host();
   Hinfoptr make_hinfo();
   Chix     subscribers, chn, true_conf;

   ENTRY ("publish_conf", "");

   chn =  chxalloc (80L, THIN, "publish chn");
   true_conf =   chxalloc (80L, THIN, "publish true_conf");
   subscribers = chxalloc (80L, THIN, "publishc subscribers");

   me = make_host ();
   me->info = make_hinfo();

   while (1) {
      if (mdprompter ("net_Ppub_conf", nullchix, 0, "net_Hpub_conf", 80,
                       true_conf, nullchix) < -1)   FAIL;
 
      if ((cnum = cnumber (true_conf, LOCALCONF, &openval)) <= 0) {
         mdwrite (XITT, "rem_Enoconf", null.md); continue; }
      break;
   }

   /*** Check to see if it's already published. */
   if (NOT unit_lock (XCNB, READ, L(cnum), 0L, 0L, 0L, nullchix)) FAIL;
   is_local = NOT unit_check (XCNB);
   unit_unlk (XCNB);
   if (NOT is_local) {
      mdwrite (XWTX, "net_Tnotlocal", null.md);
      FAIL;
   }


   /*** Get the list of CHN's allowed to subscribe.  Store it in a chix,
   /    and only when we're done getting the list add it to the end of
   /    XCUS. */
   success     = 0;
   mdwrite (XWTX, "net_Ppub_chn", null.md);
   while (1) {
      if ((result = mdprompter ("ss_inprom", nullchix, 0, "net_Hpub_chn",
                                 80, chn, nullchix)) < -2)   FAIL;
      if (result == -2) break;

      if (hnumber (chn) < 1) {
         mdwrite (XWER, "net_Tnochn", null.md); continue; }
      any=1;
      chxcat (subscribers, chxquick ("@", 0));
      chxcat (subscribers, chn);
      chxcat (subscribers, chxquick ("\n",0));
   }
   if (NOT any) FAIL;
   if (NOT unit_lock (XCUS, WRITE, L(cnum), 0L, 0L, 0L, nullchix))  FAIL;
   unit_append (XCUS);
   unit_write  (XCUS, CQ(":include\n"));
   unit_write (XCUS, subscribers);

   unit_close (XCUS);
   unit_unlk  (XCUS);


   unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, nullchix);
   get_neighbor (0, me, XSNF);
   unit_unlk (XSNF);

   /*** Create an empty XCNB "neighbors" file. */
   unit_lock  (XCNB, WRITE, L(cnum), 0L, 0L, 0L, nullchix);
   unit_make  (XCNB);
   unit_close (XCNB);
   unit_unlk  (XCNB);

   /*** Create (or add to) the XCTC "trueconf" file. */
   unit_lock   (XCTC, WRITE, L(cnum), 0L, 0L, 0L, nullchix);
   existed = unit_check (XCTC);
   unit_append (XCTC);
   if (NOT existed) {
      unit_write  (XCTC, true_conf);   unit_write (XCTC, CQ("\n"));
   }
   unit_write  (XCTC, true_conf);   unit_write (XCTC, CQ("\n"));
   unit_write  (XCTC, me->chn);     unit_write (XCTC, CQ("\n"));
   unit_close  (XCTC);
   unit_unlk   (XCTC);

   /*** Finally, use cus_dvisib() to mark XSCN (MISC/confs001) with the
   /    proper local & true name entries. */
   cus_dvisib (cnum, true_conf, openval);
   success = 1;

done:
   chxfree (chn);
   chxfree (true_conf);
   chxfree (subscribers);

   free_host (me);
   RETURN  (success);
}
