/* SUBSCR_CONF:  Ask questions leading to subscribing to a conference
/       on a neighboring host.
/
/  subscr_conf();
/
/  Purpose:
/    NetMaint handles various management tasks for the manager.  One of
/    these is managing conferences.  The manager can create, destroy,
/    publish, subscribe to, and unsubscribe from conferences.
/
/    This function handles "subscribing to a conf".
/
/  How it works:
/    When the mgr chooses to subscribe to a conf, do_conf() calls
/    this function, which prompts the mgr for the neighbor CHN, the
/    true conf name of the conf,

/    Subscr_conf() sends out a subscription request xaction to the neighbor.
/
/  Returns:
/    No return value.
/
/  Error conditions:
/    subscr_conf() validates all user input (when possible).  If it detects
/    bad input, it tells the user and warns them to try again, or enter zero.
/    The only way that send_conf should fail is if there's a problem with the
/    database.  If this happens the manager is warned of this condition.
/
/  Related functions: 
/
/  Called by: do_conf.
/
/  Home: netmaint/subscrco.c
/
/  Dictionary strings:
/     net_Psubs_chn,  net_Hsubs_chn:
/     net_Psubs_ochn,  net_Hsubs_ochn:
/     net_Tnochn:
/     net_Psubs_conf, net_Hsubs_conf:
/     net_Psubs_lconf, net_Hsubs_lconf:
/     net_Tsubs_oneconf:
/     net_Tsubs_done:
/     net_Tsubs_fail:
*/

/*: JV 10/27/91 17:54 Creat (sic) this */
/*: CR 11/18/91 17:20 Add CNUM argument to modnames() call. */
/*: CR 11/22/91 13:30 Rearrange question order. */
/*: JV 11/25/91 22:22 Return if can't lock or append XHML. */
/*: WC 11/29/91 21:14 Add 3rd arg to cnumber(). */
/*: JV 12/05/91 18:06 Add sysdirchk() so XITX can be there. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: JV  1/03/92 15:11 Declare strcpy(). */
/*: DE  4/09/92 16:18 Complete Chixification */
/*: JX  7/31/92 16:33 Add make_host(). */
/*: JX  8/09/92 16:26 Fixify. */
/*: JX  9/01/92 13:54 Change empty_xact to make_xaction(). */
/*: CP  5/06/92 12:41 Add 0L as 'item' arg to modnames() call. */
/*: JZ  9/28/93 15:42 Add file lib arg to make_conf(). */

#include <stdio.h>
#include "handicap.h"
#include "unitcode.h"
#include "derr.h"
#include "xaction.h"
#include "xfp.h"
#include "chixuse.h"
#include "conf.h"
#include "done.h"

extern union null_t null;

FUNCTION subscr_conf ()
{
   char chxname[20];
   Chix  str, chn, ochn, true_conf, local_conf;
   int   v, success;
   short i, pass;
   int   lhnum, cnum, islocal=0, error, confnum;
   Xactionptr xaction, make_xaction();
   Chix carray[TEXTFIELDS];
   HOSTPTR  me, make_host();
   Hinfoptr make_hinfo();
   Namelist add, del, nlnode();

   ENTRY ("subscr_conf", "");

   str        = chxalloc (200L, THIN, "subscrco str");
   chn        = chxalloc (100L, THIN, "subscrco chn");
   ochn       = chxalloc (100L, THIN, "subscrco ochn");
   true_conf  = chxalloc (100L, THIN, "subscrco true_conf");
   local_conf = chxalloc (100L, THIN, "subscrco local_conf");
   success    = 0;

   xaction  = make_xaction();
   me       = make_host();
   me->info = make_hinfo();

   /*** Get True conf name */
   while (1) {
      if (mdprompter ("net_Psubs_conf", null.chx, 0, "net_Hsubs_conf", 80,
                true_conf, null.chx) < -1)    FAIL;

      if ((cnum = cnumber (true_conf, TRUECONF,  &v)) > 0) {
         mdwrite (XITT, "net_Tsubs_oneconf", null.md); continue; }
      if ((cnum = cnumber (true_conf, LOCALCONF, &v)) > 0)
         islocal = 1;
      break;
   }

   /*** If this host already has a conf with a local conf name the same
   /    as the true conf name just specified, the mgr *must* choose a 
   /    different local name. */
   if (islocal)
      while (1) {
         if (mdprompter ("net_Psubs_lconf", null.chx, 0, "net_Hsubs_lconf",
                   80, local_conf, null.chx) < -1)    FAIL;

         if ((cnum = cnumber (local_conf, LOCALCONF, &v)) > 0) 
            mdwrite (XITT, "net_Tsubs_oneconf", null.md);
         else break;
      }
   else chxcpy (local_conf, true_conf);
      
   /*** Get original host's CHN. */
   if (mdprompter ("net_Psubs_ochn", null.chx, 0, "net_Hsubs_ochn", 80,
                    ochn, null.chx) < -1)    FAIL;


   /* Get Neighbor's CHN */
   while (1) {
      if (mdprompter ("net_Psubs_chn", null.chx, 0, "net_Hsubs_chn", 80, 
                       chn, null.chx) < -1)    FAIL;

      if ((lhnum = hnumber (chn)) > 0)  break;    /* Got it. */

      mdwrite (XITT, "net_Tnochn", null.md);
   }

   /*** Now that we know what it is they want us to do, 
   /    first we create an empty, closed conference as a framework
   /    for the XFU to stuff the incoming items into... */
   sysdirchk();    
   if (NOT unit_lock (XITX, WRITE, 0L, 0L, 0L, 0L, null.chx)) {
      mdwrite (XWER, "err_Ebaddata", null.md);
      FAIL;                               /* Set up empty userlist */
   }
   unit_make (XITX);
   mdwrite   (XITX, "cst_Tcfempty", null.md);
   unit_close (XITX); unit_unlk (XITX);

   if (NOT make_conf (local_conf, nullchix, 0, 0, XITX, &error)) {
      if (error == DBADDB)
         mdwrite (XWER, "err_Ebaddata", null.md);
      else if (error == DMKDIR)
         mdwrite  (XWTX, "cst_Tmkdir", null.md);
      FAIL;
   }

   /*** ...then add the TrueConf entry to the XSCD/N database... */
   cnum = cnumber (local_conf, LOCALCONF, &v);
   chxformat (str, CQ("%s@%s c#%03d#T"), L(cnum), 0L, true_conf, ochn);
   add = nlnode(1);
   nladd (add, str, 0);
   del = nlnode(1);
   modnames (XSCD, XSCN, null.chx, 0, 0L, add, del, 0);
   nlfree (add); nlfree (del);

   /*** ...then append the True Confname and Original CHN to XCTC... */
   confnum = cnumber (local_conf, LOCALCONF, &v);
   unit_lock   (XCTC, WRITE, L(confnum), 0L, 0L, 0L, null.chx);
   unit_append (XCTC);
   unit_write (XCTC, true_conf);  unit_write (XCTC, CQ("\n"));
   unit_write (XCTC, ochn);       unit_write (XCTC, CQ("\n"));
   unit_close (XCTC);             unit_unlk  (XCTC);

   /*** ...then create the XCNB file... */
   unit_lock  (XCNB, WRITE, L(confnum), 0L, 0L, 0L, null.chx);
   unit_make  (XCNB);
   chxclear   (str);
   chxformat  (str, CQ("%d\n"), L(lhnum), 0L, nullchix, nullchix );
   unit_write (XCNB, str);
   unit_close (XCNB);             unit_unlk (XCNB);

   /*** ...and finally send off the transaction requesting the
   /     entire conference. */
   xaction->typenum = TT_SUBREQ;
   chxcpy (xaction->conftname, true_conf);
   chxcpy (xaction->confochn,  ochn);
   unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, null.chx);
   get_neighbor (0, me, XSNF);
   unit_unlk (XSNF);
   chxformat (xaction->uid, CQ("%s@%s"), 0L, 0L, me->info->mgrid, me->chn);

   if (NOT unit_lock (XHML, WRITE, L(lhnum), 0L, 0L, 0L, null.chx)) {
      bug ("ADDXMESS: can't lock XHML.", 0);
      mdwrite (XITT, "err_Ebaddata", null.md);
      FAIL;
   }
   if (NOT unit_append (XHML)) {
      bug ("ADDXMESS: can't append to XHML.", 0);
      mdwrite (XITT, "err_Ebaddata", null.md);
      unit_unlk (XHML); FAIL;
   }

   for (i=0; i<TEXTFIELDS; i++) {
      sprintf (chxname, "subscr_conf %d", i);
      carray[i] = chxalloc (0L, THIN, chxname);
   }

   pass = putnextxact (XHML, xaction, carray);

   unit_close (XHML);  unit_unlk (XHML);

   for (i=0; i<TEXTFIELDS; i++) chxfree (carray[i]);

   if (pass) mdwrite (XITT, "net_Tsubs_done", null.md);
   else         mdwrite (XITT, "net_Tsubs_fail", null.md);

   SUCCEED;

 done:
   chxfree (str);
   chxfree (chn);
   chxfree (ochn);
   chxfree (true_conf);
   chxfree (local_conf);
   free_host (me);
   free_xaction (xaction);

   RETURN (success);

}
