/*** SYSBREAK.  Break (interrupt) handling routines.
/
/  The following routines are used to intercept and announce BREAK, CANCEL,
/  or whatever the vanilla interrupt signal is for a given system.  The
/  visible routines are:
/
/     sysbrkinit()        Initializes interrupt handling; must be executed
/                         once before interrupt testing is to be done.
/
/     sysbrkignore()      Causes "CANCEL" interrupt to be ignored.  Must
/                         call sysbrkinit() again to re-initialize
/                         interrupt handling.
/
/     sysbrktest()        Returns 0 if no interrupt has been encountered,
/                         or a non-zero code if an interrupt has occurred.
/                         The value depends on the type of interrupt.
/
/     sysbrkclear()       Clears the "interrupt has occurred" flag.  Note
/                         that SYSBRKTEST does NOT automatically clear.
/
/     sysbrkset()         Sets the "interrupt has occurred" flag.
/
/ NOTES:
/   (a) currently, if more than MAXQUIT un-processed interrupts are
/       encountered, the interrupt catcher will automatically exit to the
/       operating system with a status of 7.
/
/   (b) sysbrkignore() is provided for systems (such as SunOS) which
/       need to ignore interrupts while doing sensitive operations
/       like creat() or open().
/ */

/*** Copyright (C) 1986 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:44 Source code master starting point */
/*: CR  8/27/88 16:56 Remove duplicate "SU". */
/*: JV  9/02/88 17:15 Fixed the AOS code to check for MAXQUIT intrpts. */
/*: CR  9/21/88 15:25 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 13:00 Add "IX" system type. */
/*: CR 10/07/88 13:48 Change IM to use new CNSBREAK names. */
/*: CR 11/02/88 16:06 Add CX server to SX code. */
/*: JV 12/07/88 10:24 Added CA code.*/
/*: CR  1/09/89 12:49 Add CVM code. */
/*: CR  1/09/89 14:22 CVM only: void sysbrk2(). */
/*: CR  6/09/89 12:16 Add "SV" system type code. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: CR  9/17/89 23:27 Add hangup/quit detection for Sys V. */
/*: JV 10/04/89 17:47 Stripped out AOS non-server code. */
/*: CR 10/12/89 15:35 Add DU system type code. */
/*: CR  1/29/90 16:38 Add hangup detection for Primos. */
/*: JV  7/06/90 20:30 Add MIPS RISC/os (RO). */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  9/27/90 12:19 Add U3, AU system types. */
/*: JV  1/22/91 17:19 Move AOS into Unix code (add hangup detection) */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: CR  1/28/91  0:38 Hangup kicks in an alarm interrupt in 3 sec. */
/*: CR  2/01/91 16:13 Remove hangup detection to syshng.c. */
/*: CR  2/09/91 21:44 Split long #if line in two (SYSV1, SYSV2). */
/*: SB 11/29/90 14:03 Add DY system code*/
/*: CR  3/11/91 14:15 Add AX and UL system type codes. */
/*: JV  2/05/91 12:25 Add alarm handler initialization to sysbrkinit() */
/*: CR  4/06/91 17:12 Integrate recent changes. */
/*: CR 10/03/91 15:11 Move SU code into its own section. */
/*: SB 12/03/91 18:29 Add DP code. */
/*: JV  1/17/92 16:38 Change SU code: decl signal handlers (*handler)(). */
/*: CR  1/21/92 10:40 Fix SU signal handler declaration. */
/*: JV  7/16/92 15:23 Add U4. */
/*: CR  7/21/92 17:39 Remove DP (in favor of DYP). */
/*: CR 12/10/93 15:38 Add sysbrkignore(). */
/*: CR  5/05/94 16:44 sysbrkinit() should not clear break flag. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#define  MAXQUIT  10

/*---------------------------------------VM/CMS-------------------*/
#if CVM
#include <signal.h>

static int break_flag;

FUNCTION  sysbrkinit()
{
   void sysbrk2();

   ENTRY ("sysbreak", "");
   break_flag = 0;
   signal (SIGINT, sysbrk2);
   RETURN (1);
}

FUNCTION  void sysbrk2 (code, reg_area, PSW, signal_name)
   int   code, *reg_area, *PSW;
   char *signal_name;
{
   ++break_flag;
   if (break_flag > MAXQUIT)  sysexit(7);
}

FUNCTION  sysbrkignore()
{
   signal (SIGINT, SIG_IGN);
}

FUNCTION  sysbrkset()
{
   ++break_flag;
}

FUNCTION  sysbrktest()
{
   ENTRY ("sysbrktest", "");
   if (break_flag)  break_flag = 1;
   RETURN (break_flag);
}

FUNCTION  sysbrkclear()
{
   ENTRY ("sysbrkclear", "");
   break_flag = 0;
   RETURN (1);
}
#endif

/*--------------------------------FORTUNE & VAX & AOS/VS-------------------*/
#define SYSV1  A3 | LA | SV | HU | FU | MU | MV | PU | SX | TX | U4
#define SYSV2  VV | IX | CX | PX | DU | RO | U3 | AU | CA | DY | AX
#if SYSV1 | SYSV2
#include <signal.h>

static int break_flag = 0;

FUNCTION  sysbrkinit()
{
   int sysbrk2();

   ENTRY ("sysbreak", "");
/* break_flag = 0; */
   signal (SIGINT,  sysbrk2);
#if CA
   sysalinit ();
#endif
   RETURN (1);
}

FUNCTION  sysbrk2()
{
   signal (SIGINT, sysbrk2);
   ++break_flag;
   if (break_flag > MAXQUIT)  sysexit(7);
}

FUNCTION  sysbrkignore()
{
   signal (SIGINT, SIG_IGN);
}

FUNCTION  sysbrkset()
{
   ++break_flag;
}

FUNCTION  sysbrktest()
{
   ENTRY ("sysbrktest", "");
   if (break_flag)  break_flag = 1;
   RETURN (break_flag);
}

FUNCTION  sysbrkclear()
{
   ENTRY ("sysbrkclear", "");
   break_flag = 0;
   RETURN (1);
}
#endif

/*---------------------------------------Berkley Unix 4.2----------------*/
#if BU | PS | UL
#include <signal.h>
#include <setjmp.h>

int     sysb_flag = 0;   /* Interrupt flagged? */
int     sysb_read;       /* Interrupt in the middle of a read? */
jmp_buf sysb_env;        /* longjmp environment in case of sysb_read */

FUNCTION  sysbrkinit()
{
   struct sigvec vector;
   int sysbrk2();

   ENTRY ("sysbreak", "");
   vector.sv_handler = sysbrk2;
   vector.sv_mask    = 4;
   vector.sv_onstack = 0;
/* sysb_flag = 0; */
   sysb_read = 0;
   sigvec (SIGINT, &vector, 0);
   RETURN (1);
}

FUNCTION  sysbrk2()
{
   ++sysb_flag;
   if (sysb_flag > MAXQUIT)  sysexit(7);
   if (sysb_read) {
      sysb_read = 0;
      longjmp (sysb_env, 1);
   }
}

FUNCTION  sysbrkignore()
{
   signal (SIGINT, SIG_IGN);
}

FUNCTION  sysbrktest()
{
   ENTRY ("sysbrktest", "");
   if (sysb_flag)  sysb_flag = 1;
   RETURN (sysb_flag);
}

FUNCTION  sysbrkset()
{
   ++sysb_flag;
}

FUNCTION  sysbrkclear()
{
   ENTRY ("sysbrkclear", "");
   sysb_flag = 0;
   RETURN (1);
}
#endif

/*---------------------------------------SunOS---------------------------*/
#if SU
#include <signal.h>
#include <setjmp.h>

int     sysb_flag = 0;   /* Interrupt flagged? */
int     sysb_read;       /* Interrupt in the middle of a read? */
jmp_buf sysb_env;        /* longjmp environment in case of sysb_read */

FUNCTION  sysbrkinit()
{
   void   sysbrk2();

   ENTRY ("sysbreak", "");
/* sysb_flag = 0; */
   sysb_read = 0;
   signal (SIGINT, sysbrk2);
   RETURN (1);
}

FUNCTION void sysbrk2()
{
   ++sysb_flag;
   if (sysb_flag > MAXQUIT)  sysexit(7);
   if (sysb_read) {
      sysb_read = 0;
      longjmp (sysb_env, 1);
   }
}

FUNCTION  sysbrkignore()
{
   signal (SIGINT, SIG_IGN);
}

FUNCTION  sysbrktest()
{
   ENTRY ("sysbrktest", "");
   if (sysb_flag)  sysb_flag = 1;
   RETURN (sysb_flag);
}

FUNCTION  sysbrkset()
{
   ++sysb_flag;
}

FUNCTION  sysbrkclear()
{
   ENTRY ("sysbrkclear", "");
   sysb_flag = 0;
   RETURN (1);
}
#endif

/*---------------------------------------IBM PC--------------------------*/
#if  IM | PN

int  breakflag = 0;
extern struct flag_template flag;

FUNCTION  sysbrkinit()
{
   ENTRY ("sysbrkinit", "");
   cbcatch();
   RETURN (1);
}

FUNCTION  sysbrkignore()
{
}

FUNCTION  sysbrkclear()
{
   ENTRY ("sysbrkclear", "");
   if (flag.baud)  breakflag = 0;
   else            cbclear();
   RETURN (1);
}

FUNCTION  sysbrkset()
{
   ENTRY ("sysbrkset", "");
   if (flag.baud)  breakflag = 1;
   else            cbset();
   RETURN (1);
}

FUNCTION  sysbrktest()
{
   int    c;
   ENTRY ("sysbrktest", "");

   c = cbtest();
   if (NOT flag.baud)   RETURN (c);

   if (c)  sysexit(0);

   while ( (c = sysgetch()) > 0)    if (c == '\003')  breakflag = 1;

   RETURN (breakflag);
}
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR
#include <setjmp.h>

int     sysb_flag = 0;   /* Interrupt flagged? */
int     sysb_read;       /* Interrupt in the middle of a read? */
jmp_buf sysb_env;        /* longjmp environment in case of sysb_read */

FUNCTION  sysbrkinit()
{
   ENTRY  ("sysbrkinit", "");
   sysb_read = 0;
   RETURN (1);
}

FUNCTION  sysbrkignore()
{
}

FUNCTION sysbrk2(i)
   int  i;
{
   ENTRY ("sysbrk2", "");

   ++sysb_flag;
   if (sysb_flag > MAXQUIT)  sysexit(7);
   if (sysb_read) {
      sysb_read = 0;
      longjmp (sysb_env, 1);
   }
   RETURN (1);
}

FUNCTION  sysbrktest()
{
   ENTRY  ("sysbrktest", "");
   if (sysb_flag)  sysb_flag = 1;
   RETURN (sysb_flag);
}

FUNCTION  sysbrkset()
{
   ++sysb_flag;
}

FUNCTION  sysbrkclear()
{
   ENTRY  ("sysbrkclear", "");
   sysb_flag = 0;
   RETURN (1);
}
#endif
