/*** SYSCMD.  Execute system COMMAND as though it had been typed by user.
/
/   ok = syscmd (command, should_wait);
/
/   Parameters:
/     int   ok;            (Return value, see below)
/     char *command;       (Operating system command)
/     int   should_wait;   (if SYS_WAIT, wait for command to finish)
/
/   Purpose:
/      SYSCMD allows us to invoke operating system commands from within
/      Caucus or one of the CaucusLink programs.
/
/   How it works:
/      Lines beginning with "#" are treated as comments and ignored.
/
/      If WAIT is SYS_WAIT, syscmd() returns after the COMMAND is
/      complete.  Otherwise it returns immediately.  (Note: not all
/      systems support the concept of not waiting.)
/
/   Returns:
/      There are five categories of return values for syscmd():
/         A) Should_wait is not SYS_WAIT.  Returns 1 immediately.
/
/      All other cases assume should_wait == SYS_WAIT.
/
/         A) Command (appears to have) executed successfully (this includes
/            comments).
/            Returns: the exit value PLUS ONE.
/
/         B) Command failed: unknown error
/            Returns: 0
/
/         C) Command failed or couldn't be started
/            Returns negative even number:
/            -2: COMMAND specifies an incorrect command or file name
/            -4: User is not allowed to execute the command or file
/            -6: Argument list too big
/
/         D) Command failed or couldn't be started due to a temporary
/            condition (e.g. temporary resource limitation)
/            Returns negative odd number:
/            -1  O.S. error: Too many processes
/            -3: Not enough memory
/            -5: Child terminated by another process
/
/      These groupings are provided in this way so that the calling
/      program can evaluate reality quickly and easily:
/
/      1) If the return value is greater than zero, the command executed.
/         Otherwise, it didn't even run.
/         The caller can also check the return value from the command
/         by subtracting one from syscmd()'s return value.
/      2) If it failed to run, check the low-order bit:
/         1) If it's 1, it may be useful to try again in a few seconds.
/         2) If it's 0, trying again won't help.
/
/ **Errors:
/ **   WARNING: So far this more detailed version has only been implemented
/ **   for A3, IM, SU, SX, and VV!
/
/   Related functions:  sysscrip.c
/
/   Called by: gen/printit.c  main/interpre.c sys/sysclrsc.c
/              sys/sysedito.c sys/sysmstra.c  sys/syspass.c  sys/sysscrip.c
/
/   Operating System Dependencies:
/      The amount of information returned about the child is highly
/      system-dependent.  Unix provides a few different failure conditions,
/      as does DOS, but we don't know what VM returns!
/
/      See the bottom of this file for a more detailed explanation of what
/      is available per O.S.
/
/      Some systems support the WAIT argument.  These include:
/         UNIX
/      These do not support the WAIT argument:
/         DOS, VMS (VMS can, but the code hasn't been modified yet)
/
/   Home: sys1/syscmd.c
/ */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:44 Source code master starting point */
/*: CR  9/21/88 15:25 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 13:00 Add "IX" system type. */
/*: JV  9/21/88 16:26 Fixed AOS code.*/
/*: JV 11/22/88 16:26 Fixed AOS code. Again.*/
/*: CR  1/06/89 16:12 CVM changes only. */
/*: JV  2/13/89 16:08 Added CA code.*/
/*: CR  4/19/89 16:19 Changed CVM,CA,CX code to use vcsyscmd(). */
/*: CR  6/09/89 12:16 Add "SV" system type code. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JV 10/04/89 17:55 Stripped out AOS non-server code. */
/*: CR 10/12/89 15:35 Add DU system type code. */
/*: CR  2/26/90 13:56 CVM, PX: use vwin functions. */
/*: CR  6/05/90 13:13 CVM, PX: Check flag.fullscreen. */
/*: CR  6/20/90 11:58 CVM, PX: use vwinreset() to force screen redraw. */
/*: JV  7/06/90 21:09 Add MIPS RISC/os (RO). */
/*: JV  7/22/90 16:26 Add getuid decl to Sys V. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  9/27/90 12:19 Add U3, AU system types. */
/*: CJ 11/22/90 22:07 Save termstate, do command, then restore term state. */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: CR  1/27/91 17:21 Detect hangups that happen while waiting for child. */
/*: CR  2/01/91 16:22 Replace hangup with syshngtest(). */
/*: CR  2/09/91 21:44 Split long #if line in two (SYSV1, SYSV2). */
/*: SB 11/29/90 14:04 Add DY system code*/
/*: CR  3/11/91 14:15 Add AX and UL system type codes. */
/*: SB  5/16/91 10:34 Add DP system type. */
/*: JV  1/07/91 16:11 Attempt to recover from Integration Problem. */
/*: CR  1/30/92 17:37 Parenthesize operations including bit-wise &. */
/*: JV  7/16/92 15:28 Add U4, move include files. */
/*: CR  7/21/92 17:39 Remove DP (in favor of DYP). */
/*: CR  8/20/92 15:13 MV status return & 0xFE. */
/*: CV  6/02/93 11:39 Expand VV return value code. */
/*: JV  3/18/93 11:43 Add WAIT arg. */
/*: CR  6/08/93 13:51 Integrate should_wait and Vax changes. */
/*: CP  7/16/93 16:43 Handle return code for should_wait != SYS_WAIT. */
/*: CP  7/30/93 17:58 Fix screwy else's with {}'s. */

#include <stdio.h>
#include "caucus.h"
#include "sys.h"
#include "systype.h"

#define  SYSV1 A3  | LA | SV | HU | FU | MU | MV | PU | SX | TX | U4
#define  SYSV2 VMU | IX | PX | DU | RO | PS | U3 | AU | DY | AX
#if SYSV1 | SYSV2 | BU | SU | UL
#include <errno.h>
#endif

extern struct flag_template flag;
extern struct termstate_t   term_original, term_in_caucus;
extern int                  debug;

FUNCTION  syscmd (command, should_wait)
   char  command[];
   int   should_wait;
{

/*-----------------------------FORTUNE, Masscomp, A3---------------------*/
#if SYSV1 | SYSV2

   int   status, child;
   unsigned short getuid();

   ENTRY ("syscmd", "");
   if (command[0] == '#')  RETURN (1);

   /*** The Unix version spawns a task that reverts to the Real userid,
   /    and then tries to execute COMMAND via the Bourne shell. */
   systurn (&term_in_caucus, GET_TERM, 0);
   systurn (&term_original,  SET_TERM, 0);
   child  = fork();
   if (child == -1) {
      systurn (&term_in_caucus, SET_TERM, 0);
      if (errno == EAGAIN) RETURN (-1);
      if (errno == ENOMEM) RETURN (-3);
      RETURN (0);
   }

   /*** The parent task waits for the child to terminate. */
   else if (child >   0) {
      if (should_wait == SYS_WAIT)
         while (wait (&status) != child) { if (syshngtest())  break; }
   }
 
   /*** The child task reverts to the Real uid, and executes the
   /    command via the Bourne shell. */
   else {
      setuid (getuid());
      execl  ("/bin/sh", "sh", "-c", command, NULL);
      exit   (1);
   }

   systurn (&term_in_caucus, SET_TERM, 0);

#if PX | PS
   if (flag.fullscreen)  vwinreset();
#endif

   if (should_wait == SYS_WAIT) {
#if A3 | SX
      /*** Child process exited cleanly. */
      if ((status & ~0xFF00) == 0)
         RETURN ((status & ~0x00FF) + 1);

      /*** Child process terminated by signal. */
      if ((status & ~0x00FF) == 0)
         RETURN (-5);

      /*** Unknown (including the child was "stopped"). */
      RETURN (0);
#else

#if MV
      RETURN  ( (status & 0xFE00) ? 0 : 1);
#endif

      RETURN  ( (status & 0xFF00) ? 0 : 1);
#endif
   }
   RETURN (1);
#endif

/*-----------------------------Berkeley Unix---------------------*/
#if BU | SU | UL
   int   status, child;

   ENTRY ("syscmd", "");
   if (command[0] == '#')  RETURN (1);

   /*** The Unix version spawns a task that reverts to the Real userid,
   /    and then tried to execute COMMAND via the Bourne shell. */
   systurn (&term_in_caucus, GET_TERM, 0);
   systurn (&term_original,  SET_TERM, 0);
   child  = vfork();
   if (child == -1) {
      systurn (&term_in_caucus, SET_TERM, 0);
      if (errno == EAGAIN) RETURN (-1);
      if (errno == ENOMEM) RETURN (-3);
      RETURN  (0);
   }

   /*** The parent task waits for the child to terminate. */
   else if (child >   0) {
      if (should_wait == SYS_WAIT) 
         while (wait (&status) != child) { if (syshngtest())  break; }
   }
 
   /*** The child task reverts to the Real uid, and executes the
   /    command via the Bourne shell. */
   else {
      setreuid (getuid(), getuid());
      execl  ("/bin/sh", "sh", "-c", command, NULL);
      _exit  (1);
   }

   /*** If the child process exited with a status other than 0,
   /    the command failed. */
   systurn (&term_in_caucus, SET_TERM, 0);

   if (should_wait == SYS_WAIT) {
#if SU
      /*** Child process was stopped. */
      if ((status & ~0xFF00) == 0177)
         RETURN (0);

      /*** Child process terminated by signal. */
      /*   Note: the man pages for wait() are wrong.  Try it yourself! */
      if ((status & ~0x7FFF) == 1)
         RETURN (-5);

      /*** Probably finished successfully. */
      RETURN ((status & ~0x00FF) + 1);
#else
      RETURN  ( (status & 0xFF00) ? 0 : 1);
#endif
   }
   RETURN (1);
#endif

/*---------------------------------------IBM PC--------------------------*/
#if IM | JM | PN
   ENTRY ("syscmd", "");

   if (NOT syscmd (command)) RETURN (1);
   switch (errno) {
      case E2BIG: RETURN (-6);
      default:    RETURN  (0);
   }
#endif


/*---------------------------------------AOS/VS--------------------------*/
#if CA | CX
   int  success;
   ENTRY ("syscmd", "");

   systurn (&term_in_caucus, GET_TERM, 0);
   systurn (&term_original,  SET_TERM, 0);
   success = (vcsyscmd (command) ? 0 : 1);
   systurn (&term_in_caucus, SET_TERM, 0);

   RETURN (success);
#endif


/*---------------------------------------VM/CMS--------------------------*/
#if CVM
   int  status;

   ENTRY  ("syscmd", "");
   status = vcsyscmd (command);
   if (flag.fullscreen)  vwinreset();

   RETURN (1);   /* I don't know what the status values mean yet! */
#endif


/*---------------------------------------VAX-----------------------------*/
#if VV
#include <ssdef>
#include <descrip>
#include <stdio>

   struct dsc$descriptor_s desc;
   long   complete, status, flags, code;

   ENTRY ("syscmd", "");

   if (command[0] == '#')  RETURN (1);

   /*** Spawn off a task that knows all of the current symbol definitions,
   /    etc. of the current task to execute the DCL command line COMMAND. */
   desc.dsc$w_length  = strlen (command);
   desc.dsc$a_pointer = command;
   desc.dsc$b_class   = DSC$K_CLASS_S;
   desc.dsc$b_dtype   = DSC$K_DTYPE_T;

   systurn (&term_in_caucus, GET_TERM, 0);
   systurn (&term_original,  SET_TERM, 0);
   complete = -1L;
   flags    =  0L;
   status = lib$spawn (&desc, 0, 0, &flags, 0, 0, &complete);
   systurn (&term_in_caucus, SET_TERM, 0);

   if (debug  &&  status != SS$_NORMAL)
      printf ("syscmd failed: status=%d, complete=%x\n", status, complete);

   /*** Most successful Vax commands have a completion code of 1.  Some
   /    oddballs (edit/tpu, cc) have other codes, but they all seem to set
   /    bit 0 to 1 if the command succeeded, and 0 if it failed. */
   if      (status == SS$_NORMAL  &&  (complete & 0x01))  code = complete+1;
   else if (status == SS$_NORMAL)                         code = -2;
   else if (status == SS$_EXQUOTA)                        code = -1;
   else                                                   code =  0;
   RETURN (code);
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR
   struct  { short len;   char str[100]; } comline;
   short   code, status;
   fortran cp$();

   ENTRY  ("syscmd", "");

   systurn (&term_in_caucus, GET_TERM, 0);
   systurn (&term_original,  SET_TERM, 0);

   strcpy (comline.str, command);
   comline.len = strlen(comline.str);
   cp$ (&comline, &code, &status);

   systurn (&term_in_caucus, SET_TERM, 0);

   RETURN (code==0);
#endif

}

/*** Operating system dependencies:
/    What conditions are detected per O.S.:
/
/   System V, Xenix:
/      1) Child process exited cleanly
/      2) Child process terminated by another process
/      3) Other
/
/   SunOS:
/      1) Child process stopped by another process
/      2) Child process terminated by another process
/      3) Child process probably exited cleanly
/      (We're not confident of this: the man pages don't match
/       what really happens, so we have to make an empirical guess.)
/
/   MS-DOG
/      1) Child process executed successfully
/      2) Argument list was too big
/      3) Other problem
/ */
