/*** SYSGET1KEY.   Get a key-stroke from the user's terminal.
/
/    success = sysget1key (key);
/
/    Parameters:
/       int   success;   (Success/failure code.)
/       long *key;       (Key-stroke value.)
/
/    Purpose:
/       Read a single key-stroke from the keyboard, and return it in KEY.
/       Return a success or failure code value.  The key-stroke is *not*
/       echoed; that is up to the caller.
/
/       Function keys (that may send multiple bytes) are read as one "key"
/       and return a single value.
/
/       Characters (or keys) that otherwise send multiple bytes (such as
/       Japanese characters) are read one byte at a time, for each call
/       to sysget1key().
/
/    How it works:
/       Sysget1key() assumes that the caller has set the terminal driver
/       to "raw" mode, i.e. the characters are not interpreted by the
/       operating system.
/
/       If the key-stroke was a function key, sysget1key() puts a value
/       of 70000 + a special code, into KEY.  Sysget1key() does a lot of
/       special processing to detect such function keys.
/
/       Otherwise, the 1-byte value of the key, from 0 to 255, is placed
/       in KEY.
/
/    Returns:
/       0 if successful.
/      -2 if the local end-of-file key was pressed
/      -3 if the local interrupt ("<CANCEL>") key was pressed
/      -4 on hang-up, or catastrophic i/o error
/      -5 on indecipherable function key
/
/    Function key values:
/       Each of the "function key" values 70000-70199 have a special meaning.
/       They are defined in the tables below.  Note that the values shown
/       below are all offsets from 70000.  Not all keyboards can produce
/       all codes.
/ 
/       Table 1: keys common to many terminals
/                        Normal   Shift     CTRL    Shift-CTRL
/             Key Name   Value    Value     Value    Value
/          -----------------------------------------------------------
/             up arrow     101      121      141       161
/           left arrow     102      122      142       162
/           down arrow     103      123      143       163
/          right arrow     104      124      144       164
/
/       Table 2: vt100-specific keys
/                        Normal   Shift     CTRL    Shift-CTRL
/             Key Name   Value    Value     Value    Value
/          -----------------------------------------------------------
/            PF1           111      131      151       171
/            PF2           112      132      152       172
/            PF3           113      133      153       173
/            PF4           114      134      154       174
/
/       Table 3: Dasher D200/D400-specific keys
/                        Normal   Shift     CTRL    Shift-CTRL
/             Key Name   Value    Value     Value    Value
/          -----------------------------------------------------------
/               F1           1       21       41        61
/               F2           2       22       42        62
/               F3           3       23       43        63
/               F4           4       24       44        64
/               F5           5       25       45        65
/               F6           6       26       46        66
/               F7           7       27       47        67
/               F8           8       28       48        68
/               F9           9       29       49        69
/              F10          10       30       50        70
/              F11          11       31       51        71
/              F12          12       32       52        72
/              F13          13       33       53        73
/              F14          14       34       54        74
/              F15          15       35       55        75
/              F16          16       36       56        76
/              F17          17       37       57        77
/              F18          18       38       58        78
/              F19          19       39       59        79
/              F20          20       40       60        80
/              C1          111      131      151       171
/              C2          112      132      152       172
/              C3          113      133      153       173
/              C4          114      134      154       174
/              BS erase*   100       -        -         -
/ 
/              Note that the equation for the dasher "F" function keys in
/              the table above is:
/                 KEY = (FKey#) + 20*(SHIFT?) + 40*(CTRL?)
/
/    Error Conditions:  none
/ 
/    Side effects:      none
/
/    Related functions:
/
/    Called by:  Application.
/
/    Operating system dependencies:
/       Needs extern struct sysio_t sysio, which contains information
/       about how the terminal has been set.
/
/    Known bugs:
/       Sysget1key() gets confused if it sees an ESC followed immediately
/       by a legal ESC sequence.  It reports all of the characters as an
/       unknown ESC sequence.
/
/       CR would like to see more protection against possible sign extension
/       of the high order bit of "char c".
/
/    Home:  sys/sysget1k.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: JV  2/02/90 17:29 Created function. */
/*: JV  2/20/90 04:57 Changed tables to allow DBC encoding. */
/*: JV  2/20/90 06:22 Added more DG keys. */
/*: JV  2/20/90 09:20 Expand table further. */
/*: JV  9/11/90 18:52 Resurrected as sysget1key(), added VT100. */
/*: JV  9/18/90 16:54 Changed argument from *char to *long. */
/*: JV  9/20/90 10:38 Add caucus.h */
/*: CR 11/11/90 18:27 Add missing closing #endif. */
/*: JV  1/13/91 19:08 Add ^C interrupt to AOS/VS */
/*: CJ 11/22/90 21:48 Change use of sysio members. */
/*: CP 12/23/90 21:04 Add PRIME code, new READCHAR macro. */
/*: CR  2/01/91 16:40 Replace global hangup with syshngtest(). */
/*: CR  2/20/91  1:26 Fix oops in UNIX READCHAR (missing n = ) */
/*: CR  3/11/91 16:20 Add AX & UL system type codes. */
/*: CR  5/24/91 17:52 Redo comments, handle vt100 keys properly. */
/*: CR  7/24/91 21:54 Make Lint happier. */
/*: JV  8/27/92 13:21 Add code for VV. */
/*: CX  6/02/92 14:43 Make return values consistent with sysrdline(). */
/*: CI 10/07/92 14:29 Chix Integration. */

#include <stdio.h>
#include "handicap.h"
#include "caucus.h"
#include "systype.h"
#include "sys.h"
#if UNIX | CX
#include <setjmp.h>
#endif

#define  TAPELEN 6     /* Length of virtual tape on virtual tapedrive. */

#if UNIX | IM | PN | VV | CA | CX | PR
#define  ESC    '\033'   /* Escape key. */
#define  CTLH   '\010'   /* Control-H, backspace. */
#define  DEL    '\177'   /* Del key. */
#define  BELL   "\007"   /* String containing bell (^G). */
#define  RS     '\036'   /* ASCII RS code. */
#endif

#define ERASE(tape)  {for (i=0; i<TAPELEN; i++) tape[i]='\0';   tapehead=0; }

/*** The READCHAR macro handles actually reading a single byte from
/    the terminal for a particular operating system. */

#if UNIX
#define  READCHAR(n,c)    n = read (0, &c, 1)
#endif

#if BU | SU | UL
#undef   READCHAR
#define  READCHAR(n,c)    sysb_read=1, n = read (0, &c, 1), sysb_read=0
#endif

#if VV
#define  READCHAR(n,c)    n=1, vaxread(&c, 1, 1, sysio.state.o_echo, 0)
extern struct sysio_t sysio;
#endif

#if PR
#define  READCHAR(n,c)    c1in$ (&x), c = x & 0x7F, n = 1
#endif

extern struct sysio_t       sysio;

/*** Full-window systems have a no-op for sysget1key(). -------------- */
#if CVM | PX
FUNCTION  sysget1key (c)
   long  *c;
{
   ENTRY  ("sysget1key", "");
   if (syshngtest())  RETURN (-4);
   RETURN (0);
}
#endif


/*** Here's the real code for most normal operating systems. ---------- */
#if UNIX | CX | CA | PR | VV
FUNCTION  sysget1key (key)
   long  *key;
{
   static char  tapedrive[TAPELEN] = {'\0', '\0', '\0', '\0', '\0', '\0'};
   static short tapesize=0;    /* Size of valid data on tape. */
   static short tapehead=0;    /* Position of tape r/w head. */
   short  i;
   short  byte;
   char   c;
   register int n;

#if BU | SU | UL     /* Only need stupid longjmp'ing stuff for BSD. */
   extern int     sysb_flag, sysb_read;
   extern jmp_buf sysb_env;
#endif

#if PR
   short   x;
   fortran c1in$();
#endif

   ENTRY ("sysget1key", "");

/*** SYSGET1KEY is a simple automaton, in the classic computer
/    science sense.  It reads one byte at a time from the keyboard,
/    returning single-byte keystrokes to the caller.
/    It attempts to interpret multiple-byte keystrokes and translate
/    them into one of the codes above.
/
/    Result:
/       1) Function Keys: if the first byte is one of a very select
/          few (sort of like the Marines) we continue reading
/          bytes until we can decide we have a valid function key
/          or not.  If it's a valid function key, we either copy
/          a value from the above tables into KEY or take some
/          appropriate action.  If it's not a valid function key,
/          we've got a problem.  We copy the first byte into KEY
/          and store the later bytes for later calls to SYSKET1KEY.
*/
   if (syshngtest())  RETURN (-4);

   /*** SYSGET1KEY() uses a "virtual tapedrive" to record bytes from
   /    a multiple-byte keystroke.  If the keystroke is decipherable,
   /    put the f-key value into KEY and return.  Otherwise, return
   /    individual bytes from the keystroke one-byte-per-call, FIFO. */

   if (tapesize) {            /* If anything from the last keystroke... */
      *key = tapedrive[tapehead++];     /* Get byte from FIFO tape and... */
      tapesize--;                                      /* move tape head. */
      if (NOT tapesize) ERASE (tapedrive);    /* If we've read last byte. */
      RETURN (0);
   }

/*--------------------AOS/VS----------------------------------------*/
#if CA
   { short  shift=0, ctrl=0;
   n = read (0, &c, 1);         /* Get a byte */
   if (NOT n)   RETURN (-2);    /* EOF */
   if (n == -1) RETURN (-5);    /* Error */

   if (sysio.state.eightbit <= 0)  c = c & 0177;

   if (c > 31 && c < 127) {     /* If a normal char, get it out quick. */
      *key = c;
      RETURN (0);
   }
   if (c == RS) {              /* ASCII "RS" means special key. */
      tapedrive[0] = c; tapesize = 1;
      n = read (0, &c, 1);
      if (NOT n)  {               /* EOF: Shouldn't happen, but...*/
         *key = RS;
         tapesize = 0;
         RETURN (-2);
      }
      if (n== -1) {               /* Error: Neither should this...*/
         *key = RS;            /*...but we'll handle it anyway.*/
         tapesize = 0;
         RETURN (-4);
      }
      if (c > 31 && c < 177) {    /* User pressed a function key. */
         if ( (c > 95 && c < 111) || (c > 31 && c < 47) )
            shift=1;              /* 040-056, 0140-0156 */
         if (c > 31 && c < 63 && c != 47)
            ctrl=1;               /* 040-056, 060-076 */
         c %= 16;         /* These keyboards don't generate remainder=15. */
         if (NOT c) c=15;     /* Remainder=0 ==> F15. */
         *key = 70000 + c + 20*shift + 40*ctrl;
      }                                         /* End of Funct key code. */
      else if (c > 201 && c < 206)
         *key = 70000 + (c-71);           /* Shift C1-C4 */
      else if (c > 205 && c < 210)
         *key = 70000 + (c-95);           /*       C1-C4 */
      else if (c == 23) *key = 70121;           /* Shift up arrow */
      else if (c == 26) *key = 70123;           /* Shift down arrow */
      else {
         *key = RS;
         tapedrive[0] = c; tapesize = 1;
         RETURN (0);
      }
   }
   else if (c == 255) *key = CTLH;
   else if (c ==  23) *key = 70101;              /* Up arrow */
   else if (c ==  26) *key = 70103;              /* Down arrow */
   else
      *key = c;             /* Return single-byte, non-printable ASCII. */
   if (*key == 70005 || *key == 3) sysbrkset();  /* F5, ^C = <CANCEL> */
   }
   RETURN (0);
#endif


/*--------------------Unix, etc. systems----------------------------*/
#if UNIX | VV | PR
#if BU | SU | UL
   /*** Save the Current ENVironment.  If we come back from SETJMP with a non-
   /    zero return, a read from the terminal was interrupted by CANCEL. */
   if (setjmp (sysb_env)) { sysbrkset(); tapesize = 0; RETURN (-3); }
#endif

   /*** Read bytes until we've gotten a single, ordinary character, or
   /    we're at the end of a multi-byte function key. */
   while (1) {
      READCHAR (n, c);
      byte = c & 0xFF;
  
      if (sysbrktest())   RETURN (-3);     /* <CANCEL> */
      if (n == 0)         RETURN (-2);     /* EOF */
      if (syshngtest())   RETURN (-4);     /* Catastrophic Error. */
      if (n <  0)         RETURN (-4);     /* Other Catastrophic Error. */
  
      if (sysio.state.eightbit <= 0)  byte = byte & 0177;
  
      /*** If a regular ASCII character, get it out quick. */
      if (tapesize == 0  &&  byte > 31  &&  byte < 127) {
         *key = byte;
         RETURN (0);
      }
  
      /*** Test for interrupt character. */
      if (byte == sysio.c_intr)               { sysbrkset();   RETURN (-3); }
  
      /*** Test for end-of-file character. */
      if (byte == sysio.c_eof)                                 RETURN (-2);
  
      /*** Any remaining non-ESC control characters. */
      if (tapesize == 0  &&  byte != ESC)  { *key = byte;      RETURN (0); }

      /*** The remaining code handles the vt100 function keys, which produce
      /    multiple-byte sequences that begin with ESC.  The sequences are
      /    stored in a "virtual tapedrive". */

      /*** An ESC by itself starts a multi-byte function key. */
      if (tapesize == 0  &&  byte == ESC) {
         tapedrive[0] = ESC; tapesize = 1;
         tapehead = 1;
         continue;
      }
  
      /*** PF1 through PF4. */
      if (tapesize == 1) {
         if (byte == 'P')  { *key = 70111L;   break; }
         if (byte == 'Q')  { *key = 70112L;   break; }
         if (byte == 'R')  { *key = 70113L;   break; }
         if (byte == 'S')  { *key = 70114L;   break; }
      }

      /*** 2nd byte of 3-byte keys. */
      if (tapesize == 1  &&  (byte == '[' || byte == 'O')) {
         tapedrive[1] = byte; tapesize = 2;
         tapehead = 2;  continue;
      }

      /*** Arrow keys. */
      if (tapesize == 1  ||  tapesize == 2) {
         if (byte == 'A')  { *key = 70101L;   break; }
         if (byte == 'B')  { *key = 70103L;   break; }
         if (byte == 'C')  { *key = 70104L;   break; }
         if (byte == 'D')  { *key = 70102L;   break; }
      }

      /*** If we've gotten here, this is a multi-byte sequence that we
      /    can't interpret.  Use the virtual tapedrive to send all the
      /    bytes back to the caller, one per call. */
      tapedrive[tapehead] = byte;                  /* Write this byte. */
      *key = tapedrive[0];                         /* Read first byte. */
      tapehead=1;                    /* Next time read from next byte. */
      RETURN (0);                        /* TAPESIZE remains the same. */
   }

   /*** If we broke out of the loop, erase the virtual tapedrive and return. */
   ERASE (tapedrive);
   tapesize = 0;
   RETURN (0);
#endif
}
#endif
