/*** SYSHNG.  Hangup detection functions:
/
/    syshnginit (detect);
/    h = syshngtest ();
/
/    Parameters:
/       int    detect;   Detect hangups  (1), ignore hangups (0).
/       int    h;        Hangup detected (1), else 0.
/
/    Purpose:
/       Syshnginit() controls hangup detection and handling.  If DETECT
/       is 1, hangup signals are trapped and detected.  If DETECT is 0,
/       hangup signals are ignored.  Syshnginit() may be called multiple
/       times to change how hangups are treated.
/
/       Syshngtest() tells the caller if a hangup has occurred and been
/       detected.
/
/    How it works:  
/
/    Returns:  Syshnginit() always returns 1.
/       Syshngtest() returns 1 if hangups are detected, AND a hangup
/       has occurred.  It returns 0 if hangups are ignored, or if no
/       hangup has occurred.
/
/    Error conditions:
/
/    Side effects:  Some versions are coupled via setjmp() and longjmp()
/       with sysread(), as a hangup may not cause read() to return 
/       automatically.
/
/    Related functions: sysread().
/
/    Called by:
/
/    Operating system dependencies:
/       Sys V unix: use signal().
/       PRIME:      see SYSINIT in sysinit.h.
/       Others:     not implemented.  Syshngtest() always returns 0.
/
/    Known bugs:      none
/
/    Home:  sys/syshng.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CR  2/01/91 14:26 New function. */
/*: CR  2/09/91 21:44 Split long #if line in two (UX1, UX2). */
/*: CR  3/04/91 23:19 Add DY code.*/
/*: CR  3/11/91 16:43 Add AX & UL system type codes. */
/*: CR  3/20/91 23:56 Add UL code. */
/*: CR 10/03/91 15:15 Make SU same as UL. */
/*: SB  5/16/91 10:39 Add DP system type. */
/*: JV  1/17/92 16:59 Declare SU signal handlers as (*handler)(). */
/*: JV  7/16/92 15:29 Add U4. */
/*: CR  7/21/92 17:39 Remove DP (in favor of DYP). */
/*: CR  3/16/94  0:11 BU is mostly like SU. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#if UL | SU | BU
#include <setjmp.h>
extern int     sysb_flag;   /* Interrupt flagged? */
extern int     sysb_read;   /* Interrupt in the middle of a read? */
extern jmp_buf sysb_env;    /* longjmp environment in case of sysb_read */
#endif

static int hangup = 0;
static int detect = 1;

/*-------------------------------- Unixes--------------------------*/
#define UX1  A3 | LA | SV | HU | FU | MU | MV | PU | SX | TX | VV | IX | U4
#define UX2  IX | CX | PX | DU | RO | U3 | AU | CA | DY | AX | UL | SU | BU
#if UX1 | UX2

#include <signal.h>

FUNCTION  syshnginit (on)
   int    on;
{
#if SU
   void sysbrk2(), syshang();
#else
   int    sysbrk2(), syshang();
#endif

   ENTRY ("syshnginit", "");
   hangup = 0;
   detect = on;
   if (detect) { signal (SIGHUP, syshang);    signal (SIGQUIT, syshang); }
   else        { signal (SIGHUP, SIG_IGN);    signal (SIGQUIT, SIG_IGN); }
   RETURN (1);
}

FUNCTION  syshngtest ()
{
   ENTRY  ("syshngtest", "");
   RETURN (detect ? hangup : 0);
}

#if SU
FUNCTION  void syshang()
{
   void sysalarm();
#else
FUNCTION  syshang()
{
   int    sysalarm();
#endif

   hangup = 1;
   signal (SIGALRM, sysalarm);
   alarm  (3);
#if UL | SU | BU
   if (sysb_read) {
      sysb_read = 0;
      longjmp (sysb_env, 1);
   }
#endif
}

#if SU
FUNCTION void sysalarm()
#else
FUNCTION  sysalarm()
#endif
{
}
#endif

/*----------------------------------------------------------*/

#if PR
#include <setjmp.h>

/*** The PRIME version establishes syshang() as the hangup handling
/    function in the main program.  See the SYSINIT macro in sysinit.h. */

int         sysb_read;   /* Interrupt in the middle of a read? */
jmp_buf     sysb_env;    /* longjmp environment in case of sysb_read */

FUNCTION  syshnginit (on)
   int    on;
{
   ENTRY  ("syshnginit", "");
   detect = on;
   RETURN (1);
}

FUNCTION  syshngtest ()
{
   ENTRY  ("syshngtest", "");
   RETURN (detect ? hangup : 0);
}

FUNCTION  syshang (i)
   int    i;
{
   hangup = 1;

   /*** If hangup occurred in the middle of a terminal read, longjmp back
   /    out of the read.  See sysread() for details. */
   if (sysb_read) {
      sysb_read = 0;
      longjmp (sysb_env, 1);
   }
}
#endif


/*----------------------------------------------------------*/
#if CVM | PS | IM | PN

FUNCTION  syshnginit (on)
   int    on;
{
   ENTRY  ("syshnginit", "");
   RETURN (1);
}

FUNCTION  syshngtest ()
{
   ENTRY  ("syshngtest", "");
   RETURN (hangup);
}
#endif
