
/*** SYSRDLINE.   Read a LINE of bytes from the user's terminal.
/
/    code = sysrdline (buf, maxbuf);
/
/    Parameters:
/       int   code;      (return status code)
/       char  buf[];     (buffer to place bytes read from terminal)
/       int   maxbuf;    (maximum size of BUF)
/
/    Purpose:
/       Sysrdline() reads an entire line of bytes from the user's terminal
/       and places them in BUF, along with a final 0 byte.
/
/    How it works:
/       If there is a terminating newline, it is thrown away.
/       Only the first MAXBUF-1 bytes are placed in BUF; the rest
/       are thrown away.
/
/       For most systems, sysrdline() is intended to be used with input
/       character processing ("cooking" and echoing) handled by the operating
/       system.
/
/       The AOS/VS version of sysrdline() handles input one keystroke at
/       a time, and also handles special function key interrupts that invoke
/       other services.
/
/    Returns:
/       0   normal return, input line in BUF.
/      -1   normal return, input was longer than MAXBUF, input truncated
/      -2   end-of-file
/      -3   cancel/interrupt
/      -4   hangup or catastrophic error
/      -5   other/unknown error
/
/    Error Conditions:
/
/    Side effects:
/
/    Related functions:
/
/    Called by:  Application.
/
/    Operating system dependencies: lots
/
/    Known bugs: 
/
/    Home:  sys/sysrdlin.c
/ */

/*** Copyright (C) 1986 Camber-Roth.  All rights reserved. */

/*: CX  5/29/92 16:22 New function, from old sysread(). */
/*: CR  2/24/93 12:59 Add U4 to SYSV code. */
/*: CK  9/22/93 19:02 Move AOS code to end. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "sys.h"

#if CA
#include <termio.h>
#include <it_lib.h>
#endif

extern struct sysio_t       sysio;

/*** Cases that must be considered.  N is # of characters read.
/    (a) <CANCEL> found.  Empty line and go home.
/    (b) n==0.  End-of-file-char(?) at start of line, EOT==1EOF.
/    (c) n==0.  All other cases.  Ignore?  Cause error?  Which?
/    (d) n <0.  Hangup... or worse? 
/    (e) n==1.  Normal character, < maxlen chars.
/    (f) n==1.  Normal character, > maxlen chars.  Echo?  Beep?  What?
/    (g) n==1.  Control character or function key.  */


/*------------------------------- Unix, and VM-----------------------*/
#define SYSV1  A3 | LA | SV | HU  | MU | MV | PU | SX | TX | IX | U4
#define SYSV2  CX | DU | PX | CVM | RO | PS | U3 | AU | DY | AX | DP
#if SYSV1 | SYSV2

FUNCTION sysrdline (line, maxlen)
   char  line[];
   int   maxlen;
{
   int    c, last;
   char  *fgets();

   ENTRY ("sysrdline", "");

   if (maxlen == 0)  maxlen = 10;

   if (syshngtest())  { line[0] = '\0';   RETURN (-4); }

   /*** Normally, we read up to MAXLEN chars from the terminal.  If
   /    the read fails, it's either hangup, cancel, or EOF. */
   if (fgets (line, maxlen, stdin) == NULL)  {
      line[0] = '\0';
      sysputstr ("\n");
      if (syshngtest())   RETURN (-4);
      if (sysbrktest())   RETURN (-3);
      RETURN (-2);
   }

   /*** Was the line ended cleanly? */
   last = strlen(line) - 1;
   if (line[last] == '\n')  { line[last] = '\0';   RETURN (0); }

   /*** No, we truncated it.  Throw away everything up to real end of line. */
   line[maxlen-1] = '\0';
   do  { c = getchar(); }  UNTIL (c < 0  ||  c=='\n');
   if (syshngtest())   RETURN (-4);
   if (sysbrktest())   RETURN (-3);
   RETURN (-1);
}
#endif

/*---------------------------------Berkeley Unix-------------------------*/
#if BU | SU | UL
#include <setjmp.h>

FUNCTION sysrdline (line, maxlen)
   char  line[];
   int   maxlen;
{
   extern int     sysb_flag, sysb_read;
   extern jmp_buf sysb_env;
   int    ended, n,len;
   char   c;

   ENTRY ("sysrdline", "");

   if (maxlen == 0)  maxlen = 10;

   if (syshngtest())  { line[0] = '\0';   RETURN (-4); }

   /*** Save the current ENVironment.  If we come back from SETJMP with a non-
   /    zero return, a read from the terminal was interrupted by CANCEL. */
   if (setjmp (sysb_env)) { 
      line[0] = '\0';
      if (syshngtest())  RETURN (-4);
      if (sysbrktest())  RETURN (-3);
      RETURN (0); 
   }

   /*** Read the line; we may get it in pieces and have to assemble it
   /    into one line.  */
   len = 0;
   do {
      sysb_read = 1;
      n  = read (0, line+len, maxlen-len-1);
      sysb_read = 0;

      if (n < 0  ||  len+n == 0) {
         line[0] = '\0';
         sysputnl();
         RETURN  (-2);
      }

      len += n;
      line[len] = '\0';
      if (len > 0) ended = (line [len-1] == '\012');
      else         ended = 0;
   } while (NOT ended  &&  len < maxlen-1);

   if (ended) { line[len-1] = '\0';    RETURN (0); }

   /*** Here if we got more than MAXLEN-1 chars.  Read and toss
   /    everything until we see a newline. */
   line[maxlen-1] = '\0';
   while  (read (0, &c, 1) == 1   &&   ( (c & 0177) != '\012') ) ;
   if (syshngtest())  RETURN (-4);
   if (sysbrktest())  RETURN (-3);
   RETURN (-1);
}
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR
#include <setjmp.h>

FUNCTION sysrdline (line, maxlen)
   char  line[];
   int   maxlen;
{
   extern  int     sysb_flag, sysb_read;
   extern  jmp_buf sysb_env;
   struct  { short len;  char str[160]; } comline;
   short   comlength, status;
   fortran cl$get();

   ENTRY ("sysrdline", "");

   if (maxlen == 0)  maxlen = 10;

   if (syshngtest())  { line[0] = '\0';   RETURN (-4); }

   /*** Save the current ENVironment.  If we come back from SETJMP with a
   /    non-zero return, a read from the terminal was interrupted by
   /    a QUIT or a hangup. */
   if (setjmp (sysb_env)) {
      line[0] = '\0';
      if (syshngtest())  RETURN (-4);
      if (sysbrktest())  RETURN (-3);
      RETURN (0);
   }

   comlength = min (159, maxlen-1);

   comline.str[0] = '\0';
   comline.len    =   0;
   sysb_read = 1;
   cl$get (&comline, &comlength, &status);
   sysb_read = 0;
   comline.str[comline.len] = '\0';

   if ((comline.str[0] & 0177) == 3)  RETURN (-2);
   last = strlen (comline.str) - 1;
   if (comline.str[last] == '\n')  comline.str[last] = '\0';
   strcpy (line, comline.str);

   RETURN (status>0 ? -1 : 0);
}
#endif

/*---------------------------------------IBM PC--------------------------*/
#if IM | PN
FUNCTION sysrdline (line, maxlen)
   char   line[];
   int    maxlen;
{
   int    c;
   short  n, ended, len;

   ENTRY ("sysrdline", "");

   if (maxlen == 0)  maxlen = 10;

   len  = 0;
   while (1) {

      /*** Loop until the user hits a character. */
      while ( (c = sysgetch()) == 0)  ;

      /*** A -1 means loss of carrier. */
      if (c < 0)       {                line[0] = '\0';     RETURN (-4); }

      if (sysio.state.eightbit <= 0)  c = c & 0x7F;

      /*** ^C sets an interrupt, and returns an empty LINE. */
      if (c == '\003') { sysbrkset();   line[0] = '\0';     RETURN (-3); }

      /*** ^Z is end-of-text. */
      if (c == '\032') {
         if (sysio.state.c_echo > 0) { sysputch ('\015');   sysputch ('\012'); }
         RETURN   (-2);
      }

      else if (c == '\015'  ||  c == '\012') {
         line[len] = '\0';
         if (sysio.state.c_echo > 0) { sysputch ('\015');   sysputch ('\012'); }
         RETURN   (0);
      }

      else if (c == '\010') {
         if (len > 0) {
            --len;
            if (sysio.state.c_echo > 0)
               { sysputch (c);   sysputch (' ');   sysputch (c);}
         }
      }

      else if (len >= maxlen-1)   ;

      else {
         line[len++] = c;
         if (sysio.state.c_echo > 0)  sysputch (c);
      }
   }
}
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV

FUNCTION sysrdline (line, maxlen)
   char  line[];
   int   maxlen;
{
   int    length;
   char   *a, *b;
   char   buf[200];

   ENTRY ("sysrdline", "");

   if (syshngtest())  { line[0] = '\0';   RETURN (-4); }

   if (maxlen == 0)  maxlen = 10;

   vaxread (buf, 199, 0, sysio.state.o_echo > 0, sysio.state.lineedit);
   if (sysio.state.c_echo > 0)  fputs ("\n", stdout);
   if (sysbrktest())            RETURN (-3);

   /*** If ^Z found (and used as EOT), return end of text. */
   if (buf[0] == 032) {
      fputs  ("\n", stdout);
      RETURN (-2);
   }

   /*** ^H filtering. */
   for (a=b=buf;   *a;   ++a) {
      if (*a == 010) {
         if (b != buf)  --b;
      }
      else *b++ = *a;
   }
   *b = '\0';

   length = strlen (buf);

   /*** Truncate if line was too long. */
   buf[maxlen-1] = '\0';
   strcpy (line, buf);
   RETURN (length >= maxlen ? -1 : 0);
}
#endif

/*---------------------------------------AOS/VS--------------------------*/
#if CA

#define F5  '\165'
#define F6  '\166'
#define F11 '\173'
#define F14 '\176'
#define SF1 '\141'

extern short under_ceo;
extern int  menu_level;

FUNCTION  sysrdline (line, maxlen)
   char   line[];
   int    maxlen;
{
   char c, cstr[5];
   short col=0, n, ceo_return, result_code;
   int ioctl(), ioerr, success;
   struct termio termbuf, oldbuf;
   
   ENTRY ("sysrdline", "");

   line[0]='\0';            /* Empty line in case user just hits <RETURN> */   
   if (syshngtest()) { line[0] = '\0';  RETURN (-1); }
   
   if (maxlen == 0)  maxlen = 10;
   
   /*** AOS must start the alarm so it can send ANULL msgs to the server. */
   sysalstart ();

   /*** In contrast to the code for most other operating systems, the AOS
   /    version of sysrdline() actually reads the characters one at a time,
   /    and does all the "cooking" and processing. */
   success = 1;
   do {
      n = read (0, &c, 1);          /* Get a byte from the terminal. */
      if (sysio.state.eightbit <= 0)  c = c & 0x7F;

      if (n == 0) {
         if (sysaltest())  { sysalclear();
                             sysalstart (); continue;  }
         sysalstop();
         sysputnl ();
         RETURN (0);             /* EOF */
      }
      
      /*** The AOS client needs to send an occasional msg to the server. We've
      / already started the clock, the alarm handler will send the msg, so we
      / just need to detect the "ringing", clean up, and reset the alarm. */
      if (n == -1) {                                    /* read() failed: */
         if (sysaltest())  { sysalclear();
                             sysalstart (); continue;  }
         sysalstop();
         if (sysbrktest()) { line[0] = '\0'; RETURN(1); }     /* <CANCEL> */
         else RETURN (-1);                             /* Hangup or worse */
      }

      if (c > 31 && c < 127) {                       /* If a normal char, */
         if (col < maxlen-1) {                      /* turn around quick. */
            line[col++] = c;    line[col] = '\0'; }
         else success = 2;
         if (sysio.state.c_echo > 0)   write (1, &c, 1);
      }
      
      else if (c == '\n' && sysio.state.c_echo > 0) { write (1, &c, 1); break; }
      
      else if (c == '\36') {           /* ASCII "RS" means special key. */
         n = read (0, &c, 1);  /* Get next character to determine function. */
         if (n < 1) continue;  /* None: Not supposed to happen! */
 
         /*** We've got a code following the f-key prefix.  Now figure
         /    out what to do based on it. */
         switch (c) {
         case (F5):
          if (under_ceo) {
            ioerr = ioctl (0, TCGETA, &oldbuf);
            ioerr = ioctl (0, TCGETA, &termbuf);       /* Modify keyboard to */
            termbuf.c_lflag = termbuf.c_lflag | (ECHO);      /* Turn on echo */
            termbuf.c_lflag = termbuf.c_lflag | (ICANON); /* Turn on cooking */
            IT2_INTERRUPT(&result_code); sysbrkset();    /* CEO library call */
            ioerr = ioctl (0, TCSETA, &oldbuf);  /* Reset terminal "our" way */
            sysputstr ("\022");                      /* Turn scroll back on. */
            break;
          } else { sysbrkset(); line[0] = '\0';      /* <CANCEL> */
                   sysputnl();  sysalstop(); RETURN (1); }

         case (F6):  sysbrkset(); line[0] = '\0'; sysputnl();  /* <CANCEL> */
            sysalstop(); RETURN (1);

         case (F11):
            if (menu_level) {                        /* If we're in menus: */
               strcpy (line, "9"); sysputnl();       /* Back one menu level */
               sysalstop(); RETURN (1); }
            else break;
    
         case (F14):    /* User wants CEO calculator, for some inane reason */
            if (menu_level && under_ceo) {
               ioerr = ioctl (0, TCGETA, &oldbuf);      /* Get present term. */
               ioerr = ioctl (0, TCGETA, &termbuf);    /* Modify keyboard to */
               termbuf.c_lflag = termbuf.c_lflag | (ECHO);   /* turn on echo */
               termbuf.c_lflag = termbuf.c_lflag | (ICANON);   /* and cookng */
               IT2_CALCULATOR (&ceo_return);
               ioerr = ioctl (0, TCSETA, &oldbuf);    /* Put our keybrd back */
               sysputstr ("\022");                    /* Turn scroll back on */
               sysbrkset (); }                        /* Repaint screen */
            break;

         case (SF1):
            strcpy (line, "help");                    /* ...I need somebody */
            sysputnl();  sysalstop();
            RETURN (1);

         default:  continue; 
         }
      }

      /*** ^H or DEL anywhere but column zero backspaces and erases. */
      else if ( (c=='\010' || c=='\177') &&  col > 0) {
         sysputstr ("\b \b"); /* This should backspace according to the OS */
         line [--col] = '\0';
      }
      
      /*** ^C generates an interrupt, but not like F5! */
      else if (c=='\03') { sysbrkset(); line[0] = '\0';      /* <CANCEL> */
                           sysputnl();  sysalstop(); RETURN (1); }

   } while (c != '\n' && NOT sysbrktest());

   sysalstop();
   RETURN (1);
}
#endif
