/*** UNIT_LKFX.   Open proper lockmap file for use by unit_lock().
/
/    ok = unit_lkfx (n, nuser, nconf);
/
/    Parameters:
/       int    ok;          (Success?)
/       int    n;           (type of lockmap file: LCNF, LUSR... )
/       Chix   nuser;       (if LUSR, userid of user)
/       long   nconf;       (conference number, host number, etc.)
/
/    Purpose:
/       For Unix system V and related systems, unit_lock() operates
/       by mapping a particular file into a byte number, and then
/       locking that byte in a selected lock map file.
/
/       Unit_lkfx() ensures the the proper lockmap file is open and
/       ready to be used by unit_lock().
/
/    How it works:
/       N must be one of the types:
/          LUSR    locking a file about a particular user
/          LSYS    locking a system-wide file
/          LCNF    locking a file about a particular conference
/          LHST    locking a file about a particular host
/          LSHP    locking a file about a particular shipping bay
/          LRCV    locking a file about a particular receiving bay
/
/       If N is LSYS, NUSER and NCONF are ignored.
/       If N is LUSR, NUSER must be the user's userid.  NCONF is ignored.
/       For all other cases, NCONF is the number of the particular
/       conference, host, shipping bay, or receiving bay.  NUSER is ignored.
/
/       Unit_lkfx() checks the global locktype[], opens the proper lockmap
/       file (if not already open), and marks locktype[] appropriately.
/ 
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions:
/       Cannot open or create lockmap file.
/ 
/    Side effects: may change data in global locktype[].
/
/    Related functions:
/
/    Called by:  unit_lock()
/
/    Operating system dependencies: no-op for non system-V
/
/    Known bugs:
/
/    Home:  unit/unitlkfx.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:54 Source code master starting point */
/*: CR  8/04/88 12:39 move <stdio> to top */
/*: JV  8/05/88 13:10 Isolated the 'locktype' def to satisfy linker.*/
/*: CR  9/21/88 15:42 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 12:59 Add "IX" system type. */
/*: CR 11/22/88 17:33 Add PN code. */
/*: CR 12/07/88 15:12 Remove extern int errno; */
/*: CR  4/29/89 14:10 Break SU into SS (lockf) and SN (guardian files). */
/*: CR  6/09/89 12:21 Add "SV" system type code. */
/*: CR  7/17/89 15:08 Add default case to sys v switch. */
/*: CR  8/08/89 13:38 Call sysudnum() to get userid directory number. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CW  5/19/89 12:51 Add XW unit codes. */
/*: CR 10/12/89 15:43 Add DU system type code. */
/*: CR 10/24/89 13:23 Add 3rd diagnostic argument to sysudnum(). */
/*: CR 12/15/89 16:15 PN code must use USER, not USER001. */
/*: JV  7/08/90 18:26 Add MIPS RISC/os (RO). */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CR  9/27/90 12:23 Add U3, AU system types. */
/*: CJ 11/22/90 16:36 Sysudnum() complains only if debug on. */
/*: CJ 11/26/90 21:46 Add LA system type. */
/*: CR  2/09/91 21:44 Split long #if line in two (SYSV1, SYSV2). */
/*: SB 11/30/90 11:08 Add DY system type*/
/*: CR  3/11/91 14:10 Add AX and UL system type codes. */
/*: JV  5/12/91 13:38 Add LHST, LSHP, LREC. */
/*: CR  7/02/91 12:38 Include unitwipe.h. */
/*: JV  7/15/91 10:12 Fix LHST. */
/*: JV  7/19/91 14:28 Split debugging into two parts. */
/*: CR  9/04/91 16:15 Many changes to be sensible for CaucusNet. */
/*: CR  9/09/91 10:52 Add more 'debug' diagnostics, always set ret_val! */
/*: UL 12/11/91 18:53 Expand nconf to a long. */
/*: CR 12/12/91 20:23 LSHP lock file is now Hnnn/lockship. */
/*: CR 12/26/91 19:31 Last arg of unit_lkop is long. */
/*: JV  7/17/92 15:22 Add U4. */
/*: JV  9/15/92 15:01 Change name of LHST to confid/lockhNNN. */
/*: CR  9/07/92 22:22 MV requires 0770 mask on lockmap files. */
/*: DE  4/09/92 13:17 Chixified */
/*: CX  6/05/92 17:28 Include handicap.h to get L(). */
/*: JX 10/01/92 12:19 Change NUSER to Chix, null.chx to nullchix. */
/*: CI 10/08/92  9:33 Chix Integration. */
/*: JV 10/14/92 18:42 Check NUSER before ascofchx. */
/*: CR 10/22/92 14:37 Fix unit_write (...disp). */
/*: CR 10/22/92 16:23 No, remove disp entirely! */
/*: CR 11/15/92 17:59 NUSER may be NULL. */
/*: CL 12/10/92 17:07 Long ascofchx args. */
/*: CP 11/21/93 19:12 Test debug anded with DEB_LOCK. */
/*: CR 12/10/93 15:51 Add sysbrkignore()/init() around creat() call. */

#include <stdio.h>
#include "caucus.h"
#include "handicap.h"
#include "systype.h"
#include "units.h"
#include "unitwipe.h"

#define SYSV1  A3 | LA | SV | HU | FU | MU | MV | PU | SX | SS | TX | IX
#define SYSV2  PX | DU | RO | PS | U3 | AU | DY | AX | DP | U4
#if SYSV1 | SYSV2 | PN
extern struct locktype_t locktype[];
#endif

#if PN
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
#endif

extern Chix   confid;
extern int    debug;

FUNCTION  unit_lkfx (n, nuser, nconf)
   int    n;
   long   nconf;
   Chix   nuser;
{
   char  *bugtell();
   char   confidstr[200], nuser_str[60];

/*------------------------------Most UNIXes------------------------------*/
#if SYSV1 | SYSV2 | PN
   int    udnum, ret_val;

   ENTRY ("unit_lkfx", "");

   if (nuser==NULL  ||  EMPTYCHX(nuser))  nuser_str[0] = '\0';
   else   ascofchx (nuser_str, nuser, 0L, L(58));

   ret_val = 1;
   if (debug & DEB_LOCK) {
      char temp[300];
      sprintf (temp, "UNIT_LKFX: type=%d, nuser='%s', nconf=%ld\n",
                      n, nuser_str, nconf);
      unit_write (XWDB, chxquick(temp,2));
   }

   ascofchx (confidstr, confid, 0L, 200L);

   switch (n) {

      case (LSYS):
         if (locktype[LSYS].fd >= 0)  break;
         sprintf (locktype[n].file, "%s/locksyst", confidstr);
         ret_val =  (unit_lkop (n, locktype[n].file, "", 0L));
         break;

      case (LUSR):
         if (streq (locktype[LUSR].user, nuser_str))  break;
#if SYSV1 | SYSV2
         udnum = sysudnum (nuser, (debug & DEB_LOCK ? 0 : -1), n);
         sprintf (locktype[n].file, "%s/USER%03d/%s/lockuser",
                   confidstr, udnum, nuser_str);
#endif
#if PN
         sprintf (locktype[n].file, "%s/USER/%s/lockuser", confidstr,
                  nuser_str);
#endif
         ret_val =  (unit_lkop (n, locktype[n].file, nuser_str, 0L));
         break;

      case (LCNF):
         if (locktype[LCNF].nconf == nconf)  break;
         sprintf (locktype[n].file, "%s/C%03ld/lockconf", confidstr, nconf);
         ret_val =  (unit_lkop (n, locktype[n].file, "", nconf));
         break;

      case (LHST):
         if (locktype[LHST].nconf == nconf)  break;
         sprintf (locktype[n].file, "%s/lockh%03ld", confidstr, nconf);
         ret_val =  (unit_lkop (n, locktype[n].file, "", nconf));
         break;

      case (LSHP):
         if (locktype[LSHP].nconf == nconf)  break;
         sprintf (locktype[n].file, "%s/H%03ld/lockship", confidstr, nconf);
         ret_val =  (unit_lkop (n, locktype[n].file, "", nconf));
         break;

      case (LRCV):
         if (locktype[LRCV].nconf == nconf)  break;
         sprintf (locktype[n].file, "%s/RECV%03ld/lockrecv", confidstr, nconf);
         ret_val =  (unit_lkop (n, locktype[n].file, "", nconf));
         break;

      default:
         buglist (bugtell ("Bad case for unitlkfx!", n, nconf, nuser_str));
         sysexit (1);
   }

   if (debug & DEB_LOCK) {
      char temp[300];
      sprintf (temp, "    ret_val=%d, lt.file='%s', lt.user='%s'\n",
                          ret_val, locktype[n].file, locktype[n].user);
      unit_write (XWDB, chxquick(temp, 2));
   }

   RETURN (ret_val);
#endif
}


#if SYSV1 | SYSV2 | PN

/*** UNIT_LKOP actually opens the selected type of lockmap file. */

FUNCTION  unit_lkop (n, file, nuser, nconf)
   int    n;
   long   nconf;
   char   file[], nuser[];
{
   int    fd, mask;

   ENTRY ("unit_lkop", "");

   if (locktype[n].fd >= 0)  close (locktype[n].fd);

#if SYSV1 | SYSV2
   mask = 0700;
#if MV
   mask = 0770;
#endif
   sysbrkignore();
   if ( (fd = open (file, 2)) < 0)  fd = creat (file, mask);
   sysbrkinit();
#endif
#if PN
   fd = sopen (file, O_BINARY | O_CREAT | O_RDWR, SH_DENYNO,
                     S_IREAD  | S_IWRITE);
#endif
   if ((debug & DEB_LOCK)  &&  fd < 0)   perror ("unit_lkfx: fd < 0");
   locktype[n].fd = fd;
   strcpy (locktype[n].user, nuser);
   locktype[n].nconf = nconf;
   if (fd < 0)  locktype[n].user[0] = locktype[n].nconf = 0;
   RETURN (fd >= 0);
}
#endif
