/*** UNIT_LOCK.   Name and lock a specific file, by unit number.
/
/    UNIT_LOCK locks the file specified by unit number N.  The file
/    is further specified, if necessary, by conference, item, response,
/    and line-numbers CNUM, ITEM, RESP, and LNUM; and by the string WORD.
/
/    The parameter READWRIT indicates the kind of lock requested.  There
/    are three possibilities:
/       WRITE   lock for writing, only one locker allowed
/       READ    lock for reading, many simultaneous read locks allowed
/       TLOCK   request a WRITE lock, but return immediately if already
/               locked by someone else.
/    Caucus.h defines the macros WRITE, READ, and TLOCK that should be
/    used for this parameter.
/
/    Once a file is locked, it stays locked, until UNIT_UNLK or
/    UNIT_WIPE is called.  On some systems, the lock may also be
/    removed if the process exits.
/
/    The "Arguments" table below describes the arguments that are
/    required for each file unit code.  The meanings of the entries
/    in the table are as follows:
/       Cnum      conference number.
/       Item      item number.
/       Resp      response number.
/       Mess      message number.
/       Lnum      starting line number of a particular partfile.
/       Pfile     part-file number (as in names001)
/       x         number (as in number of a names file).
/       File      name of a (usually external to Caucus) file.
/       Uid       userid.
/       ""        empty string.
/       Uid/""    userid, or uses current userid if empty string.
/       host      host number, 1 to MAXHOST-1
/       ship      shipping method, 1 to 999
/       recv      receiving method, 1 to 999
/       seq       sequence number for CaucusLink, 0 to MAXSEQ-1
/       chunk     chunk number for CaucusLink, 0 to MAXSEQ-1.
/
/    If an argument appears in parentheses (e.g., "(cnum)"), it means
/    that the argument may be used in a call to unit_lock, but is
/    actually ignored.
/
/=======Arguments===================================================
/  N    cnum   item    resp    lnum   word       Limits, Comments
/===================================================================
/ XIRE    0      0       0       0    file
/ XIPT    0      0       0       0    file
/ XITX    0      0       0       0      ""
/ XITY    0      0       0       0      ""
/ XICE    0      0       0       0      ""
/ XILM    0      0       0       0      ""
/ XICF    0      0       0       0    file
/ XISF    0      0       0       0    file
/ XIBB    0      0       0       0      ""
/ XURG    0      0       0       0     uid/""
/ XUMD    0      0       0       0     uid/""
/ XUMF    0      0     mess    lnum    uid
/ XUND (cnum)    0       0       0     uid/""  See [1]: psub's in one file
/ XUNN (cnum)  pfile     0       0     uid/""  See [1]
/ XUSD (cnum)    0       0       0     uid/""  See [1];
/ XUSF (cnum)    0    pfile    lnum    uid     See [1], [2].
/ XUOD    0      0       0       0     uid/""
/ XUON (cnum)  pfile     0       0     uid/""  See [1].
/ XUPA  cnum     0       0       0     uid
/ XUOU    0      0       0       0     uid/""
/ XUXF    0      0       0       0     uid
/ XUXT    0      0       0       0     uid
/ XUDD    0      0       0       0     uid
/ XCRD  cnum   item      0       0      ""
/ XCRF  cnum   item    resp    lnum      ""
/ XCMD  cnum     0       0       0      ""
/ XCMN  cnum   pfile     0       0      ""
/ XCTD  cnum     0       0       0      ""
/ XCTN  cnum   pfile     0       0      ""
/ XCAD  cnum     0       0       0      ""
/ XCAN  cnum   pfile     0       0      ""
/ XCND  cnum     0       0       0      ""
/ XCNN  cnum   pfile     0       0      ""
/ XCSD  cnum     0       0       0      ""
/ XCSF  cnum     0     pfile   lnum     ""     See [2].
/ XCOD  cnum     0       0       0      ""
/ XCON  cnum   pfile     0       0      ""
/ XCMA  cnum     0       0       0      ""
/ XCIN  cnum     0       0       0      ""
/ XCGR  cnum     0       0       0      ""
/ XCUS  cnum     0       0       0      ""
/ XCDD  cnum     0       0       0      ""
/ XCLG  cnum     0       0       0      ""
/ XCXT  cnum     0       0       0      ""
/ XCNB  cnum     0       0       0      ""
/ XCTC  cnum     0       0       0      ""
/ XCED  cnum     0       0       0      ""
/ XCEN  cnum   pfile     0       0      ""
/ XCFD  cnum   item      0       0      ""
/ XCFN  cnum   pfile   item      0      ""     See [3].
/ XSMD    0      x       0       0      ""
/ XSND    0      0       0       0      ""
/ XSNN    0      x       0       0      ""
/ XSPC    0      0       0       0      ""
/ XSMU    0      x       0       0      ""
/ XSPD    0      0       0       0      ""
/ XSPN    0      x       0       0      ""
/ XSBG    0      0       0       0      ""
/ XSLG    0      0       0       0      ""
/ XSCD    0      0       0       0      ""
/ XSCN    0      x       0       0      ""
/ XSCK    0      0       0       0      ""
/ XSMK    0      0       0       0      ""
/ XSPV    0      0       0       0      ""
/ XSMO    0      0       0       0    file
/ XSUG    0      0       0       0    file
/ XSPB   seq     0       0       0      ""
/ XSHD    0      0       0       0      ""
/ XSHN    0      x       0       0      ""
/ XSNF    0      0       0       0      ""
/ XSSM    0      0       0       0      ""
/ XSSS   ship    0       0       0      ""
/ XSRM    0      0       0       0      ""
/ XSRS   recv    0       0       0      ""
/ XSPL    0      0       0       0      ""
/ XSUL    0      0       0       0      ""
/ XSRP   seq     0       0       0      ""
/ XSR2   seq     0       0       0      ""
/ XSRD    0      0       0       0      ""
/ XSRN    0      x       0       0      ""
/ XSUP    0      0       0       0    file
/ XSAM    0      0       0       0    file
/ XHI1   host    0       0       0      ""
/ XHML   host    0       0       0      ""
/ XHXA   host   seq      0       0      ""
/ XHI2   host    0       0       0      ""
/ XHSV   host   ship     0       0      ""
/ XHRV   host   recv     0       0      ""
/ XHSL   host    0       0       0      ""
/ XHDD   host    0       0       0      ""
/ XPAK   ship   host    seq      0      ""
/ XPXH   ship   host    seq      0      ""
/ XPXT   ship   host    seq      0      ""
/ XPXJ   ship   host    seq    chunk    ""
/ XPRH   ship   host    seq      0      ""
/ XPRT   ship   host    seq      0      ""
/ XPNH   ship   host    seq      0      ""
/ XPNT   ship   host    seq      0      ""
/ XRXC   recv   seq      0       0      ""
/ XRRX   recv   host    seq      0      ""
/ XRCC   recv     0      0       0      ""
/ XRCH   recv     0      0       0      ""
/ XRRL   recv     0      0       0      ""
/ XAHMI  lhnum    0      0       0      ""
/ XAHMO  lhnum    0      0       0      ""
/ XAHCI  lhnum    0      0       0      ""
/ XAHCO  lhnum    0      0       0      ""
/ XAHPI  lhnum    0      0       0      ""
/ XAHPO  lhnum    0      0       0      ""
/ XAUCO    0      0      0       0     userid
/ XAUMI    0      0      0       0     userid
/ XAUMO    0      0      0       0     userid
/ XACCI   cnum    0      0       0      ""
/ XACCO   cnum    0      0       0      ""
/ XADD     0      0      0       0      ""
/ XACD    cnum    0      0       0      ""
/ XAUD     0      0      0       0     userid
/ XARF    cnum  item    resp     0      ""
/ XAMF     0      0     mess     0     userid
/
/ Notes:
/ [1]. All of the PSUBJECTs for a single user are lumped together into
/      one set of files, regardless of which conference they came from.
/      (The conference name is embedded as part of the psubject name.)
/      This means that unit_lock can cheerfully ignore the CNUM argument.
/
/ [2]. The use of pfile in XUSF may appear inconsistent with pfile in
/      XUNN.  There are historical reasons for this, because names partfiles
/      always use the partfile # as the item number, whereas text partfiles
/      use the partfile # as the response number.
/
/ [3]. XCFN uses the partfile number as the "item" number to be consistent
/      with other names partfiles.  Unfortunately, this means it has to
/      use the item number as the "response" number!  *Sigh*.
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:54 Source code master starting point */
/*: CR  8/04/88 12:44 Add VM code (no other changes) */
/*: JV  8/01/88 14:35 Chg'd 'ff' to '&&' that kermit 'broke'.*/
/*: JV  9/02/88 11:20 Fixed 'lack of close of lockname' bug to AOS. */
/*: CR  9/21/88 15:42 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 12:59 Add "IX" system type. */
/*: CR  9/28/88 12:49 #define F_TLOCK for PU. */
/*: CR  9/30/88 15:04 Add server code: "#if SRV" type. */
/*: CR 10/07/88 16:29 Change IM to use QuickC. */
/*: CR 10/18/88 15:24 Remove namecode.h from SRV code. */
/*: CR 11/09/88 15:43 Add #include "errchk.h" to SRV code. */
/*: JV 11/22/88 17:04 Fixed bug in AOS.*/
/*: CR 11/22/88 17:20 Add PN code; BAD macro to UNIX code. */
/*: CR 12/16/88 14:56 PN #includes, UNIX | PN calls syssleep(). */
/*: CR  1/31/89 14:45 Add XCLG test to UNIX locking code. */
/*: CR  3/21/89 15:26 Add use of servername to #if SRV code. */
/*: JV  3/09/89 13:13 Fixed SRV code. Again. */
/*: JV  4/05/89 21:28 Added code to stop client if server is dead. */
/*: CR  4/27/89 15:48 Add lock-consistency check code. */
/*: CR  4/29/89 14:10 Break SU into SS (lockf) and SN (guardian files). */
/*: CR  6/09/89 12:21 Add "SV" system type code. */
/*: CR  7/02/89 10:41 Add use of GUARD macro for BSD locking on sys V. */
/*: CR  7/17/89 15:02 Add default case to sys V switch. */
/*: CR  8/09/89 15:20 Sys V only: add XSMK. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CW  5/16/89 12:45 VV code: allow for very long pathnames. */
/*: CW  5/19/89 12:51 Add XW unit codes. */
/*: CR 10/12/89 15:43 Add DU system type code. */
/*: CR 10/30/89 12:34 Replace sleep() with syssleep(). */
/*: CR 11/14/89 17:53 Unix code: add XSMO entry. */
/*: CR  1/30/90 14:32 Remove sysbrkclear() from Primos code. */
/*: CR  3/24/90 12:57 Add table of required args. to header doc. */
/*: CR  5/07/90 13:55 VV: TLOCK affects lockflags argument. */
/*: JV  7/08/90 16:29 Add MIPS RISC/os (RO). */
/*: CR  7/16/90  8:20 Add XSUG to comments. */
/*: CR  8/23/90 14:21 If locking file already locked, include access value. */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CR  9/27/90 12:23 Add U3, AU system types. */
/*: CJ 11/26/90 21:46 Add LA system type. */
/*: CR  2/09/91 21:44 Split long #if line in two (SYSV1, SYSV2). */
/*: CR  3/02/91 19:08 Add XIBB to comments. */
/*: SB 11/29/90 13:03 Add DY system code*/
/*: CR  3/11/91 14:10 Add AX and UL system type codes. */
/*: CR  3/22/91  0:29 Add TLOCK to BU code. */
/*: CR  4/08/91 19:04 Add new cnum argument. */
/*: JV  5/10/91 16:51 Add CaucusLink codes to Unix. */
/*: JV  5/23/91 17:40 Add CaucusLink XCTC, chg XHST->XHI1, XHI2. */
/*: JV  6/14/91 14:46 Add humor, fix Unix CaucusLink codes. */
/*: CR  7/02/91 12:39 Include unitwipe.h. */
/*: JV  7/15/91 19:04 Add XSRP, remove XRRP. */
/*: JV  7/17/91 15:37 Add XPXT. */
/*: JV  7/23/91 13:16 Add damn ENTRY to SN so it'll stop crashing. */
/*: JV  7/27/91 16:44 Add ranges on Net codes. */
/*: CR  7/31/91 17:50 Add XHSV. */
/*: JV  8/13/91 08:16 Add debug.h stuff to Unix (only) */
/*: CR  8/02/91 17:38 Add XHSL. */
/*: CR  8/07/91 14:14 Expand comment defn of XPAK syntax. */
/*: CR  8/08/91 22:18 XPNO -> XPNH, XPNT. */
/*: CR  8/19/91 22:23 Add XRCC, XRCH for sysv unix. */
/*: CR  8/22/91 13:38 Remove duplicate XHSV. */
/*: JV  8/29/91 15:00 Lintify */
/*: CR  9/05/91 15:59 Change system V mappings for CaucusLink stuff. */
/*: CR  9/06/91 14:05 Finish system V changes. */
/*: CR  9/06/91 17:42 Declare long lseek(). */
/*: CR  9/09/91 11:01 Complain if unit_lkfx() fails! */
/*: CR  9/09/91 12:23 Put back missing "i" from SU code. */
/*: JV 10/08/91 12:00 Add XSRD/XSRN to SYS V. */
/*: JV 10/14/91 11:44 Return 1 if already locked. */
/*: JV 10/14/91 12:00 Add XSR2, the 2nd XSRP file. */
/*: JV 10/17/91 13:36 Add Audit units: XAxxx. */
/*: CR 11/22/91 16:17 Check MAX_AGE of Sun/BSD lockfiles for TLOCK, too. */
/*: JV 12/10/91 18:20 Add XHDD. */
/*: UL 12/11/91 18:50 Expand args 3-6 to long. */
/*: CR 12/12/91 20:23 Sysv: lock map allows MAXSEQ of 36**4. */
/*: CR  1/06/92 19:22 Add DP system type. */
/*: JV  1/20/92 11:35 Add XCUX. */
/*: CR  1/22/92 14:44 Paren SYSV stuff; move GUARD includes to top. */
/*: JV  1/28/92 14:23 Only print "Pause for Lock" during debug. */
/*: CR  8/21/92 12:24 Move MV to UNISTD group. */
/*: DE  4/09/92 13:16 Chixified */
/*: CX  6/05/92 11:47 Add XISF to comments. */
/*: CX  6/05/92 17:28 Include handicap.h to get L(). */
/*: CX  6/08/92 12:22 Initialize newstr to nullchix. */
/*: JX  8/05/92 16:05 Add XSUP. */
/*: JX 10/01/92 12:30 Change null.chx to nullchix. */
/*: CI 10/08/92  9:55 Chix Integration. */
/*: JV 10/13/92 15:00 Fix bug in SysV bugtell call. */
/*: JV 10/14/92 12:45 Increase lock elements to XU** and XC** for 10K I/R. */
/*: CR 10/22/92 14:39 Chxformat() needs L(int...). */
/*: CR 10/22/92 16:19 Add U4 to SYSV defn. */
/*: JV 10/26/92 14:30 Add XHRV. */
/*: CR 11/03/92 13:23 Sun/BSD: TLOCK guardian file gets longer max age. */
/*: CR 11/15/92 18:13 Remove unused bugword, simplify. */
/*: JV  2/26/93 23:03 Add Attachment units to Unix, VMS, PC/Novell */
/*: JV  2/26/93 23:05 Move AOS near end. */
/*: JV  3/03/92 17:13 Add XSAM to Unix, VMS, PC/Novell. */
/*: JV  3/26/93 15:45 Add XUDD to Unix, VMS, PC/Novell. */
/*: JV  4/08/93 14:04 Fix XUSF in Unix to use CNUM. */
/*: CN  4/10/93 16:15 Expand XU.. comments; Unix code uses X...LAST macros. */
/*: JV  5/24/93 14:32 Add XART, XAMT. */
/*: CV  5/10/93 11:50 VV: Add no syslck condition. */
/*: CV  6/03/93 14:13 VV: Expand no syslck warning. */
/*: CP  4/14/93 14:35 Use DEF macro to define lock map ranges. */
/*: CP  5/06/92 11:36 Add XCED,XCEN, XCFD,XCFN entry-time files. */
/*: CT  5/28/93 11:36 Use lmap array for new DEF definition. */
/*: CP  7/08/93 17:15 Integrate 2.5 changes. */
/*: J3  8/03/93 09:56 Add XULG to Sys V. */
/*: CP 11/21/93 19:08 Test debug anded with DEB_LOCK. */
/*: CR 12/10/93 14:43 BSD/Sun: retry failed creat() if hit with signal. */
/*: CR  3/15/94 12:47 'BU' fixes. */

#include <stdio.h>
#include "caucus.h"
#include "handicap.h"
#include "systype.h"
#include "units.h"
#include "unitwipe.h"

#define  MAXCANCEL  12

/*** SysV types use lockf() routine.  UNISTD types use lockf() *and*
/    need <unistd.h> included. */
#define  SYSV1   A3 | LA | SV | FU | MU | SX | SS | TX | HU | IX
#define  SYSV2   PX | DU | RO | PS | U3 | AU | DY | AX | DP | MV | U4
#define  SYSV3   PU | TX
#define  SYSV    (SYSV1 | SYSV2 | SYSV3)
#define  UNISTD  (SYSV1 | SYSV2)

#if UNISTD
#include <unistd.h>
#endif

#if PU
#define  F_TLOCK   2      /* Test and lock if possible. */
#endif

#if PN
#include <sys\locking.h>
#include <errno.h>
extern int  errno;
#endif

#if AOS
#include <fcntl.h>
#include <errno.h>
#endif

#if BU | SN | GUARD | UL
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#endif

extern struct unit_template  units[];
extern Chix                  confid;
extern Flag                  shouldlock[];
extern int                   debug;

FUNCTION  unit_lock (n, readwrit, cnum, item, resp, lnum, word)
   int   n, readwrit;
   long  cnum, item, resp, lnum;
   Chix  word;
{
   char *bugtell();
   Chix  newstr = nullchix;
   int   success;

/*------------------------------Most UNIXes------------------------------*/
#if (SYSV & (1-GUARD)) | PN

/*** Maximum range of host and sequence numbers used in locking CaucusLink
/    shipper and receiver files. */
#define MAXHOST    1000L
#define MAXSEQ  1679616L   /* 36 ** 4 */

/*** Under UNIX, there are six possible kinds of locking done:
/      LSYS:   lock files common to the Caucus system.
/      LUSR:   lock files common to a single user.
/      LCNF:   lock files common to a single conference.
/      LHST:   lock files common to a single host.
/      LSHP:   lock files common to a shipping bay (method).
/      LRCV:   lock files common to a receiving bay (method).
/
/    For each of these types, we maintain an open "lockmap" file.  Each
/    byte in the relevant lockmap file acts as a one-bit semaphore that
/    we can lock or unlock.  The semaphore is used as a logical "lock"
/    for the file that is mapped onto that semaphore.  The mappings
/    are coded below.
/
/    This version ignores the READWRIT parameter, and treats all locks
/    as exclusive. */

   extern struct locktype_t locktype[];
   short  type, i;
   long   lock;
   char  *bugtell();
   long   lseek();

   static struct { long first;   long range; } lmap[XLAST+1];
   static int lmap_initialized = 0;

/*** The DEF macro is used to define the lock map byte ranges for various
/    unit codes.  For a given user or a given conference, each unit code
/    number gets a certain block or range of byte #s.  The DEF macro
/    defines a struct of longs for each unit that contains the starting
/    lock map byte number, and the size of the range.  */
#define DEF(xt, xp, xr) \
   { lmap[xt].first = lmap[xp].first + lmap[xp].range;   lmap[xt].range = xr; }

   if (NOT lmap_initialized) {
      lmap_initialized = 1;
      lmap[0].first = 0L;   lmap[0].range = 1000L;   /* 1000 unit codes. */

      /*** Define the lock map byte number ranges, by lower-case unit code. */
      DEF (XUNN,    0, 1000L);   /* 1000 psubj names files. */
      DEF (XUON, XUNN, 1000L);   /* 1000 psubj alpha files. */
      DEF (XUPA, XUON, 1000L);   /* 1000 conf partic. records. */
      DEF (XUSF, XUPA, 1000L);   /* 1000 psubjects. */
      DEF (XUMF, XUSF, 1000L);   /* 1000 messages.  Ignore lnums. */
      DEF (XAMF, XUMF, 1000L);   /* 1000 message attachments. */
  
      DEF (XCMN,    0, 1000L);     /* 1000 membership partfiles. */
      DEF (XCTN, XCMN, 1000L);     /* 1000 title partfiles. */
      DEF (XCAN, XCTN, 1000L);     /* 1000 author partfiles. */
      DEF (XCNN, XCAN, 1000L);     /* 1000 subject name partfiles. */
      DEF (XCON, XCNN, 1000L);     /* 1000 alpha subject name partfiles. */
      DEF (XCSF, XCON, 1000L);     /* 1000 subject definition partfiles. */
      DEF (XCEN, XCSF, 1000L);     /* 1000 item-entry-time name partfiles. */
      DEF (XCFD, XCEN, MAXITEMS);  /* maxitems resp-entry-time directories. */
      DEF (XCFN, XCFD, MAXITEMS * 1000L); /* resp-entry-time name partfiles. */
      DEF (XCRD, XCFN, MAXITEMS);         /* response 'directories'. */
      DEF (XCRF, XCRD, MAXITEMS * MAXRESP + MAXLNUM); /* individ. resp's. */
      DEF (XARF, XCRF, MAXITEMS * MAXRESP);           /* resp attachments. */
   }

   ENTRY ("unitlock", "");

   newstr    = chxalloc (160L, THIN, "unitlock newstr");
   success   = 0;

   /*** Save the true name of the file in the UNITS structure.  */
   unit_name (units[n].nameis, n, cnum, item, resp, lnum, word);
   if (units[n].access != UNLKED)
      buglist (bugtell ("Unitlock:", n, (long) units[n].access,
                        "Locking file not unlocked!\n"));

   /*** Many file types do not *actually* get locked. */
   if (NOT shouldlock[n]) {
      if (debug & DEB_LOCK)  bugtell ("pLok", n, 0L, units[n].nameis);
      units[n].access = LOCKED;
      SUCCEED;
   }

   /*** Figure the lock TYPE and semaphore mapping number (LOCK) of the
   /    files that do get locked. */
   switch (n) {
      case (XUDD):
      case (XURG):
      case (XUMD):
      case (XUND):
      case (XUSD):
      case (XUOD):
      case (XUXF):
      case (XUXT):
      case (XAUCO):
      case (XAUMO):
      case (XAUMI):
      case (XULG):
      case (XUOU):   type = LUSR;    lock =  n;                          break;

      case (XUNN):   type = LUSR;    lock = lmap[XUNN].first + item;     break;
      case (XUON):   type = LUSR;    lock = lmap[XUON].first + item;     break;
      case (XUPA):   type = LUSR;    lock = lmap[XUPA].first + cnum;     break;
      case (XUSF):   type = LUSR;    lock = lmap[XUSF].first + resp;     break;
      case (XUMF):   type = LUSR;    lock = lmap[XUMF].first + resp;     break;
      case (XAMF):   type = LUSR;    lock = lmap[XAMF].first + resp;     break;

      case (XCMA):
      case (XCTD):
      case (XCAD):
      case (XCMD):
      case (XCND):
      case (XCSD):
      case (XCOD):
      case (XCIN):
      case (XCGR):
      case (XCLG):
      case (XCXT):
      case (XCNB):
      case (XCTC):
      case (XACCI):
      case (XACCO):
      case (XCUX):
      case (XCED):
      case (XCUS):   type = LCNF;    lock =  n;                        break;

      case (XCMN):   type = LCNF;    lock = lmap[XCMN].first + item;   break;
      case (XCTN):   type = LCNF;    lock = lmap[XCTN].first + item;   break;
      case (XCAN):   type = LCNF;    lock = lmap[XCAN].first + item;   break;
      case (XCNN):   type = LCNF;    lock = lmap[XCNN].first + item;   break;
      case (XCON):   type = LCNF;    lock = lmap[XCON].first + item;   break;
      case (XCSF):   type = LCNF;    lock = lmap[XCSF].first + resp;   break;
      case (XCEN):   type = LCNF;    lock = lmap[XCEN].first + item;   break;
      case (XCFD):   type = LCNF;    lock = lmap[XCFD].first + item;   break;

      case (XCFN):   type = LCNF;
                     lock = lmap[XCFN].first + (resp * 1000L) + item;
                     break;

      case (XCRD):   type = LCNF;    lock = lmap[XCRD].first + item;   break;

      case (XCRF):   type = LCNF;
                     lock = lmap[XCRF].first + (item * MAXRESP) + resp + lnum;
                     break;

      case (XARF):   type = LCNF;
                     lock = lmap[XARF].first + (item * MAXRESP) + resp;
                     break;

      case (XSND):
      case (XSPC):
      case (XSPD):
      case (XSBG):
      case (XSLG):
      case (XSCK):
      case (XSMK):
      case (XSPV):
      case (XSMO):
      case (XSSM):
      case (XSRM):
      case (XSNF):
      case (XSPL):
      case (XSUL):
      case (XSHD):
      case (XSRD):
      case (XSUP):
      case (XSAM):
      case (XSCD):   type = LSYS;    lock = n;                         break;

      case (XSNN):   type = LSYS;    lock = 1000L + item;              break;
      case (XSMU):   type = LSYS;    lock = 2000L + item;              break;
      case (XSPN):   type = LSYS;    lock = 3000L + item;              break;
      case (XSCN):   type = LSYS;    lock = 4000L + item;              break;
      case (XSHN):   type = LSYS;    lock = 5000L + item;              break;
      case (XSSS):   type = LSYS;    lock = 6000L + cnum;              break;
      case (XSRS):   type = LSYS;    lock = 7000L + cnum;              break;
      case (XSRN):   type = LSYS;    lock = 8000L + item;              break;

      case (XSPB):   type = LSYS;    lock =     MAXSEQ + cnum;   break;
      case (XSRP):   type = LSYS;    lock = 2 * MAXSEQ + cnum;   break;
      case (XSR2):   type = LSYS;    lock = 3 * MAXSEQ + cnum;   break;

      case (XHML):
      case (XHSL):
      case (XHDD):
      case (XHI2):
      case (XAHMI):
      case (XAHMO):
      case (XAHCI):
      case (XAHCO):
      case (XAHPI):
      case (XAHPO):
      case (XHI1):   type = LHST;    lock = n;              break;

      case (XHSV):   type = LHST;    lock = 1000L + item;   break;
      case (XHRV):   type = LHST;    lock = 2000L + item;   break;
      case (XHXA):   type = LHST;    lock = 3000L + item;   break;

      /*** LSHP (shipment) files can ignore shipping method #, as there
      /    is only one shipment method per host.   XRRX is included with
      /    the LSHP files, even though it is a receive file. */
      case (XPXH): type = LSHP;  cnum=item;  lock = resp +     MAXSEQ;  break;
      case (XPXT): type = LSHP;  cnum=item;  lock = resp + 2 * MAXSEQ;  break;
      case (XPRH): type = LSHP;  cnum=item;  lock = resp + 3 * MAXSEQ;  break;
      case (XPRT): type = LSHP;  cnum=item;  lock = resp + 4 * MAXSEQ;  break;
      case (XPNH): type = LSHP;  cnum=item;  lock = resp + 5 * MAXSEQ;  break;
      case (XPNT): type = LSHP;  cnum=item;  lock = resp + 6 * MAXSEQ;  break;
      case (XPAK): type = LSHP;  cnum=item;  lock = resp + 7 * MAXSEQ;  break;
      case (XRRX): type = LSHP;  cnum=item;  lock = resp + 8 * MAXSEQ;  break;

      case (XRCC):
      case (XRCH):
      case (XRRL): type = LRCV;              lock = n;                  break;

      case (XRXC): type = LRCV;              lock = item + MAXSEQ;      break;

      default:
         buglist (bugtell ("Bad LOCK case!", n, 0L, units[n].nameis));
         sysexit (1);
   }

   units[n].locknum  = lock;
   units[n].locktype = type;

   /*** If debugging, tell user about lock attempt. */
   if (debug & DEB_LOCK)  bugtell ("LOCK", n, lock, units[n].nameis);

   /*** Make sure the locking file for this TYPE is open. */
   if (NOT unit_lkfx (type, word, cnum)) {
      bugtell ("Unit_lock: can't open lockmap!", type, cnum, ascquick(word));
      FAIL;
   }

   /*** Lseek to the locking record that corresponds to this file,
   /    and try to lock it, waiting 2 seconds between failed attempts. */
   if (lseek (locktype[type].fd, lock, 0) < 0)  {
      chxclear  (newstr);
      chxformat (newstr, CQ("Unit_lock lseek error: n=%d, fd=%d,"), L(n),
                  L(locktype[type].fd), nullchix, nullchix);
      chxformat (newstr, CQ(" lock=%d\n"), L(lock), 0L, nullchix, nullchix);
      unit_write (XWER, newstr);
      FAIL;
   }

#define LK1  A3 | LA | SV | FU | MU | MV | PU | SX | SS | HU | IX
#define LK2  PX | DU | RO | PS | U3 | AU | DY | AX | U4
#if LK1 | LK2
#define LOCKF   lockf   (locktype[type].fd, F_TLOCK, 1L)
#define BAD     (0)
#endif

#if TX
#define LOCKF   locking (locktype[type].fd, 2, 1L)
#define BAD     (0)
#endif

#if PN
#define LOCKF   locking (locktype[type].fd, LK_NBLCK, 1L)
#define BAD     (errno != EACCES)
#endif

   success   = 0;

   for (i=0;   LOCKF < 0;   ++i) {
      if (readwrit == TLOCK  ||  BAD)  { units[n].locknum = -1L;   FAIL; }
      if (sysbrktest()  &&  i>MAXCANCEL)  sysexit(1);
      chxclear  (newstr);
      chxformat (newstr, CQ("Please wait for lock on %d: %d\n"), L(n), L(lock),
                      nullchix, nullchix);
      if (debug & DEB_LOCK) unit_write (XWAN, newstr);
      syssleep (2);
   }
   units[n].access = LOCKED;
   SUCCEED;
#endif

/*-----------------------------------Berkeley Unix-------------------------*/
#if BU | SN | GUARD | UL
#if BU | SN
#include <sys/file.h>
#endif

/*** Oddly enough, Unix 4.2 BSD has no semaphore or record-locking capability.
/    Therefore, to do Caucus locking, we're back to the old guardian-file
/    scheme.
/
/    How it works: to lock a file "x", create a file called "x_l", permitted
/    read-only.  This means that once the file is created, no-one else can
/    create it until we unlock the lock by deleting the guardian file.
/
/    The rub is that if Caucus or the system crashes before we unlock, we
/    end up leaving behind a permanently-locked file.  So: if we can't
/    lock a file, we try several times, sleeping between each attempt.  If
/    we still can't lock it, we check to see if it's locked by FLOCK.  This
/    means that a running task has still got the file locked.  If it's not
/    locked, then check how old the file is.  If it is more than MAX_AGE
/    seconds, then it's left over from a crash, and we can delete the guardian
/    file.
/
/    (For SN, ignore FLOCK, and just use the time since creation.) */

#define  MAX_WRITE_AGE   300L  /* Max age of WRITE-lock guardian file. */
#define  MAX_TLOCK_AGE 86400L  /* Max age of TLOCK-lock guardian file. */

   extern int  errno;
   struct stat buf;
   char   file[80];
   int    fd, fe, i;
   long   guardian_age;
   long   time();

   ENTRY ("unit_lock", "");

   /*** Save the true name of the file in the UNITS structure.  */
   unit_name (units[n].nameis, n, cnum, item, resp, lnum, word);
   strconc   (file, units[n].nameis, "_l");
   if (units[n].access != UNLKED) {
      buglist (bugtell ("Unitlock:", n, (long) units[n].access,
                        "Locking file not unlocked!\n"));
      SUCCEED;
   }

   /*** Many file types do not *actually* get locked. */
   if (NOT shouldlock[n]) {
      if (debug & DEB_LOCK)  bugtell ("pLok", n, 0L, units[n].nameis);
      units[n].access = LOCKED;
      SUCCEED;
   }

   /*** If debugging, tell user about lock attempt. */
   if (debug & DEB_LOCK)  bugtell ("LOCK", n, 0L, file);

   /*** Open the guardian file.  Retry on failure, in case something
   /    funny happened on the Way to the Forum. */
   do {
      sysbrkignore();
      for (i=0;   i<6;   ++i) {
         if ( (fd = creat (file, 0400)) >= 0)       break;
         if (errno    != EACCES)  { sysbrkinit();   FAIL; }
         if (readwrit == TLOCK)   break;
         if (debug & (DEB_LOCK | DEB_WAIT)) {
            unit_write (XWAN, CQ("Please wait for lock on '"));
            unit_write (XWAN, CQ(file));
            unit_write (XWAN, CQ("'\n"));
         }
         syssleep (2);
      }
      sysbrkinit();

      if (fd < 0) {
#if BU
         int locked;
         if ( (fe = open (file, 0)) < 0)  continue;
         locked = (flock (fe, LOCK_EX | LOCK_NB) != 0);
         close (fe);
         if (locked)  { syssleep (2);  continue; }
#endif

         stat (file, &buf);
         guardian_age = time((long *) 0) - buf.st_mtime;

         if      (guardian_age > MAX_TLOCK_AGE)   unlink (file);
         else if (readwrit == TLOCK)  { units[n].locknum = -1L;   FAIL; }
         else if (guardian_age > MAX_WRITE_AGE)   unlink (file);
      }
   } UNTIL (fd > 0);

#if BU
   flock (fd, LOCK_EX);
#endif
   units[n].locknum = fd;

   units[n].access = LOCKED;
   SUCCEED;
#endif

/*---------------------------------------IBM PC--------------------------*/
#if IM
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
   char  locker[80];

   ENTRY ("unitlock", "");

   /*** Save the true name of the file in the UNITS structure.  */
   unit_name (units[n].nameis, n, cnum, item, resp, lnum, word);
   if (units[n].access != UNLKED)
      buglist (bugtell ("Unitlock:", n, (long) units[n].access,
                        "Locking file not unlocked!\n"));

   /*** If debugging, tell user about lock attempt. */
   if (debug & DEB_LOCK)
      bugtell ("LOCK", n, 0L, units[n].nameis);

   /*** The one lock we MUST try to make happen on the theoretically
   /    single-user MSDOS is the "maximum # users" control file. */
   if (n==XSMU) {
      strconc (locker, units[n].nameis, ".lk");
      while ( (units[n].locknum = open (locker, O_CREAT | O_BINARY | O_RDWR,
                                                S_IREAD | S_IWRITE)) < 0)
         syssleep(2);
   }

   units[n].access = LOCKED;
   SUCCEED;
#endif


/*---------------------------------------PRIME---------------------------*/
#if PR

/***   The PRIME version opens a guardian file with the prefix "_" to
/   to indicate that the file proper is locked.  This file is left open
/   in read/write mode during the course of the use of the actual data
/   file.  Since PRIMOS only allows one writer on an open file, this
/   will block any other attempts to open the guardian file. */

   fortran tsrc$$();
   short   key, funit, filetype, code, chrpos[2];
   char    file[80];
   char   *s;
   extern  int errno;

   ENTRY ("unitlock", "");

   /*** Save the true name of the file in the UNITS structure. */
   unit_name (units[n].nameis, n, cnum, item, resp, lnum, word);
   if (units[n].access != UNLKED)
      buglist (bugtell ("Unitlock:", n, (long) units[n].access,
                        "Locking file not unlocked!\n"));

   /*** If debugging, tell user about lock attempt. */
   if (debug & DEB_LOCK)
      bugtell ("LOCK", n, 0L, units[n].nameis);

   /*** Many file types do not *actually* get locked. */
   if (NOT shouldlock[n]) {
      units[n].access = LOCKED;
      SUCCEED;
   }

   /*** Make up the name of the guardian FILE by prefixing it
   /    with a "_". */
   strcpy (file, units[n].nameis);
   if (strindex (file, ">") > 0)  {
      for (s=file+strlen(file)-1;   *s != '>';   --s) ;
      stralter (s, ">", ">_");
   }
   else strconc (file, "_", file);

   /*** Try to open the guardian FILE for write.  Return on success
   /    or unknown error.  Loop while FILE is busy (opened by someone
   /    else).  */
   while (1) {
      key = KWRIT + KGETU + KNSAM;
      chrpos[0] = 0;
      chrpos[1] = strlen(file);
      tsrc$$ (&key, file, &funit, chrpos, &filetype, &code);
      units[n].locknum = funit;

      if (code ==     0)  break;
      if (code != EFIUS) FAIL;  /* Errors other than file in use */
      if (readwrit == TLOCK)  { units[n].locknum = -1L;   FAIL; }
      if (debug & DEB_LOCK) unit_write (XWAN, CQ("Please wait for a lock...\n"));
      syssleep (3);
   }
   units[n].access = LOCKED;
   SUCCEED;
#endif

/*------------------------------Vax/VMS----------------------------------*/
#if VV
#include <ssdef>
#include <descrip>
#include <lckdef>
   struct {short cond;   short reserv;   long  lockid;}  lksb;
   struct dsc$descriptor_s  desc;
   static int        nosyslock=0;
   char   file[100], resource[100];
   char  *bracket, *dot;
   int    status;
   long   lockmode, lockflags;

   ENTRY ("unitlock", "");

   newstr    = chxalloc (160L, THIN, "unitlock newstr");
   success   = 0;

   /*** Vax/VMS locks the file by using the $ENQ lock system service
   /    semaphore to lock the file name as a system-wide resource. */

   /*** Save the true name of the file in the UNITS structure.  */
   unit_name (file, n, cnum, item, resp, lnum, word);
   strcpy (units[n].nameis, file);
   if (units[n].access != UNLKED)
      buglist (bugtell ("Unitlock:", n, (long) units[n].access,
                        "Locking file not unlocked!\n"));

   /*** Many file types do not *actually* get locked. */
   if (NOT shouldlock[n]  ||  nosyslock) {
      if (debug & DEB_LOCK)  bugtell ("pLok", n, 0L, file);
      units[n].access = LOCKED;
      SUCCEED;
   }

   /*** If debugging, tell user about lock attempt. */
   if (debug & DEB_LOCK)  bugtell ("LOCK", n, (long) item, file);

   /*** We're going to lock the file with the ENQ system service routine.
   /    Alas, it only allows names of things to be locked up to 31 chars.
   /    Therefore, we need to trim the name of the file down to a unique
   /    minimum.  To do this, we throw away everything before the
   /    next-to-last dot in the full pathname of the file. */
   bracket = file     + strindex (file,     "[") + 1;
   strcpy (resource, bracket);
   bracket = resource + strindex (resource, "]");

   for (dot=bracket;   *dot!='.'  &&  dot!=resource;   --dot) ;
   if  (dot!=resource)                                 --dot;
   for (           ;   *dot!='.'  &&  dot!=resource;   --dot) ;
   strcpy (resource, dot);

   /*** Set up the descriptor for the name of the file we're locking. */
   desc.dsc$w_length  = strlen (resource);
   desc.dsc$a_pointer = resource;
   desc.dsc$b_class   = DSC$K_CLASS_S;
   desc.dsc$b_dtype   = DSC$K_DTYPE_T;

   lockmode  = LCK$K_EXMODE;
   lockflags = LCK$M_SYSTEM | (readwrit==TLOCK ? LCK$M_NOQUEUE : 0L);
   status = sys$enqw (0, lockmode, &lksb, lockflags,
                      &desc, 0, 0, 0, 0, PSL$C_USER, 0);

   if (debug & DEB_LOCK) {
      chxclear  (newstr);
      chxformat (newstr, chxquick("status=%d, lockid=%d, res='%s'\n",0),
               L(status), L(lksb.lockid), chxquick(resource,1), nullchix);
      unit_write (XWDB, newstr);
   }

   units[n].locknum = lksb.lockid;

   if (status == SS$_NORMAL) {
      units[n].access = LOCKED;
      SUCCEED;
   }
   else if (status == SS$_NOSYSLCK) {
      char *warn;
      units[n].access = LOCKED;
      nosyslock = 1;
      warn = " ****************************\n * WARNING!  No SYSLCK priv.*\n";
      unit_write (XWDB, CQ(warn));
      warn = " * Proceeding in Test mode! *\n ****************************\n";
      unit_write (XWDB, CQ(warn));
      SUCCEED;
   }
   else  FAIL;
#endif

/*--------------------------AOS, MV/UX--------------------------------*/
#if AOS
    char lockname[60];
    int i, lockopen;
    ENTRY ("unitlock", "");

    newstr    = chxalloc (160L, THIN, "unitlock newstr");
    success   = 0;

    /*** Save the true name of the file in the UNITS structure.  */
    unit_name (units[n].nameis, n, cnum, item, resp, lnum, word);

    if (units[n].access != UNLKED)  buglist (bugtell ("Unitlock:",
              n, (long) units[n].access, "Locking file not unlocked!\n"));

    /*** Many file types do not *actually* get locked. */
    if (NOT shouldlock[n]) {
       if (debug & DEB_LOCK)  bugtell ("pLok", n, 0L, units[n].nameis);
       units[n].access = LOCKED;
       SUCCEED;
    }

#define LOCKF (lockopen = open (lockname, O_CREAT | O_EXCL, 700))
    strconc (lockname, units[n].nameis, "_l");
    sysbrkclear();
    if (debug & DEB_LOCK) bugtell ("Lock", n, 0L, lockname);
    for (i=0;   LOCKF < 0;   ++i) {
       if (readwrit == TLOCK)  FAIL;
       if (errno != EEXIST) { /* if can't open and !exists, fail */
          if (debug & DEB_LOCK) {
             perror ("unit_lock failed: EEXIST");
             printf ("Errno: %d\n", errno);
	   }
           FAIL; }
       if (sysbrktest()  &&  i>8)  sysexit(1);
       if (debug & DEB_LOCK) {
          chxclear  (newstr);
          chxformat (newstr, CQ("Please wait for lock on %d\n"), L(n), 0L,
                     nullchix, nullchix);
          unit_write (XWAN, newstr);
       }
       syssleep (2);
    }
    units[n].locknum = lockopen;
    units[n].access  = LOCKED;
    SUCCEED;
#endif

/*-------------------------------Server-based systems-------------------*/
#if SRV
#include "actions.h"
#include "errchk.h"
   extern int   vcernum;
   extern char *vcernam;
   extern Chix  localfile[];
   extern Chix  servername;
   struct namecode_t namecode;
   char   reply[10], str[80];
   short  class;

   ENTRY ("unitlock", "");

   if (localfile[n])
          unit_name (units[n].nameis, n, cnum, item, resp, lnum, word);
   else              units[n].nameis[0] = '\0';
   if (units[n].access != UNLKED)  buglist (bugtell ("Unitlock:",
             n, (long) units[n].access, "Locking file not unlocked!\n"));

   if (debug & DEB_LOCK)
      sprintf (str, "%d %d %d %d %s", cnum, item, resp, lnum, ascquick(word));

   /*** Many file types do not *actually* get locked. */
   if (localfile[n]  ||  NOT shouldlock[n]) {
      if (debug & DEB_LOCK)  bugtell ("pLok", n, 0L, str);
      units[n].access = LOCKED;
      SUCCEED;
   }

   /*** If debugging, tell user about lock attempt. */
   if (debug & DEB_LOCK)  bugtell ("LOCK", n, (long) item, str);

   class = ALOCK + n;
   namecode.confnum  = cnum;
   namecode.item     = item;
   namecode.resp     = resp;
   namecode.lnum     = lnum;
   namecode.readwrit = readwrit;
   if (EMPTYCHX (word))   sysuserid (namecode.word);
   else                   chxcpy    (namecode.word, word);

   if (NOT vcsend (servername, class, (char *) &namecode, sizeof(namecode),
                    reply, 1)) {
      ERRCHK("unitlock");
      unit_write (XWER, CQ("Caucus Server has terminated.\n"));
      unit_write (XWER, CQ("Caucus is now aborting.\n"));
      sysexit (0);
   }

   if (reply[0])   units[n].access = LOCKED;
   DONE(reply[0]);
#endif

 done:

   if (chxtype(newstr)) chxfree ( newstr );

   RETURN ( success );
}
