/*** UNIT_WRITE.  Write text in STR to file open on unit N.
/
/    unit_write (n, str);
/
/    Parameters:
/       int   n;    (unit number of open file)
/       Chix  str;  (text to be written)
/
/    Purpose:
/       Write text of chix STR to file open on unit N.
/
/    How it works:
/       STR may contain any text, including multiple lines.
/
/       Special considerations:
/          Unit N may be an open file, the bit bucket (XIBB), or
/          one of the unit codes that goes to the terminal (XITT,
/          XWTX, etc.)
/
/          If the 1st character of STR is '\01' (NEWCPAGE), this
/          signifies "start a new page" to files & devices for which
/          this has meaning.
/
/          Output sent to the terminal will be word-wrapped according
/          to the value of the global screen.width.
/
/       Unit_write() calls unit_control() to find out what kind of
/       character set is used to write the output.
/
/       Output to the terminal passes through the following chain
/       of functions:
/          unit_write()      (decide where output is going)
/          unit_put()        (perform word-wrapping and do paging)
/          unit_term()       (convert to proper character representation)
/          sysputstr()       (really write to the terminal)
/       
/    Returns:
/
/    Error Conditions:
/
/    Side effects:
/
/    Related functions:
/
/    Called by:  Application.
/
/    Operating system dependencies: none
/
/    Known bugs:
/
/    Home:  unit/unitwrit.c
/ */

/*** Copyright (C) 1985 Camber-Roth.  All rights reserved.  */

/*: CX  5/12/92 15:49 Extensively rewritten for chix. */
/*: JX  6/05/92 23:30 Return after writing if one of the terminal codes. */
/*: CR 11/09/92 17:27 Check for FILEOUTPUT terminal output redirection. */
/*: CL 12/10/92 17:07 Long ascofchx args. */
/*: CL 12/11/92 13:48 Long Chxvalue N. */
/*: CR 12/16/92 15:38 unit_control should check unit N. */
/*: JV  4/23/93 14:06 Add comment to header. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"
#include "diction.h"

/*** Define PUTS macro for writing bytes to a file. */
#if UNIX | IM | JM | PN | VV | AOS | PX | PS
#define  PUTS(s)  fputs (s, units[n].fp)
#endif

#if PR
#define  PUTS(s)  put2s (units[n].fd, s)
#endif

#if SRV
#include "actions.h"
#include "errchk.h"
#define  PUTS(s)  unitwts (n, s)
extern int   debug;
extern int   vcernum;
extern char *vcernam;
extern char  localfile[];
extern char  confid[];
extern char  servername[];
#endif

extern struct unit_template units[];
extern struct flag_template flag;
extern Userreg              thisuser;

static char firstpage = 1;

FUNCTION  unit_write (n, str)
   int  n;
   Chix str;
{
#define   BUFSIZE  512L
   char   buf[BUFSIZE];
   int    lang, inc;
   long   pos;
   char  *bugtell();

   ENTRY ("unitwrite", "");

   if (n <= XITT)  n = unit_control (n, UNIT_GETSUB);

   /*** Writes to the bit bucket are thrown away. */
   if (n == XIBB)   RETURN (1);

   if (n > XITT  &&  units[n].access != WRITE)
      buglist (bugtell ("Unitwrite: ", n, 0L, "writing to unopened file!\n"));

#if PX | CVM | PS
   if (n <= XITT  &&  flag.fullscreen)  { vmiwrite (n, str);   RETURN (1); }
#endif

   if (n==XKEY  ||  n==XWTZ)   RETURN(1);

#if PX | CVM | PS
   if (n <= XITT) {
      Chix  line;
      int   code;

      line = chxalloc (100L, THIN, "unit_write line");

      /*** If fullscreen is OFF, we write STR to the terminal WITHOUT any of
      /    the pf key map lines (which all begin with '&'). */
      for (pos=0;   code = chxnextline (line, str, &pos);   ) {
         if (chxvalue (line, 0L) != '&')  unit_put (line,     n!=XWTY);
         if (code == 2)                  unit_put (CQ("\n"), n!=XWTY);
      }

      chxfree (line);
      RETURN  (1);
   }
#endif

   /*** New page indicator?  If so, and output is going to the printer
   /    unit (XIPT), and this is not the very first newpage to be sent
   /    (i.e., printer starts at top of form!), write the user's selected
   /    new page character. */
   if (chxvalue (str, 0L) == '\01') {
      if (n==XIPT  &&  thisuser->newpage>1  &&  NOT firstpage) {
         buf[0] = thisuser->newpage;
         buf[1] = '\0';
         PUTS (buf);
      }
      firstpage = 0;
      RETURN (1);
   }

   /*** If one of the terminal unit codes, have unit_put() do the
   /    dirty work. */
   if      (n==XWTY  ||  n==XWPM)  { unit_put (str, 0);   RETURN (1); }
   else if (n<=XITT)               { unit_put (str, 1);   RETURN (1); }

   else if (EMPTYCHX (str))        RETURN (1);

   /*** Otherwise, if there's anything at all to write, convert it
   /    to bytes a BUFSIZE at a time, and write it out with PUTS. */
   lang = unit_control (XIRE, UNIT_GET);
   for (pos=0;   (1);   pos += inc) {
      switch (lang) {
         case (LANG_NORM):
         case (LANG_ISO):  inc = ascofchx  (buf, str, pos, BUFSIZE);  break;
         case (LANG_EUC):  inc = eucofjix  (buf, str, pos, BUFSIZE);  break;
         case (LANG_SJIS): inc = sjisofjix (buf, str, pos, BUFSIZE);  break;
         default:          inc = -1;                                  break;
      }
 
      PUTS (buf);
      if (inc <= 0)  break;
   }

   RETURN (1);
}


/*** For server based systems, strings get written to a large buffer,
/    and the buffer sent to the server when full. */
#if SRV

FUNCTION  unitwts (n, str)
   int    n;
   char  *str;
{
   short  len, class;

   if (localfile[n])  { fputs (str, units[n].fp);   return; }

   /*** If STR is NULL, it means that we're closing the file and
   /    need to flush the buffer by sending it to the server. */
   if (str==NULL) {
      class = AWRIT + n;
      if (NOT vcsend (servername, class, units[n].buffer, units[n].buflen+1, 
                      NULL, 0)) {
         ERRCHK ("unitwrite");
         unit_write (XWER, "Caucus Server has terminated.\n");
         unit_write (XWER, "Caucus is now aborting.\n");
         sysexit (0);
      }
   }
      
   /*** If STR would overflow the buffer, empty the buffer by sending
   /    its contents to the server. */
   else {
      len = strlen(str);
      if (len + units[n].buflen + 1 > MEMBUF-8) {
         class = AWRIT + n;
         if (NOT vcsend (servername, class, units[n].buffer, units[n].buflen+1,
                      NULL, 0)) {
            ERRCHK ("unitwrite");
            unit_write (XWER, "Caucus Server has terminated.\n");
            unit_write (XWER, "Caucus is now aborting.\n");
            sysexit (0);
	 }
         units[n].buflen = 0;
         units[n].p = units[n].buffer;
        *units[n].p = '\0';
      }

      /*** Add STR onto the end of the buffer. */
      strcat (units[n].p, str);
      units[n].buflen += len;
      units[n].p      += len;
    }
}
#endif


#if PR

/*** Under PRIMOS, PUT2S adds its respective bytes onto the
/    end of an accumulating line of text.  The line is written out with
/    wtlin$() when it is complete. */

static char  outbuf[140];
static short p     = 0;
static short funit = 0;

FUNCTION  put2s (unit, str)
   int    unit;
   char   str[];
{
   short   size, code;
   fortran wtlin$();
   char   *s;

   if (funit) { 
      if (funit != unit)  bugtell("PUT2S: dup write", funit, (long) unit, "");
   }

   for (s=str;   *s;   ++s) {
      if (*s != '\n')  outbuf[p++] = *s;
      else {
         outbuf[p++] = ' ';
         size  = p/2;
         if (NOT funit)  funit = unit;
         wtlin$ (&funit, outbuf, &size, &code);
         p = funit = 0;
      }
   }
}
#endif
