
/*** MDRECORD.  Try to write a header record to the dictionary.
/
/    ok = mdrecord (fd, mdrec, text, collisions);
/
/    Parameters:
/       int    ok;              (Success?)
/       int    fd;              (unit number of open dictionary)
/       struct mdrec_t *mdrec;  (pointer to dictionary header record)
/       long   text;            (next place in dictionary to write text)
/       int   *collisions;      (number of collisions in hashing scheme);
/
/    Purpose:
/       Write header record MDREC into the proper place in the dictionary.
/
/    How it works:
/       Mdrecord() hashes the name in MDREC, and selects a position in
/       the hash table area of the dictionary file open on FD.  Mdrecord()
/       follows a very traditional manner of trying to find an open slot
/       in the hash table to write MDREC.
/
/       If that position (or in case of collision, any subsequently 
/       suggested position) contains a header record with the same name
/       as in MDREC, mdrecord() does *not* write the record and
/       immediately returns.
/
/       Just before mdrecord() returns, it moves the file pointer (the
/       place where the next write will occur) to TEXT, which is presumably
/       the next open area in the text part of the dictionary file.
/
/       If the record is written, any one (or more) collisions with
/       existing records occurred in the hash table, COLLISIONS is
/       incremented by 1.
/
/    Returns: 1 on success.
/             0 if a record with the same string name already exists.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  procfile()
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  xdiction/procfile.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:58 Source code master starting point */
/*: JV  8/02/88  8:51 Used 'sizeof(this)' to make more portable.*/
/*: JV  9/08/88 14:23 Took out extra #def of MWRITE.*/
/*: CR  9/21/88 15:04 Put in MWRITE #def for UNIX and PN. */
/*: CR 11/22/88 20:42 Add new PN code. */
/*: CR  1/09/89 15:27 Add CVM code. */
/*: CR  4/17/89 14:44 Merge SEEK etc defn's for all systems. */
/*: JV  8/03/89 10:46 Added CA back into SEEK, etc. def's. */
/*: CX  5/07/92 18:28 Major changes for chixification. */

#include "caucus.h"
#include "systype.h"
#include "diction.h"

#if UNIX | IM | JM | VV | PR | PN | CX | CA | CVM
#define  SEEK(pos)    lseek (fd, pos, 0)
#define  MREAD(w,n)   read  (fd, (char *)w, (int) (n))
#define  MWRITE(w,n)  write (fd, (char *)w, (int) (n))
#endif

FUNCTION  mdrecord (fd, mdrec, text, collisions)
   int    fd;
   struct mdrec_t *mdrec;
   long   text;
   int   *collisions;
{
   struct mdrec_t this;
   long   hash, hashbyte;
   int    tries;
   char  *s;

   /*** Hash the string name into an address in the Master Dictionary. */
   for (hash=0L, s=mdrec->name;   *s;   ) {
      hash += (*s++ & 0xFF) * 256L;
      if (NOT *s)  break;
      hash += (*s++ & 0xFF) *   5L;
   }

   /*** Seek to that address, and read what's there.  If it's not an empty
   /    record (a collision), try the next record. */
   tries = 0;
   do {
      hashbyte = (hash++ % HASHSIZE) * (long) sizeof(this);
      SEEK  (hashbyte);
      MREAD (&this, (long) sizeof(this));
      if (streq (this.name, mdrec->name))  { SEEK(text);   return (0); }
      ++tries;
   } while (this.start);

   if (tries > 1)  ++(*collisions);

   /*** Once we've got an empty slot, write our MDREC there. */
   SEEK (hashbyte);
   MWRITE (mdrec, (long) sizeof(this));

   /*** Finally, restore the file pointer to the TEXT byte where we left
   /    off writing. */
   SEEK (text);
   return (1);
}
