
/*** Process a dictionary source text file.
/
/    ok = procfile (file, fd, text, collisions, strings, top_level, language,
/                   verbose);
/
/    Parameters:
/       int    ok;          (Success?)
/       char  *file;        (name of dictionary file to read & process)
/       int    fd;          (unit number of open output binary file)
/       long  *text;        (pointer to text area in output binary file)
/       int   *collisions;  (record # of collisions that occur)
/       int   *strings;     (record # of strings found)
/       int    top_level;   (is this the top-level source text file?)
/       int    language;    (current default language type)
/       int    verbose;     (display verbose diagnostics?)
/
/    Purpose:
/       Procfile() does most of the work of reading & processing 
/       dictionary strings.
/
/    How it works:
/
/    Returns: 1 on success.
/             0 if FILE cannot be opened.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  makemd
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  xdiction/procfile.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:58 Source code master starting point */
/*: CR  4/21/89 15:05 Add code to remove '$' line-end-marker. */
/*: CR 10/11/89 18:02 Warn user when too-long strings are truncated. */
/*: JV  7/22/90 17:00 Add strtoken() decl. */
/*: CR  9/24/90 14:17 Use mdname() to figure name of ":<" include file. */
/*: CR 10/05/90 23:52 Fix strtoken decl! */
/*: CS 11/30/90 13:38 Add top_level argument. */
/*: CR  6/07/91 17:17 Replace mdwrite() calls with mdput(). */
/*: CX  5/07/92 18:28 Major changes for chixification. */
/*: CX  6/07/92 20:29 Put proper chx type in mdrec.width. */
/*: CR 10/07/92 23:21 Treat tabs as spaces in ":name opt" control lines. */
/*: JV 10/10/92 10:59 Allocate CONVERT, THISCHIX. */

#include <stdio.h>
#include "caucus.h"
#include "diction.h"

#define   LINESIZE  200

static struct mdrec_t mdrec;
static Chix   thischix, convert, dollar, nothing, newline;
static int    thislang;


FUNCTION  procfile (file, fd, text, collisions, strings, top_level, language,
                    verbose)
   char   file[];
   int    fd, top_level;
   long  *text;
   int   *collisions;
   int   *strings;
   int    language, verbose;
{
   FILE  *in, *fopen();
   char   line[LINESIZE], name[LINESIZE], opt[LINESIZE], incfile[LINESIZE],
          word[LINESIZE];
   char  *line0, *strtoken();
   int    i, newlang, width;
   short  firstline;

   if ( (in = fopen (file, "r")) == NULL)  return(0);

   if (verbose) printf ("Processing file '%s'.\n", file);

   if (top_level) {
      dollar   = chxalloc (  4L, THIN, "procfile dollar");
      nothing  = chxalloc (  4L, THIN, "procfile nothing");
      newline  = chxalloc (  4L, THIN, "procfile newline");
      convert  = chxalloc ( 16L, THIN, "procfile convert");
      thischix = chxalloc ( 16L, THIN, "procfile thischix");
      chxofascii (dollar,   "$");
      chxofascii (newline, "\n");
   }

   /*** Get each line from the current input file and process it. */
   while (fgets (line, LINESIZE-2, in) != NULL) {
      line [ strlen(line)-1 ] = '\0';
      strtoken (word, 1, line);

      /*** Ignore comment lines. */
      if      (line[0] == '#')  continue;


      /*** Process included files recursively. */
      else if (line[0] == ':'  &&  line[1]=='<') {
         mdname   (incfile, 4, "", line+2, 0);
         procfile (incfile, fd, text, collisions, strings, 0, language, 
                            verbose);
         if (verbose) printf ("Continuing with '%s'.\n", file);
      }
         

      /*** Set options for this file (and below)... */
      else if (streq (word, ":")) {
         while (stralter (line, "\t", " "));
         for (i=2;   strtoken (word, i, line) != NULL;   ++i) {
            if (strindex (word, "lang") == 0) {
               strbreak  (word, name, opt, "=");
               newlang = strmatch (LANG_STRING, opt) + 1;
               if (newlang > 0)  language = newlang;
            }

            else printf ("Unknown option: %s\n", word);
         }
      }


      /*** Start of a new string.... */
      else if (line[0] == ':') {
         while (stralter (line, "\t", " "));

         /*** Try to write the old string (if there is one) to the hash
         /    table.  If that succeeds, write the data to the text area. */
         if (mdrec.start != 0L) {
            mdrec.width  =  chxtype (thischix);
            mdrec.length = (chxlen  (thischix) + 1) * mdrec.width;
            if (mdrecord (fd, &mdrec, *text, collisions)) {
               mdput (fd, thischix->p, mdrec.length);
               *text += mdrec.length;
            }
            chxfree (thischix);
            chxfree (convert);
         }

         /*** Get name of new string. */
         ++(*strings);
         strtoken (name, 1, line+1);
         if (strlen (name) > DICNAMELEN) {
            printf ("  The name of string '%s' is too long.\n", name);
            name[DICNAMELEN] = '\0';
            printf ("  Truncating it to:  '%s'\n", name);
         }

         /*** Initialize the header record for the string. */
         strcpy (mdrec.name, name);
         mdrec.start  = *text;
         mdrec.status = 0;

         /*** Detect RES, TABLE, or LANG options for this string. */
         thislang = language;
         for (i=2;   strtoken (word, i, line) != NULL;   ++i) {
            if (word[0] == 'r')  mdrec.status |= MDRES;
            if (word[0] == 't')  mdrec.status |= MDTABLE;
            if (word[0] == 'l') {
               strbreak  (word, name, opt, "=");
               newlang = strmatch (LANG_STRING, opt) + 1;
               if (newlang > 0)  thislang = newlang;
            }
         }

         width = THIN;
         if (thislang==LANG_EUC  ||  thislang==LANG_SJIS)  width = WIDE;
         thischix = chxalloc (200L, width, "procfile thischix");
         convert  = chxalloc (100L, width, "procfile convert");

         firstline = 1;
      }


      /*** Add a line of text to this string... */
      else {
         /*** Everthing but tables and start of string gets a newline. */
         if (NOT firstline  &&  NOT (mdrec.status & MDTABLE)) 
            chxcat (thischix, CQ("\n") );
         firstline = 0;

         /*** In tables, a newline is the same as a space. */
         if (mdrec.status & MDTABLE)  chxcat (thischix, CQ(" ") );

         /*** Handle \ processing. */
         line0 = (line[0]=='\\' ? line+1 : line);

         chxclear (convert);
         switch (thislang) {
            case (LANG_EUC):   jixofeuc   (convert, line0, 0, opt);   break;
            case (LANG_SJIS):  jixofsjis  (convert, line0, 0, opt);   break;
            case (LANG_NORM):
            case (LANG_ISO):   chxofascii (convert, line0);           break;
         }

         /*** Remove trailing $ ("blank protector") if there is one.  Then
         /    tack the line onto the end of THISCHIX. */
         chxalter (convert, chxlen(convert)-1, dollar, nothing);
         chxcat   (thischix, convert);
      }

   }

   /*** For the top-level file only, once we've reached the end of
   /    the file, close off the last string. */
   if (top_level) {
      mdrec.width  =  chxtype (thischix);
      mdrec.length = (chxlen  (thischix) + 1) * mdrec.width;
      if (mdrecord (fd, &mdrec, *text, collisions)) {
         mdput (fd, thischix->p, mdrec.length);
         *text += mdrec.length;
      }
      chxfree (thischix);
      chxfree (convert);
      chxfree (dollar);
      chxfree (nothing);
      chxfree (newline);
   }

   fclose  (in);
   return  (1);
}
