/*** NEXTCONF.  Find the "next" conf.
/
/    int n = nextconf (pconfnum)
/
/    Parameters:
/       int *pconfnum;
/
/    Purpose:
/       This function gets the "next" conference number.
/       It assumes that the caller will set *PCONFNUM to zero before
/       the first call, and that the caller will not disturb the
/       contents between calls.  NEXTCONF returns 1 on success,
/       zero if there aren't any more conferences, -1 on errors.
/
/       NEXTCONF puts the local conference number into *PCONFNUM.
/
/    Returns:
/       1 on success
/       0 if there were no more users
/      -1 on any error.  In this case nextconf() also writes a message
/         to the "Distributed Bug" file.
/
/    Error Conditions:
/      Can't lock or load XSCD, or bad value in XSCD
/      Can't lock or load XSCN
/
/    Side Effects: changes the contents of *PCONFNUM
/
/    Called by: xfp (main routine)
/
/    Operating System Dependencies: none
/
/    Home: xfp/nextconf.c
/
/    Warning: There is a rare condition which will make NEXTCONF() fail
/    to find a set of confs.  But since this function is only for the
/    XFP, that set of confs will be found next time around.
/    The circumstances: we load nmdir, but don't reload it every time
/    we find the "next" conf.  But if the nmdir changes because Caucus
/    is splitting one of the XSCN files, under certain boundary conditions
/    we will skip one of the new "child" partfiles.
/
/    CHIX Warning: To avoid the "!confs" entry in XSCD, I compare the
/    1st byte in the confname entry against "!".  This must be
/    explicitly CHIXed eventually.
*/

/*: JV  4/08/91 15:55 Created function. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/02/91 17:17 Remove globals.h. */
/*: JV  7/04/91 17:53 Removed lint. */
/*: JV  7/22/91 16:09 Added ENTRY. */
/*: CR  7/24/91 22:06 Make Lint happier. */
/*: JV 11/01/91 16:06 Deal with Local Confnames vs TrueConfnames. */
/*: JV 11/12/91 17:19 Fix previous fix. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JV  8/09/92 15:45 Only check TrueConf entries. */
/*: DE  6/09/92 11:55 Started chixifying... */
/*: JX  7/24/92 12:05 Finish chixifying. */
/*: JX  9/01/92 16:18 Don't free namdir: it's static. */
/*: CI 10/06/92 16:27 Chix Integration. */
/*: CR 10/11/92 13:32 Fix use of *pconfnum in chxnum() call! */
/*: CR 12/14/92 15:15 Long chxvalue arg. */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */

#include <stdio.h>
#include "handicap.h"
#include "goodlint.h"
#include "chixuse.h"
#include "done.h"
#include "unitcode.h"
#include "namelist.h"
#include "xfp.h"

#define  THISLINE cbuf->line[lnum]

extern union  null_t        null;

FUNCTION nextconf (pconfnum)
   int *pconfnum;
{
   static struct namlist_t *namdir, *this;
   static int               lnum;
   static  first=1;
   static  Textbuf cbuf;
   int    success, ok;
   long   temp;
   Chix   name, num, cptr;
   struct namlist_t  *nlnode();
   Textbuf make_textbuf();

   ENTRY ("nextconf", "");

   num  = chxalloc (20L, THIN, "nextconf num");
   cptr = chxalloc (80L, THIN, "nextconf cptr");
   name = chxalloc (80L, THIN, "nextconf name");

   if (first) {
      cbuf  = make_textbuf (TB_RESPS, TB_LINES, "nextconf cbuf"); 
      first = 0;
   }
   success = 0;

   /*** On the 1st call, load nmdir.  Any changes made to XSCD after this
   /    won't be made to this copy of the data. */
   if (*pconfnum == 0) {
      if (NOT unit_lock (XSCD, READ, 0L, 0L, 0L, 0L, nullchix)) {
         bug ("NEXTCONF: can't lock XSCD.", 0);
         DONE (-1); }
      namdir = nlnode (4);
      ok = loadnmdir (XSCD, namdir);
      unit_unlk (XSCD);
      if (NOT ok) {
         bug ("NEXTCONF: can't loadnmdir XSCD.", 0);
         DONE (-1); }
      this = namdir;
      lnum = -1;
      cbuf->l_used = 0;
   }
   /*** For 1st time thru, now we have a valid list of partfiles
   /    that make up the namelist XSCD.  It's already out of date,
   /    but we don't care.  So we load the first partfile.
   /    For subsequent calls, we need to check if we've already
   /    used up the present partfile, and load the next if necessary. */

   while (TRUE) {
      /* First pre-increment from any previous call. */
      for (++lnum; lnum < cbuf->l_used; ++lnum) {
         /*** Each time get the "next" conference entry in the file and
         /    return the conference number.  Don't forget to skip the
         /    "~~~~~~", partfile terminators, and Local Confnames!  */
         if (chxtoken (num, nullchix, 2, THISLINE) == -1) continue;
         chxtoken (name, nullchix, 1, THISLINE);
         if (chxvalue (name, 0L) != '!' && chxvalue (num, 6L) == 'T') {
             chxcatsub (cptr, num, 2L, ALLCHARS);
             chxnum (cptr, &temp);
             *pconfnum = temp;
             if (*pconfnum == 0) DONE (0)    /* Ignore End Marker.*/
             else                DONE (1)            /* Success!! */
         }
      }         /* At this point we need to load the next pfile, if any. */
      this = this->next;                         /* (1st node is empty). */
      if (this == null.nl || this->num == 0) {
         DONE (0); }         /* No more pfiles: we're done. */
      if (this->num < 0) {
         bug ("NEXTCONF: bad value in XSCD.", 0);
         DONE (-1);  }
      if (NOT unit_lock (XSCN, READ, 0L, L(this->num), 0L, 0L, nullchix)) {
         bug ("NEXTCONF: can't lock XSCN.", 0);
         DONE (-1); }
      ok = loadnpart (XSCN, this->num, cbuf);
      unit_unlk (XSCN);
      if (NOT ok) {
         bug ("NEXTCONF: can't loadnpart XSCN.", 0);
         DONE (-1); }
      lnum = 0;
   }

   /*NOTREACHED*/
   bug ("NEXTCONF: got to end of function!", 0);
   success = -1;

done:
   chxfree (num);
   chxfree (cptr);
   chxfree (name);
 
   RETURN (success);
}
