/*** NEXTNEIGHBOR.  Get the "next" neighbor.
/
/  int n = nextneighbor (neighbor)
/
/  Parameters:
/    HOSTPTR *neighbor;
/
/  Purpose:
/    This function gets the neighbors of this host, one
/    at a time.  It assumes that the caller will assign NEIGHBOR to 
/    (int *)NULL before the first call, and that the caller will not disturb
/    the contents of NEIGHBOR between calls.  NEXTNEIGHBOR returns
/    1 on success, 0 if there aren't any more neighbors or an error occurs.
/
/    On a "first" call to nextneighbor, it first loads all the neighbors
/    from the HOSThhh files into a linked list of neighbors.  Subsequent
/    calls just return the "next" neighbor in the linked list.  Since
/    the HOSThhh files are sorted alphabetically by CHN, the linked list
/    will be sorted the same way.
/
/  How it Works:
/    On the first call, Nextneighbor() goes to the XSHD/XSHN namepart
/    file pair and builds a linked list of all the neighbors.  On
/    subsequent calls Nextneighbor just moves to the next node in the list.
/
/  Returns:
/    1 on success
/    0 on failure (no more neighbors or error (which produces bug msg))
/
/  Error Conditions:
/
/  Side effects:
/    Changes the contents of the structure pointed to by NEIGHBOR.
/
/  Related functions: nextuser(), nextconf()
/
/  Called by: xfp (main)
/
/  Home: xfp/nextneig.c
*/

/*: JV  3/03/91 12:09 Created Function. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/02/91 17:19 Remove globals.h. */
/*: JV  7/04/91 19:18 Remove lint. */
/*: JV  7/17/91 11:48 Initialize hostptr->next, fix textbuf checker. */
/*: JV  7/22/91 16:10 Add ENTRY. */
/*: JV  7/29/91 18:04 Allocate HOST, de-allocate linked list! */
/*: JV  8/07/91 12:28 Strip # off of hostnumber in "hostnnn". */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: DE  6/09/92 12:15 Started chixifying... */
/*: JX  7/24/92 12:30 Finished chixifying. */
/*: CR 12/14/92 15:15 Long chxvalue arg. */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */

#include <stdio.h>
#include "handicap.h"
#include "unitcode.h"
#include "chixuse.h"
#include "done.h"
#include "xfp.h"
#include "namelist.h"

extern union  null_t null;

FUNCTION nextneighbor (neighbor)
   HOSTPTR *neighbor;                     /* Yup, ptr to a ptr. */
{
   int     ok, i, success;
   long    lhnum;
   Chix    id, left, number; 
   char   *sysmem();
   struct  namlist_t *nlnode(), *namdir, *this;
   static  HOST hosthead;          /* A null host: head of linked list */
   static  int     first=1;
   static  Textbuf nbuf;
   Textbuf make_textbuf();
   HOSTPTR hostptr, nextptr, make_host();
   Hinfoptr make_hinfo();
 
   ENTRY ("nextneighbor", "");
     
   id      = chxalloc (40L, THIN, "nextneig id");
   left    = chxalloc (20L, THIN, "nextneig left");
   number  = chxalloc (20L, THIN, "nextneig number");

   if (first) {
      nbuf = make_textbuf (TB_RESPS, TB_LINES, "nextneig nbuf"); 
      first=0;
   }
   success = 0;

   if (*neighbor == (HOSTPTR) NULL) {  /* We have to load all the neighbors. */
      if (NOT unit_lock (XSHD, READ, 0L, 0L, 0L, 0L, nullchix))  {
         bug ("NEXTNEIGHBOR: can't lock XSHD.", 0);   FAIL; }
      namdir = nlnode(4);
      ok = loadnmdir (XSHD, namdir);
      unit_unlk (XSHD);
      if (NOT ok)     {
         nlfree (namdir);
         bug ("NEXTNEIGHBOR: can't load XSHD.", 0);     FAIL; }
   
      /*** Load each namepart file in turn and store LH#. */
      for (this=namdir->next;   this != null.nl;   this = this->next) {
         if (this->num <= 0)  continue;
         if (NOT unit_lock (XSHN, READ, 0L, L(this->num), 0L, 0L, nullchix))    
          continue;
         if (NOT loadnpart (XSHN, this->num, nbuf))
               { unit_unlk (XSHN); continue; }
         unit_unlk (XSHN);
 
         /* Aaah! Finally we find some host data and load each one.
         /  LH# is word 2 in LINE.  Then get the host with get_neighbor()
         /  and add the host to the end of the linked list. */  
         for (i=0, nextptr = &hosthead;   i<nbuf->l_used;   ++i) {
            if (chxtoken (id, nullchix, 2, nbuf->line[i]) == -1)
               continue;
            if (chxvalue (nbuf->line[i], 0L) == '!')
               continue;                             /* Ignore "!host nnn" */
            chxbreak (id, left, number, CQ("#"));
            if (NOT chxnum (number, &lhnum)) {
               bug ("NEXTNEIGHBOR: error loading LH#'s.", 0); FAIL; }
            hostptr =  make_host();
            hostptr->info = make_hinfo();
            unit_lock (XHI1, READ, L(lhnum), 0L, 0L, 0L, nullchix);
            get_neighbor ((int) lhnum, hostptr, XHI1);
            unit_unlk (XHI1);
            nextptr->next = hostptr;
            nextptr = hostptr;
            hostptr->next = (HOSTPTR) NULL;
         }
      }  /* for each partfile */
      nlfree (namdir);
      *neighbor = hosthead.next;

   } else                /* Just return the next one in the linked list. */
      *neighbor = (*neighbor)->next;           /* Might be NULL. */

   if (*neighbor == (HOSTPTR) NULL) {
      /*** Not only do we have to return a failure, we need to de-allocate
      /    the entire linked list first. */
      for (hostptr = hosthead.next; hostptr != (HOSTPTR) NULL;
           hostptr = nextptr) {
         nextptr = hostptr->next;
         free_host (hostptr);
      }
      FAIL;
   }
   else SUCCEED;     /*  Found one! */

 done:
   chxfree (id);
   chxfree (left);
   chxfree (number);

   RETURN (success);
}

