/*** XFP. (Main)  This is the MAIN routine for the Xaction File Packager.
/
/   The duty of the XFP is to take all of the raw transactions in each
/   user's transaction file, from the mail transaction files, and
/   from the xport files we've received that have to be re-distributed.
/   create transport files out of them.   Each conference transaction file
/   will have transactions for one or more conferences, each of which must
/   be a distributed conf.
/
/   A short description of what the XFP does:
/   1) Ensure enough disk space or stop.
/   2) Cleanup from any previous sessions XFP couldn't finish due to
/      lack of disk.  Move them.  Check disk space again.
/   3) For each user's xaction file:
/      A) Append each xaction to the correct conference user xaction file.
/      B) Check disk space.
/   4) For each distributed conference:
/      A) Append each conf xaction file to to its conf user xaction file.
/      B) Turn the conf user xaction file into a xport file and put that 
/         into each of the shipping bay(s) in which it needs to go,
/         in order to get to each neighbor that it should.
/      B) Check disk space.
/   5) For each neighboring host:
/      A) Add each conf xport file that came from another host to the correct
/         shipping bay(s).
/      B) Check disk space.
/   6) For each neighboring host:
/      A) Add a header to the mail xaction file for that host and copy
/         this new transport file to the correct shipping bay.
/      B) Check disk space.
/
/   Each of steps 3, 4, 5 and 6 above check each entity of an object type.
/   This is implemented below by functions nextuser(), nextconf(), and
/   nextneighbor().
/
/   **It does not matter to XFP how the entities are ordered.**
/   For example, the conferences might be ordered alphabetically by
/   conference name or numerically by conference number.  The point is that
/   it doesn't matter to XFP, as long as *all* of them are eventually
/   mentioned during the loop.
/ */

/*: JV  3/02/91 17:39 Began XFP work. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/04/91 17:53 Removed lint. */
/*: JV  7/18/91 10:55 Increase debug granularity. */
/*: JV  7/22/91 16:24 Add ENTRY. */
/*: CR  7/24/91 22:08 Make Lint happier. */
/*: JV  8/09/91 12:53 Update seqnum, add 2nd arg to set_neighbor */
/*: JV 10/10/91 17:00 Conditionalized some debug code. */
/*: JV 10/17/91 10:27 Add audit trail gobal variable. */
/*: JV 10/22/91 15:10 Move audit glob var to xfpgloba.c, add net_argchk */
/*: JV 10/30/91 12:02 Add net_flag global declaration. */
/*: JV 11/22/91 12:39 Add xfp_protect and xfp_unprotect. */
/*: JV 12/05/91 11:41 Remove the ever-popular "Hi There". */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JV  1/03/92 15:04 Remove unused 'dlevel'. */
/*: JV  1/20/92 10:18 Process XHXA before XHML. */
/*: DE  6/09/92 15:19 Chixified */
/*: CR 10/09/92 17:55 Remove dstdfinit(), dstdfree() decl's. */
/*: CR 12/14/92 15:15 Long chxvalue arg. */
/*: CP  3/04/93 23:16 Remove nested comment. */
/*: CK  3/19/93 16:07 Save & restore terminal state; merge done, DONE2. */
/*: J8  8/02/93 10:34 Change arg to net_argchk(). */
/*: JV  8/09/93 10:54 Change diskfree to sysavail(). */
/*: CK  9/23/93 17:21 Sysavail requires (char *) 1st arg. */
/*: JZ 10/18/93 10:35 Remove dust bunnies. */
/*: CR 10/27/93 12:20 Remove initial ascofchx()! */
/*: CP  2/09/94 13:53 If (debug)... use ascquick(neigh->chn).  */

#include <stdio.h>
#include "caucus.h"
#include "syserror.h"
#include "goodlint.h"
#include "unitcode.h"
#include "xfp.h"
#include "done.h"

extern struct termstate_t  term_original, term_in_caucus;
extern struct net_flag_t   net_flag;
extern Chix                confid;

int debug;
Chix ss_pause, ss_abort;

FUNCTION main (argc, argv)
int argc; char *argv[];
{
   long freedisk, files;
   Chix user;
   char confidstr[300];
   int  confnum=0, success=1, error;
   HOSTPTR neigh=(HOSTPTR) NULL;

   ENTRY ("main", "");

   ss_pause  = chxalloc (80L, THIN, "xfp ss_pause");  /* Global */
   ss_abort  = chxalloc (80L, THIN, "xfp ss_abort");  /* Global */
   user      = chxalloc (80L, THIN, "xfp user"); 

   if (argc < 2) {printf ("Usage: xfp_x confid [-d [#]] [-y net.opt]\n");
                  exit(0); }

   confid = chxalloc (40L, THIN, "xfp confid");
   chxofascii (confid,    argv[1]);
   strcpy     (confidstr, argv[1]);
   
   net_flag.optfile = chxalloc (40L, THIN, "xfp netflag");
   net_argchk (&argc, argv, 2);

   systurn (&term_original,  GET_TERM, 0);
   systurn (&term_in_caucus, GET_TERM, 0);
   unitglobals();
   utilglobals();
   xfpglobals();
   net_optchk ();

   if (NOT xfp_protect())  exit (0);

   chxclear (user);                /* Initialize for nextuser(). */

   if (debug) printf ("Nextuser:\n");
   while (nextuser (user) > 0) {
      if (chxvalue(user, 0L) == L('\0')) {
         bug ("XFP: Nextuser returned 1 but no data.", 0); break; }
      if (debug) printf ("XFP: Got user: %s\n", ascquick(user));
      sysavail (confidstr, &freedisk, &files, &error);
      if (error != SE_NOTIMP && freedisk < MINDISK) {
         nodisk (freedisk);           /* Report lack of disk to CaucusMgr */
         FAIL;
      }
      moveuxact (user);
   }
   if (debug) printf ("XFP: Finished users.\n");

   if (debug) printf ("XFP: Starting conf:\n");
   /*** For each conf, turn the conf-xaction-file into a xport file. */
   while (nextconf (&confnum)) {
      if (debug) printf ("XFP: Got conf: %d\n", confnum);
      xp_ship (confnum);             /* This makes one copy per naybor */
      sysavail (confidstr, &freedisk, &files, &error);
      if (error != SE_NOTIMP && freedisk < MINDISK) {
         nodisk (freedisk);
         FAIL;
      }
   }

   /*** The order we process the next two types of xaction files is
   /  important.  By processing the XHXA files before the XHML files
   /  we'll pick up NEW xactions before {CHANGE | DELETE} TRANSIT
   /  xactions. */
   while (nextneighbor (&neigh)) {
      if (debug) printf ("Next: Neigh->lhnumber: %d.\n", neigh->lhnumber);

      /* First check each neighbor for conf xactions coming from it
      /  that must be dispersed to other naybors sharing the conf.
      /  First we re-get the neighbor's host info in case anything
      /  has changed. */
      if (NOT unit_lock (XHI2, WRITE, L(neigh->lhnumber), 0L, 0L, 0L, 
                         nullchix)) {
         bug ("XFP: can't lock XHI2.", 0); continue; }
      if (debug) printf ("XFP: Got neighbor: %s for XHXA.\n", 
                                    ascquick(neigh->chn));
      get_neighbor (neigh->lhnumber, neigh, XHI2);
      xpthru_ship  (neigh);
      set_neighbor (neigh, XHI2);
      unit_unlk (XHI2);

      sysavail (confidstr, &freedisk, &files, &error);
      if (error != SE_NOTIMP && freedisk < MINDISK) {
         nodisk (freedisk);
         FAIL;
      }
   }

   neigh = (HOSTPTR) NULL;
   while (nextneighbor (&neigh)) {
      /*** Now check each neighbor for mail headed to/thru it. */
      if (NOT unit_lock (XHI1, WRITE, L(neigh->lhnumber), 0L, 0L, 0L,
                         nullchix)) {
         bug ("XFP: can't lock XHI1.", 0); FAIL; }
      if (debug) printf ("XFP: Got neighbor: %s for XHML.\n",
                         ascquick(neigh->chn));
      mailxp_ship (neigh);
      unit_unlk (XHI1);

      sysavail (confidstr, &freedisk, &files, &error);
      if (error != SE_NOTIMP && freedisk < MINDISK) {
         nodisk (freedisk);
         FAIL;
      }
   }
   if(debug) printf ("Finished neighbors.\n");

   SUCCEED;

done:
   xfp_unprotect ();
   chxfree (user);
   systurn (&term_original, SET_TERM, 0);
   exit    (success);
   RETURN  (success); /* Make lint happy. */
}
