/*** FORW_XACTION.  Put this xaction in a place where it will get sent
/                   on (forwarded) to appropriate neighboring hosts.
/
/   int n = forw_xaction (pheader, pxact, error)
/
/   Parameters:
/      XPORTHDR    *pheader;
/      XACTIONPTR   pxact;
/      int         *error;
/
/   Purpose:
/     After the XFU has parsed and possibly digested a xaction, it
/     needs to decide whether, and if so, to whom, to forward it.
/     This function first makes the decisions and then performs the
/     actions.
/
/      Xactions that do go in XHXA:
/         "Normal" conf additions
/         Conf changes being dispersed from the original host.
/
/      Xactions that do not go in XHXA:
/         "In transit" xactions (they go in XHML toward original host).
/         Conf changes being dispersed from the original host.
/            These go in XCXT for the correct conf.  For more on this,
/            see comments starting about 15 lines after the tag DISPERSE.
/         Mail, which also goes in XHML. (But this is handled by
/            the XFU directly, when it calls parse_xaction().)
/         People changes (I'm not sure where they go-JV:5-AUG-91).
/
/   How it works:
/      Forw_xaction() assumes that XHXA has been locked and opened,
/      and will be unlocked and closed by the caller.
/      It also assumes that the caller has written the header to XHXA,
/      although this code won't barf if it hasn't.  But if the header
/      isn't there, something else *will*, and it won't be pretty.
/
/   Returns:
/      1  if any of the xactions were copied into XHXA.
/      0  if no xactions needed to be copied into XHXA.
/      -1 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions:
/
/   Called by:
/
/   Home: xfu/forwxact.c
/ */

/*: JV  8/05/91 08:13 Create this. */
/*: JV  8/29/91 15:00 Cast locnum, seqnum_thru_hi to int for now.  Fix later.*/
/*: JV  9/12/91 12:37 Fix XHML lock. */
/*: JV 10/11/91 18:00 Fix some RETURNS */
/*: JV 10/31/91 12:15 Add "break"s. */
/*: JV 11/07/91 12:23 Add TT_SUB* types. */
/*: CR 11/25/91 14:03 Add RETURN(0) at bottom (to make Lint happy?) */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR 12/26/91 12:39 Use FAIL/SUCCEED and free all chix!! */
/*: JV  1/15/92 11:30 Handle TT_HOST, _SUBDEL, _ROUTES, init ERROR. */
/*: DE  6/10/92 11:50 Chixified */
/*: JX  7/31/92 16:54 Fix Chix. */
/*: JX  9/17/92 15:19 FAIL for TT_HOST, etc. */
/*: CR 10/09/92 17:34 ERROR is (int *), not int. */
/*: CP  3/04/93 23:16 Remove nested comment. */
/*: CK  3/20/93 19:22 Treat SUBACK like SUBNAK (i.e., toss it!) */

#include <stdio.h>
#include "handicap.h"
#include "chixuse.h"
#include "xaction.h"
#include "xfp.h"
#include "done.h"

FUNCTION forw_xaction (pheader, pxact, error)
   XPORTHDR   *pheader;
   XACTIONPTR  pxact;
   int        *error;
{
   Chix locname, tname, orighost, mychn;
   char chxname[20];
   int  neighnum, i, success;
   Chix carray[2];
   HOSTPTR neighbor, make_host();
   Hinfoptr make_hinfo();

   ENTRY ("FORW_XACTION", "");

   neighbor = make_host();
   neighbor->info = make_hinfo();

   locname    = chxalloc (80L, THIN, "forwxact locname");
   tname      = chxalloc (80L, THIN, "forwxact tname");
   orighost   = chxalloc (80L, THIN, "forwxact orighost");
   mychn      = chxalloc (80L, THIN, "forwxact mychn");

  *error=0;

   /*** Allocate the carray fields. */
   for (i=0; i< TEXTFIELDS; i++) {
      sprintf (chxname, "forw_x carray[%d]", i);
      carray[i] = chxalloc (0L, THIN, chxname);
      if (pxact->datasize[i])  chxcpy (carray[i], pxact->data[i]);
   }

   /*** First take care of the case of conf changes in transit. */
   if (pxact->transit) {
      get_trueconf ((int) pxact->conflocnum, locname, tname, orighost);
      get_chn (mychn, XSNF, 0);
      if (chxeq (orighost, mychn)) goto DISPERSE;  /* So Sue Me! */

      get_course (orighost, &neighnum);
      unit_lock    (XHI1, READ, L(neighnum), 0L, 0L, 0L, nullchix);
      get_neighbor (neighnum, neighbor, XHI1);
      unit_unlk    (XHI1);

      unit_lock   (XHML, WRITE, L(neighnum), 0L, 0L, 0L, nullchix);
      unit_append (XHML);
      putnextxact (XHML, pxact, carray);
      unit_close  (XHML);
      unit_unlk   (XHML);
      FAIL;
   }

DISPERSE:
   /*** Put most of these in the XHXA of the neighbor who sent it to us. */
   switch (pxact->typenum) {
   case (TT_ITEM):
   case (TT_RESPONSE):
   case (TT_OSUBJECT):
   case (TT_CONFERENCE):

      /*** Unfortunately, an "in transit" change (for which this is the
      /    original host) must be re-directed to XCXT instead of XHXA,
      /    since the header for a xport file that included "in transit"
      /    changes was an XHML, whose header would be MSGS and would
      /    not have a CONF field.  To make matters worse, one XHML file
      /    might contain "in transit" changes for many different confs,
      /    and at some point we need to split them out into the different
      /    confs.  *This* is that point.  */
      if (pxact->transit) {
         pxact->transit = 0;     /* It has achieved its purpose */
         unit_lock   (XCXT, WRITE,  pxact->conflocnum, 0L, 0L, 0L, nullchix);
         unit_append (XCXT);
         putnextxact (XCXT, pxact, carray);
         unit_close  (XCXT);
         unit_unlk   (XCXT);
         FAIL;               /* MUST do this so that XFU counts properly */
      }
      else {
         putnextxact (XHXA, pxact, carray);
         SUCCEED;
      }
      break;

   case (TT_MESSAGE):               /* These get handled by parse_xact */
   case (TT_SUBREQ):
   case (TT_SUBNAK):
   case (TT_SUBACK):
   case (TT_HOST):
   case (TT_SUBDEL):
   case (TT_ROUTES):   FAIL;  /* MUST do this so that XFU counts properly. */
   default:
      bug ("FORW_XACTION: message type not dealt with:", pxact->typenum);
      printxact (pxact);
      DONE (-1);
   }  

   FAIL;

done:
   for (i=0;   i<TEXTFIELDS;   ++i)  chxfree (carray[i]);

   chxfree (locname);
   chxfree (tname);
   chxfree (orighost);
   chxfree (mychn);

   free_host (neighbor);

   RETURN (success);
}   
