/* Mess_PARSE.  Parse a xaction that the XFU has determined to be a message. 
/
/   int n = mess_parse (xptr)
/
/   Parameters:
/      XACTIONPTR xptr;
/
/   Purpose:
/      After the XFU has decided that an incoming xaction is a message,
/      mess_parse() further parses it and either delivers it to a local
/      user or forwards it on to a neighboring host.
/
/      If the message was addressed to a user on this host and it can't
/      be delivered because that is an invalid address or the recipient's
/      mailbox is full, the XFU sends the message back to the host it
/      came from with an error code in the MAILCTRL field.  The XFU
/      at the originating host parses this field and replies to the
/      message sender with an appropriate message.  This method enables
/      us to reply to the user using their own dictionary.
/
/      If the message can't be delivered because there were multiple
/      matches of that address, the "receiving" XFU replies to the
/      message with a list of matches, one per line.  The "originating"
/      XFU turns this into a reply by adding appropriate text to the reply.
/
/   How it works:
/      A message xaction can be one of two xaction types: XT_NEW, & XT_REJECT.
/      XT_NEW is for any normal mail message being sent, routed, or delivered.
/      XT_REJECT is for any message that could not be delivered for some
/      reason and is being sent back to the original author.
/
/      Either of these two xaction types can be one of three message types:
/      SEND, REPLY, FORWARD.  Since a "message" can contain more than one
/      message "part", we concatenate all of the parts together into
/      the DATA field, and keep track of the number of "original" parts.
/      By counting the number of parts that are in DATA and subtracting
/      the number of original parts, we can tell whether there is a "new"
/      part to this message or this message is just made up of original parts.
/
/      When the XFU processes a message of either xaction type and of any
/      message type, it tries to do one of two things with it.  If the
/      intended recipient should be on this host, the XFU tries to deliver
/      it via add_caumess().  If the intended recipient should be on another
/      host, the XFU tries to forward it via add_xmessage().
/
/      If either of these fail the XFU tries to send the message back to 
/      the original author.  If the message cannot be delivered back to 
/      the author (either here or somewhere along the way) the XFU sends
/      it to the local Caucus manager.
/
/      Be careful of the code that sends a badly addressed message back
/      to the originating XFU.  It sets the "recipient" to the original
/      sender so that the code knows where to send the message back.
/      It also sets the "sender" to the original recipient so that we
/      don't lose that information.
/      The "originating" XFU must detect the error in MAILCTRL and do
/      something appropriate.
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions:
/
/   Called by: parse_xaction()
/
/   Known bugs:
/      add_caumess() is called with the explicit string "Caucus Manager"
/      for certain reject messages.  This should be replaced with a
/      dictionary string.
/
/   Home: xfu/messpars.c
/ */

/*: JV  5/21/91 10:44 Create this. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/04/91 19:10 Remove lint. */
/*: JV  7/10/91 11:12 Add code to deal with exceptional conditions. */
/*: JV  7/10/91 15:28 Make mychn[] local. */
/*: JV  7/11/91 15:03 Remove lots of code into messforw.c */
/*: JV  7/15/91 21:50 Add multiple text fields. */
/*: CR  7/24/91 22:10 Make Lint happier.   Correct add_dmess calls. */
/*: CR  7/28/91 14:10 Remove unused variables. */
/*: CR  7/28/91 15:48 chxalloc() first argument is long. */
/*: JV  8/23/91 16:26 Remove return() notreached. */
/*: JV 10/10/91 16:10 Fix addressing of new messages. */
/*: JV 10/24/91 14:43 Fix multiple-hop sends. */
/*: JV 10/31/91 12:18 Add "break"s. */
/*: JV 12/05/91 16:18 Fix addxmessage() call on DNOUSER. */
/*: JV 12/16/91 15:43 Random fixes to MMANY. */
/*: CR 12/17/91 13:33 Much cleanup, & rewriting of MNONE, MFULL, MMANY. */
/*: CR 12/18/91 23:30 Ensure original message parts have preface. */
/*: CR 12/26/91 12:10 Declare sysmem() & systime(), cast sysfree() arg. */
/*: DE  6/10/92 15:49 ReChixified */
/*: JX  7/24/92 15:47 Use make_hinfo(). */
/*: CR 10/11/92 22:43 Remove CQ from add_dmess() calls. */
/*: CR 10/21/92 13:31 Chxalloc 'temp'. */
/*: CR 12/14/92 14:51 Long chxalter arg. */
/*: CR  2/27/93 14:16 Replace add_dmess() with add_caumess(). */
/*: CP  3/04/93 23:13 Fixed nested comments. */
/*: JP  5/05/93 14:57 Check return from get_u_dicti(). */
/*: CK  7/27/93 17:37 Add Attachment arg to add_caumess() calls. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "chixuse.h"
#include "common.h"
#include "namelist.h"
#include "xaction.h"
#include "caucus.h"
#include "xfp.h"

#define BUG        printxact (xptr)
#define AVGNMSIZ   40
#define LOCAL      1

extern union  null_t    null;
extern Userreg thisuser;

FUNCTION  mess_parse (xptr)
XACTIONPTR xptr;
{
   HOSTPTR  myptr, make_host();
   Hinfoptr make_hinfo();
   Namelist nlmatches, pnl, nlnode();
   Chix  host, uid, date, time, mychn, reply_text, userid, userchn,
         manager, auth_id, auth_chn, preface, dateorig, timeorig,
         full_name, matchid, corig, cnew, temp;
   int error, mailctrl=NOERR, matches, success, parts;
   char *sysmem();
   long  systime();

   ENTRY ("mess_parse", "");

   myptr = make_host();
   myptr->info = make_hinfo();

   temp       = chxalloc (80L,  THIN, "messpars temp");
   host       = chxalloc (80L,  THIN, "messpars host");
   uid        = chxalloc (80L,  THIN, "messpars uid");
   date       = chxalloc (20L,  THIN, "messpars date");
   time       = chxalloc (20L,  THIN, "messpars time");
   mychn      = chxalloc (80L,  THIN, "messpars mychn");
   reply_text = chxalloc (200L, THIN, "messpars reply_text");
   userid     = chxalloc (80L,  THIN, "messpars userid");
   userchn    = chxalloc (160L, THIN, "messpars userchn");
   manager    = chxalloc (80L,  THIN, "messpars manager");
   auth_id    = chxalloc (80L,  THIN, "messpars auth_id");
   auth_chn   = chxalloc (80L,  THIN, "messpars auth_chn");
   preface    = chxalloc (200L, THIN, "messpars preface");
   dateorig   = chxalloc (20L,  THIN, "messpars dateorig"); 
   timeorig   = chxalloc (20L,  THIN, "messpars timeorig");
   full_name  = chxalloc (160L, THIN, "messpars full_name");
   matchid    = chxalloc (80L,  THIN, "messpars matchid");

   if (NOT get_chn (mychn, XSNF, 0)) {
      bug ("MESSPARS: can't get XSNF.", 0); FAIL; }

   corig  = chxalloc (0L, THIN, "messpars corig");
   cnew   = chxalloc (0L, THIN, "messpars cnew");
   nlmatches = nlnode (1);

   switch (xptr->actnum) {
   case XT_NEW:

      /*** If new message for another host, forward it on. */
      if (NOT chxeq (mychn, xptr->rechost)) { mess_forward (xptr);  SUCCEED; }

      /*** Otherwise, the message is destined here: deal with it. */
      if (xptr->m_oparts)         chxcpy (corig, xptr->data[1]);
      if (xptr->datasize[0] > 0)  chxcpy (cnew,  xptr->data[0]);

      chxbreak (xptr->date, date, time, CQ(" "));
      chxbreak (xptr->recid, uid, host, CQ("@"));
      chxbreak (xptr->uid, auth_id, auth_chn, CQ("@"));
      name_to_netid (xptr->recname, uid, nullchix, LOCAL, nlmatches, &error);

      /*** If we get an error back from name_to_netid, give up. */
      if (error) { bug ("mess_parse: name_to_net_id failed.", 0);  FAIL; }

      matches = nlsize (nlmatches);
      switch (matches) {
         case 0: mailctrl = DNOUSER; break;
         case 1:
            add_caumess (xptr->uname, auth_id, auth_chn,
                  xptr->title, xptr->mtypenum, date, time,
                  corig, xptr->m_oparts, xptr->rrlength,
                  nlmatches->next->str, cnew, (Attachment) NULL, &error);
            mailctrl = error; break;
         default: mailctrl = DMANYUSER; break;
      }

      switch (mailctrl) {
         case DNOUSER:      /* No such user to send to, or */
         case DBOXFULL:     /* User's mail box is full. */
            xptr->mailctrl = (mailctrl==DNOUSER ? MNONE : MFULL);
            addxmessage (xptr->recname, xptr->recid, xptr->title,
                XT_REJECT, corig, xptr->m_oparts, -1, xptr->mailctrl,
                xptr->uname, auth_id, auth_chn, cnew,
                date, time, &error);
            break;

         case DMANYUSER:    /* Multiple users by that name. */

            /*** We're going to take the user's message and make it the
            /    "original" part.  The "new" part will contain the names
            /    of everyone who matched the recipient name. */

            /*** If the message has a "new" part, put a preface on it and
            /    call it the original part.  Otherwise, leave the original
            /    part alone (it was probably from a FORWARD). */
            if (NOT EMPTYCHX(cnew)) {
               build_preface (preface, 0, MSEND, xptr->uname, auth_id, auth_chn,
                              date, time, 0);
               chxcpy (corig, preface);
               chxcat (corig, cnew);
            }
            chxclear   (cnew);

            /*** Pick up the userids from NLMATCHES, and then pull the
            /    full name from each user's XURG file. */
            for (pnl=nlmatches->next;  pnl!=null.nl;  pnl=pnl->next) {
               chxcpy (matchid, pnl->str);
               while  (chxalter (matchid, 0L, chxquick(" ",0), 
                                 chxquick("",1))) ;
               if (NOT unit_lock (XURG, READ, 0L, 0L, 0L, 0L, matchid))
                                                                continue;
               if (NOT unit_view (XURG))  { unit_unlk (XURG);   continue; }
               unit_read  (XURG, full_name, 0L);
               unit_close (XURG);
               unit_unlk  (XURG);
               chxcat (full_name, CQ("@"));
               chxcat (full_name, mychn);
               chxcat (full_name, CQ("\n"));
               chxcat (cnew, CQ("    "));
               chxcat (cnew, full_name);
            }
            addxmessage (xptr->recname, xptr->recid, xptr->title, XT_REJECT,
                corig, 1, -1, MMANY, xptr->uname, auth_id, 
                auth_chn, cnew, date, time, &error);
            break;

         case NOERR: SUCCEED;

         default:    logx ("Couldn't add message to DB:", xptr);
                     FAIL;
      }

      switch (error) {              /* (on trying to return a bad msg) */
         case DBADXACT:logx ("Messpars: Couldn't return bad msg:", xptr);
                         FAIL;
         case DLOCK:
         case DFILE:     tellmgr (xptr, error);   /* These 3 RETURN 1 */
         case DMISC:     tellmgr (xptr, error);
         case DNOCHN:    reparse (xptr);
         case NOERR:     SUCCEED;
         default:        FAIL;
      }
   break;

   case XT_REJECT:
      /*** Reject messages aimed for another host should be forwarded on. */
      if (NOT chxeq (mychn, xptr->rechost)) { mess_forward (xptr); SUCCEED; }

      /*** The rest of this section massages the reject message and puts
      /    it, as a reply, in the original author's mailbox on this host. */

      /* GET sender's dictionary number and set it. */
      if ((thisuser->dict = get_u_dicti (xptr->recid)) == -1) {
         logx ("Messparse: couldn't get user's dictionary.", xptr);
         FAIL;
      }

      /*** Use get_neighbor() to set local caucus manager userid. */
      get_neighbor (0, myptr, XSNF);
      chxcpy  (manager, myptr->info->mgrid);

      /*** Prepare recipient's userid, current date & time. */
      chxbreak (xptr->recid, userid, userchn, CQ("@"));
      sysdaytime  (date, time, 0, systime());

      switch (xptr->mailctrl) {                       /* What went wrong? */
      case MNONE:  /* Build the message "No-one matched this name".    */
      case MFULL:  /* Build the message "This user's mailbox is full." */
         mdstr (reply_text, (xptr->mailctrl==MNONE ? "net_mnone" : "net_mfull"),
                null.md);
         chxcpy (cnew, reply_text);
         chxformat (cnew, CQ("    %s (%s)\n"), 0L, 0L, xptr->uname, xptr->uid);
         chxbreak   (xptr->date, dateorig, timeorig, CQ(" "));

         /*** Put the user's message into the "original" part.  A forwarded
         /    message just stays in the original part.  For a new msg or reply,
         /    we build a preface, then put it all into the original part. */
         if (EMPTYCHX(xptr->data[0])) {
            chxcpy (corig, xptr->data[1]);
            parts = xptr->m_oparts;
         }
         else {
            build_preface (preface, 0, MSEND, xptr->recname, userid, userchn,
                           dateorig, timeorig, 0);
            chxcpy (corig, preface);
            chxcat (corig, xptr->data[0]);
            parts = 1;
         }
         chxcpy (temp, CQ("Caucus Manager"));
         add_caumess  (temp, manager, null.chx, xptr->title,
                       MREPLY, date, time, corig, parts, -1,
                       userid, cnew, (Attachment) NULL, &error);
         SUCCEED;

      case MMANY:
         /*** Build the message "Many people matched the name XXX.
         /    They are: (list)".  Take (list) from the new text of the
         /    reject message.   Send the result to this user, and pretend
         /    it's from the local manager. */
         mdstr      (reply_text, "net_mmany1", null.md);
         chxcpy     (cnew, reply_text);
         chxformat  (cnew, CQ("    %s (%s)\n"), 0L, 0L, xptr->uname, 
                     xptr->uid);
         mdstr      (reply_text, "net_mmany2", null.md);
         chxcat     (cnew, reply_text);
         chxcat     (cnew, xptr->data[0]);
         chxcpy     (corig, xptr->data[1]);
         chxcpy     (temp, CQ("Caucus Manager"));
         add_caumess (temp, manager, null.chx, xptr->title, 
                      MREPLY, date, time, corig, xptr->m_oparts, -1,
                      userid, cnew, (Attachment) NULL, &error);
         SUCCEED;

      default: logx ("Messparse: couldn't parse xact, bad mailctrl.", xptr);
               FAIL;
      }
      break;

   default:  /* Unknown xaction type! */
      BUG; FAIL;
   }

done:
   chxfree (corig);
   chxfree (cnew);
   chxfree (host);
   chxfree (uid);
   chxfree (date);
   chxfree (time);
   chxfree (mychn);
   chxfree (reply_text);
   chxfree (userid);
   chxfree (userchn);
   chxfree (manager);
   chxfree (auth_id);
   chxfree (auth_chn);
   chxfree (preface);
   chxfree (dateorig);
   chxfree (timeorig);
   chxfree (full_name);
   chxfree (matchid);
   chxfree (temp);
   nlfree  (nlmatches);
   
   RETURN  (success);
}   
