/*** PARSE_XACTION.  Parse a xaction that the XFU has plucked out.
/
/   int n = parse_xaction (xactptr)
/
/   Parameters:
/     XACTIONPTR xactptr;
/
/   Purpose:
/      After the XFU has plucked a xaction out of a xaction file, it
/      needs to parse the xaction.  Due to the large number of xaction
/      types and possibilities for what can be done with them,
/      this parsing is spread out over these routines:
/      parse_xaction, which calls one of:
/         item_parse(), resp_parse(), mess_parse(), person_parse(),
/         conf_parse(), subscr_parse()
/      These *_parse() routines further parse the xaction.
/
/      Note that the XFU deals with routing the info on to other hosts
/      if necessary.  This includes any In Transit xactions:
/      If this is the original host, the xaction gets digested here,
/      and forw_xaction will later disperse Change xactions.
/      If this is not the original host, forw_xaction will forward
/      it onto the original host.
/
/   How it works:
/     Parse_xaction just figures out which type of object we're acting on
/     and hands off the processing to the appropriate routine which will
/     do the brunt of the parsing and processing of the xaction.
/
/     There are exceptions, cases where there is no further parsing
/     and the processing is minimal.  These exceptions are the TT_OSUBJECT
/     xaction types.
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions:
/
/   Called by: xfu (Main)
/
/   Home: xfu/parsexac.c
/ */

/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/04/91 11:35 Fix DINOITEM */
/*: JV  7/15/91 22:42 Add multiple text fields. */
/*: JV  7/26/91 22:00 Add docs. */
/*: CR  7/28/91 14:11 Add RETURN for Lint.  Use chg_dsubject(). */
/*: CR  7/28/91 15:48 chxalloc() first argument is long. */
/*: JV  7/29/91 12:39 Remove 2nd arg from reparse */
/*: JV  8/01/91 17:21 Fix bug. */
/*: JV  8/05/91 11:15 Add check for In Transit, comments */
/*: JV  8/29/91 15:30 Cast locnum to int for now, fix later. */
/*: JV 10/29/91 15:46 Add subreq_parse(). */
/*: JV 10/31/91 12:19 Add "break"s. */
/*: JV 11/07/91 12:33 Add SUBNAK. */
/*: CR 11/07/91 14:46 Add locname arg to chg_dsubj, via get_trueconf(). */
/*: JV 11/18/91 22:26 Add Delete Osubject. */
/*: JV 12/07/91 20:40 Deallocate chixes. */
/*: JV 12/12/91 13:44 Add tellmgr() calls, TT_HOST. */
/*: JV 12/23/91 10:26 Add req_report. */
/*: JV  1/03/92 12:20 Notify local mgr of any SUBREQ. */
/*: CR  1/08/92 12:29 Fix 3rd arg in mdstr() call. */
/*: DE  6/10/92 16:52 Chixified */
/*: JX  9/24/92 10:31 Only alloc/dealloc "chix" once! */
/*: CR  8/13/92 14:29 Remove double ";;". */
/*: CP  3/04/93 23:12 Fixed nested comments. */
/*: CK  3/18/93 17:07 Succeed on SUBACK (subscription acknowledgement). */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "unitcode.h"
#include "chixuse.h"
#include "xaction.h"
#include "null.h"
#include "done.h"

#define BUG bug ("PARSE_XACTION: couldn't parse it.", 0); printxact (xactptr);

extern union null_t null;

FUNCTION parse_xaction (xactptr)
XACTIONPTR xactptr;
{
   Chix locname, tname, orighost, mychn, chn, junk, textform;
   int  error, ok, success;
   Chix chix, ctext, ctitle;

   ENTRY ("parse_xaction", "");

   locname   = chxalloc (80L,  THIN, "parsexac locname");
   tname     = chxalloc (80L,  THIN, "parsexac tname");
   orighost  = chxalloc (80L,  THIN, "parsexac orighost");
   mychn     = chxalloc (80L,  THIN, "parsexac mychn");
   chn       = chxalloc (80L,  THIN, "parsexac chn");
   junk      = chxalloc (80L,  THIN, "parsexac junk");
   textform  = chxalloc (256L, THIN, "parsexac textform");
   chix      = chxalloc (100L, THIN, "parsexac chix");
   ctext     = chxalloc (100L, THIN, "parsexac ctext");
   ctitle    = chxalloc (80L,  THIN, "parsexac ctitle");
   success   = 0;

   if (xactptr->transit) {
      get_trueconf ((int)xactptr->conflocnum, locname, tname, orighost);
      get_chn (mychn, XSNF, 0);
      if (NOT chxeq (mychn, orighost))  SUCCEED;
   }    /* This host need only forward it on, which is done by forw_xact */

   switch (xactptr->typenum) {
   case TT_ITEM:
      if (NOT item_parse (xactptr)) {
         tellmgr (xactptr, 0);
         FAIL; }
      break;

   case TT_RESPONSE:
      if (NOT resp_parse (xactptr)) {
         tellmgr (xactptr, 0);
         FAIL; }
      break;

   case TT_MESSAGE:
      if (NOT mess_parse (xactptr)) {
         tellmgr (xactptr, 0);
         FAIL; }
      break;

   case TT_PERSON:
      if (NOT pers_parse (xactptr)) {
         tellmgr (xactptr, 0);
         FAIL; }
      break;

   case TT_OSUBJECT:
      get_trueconf ( (int) xactptr->conflocnum, locname, tname, orighost);
      switch (xactptr->actnum) {
      case XT_NEW:
         chg_dsubject (XT_NEW, (int) xactptr->conflocnum, locname, null.chx, 
                        xactptr->title, null.chx, &error);
         switch (error) {
            case DNOCONF:
            case DINOITEM: reparse (xactptr);
            case DOEXIST:
            case NOERR:       SUCCEED;
            default:          FAIL;
         }

      case XT_CHANGE:
      case XT_DELETE:
         if (xactptr->actnum==XT_CHANGE)
            chxcpy (chix, xactptr->data[0]);
         chg_dsubject (xactptr->actnum, (int) xactptr->conflocnum, locname, 
                       null.chx, xactptr->title, chix, &error);
         switch (error) {
            case DNOCONF:
            case DINOITEM: reparse (xactptr);
            case DOEXIST:
            case NOERR:       SUCCEED;
            default:          FAIL;
         }
         break;

      default:
         BUG; FAIL;
      } break;

   case TT_CONFERENCE:
      if (NOT conf_parse (xactptr)) {
         tellmgr (xactptr, 0);
         FAIL;
      } break;

   case TT_SUBREQ:
      mdstr (textform,  "net_Fsubreq", null.md);
      chxbreak (xactptr->uid, junk, chn, CQ("@")); chxupper (chn);
      chxcpy (tname, xactptr->conftname);      chxupper (tname);
      chxclear  (ctext);
      chxformat (ctext, textform, 0L, 0L, chn, tname);
      mdstr (ctitle, "net_Tsubreqt", null.md);
      ok = subreq_parse (xactptr);              /* Notify local mgr. */
      if (ok) {
         mdstr  (chix, "net_Tsubreqsucc", null.md);
         chxcat (ctext, chix);
         sendmgr (ctext, ctitle);
      } else {
         mdstr  (chix, "net_Tsubreqfail", null.md);
         chxcat (ctext, chix);
         sendmgr (ctext, ctitle);
      }
      DONE (ok);
      break;

   case TT_SUBACK:
      req_report (xactptr);
      SUCCEED;

   case TT_SUBNAK:
      req_report (xactptr);
      FAIL;

   case TT_HOST:
      if (NOT host_parse (xactptr)) {
         tellmgr (xactptr, 0);
         FAIL;
      } break;

   default:
      BUG; FAIL;
   }

done:
   chxfree (locname);
   chxfree (tname);
   chxfree (orighost);
   chxfree (mychn);
   chxfree (chn);
   chxfree (junk);
   chxfree (textform);
   chxfree (chix);
   chxfree (ctext);
   chxfree (ctitle);

   RETURN (success);
}
