/*** SUBREQ_PARSE.  Parse a subscription request xaction and act accordingly.
/
/   int n = subreq_parse (xptr)
/
/   Parameters:
/     XACTIONPTR xptr;
/
/   Purpose:
/      After the XFU has decided that an incoming xaction is a subscription
/      request, subreq_parse() further parses it and responds accordingly.
/      First it validates that we have that conf here, and that the remote
/      host is allowed to subscribe to it.  If all is well with the world,
/      it will call send_conf() to actually send the conf to the subscriber.
/ 
/      If something is wrong, a message is sent back to the mgr on the
/      remote host indicating that this is not allowed, and why.
/
/   How it works:
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions: mess_parse() item_parse()
/
/   Called by: parse_xaction()
/
/   Home: xfu/resppars.c
/ */

/*: JV 10/29/91 15:46 Create this. */
/*: JV 11/05/91 15:28 Remove mgrid from uid. */
/*: CR 11/22/91 15:16 put spaces around "ok=-1". */
/*: JV 12/07/91 20:45 Deallocate chixes. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR 12/21/91 14:55 Add SUCCEED, FAIL; always free carray! */
/*: CR 12/26/91 13:09 Use done.h. */
/*: JV  1/03/92 10:41 Also copy over confochn on SUBNAK, declare strcpy(). */
/*: JV  1/23/92 14:55 Send SUBACK after sending conf. */
/*: DE  6/11/92 14:03 Chixified */
/*: JX  8/09/92 16:46 Fixify hnumber arg. */
/*: JX  9/01/92 13:49 Change empty_xact to make_xaction(). */
/*: JX 10/09/92 18:04 Fix pointer. */
/*: CP  3/04/93 23:16 Remove nested comment. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "chixuse.h"
#include "null.h"
#include "xaction.h"
#include "unitcode.h"
#include "done.h"

#define BUG printxact (xptr)

extern union null_t null;

FUNCTION  subreq_parse (xptr)
XACTIONPTR xptr;
{
   Chix date, time;
   Chix mgrid, bugmsg, chn, str;
   char chxname[20];
   int  ok = -1, i, lhnum, success;
   long systime();
   Chix carray[TEXTFIELDS], chxalloc();
   Xactionptr xaction, make_xaction();

   ENTRY ("subreq_parse", "");

   chn      = chxalloc (20L,  THIN, "subreqpa chn");
   str      = chxalloc (20L,  THIN, "subreqpa str");
   date     = chxalloc (30L,  THIN, "subreqpa date");
   time     = chxalloc (30L,  THIN, "subreqpa time");
   mgrid    = chxalloc (80L,  THIN, "subreqpa mgrid");
   bugmsg   = chxalloc (200L, THIN, "subreqpa bugmsg");

   xaction = make_xaction();

   for (i=0; i<TEXTFIELDS; i++) {
      sprintf (chxname, "subscr_conf %d", i);
      carray[i] = chxalloc (0L, THIN, chxname);
   }

   chxbreak (xptr->uid, mgrid, chn, CQ("@"));
   lhnum = hnumber (chn);
   if (xptr->conflocnum > 0) {
      ok = ok_to_send_conf ((int) xptr->conflocnum, chn);
      if (ok == 1) {
         unit_lock (XCNB, WRITE,  xptr->conflocnum, 0L, 0L, 0L, null.chx);
         unit_append (XCNB);
         chxformat (str, CQ("%d\n"), L(lhnum), 0L, null.chx, null.chx);
         unit_write  (XCNB, str);
         unit_close  (XCNB);      unit_unlk (XCNB);
         if (send_conf ((int) xptr->conflocnum, chn)) {
            xaction->typenum = TT_SUBACK;
            chxcpy (xaction->conftname, xptr->conftname);
            chxcpy (xaction->confochn,  xptr->confochn);
            sysdaytime (date, time, 0, systime());
            chxclear  (xaction->date);
            chxformat (xaction->date, CQ("%s %s"), 0L, 0L, date, time);

            unit_lock (XHML, WRITE, L(lhnum), 0L, 0L, 0L, null.chx);
            unit_append (XHML);
            putnextxact (XHML, xaction, carray);
            unit_close (XHML);  unit_unlk (XHML);
            SUCCEED;
         }
         else {
            bug ("SUBREQ_PARSE: sendconf failed:", 0);
            chxformat (bugmsg, CQ("CONF=%s, CHN=%s."), 0L, 0L, xptr->conftname,
                       chn);
            bug (ascquick(bugmsg), 0);
         }
      }
   }

   /*** At this point either xptr->conflocnum was bad,
   /    or ok_to_send_conf failed, or send_conf failed.
   /    We need to send a msg to the subscribing manager. */
   xaction->typenum = TT_SUBNAK;
   chxcpy (xaction->conftname, xptr->conftname);
   chxcpy (xaction->confochn,  xptr->confochn);
   sysdaytime (date, time, 0, systime());
   chxclear  (xaction->date);
   chxformat (xaction->date, CQ("%s %s"), 0L, 0L, date, time);
   if (ok == -1) chxcpy  (xaction->title, CQ("NOCONF"));
   else          chxcpy  (xaction->title, CQ("NOPERM"));

   unit_lock (XHML, WRITE, L(lhnum), 0L, 0L, 0L, null.chx);
   unit_append (XHML);
   putnextxact (XHML, xaction, carray);
   unit_close (XHML);  unit_unlk (XHML);

   chxformat (bugmsg,
       CQ("SUBREQ_PARSE: got a subscription request from: %s\nfor conf: %s."),
       0L, 0L, xptr->uid, xptr->conftname);
   bug (ascquick(bugmsg), 0);
   FAIL;

done:
   for (i=0; i<TEXTFIELDS; i++) chxfree (carray[i]);
   chxfree (chn);
   chxfree (str);
   chxfree (date);
   chxfree (time);
   chxfree (mgrid);
   chxfree (bugmsg);
   free_xaction (xaction);

   RETURN (success);
}   
