/*** XFU. (Main)  This is the MAIN routine for the Xaction File Unpackager.
/
/   The duty of the XFU is to parse all of the xport files in the
/   ParseBay (unit XSPB).  Each file has a xport header, which is followed
/   by either conf or mail xactions.  Xactions are terminated by
/   an equals sign (=) in the first column.
/
/   A short description of what the XFU does:
/   1) Ensure enough disk space or complain and stop.
/   2) For each xport file in the ParseBay:
/      1) If it's e-mail for a user on this host, deliver it.
/      2) If it's e-mail for a user on another host, use the routing
/         table to figure out which way to send it, and send it.
/      3) If it's shared conf stuff: ***
/         1) If it's "in transit":
/            1) If this is the original host, digest and disperse
/               the xaction.
/            2) Otherwise send it on "towards" the original host.
/         2) If it's not in transit add it to the local db.  If necessary,
/            put it in the sender's XHXA so the xfp will disperse it.
/   3) Process the ReParseBay (XSRP) file.
/
/ ***: A more detailed description of how this part of the code works
/      might be very helpful, so here it is:
/
/   Every xaction gets parsed by parse_xaction().  Unfortunately, not
/   all xactions from each xport file should go on to the next host,
/   so we need to append each xaction to XHXA individually (if at all).
/
/   Xactions that do go in XHXA:
/      "Normal" conf additions
/      Conf changes being dispersed from the original host.
/   
/   Xactions that do not go in XHXA:
/      "In transit" xactions (they go in XHML toward original host).
/      Conf changes being dispersed from the original host.
/         These go in XCXT for the correct conf.  (See forwxact.c)
/      Mail, which also goes in XHML.
/      People changes (I'm not sure where they go-JV:5-AUG-91).
/
/   If any of the xactions get appended to XHXA, XHXA must start with
/   the header that prepended the xport file these xactions came from.
/   This enables the xfp to efficiently grab that file and disperse it.
/   (Of course, since prepending to a file is stupidly difficult, we
/    write the header first and then destroy it if we don't need it.)
/
/   All of the work of adding conf stuff to the local db, as well as
/   delivering mail on this host, is done by a library of db routines.
/   The XFU and its direct support routines take care of getting info
/   into and out of xport files and also take care of routing.  The XFU
/   does a certain amount of logic checking, but security is not one of
/   its duties.  As examples, the XFU replies to senders of undeliverable
/   mail, but doesn't check the validity of userlist changes.
/
/   WARNING: currently (7/26/91) the XFU doesn't handle seq#'s that
/            have "wrapped around".  This must be fixed soon!!
/ */

/*: JV  5/20/91 18:10 Begin XFU work. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/04/91 17:58 Remove lint. */
/*: CR  7/16/91 22:05 Jeff made unknown changes. */
/*: JV  7/26/91 16:32 Fix sequence # handling & various bugs.. */
/*: CR  7/24/91 22:15 Make Lint happier with return. */
/*: JV  7/29/91 08:21 Fix locking of XSNF. */
/*: JV  7/30/91 16:07 Remove 3rd arg of getnextxact, seqnum fix */
/*: JV  8/05/91 08:05 Add code to pass xact's to neighbors. */
/*: JV  8/09/91 12:54 Add 2nd arg to set_neighbor */
/*: JV  8/13/91 09:35 Add debug.h stuff. */
/*: JV  8/29/91 15:30 Cast seqnum_thru to int for now, fix later. */
/*: JV  9/10/91 15:06 Add comment. */
/*: JV  9/12/91 12:57 Add more comments, notate proc_reparse. */
/*: JV  9/19/91 08:55 Delete XSPB file when done, use seqnum macro ops. */
/*: CR  9/18/91 16:03 Requires all the "...globals()" functions. */
/*: CR  9/18/91 16:21 Move debug setting *after* globals calls. */
/*: JV 10/11/91 18:00 Remove some debuf stuff, clean. */
/*: JV 10/14/91 14:20 Fix XSRP seqnum updating. */
/*: JV 10/17/91 10:26 Add audit trail global variable. */
/*: JV 10/22/91 15:10 Move previous to xfpgloba.c. */
/*: JV 11/22/91 12:17 Add xfu_protect and xfu_unprotect. */
/*: JV 12/07/91 20:51 Deallocate text fields of xactions. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JV 12/20/91 15:22 Add audit_xaction. */
/*: JV  1/22/92 15:42 Log which file we're parsing. */
/*: DE  6/11/92 15:44 ReChixified */
/*: JX  7/24/92 15:55 Use make_hinfo(). */
/*: JX  8/09/92 16:46 Chixify hnumber arg. */
/*: JX  9/01/92 14:18 Allocate xaction, confid, net_flag.optfile. */
/*: CR 10/09/92 17:37 Remove unused 'i'. */
/*: JV 10/12/92 15:06 Call chx_init(). */
/*: CR 10/15/92 18:02 Use #include text.h. */
/*: CP  3/04/93 21:54 Fix apparent nested comments.  Make debug extern. */
/*: CK  3/19/93 15:36 Save & restore terminal state. */
/*: J8  8/02/93 10:33 Change arg to net_argchk(). */

#include <stdio.h>
#include "caucus.h"
#include "goodlint.h"
#include "unitcode.h"
#include "xaction.h"
#include "xfp.h"
#include "text.h"
#include "sequence.h"
#include "audit.h"

extern  struct termstate_t  term_original, term_in_caucus;
extern  struct net_flag_t   net_flag;
extern  int                 debug;
extern  Chix                confid;
Userreg                     thisuser;

FUNCTION main (argc, argv)
int argc; char *argv[];
{
   char     buf[16], *strito36();
   int      fxacts=0, new_forward, neighnum, error, success=1;
   long     seqnum, seqnum_thru;
   HOSTPTR  host, neighbor, make_host();
   Hinfoptr make_hinfo();
   XPORTHDR *xheader, *make_xporthdr();
   Xactionptr  xaction, make_xaction();

   if (argc < 2) {
      printf ("Usage: xfu_x confid [-d [#]]-y net.opt\n");
      exit(0);
   }

   /* Set up data about this host and for neighbors. */
   host     = make_host();  host->info =     make_hinfo();
   neighbor = make_host();  neighbor->info = make_hinfo();
   xaction  = make_xaction();

   confid = chxalloc (40L, THIN, "xfu confid");
   chxcpy (confid, CQ(argv[1]));

   net_flag.optfile = chxalloc (40L, THIN, "xfp netflag");
   net_argchk (&argc, argv, 2);

   if (argc > 2) {
      printf ("Usage: xfu_x confid [-d [#]]-y net.opt\n");
      exit(0);
   }

   systurn (&term_original,  GET_TERM, 0);
   systurn (&term_in_caucus, GET_TERM, 0);
   unitglobals();
   utilglobals();

   chx_init (JIX);
   
   xfpglobals();
   mainglobals();
   sysglobals();
   utilglobals();
   net_optchk ();

   if (NOT xfu_protect()) exit (0);

   xheader = make_xporthdr ();

   if (NOT unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, nullchix)) {
      bug ("XFU: can't lock XSNF for read.", 0);
      FAIL;
   }
   if (NOT get_neighbor (0, host, XSNF)) {
      bug ("XFU: can't get XSNF info.", 0); unit_unlk (XSNF);
      FAIL;
   }
   unit_unlk (XSNF);
   seqnum = host->info->seqnum_from;

   while (1) {
      if (NOT unit_lock (XSPB, READ, seqnum, 0L, 0L, 0L, nullchix)) {
         bug ("XFU: Can't lock XSPB.", 0);
         FAIL;
      }
      if (NOT unit_check (XSPB)) {
         unit_unlk (XSPB); break; }

      strito36 (buf, seqnum, 8);
      printf ("Unpacking file: x%7s.\n", buf);
      unit_view (XSPB);
      get_xpheader (XSPB, xheader);

      neighnum = hnumber (xheader->origchn);
      unit_lock (XHI1, READ, L(neighnum), 0L, 0L, 0L, nullchix);
      get_neighbor (neighnum, neighbor, XHI1);
      seqnum_thru = neighbor->info->seqnum_thru_hi;
      unit_unlk (XHI1);

      unit_lock (XHXA, WRITE, L(neighbor->lhnumber), seqnum_thru, 0L, 0L,
                 nullchix);
      unit_make (XHXA);
      set_xpheader (XHXA, xheader);

      while (getnextxact (XSPB, xaction)) {
         audit_xaction (xaction, neighbor->lhnumber, AUDIT_IN);
         parse_xaction (xaction);
         new_forward = forw_xaction  (xheader, xaction, &error);
         if (new_forward) fxacts++;
      }
      if (fxacts) {
         unit_close (XHXA); unit_unlk  (XHXA);
         seqnum_thru = INC(seqnum_thru, SEQ_THRU_MOD);
         fxacts=0;
      }
      else {
         unit_close (XHXA);  unit_kill (XHXA);  unit_unlk (XHXA);
      }

      /* Reset neighbor */
      unit_lock (XHI1, WRITE, L(neighnum), 0L, 0L, 0L, nullchix);
      get_neighbor (neighnum, neighbor, XHI1);
      neighbor->info->seqnum_thru_hi = seqnum_thru;
      set_neighbor (neighbor, XHI1);
      unit_unlk (XHI1);

      unit_close (XSPB);
      unit_kill  (XSPB);
      unit_unlk  (XSPB);
      seqnum = INC(seqnum, SEQ_FROM_MOD);        /* Are there any more? */
   }
   if (NOT unit_lock (XSNF, WRITE, 0L, 0L, 0L, 0L, nullchix)) {
     bug ("XFU: can't lock XSNF for write.", 0);
     FAIL;
   }
   if (NOT get_neighbor (0, host, XSNF)) {
      bug ("XFU: can't get XSNF info for writing.", 0);
      unit_unlk (XSNF);
      FAIL;
    }

   host->info->seqnum_from = seqnum;
   set_neighbor (host, XHI1);      /* Reset the seqnum for next time. */
   unit_unlk (XSNF);   /* ^ Should this be XSNF? I don't think it matters...*/

   for (seqnum = host->info->seqnum_thru_low;
        LT (seqnum, host->info->seqnum_thru_hi, SEQ_THRU_MOD);
        seqnum = INC(seqnum, SEQ_THRU_MOD))
      proc_reparse (seqnum);

   /*** Note that any xactions which we were reparsing that are still
   /    unparseable got stored for later parsing by proc_reparse.  This allows
   /    proc_reparse to toss all of the xport files which we reparsed above.  */
   if (NOT unit_lock (XSNF, WRITE, 0L, 0L, 0L, 0L, nullchix)) {
     bug ("XFU: can't lock XSNF for write.", 0);
     FAIL;
   }
   if (NOT get_neighbor (0, host, XSNF)) {
      bug ("XFU: can't get XSNF info for writing.", 0);
      unit_unlk (XSNF);
      FAIL;
   }

   host->info->seqnum_thru_low = seqnum;
/* host->info->seqnum_thru_low = host->info->seqnum_thru_hi = seqnum; B4 10/91 */

   set_neighbor (host, XHI1);           /* Reset the seqnum for next time.  */
   unit_unlk (XSNF);

done:
   free_xporthdr (xheader);
   xfu_unprotect ();
   free_host    (host);
   free_host    (neighbor);
   free_xaction (xaction);
   systurn (&term_original, SET_TERM, 0);

   exit   (success);
   return (success);  /* Make Lint happy. */
}
