
/*** LICE2.    New Caucus licensing program.
/
/    Invoked as:
/       lice2   caucus  password  ship  code  options
/
/    Arguments:
/       caucus     name of caucus_x file.
/       password   licensing password as rendered by licepass.
/       ship       first part of serial code, e.g. "TY01".
/       code       numeric part of serial code, e.g. "023".
/       options    licensing options.
/
/    Purpose:
/       Lice2 is the one-stop licenser for all Caucus licenses.  It
/       contains no user interface code, and is meant to be run from
/       a shell script that asks all of the questions about how Caucus
/       should be licensed for a particular customer.
/
/       Because Lice2 *requires* a password, it can double as a
/       re-licenser that can be shipped with Caucus to a customer site.
/
/    How it works:
/       Any or all of the following "options" may be specified.  The
/       "-s" option may be specified more than once to allow multiple
/       sysids.
/
/          -e days         license expires in this many days from now.
/          -s id           license runs on this system id.
/          -u users        specify maximum number of simultaneous users
/          -a address      specify address of license block
/          -f mask         specify function mask value as integer
/          -v pfile        name of file for "patch" command script
/          -c confmax      max number of conferences
/          -i itemmax      max number of items
/          -r respmax      max number of responses
/
/       Vax/VMS requires the -a and -v options, as it must write
/       a "PATCH" command script to actually modify caucus_x.
/       For Vax, the "-a address" argument must be hexadecimal.
/
/       Unix versions of Lice2 can scan caucus_x for the license block
/       locator and thus do not need -a.  If it is supplied, the address
/       argument must be decimal.
/
/       The meaning of the bits in "mask" are as follows:
/          1     On => Allow CMI
/          2     On => Allow CaucusLink
/          4     On => Display Caucus copyright notice
/
/       The values of "days", "id", "users", "confmax", "itemmax", "respmax",
/       and "mask" are stored in an encrypted form in the license block
/       in caucus_x.  "Address" is never stored.
/
/       When Lice2 is finished, it writes the licensing information to stdout.
/       The first line contains the max users, expiration date, and serial code.
/       Subsequent lines contain the system id(s).
/
/    Exit status:
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xlice2/lice2.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved.  */

/*: CR  1/14/93 12:03 New program. */
/*: CR  1/26/93 16:15 Add encryption stuff (PLAIN, strplain). */
/*: CP  4/01/93 11:43 Add Vax/VMS support. */
/*: CV  6/15/93 17:31 Map " into + for Vax. */
/*: CK 10/13/93 17:33 Add confmax, itemmax, respmax. */
/*: CR 10/17/93 17:56 Rename search() -> searchfile(). */
/*: JZ 10/18/93 11:00 lseek arg3 is int, not long; declare strcpy(). */

#include <stdio.h>
#include "lice2.h"
#include "license.h"
#include "systype.h"

#define MAXSYSID 16

#if UNIX | PR | PX
#define  OPEN(x)   open (x, 2)
#endif

#if IM | PN
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#define  OPEN(x)   open (x, O_BINARY | O_RDWR)
#endif

#if AOS | CA
#include <fcntl.h>
#define OPEN(x)  open (x, O_RDWR)
#endif

#if CVM
#include <file.h>
#define  OPEN(x)   open (x, O_RDWR)
#endif

static struct license_t license =
   {0x11, 0x22, 0x33, 0x44, 0xAA, 0xBB, 0xCC, 0xDD};

FUNCTION  main (argc, argv)
   int   argc;
   char *argv[];
{
   FILE *pf;
   char  caucus_x[200], result[40], result2[40], pfile[200];
   long  address;
   long  mask, expires_in, users, confmax, itemmax, respmax;
   long  searchfile(), systime();
   int   fd, i, j;
   char  str[40];
   char *strplain(), *strcpy();

#if UNIX | PX | PR | IM | PN | AOS | CA | VV
   if (argc < 5) {
      printf ("Usage: lice2 caucus_x password ship code options\n");
      exit   (1);
   }
   strcpy (caucus_x, argv[1]);
#endif

/*=====Section 1: check password. */
   /*** Start with "abcdefg".  Permute by SHIP and CODE. */
   strcpy      (result, "abcdefg");
   lstrpermute (result, argv[3]);
   lstrpermute (result, argv[4]);

   /*** Permute by expiration, mask, and users. */
   expires_in = mask = users = 0;
   for (i=5;   i<argc;   ++i) {
      if (streq (argv[i], "-e"))  sscanf (argv[++i], "%ld", &expires_in);
      if (streq (argv[i], "-f"))  sscanf (argv[++i], "%ld", &mask);
      if (streq (argv[i], "-u"))  sscanf (argv[++i], "%ld", &users);
      if (streq (argv[i], "-c"))  sscanf (argv[++i], "%ld", &confmax);
      if (streq (argv[i], "-i"))  sscanf (argv[++i], "%ld", &itemmax);
      if (streq (argv[i], "-r"))  sscanf (argv[++i], "%ld", &respmax);
   }
   lnumpermute (result, expires_in / 30);
   lnumpermute (result, expires_in % 30);
   lnumpermute (result, mask);
   lnumpermute (result, users);
   lnumpermute (result, confmax);
   lnumpermute (result, itemmax);
   lnumpermute (result, respmax);

   /*** Permute by each sysid. */
   for (i=5;   i<argc;   ++i) {
      if (streq (argv[i], "-s"))   lstrpermute (result, argv[i+1]);
   }

   /*** On some systems, (e.g., Vax/VMS), a " cannot be used in the
   /    command line, so we map them into + and try that result as well. */
   strcpy (result2, result);
   for (i=0;   result2[i];  ++i)  if (result2[i]=='"')  result2[i] = '+';

   if (NOT streq (result, argv[2])  &&  NOT streq (result2, argv[2])) {
      printf ("Incorrect password: '%s'.\n", argv[2]);
/*    printf ("Correct password is '%s'.\n", result); */
      exit   (1);
   }


/*=====Section 2: open caucus_x, and find license block. */
#if UNIX | PX | PR | IM | PN
   if ( (fd = OPEN (caucus_x)) < 0) {
      printf ("Can't open file '%s'\n", caucus_x);
      exit   (1);
   }
#endif

   address = -1L;
   for (i=3;   i<argc;   ++i)
      if (streq (argv[i], "-a"))
#if UNIX
         sscanf (argv[++i], "%ld", &address);
#endif
#if VV
         sscanf (argv[++i], "%lx", &address);
#endif

#if UNIX | PX | PR | IM | PN
   if (address < 0)  address = searchfile (fd, 0L);
#endif

   if (address < 0)  {
      printf ("Can't find locator.\n");
      exit   (1);
   }


/*=====Section 3: put data into license block. =======*/
   sprintf (license.serial, "%s%s", argv[3], argv[4]);
   strcpy  (license.serial, strplain(license.serial));
   license.maxusers = PLAIN(users);
   if (expires_in == 0)  license.expires = 0L;
   else                  license.expires = systime() + expires_in * 24L * 3600L;
   license.expires       = PLAIN(license.expires);
   license.function_mask = PLAIN(mask);
   license.confmax       = PLAIN(confmax);
   license.itemmax       = PLAIN(itemmax);
   license.respmax       = PLAIN(respmax);

   /*** For each sysid on the command line, add it to license. */
   for (j=0, i=5;   i<argc;   ++i) {
      if (streq (argv[i], "-s")) {
         strcpy    (str, argv[++i]);
         strlower  (str);
         sysidform (str);
         strcpy (license.sysid[j++], strplain(str));
      }
   }
   strcpy (license.sysid[j], "");



/*=====Section 4: write license block to caucus_x===== */
#if UNIX | PX | PR | IM | PN
   lseek (fd, address, 0);
   write (fd, (char *) &license, sizeof(license));
   close  (fd);
#endif

#if VV
   /*** Get the name of the PATCH script file, and open it. */
   pfile[0] = '\0';
   for (i=3;   i<argc;   ++i)
      if (streq (argv[i], "-v"))  strcpy (pfile, argv[i+1]);
   if (NULLSTR(pfile)) {
      printf ("No -v patch file.\n");
      exit   (1);
   }
   if ( (pf = fopen (pfile, "w")) == NULL) {
      printf ("Cannot write to file '%s'.\n", pfile);
      exit   (1);
   }

   /*** Now write the PATCH instructions for each field in the license
   /    structure. */
   fprintf (pf, "%s\n", caucus_x);
   fprintf (pf, "set module/all\n");
   fprintf (pf, "deposit 0%lx=%09lx\n", address+8,  license.expires);
   fprintf (pf, "deposit 0%lx=%09lx\n", address+12, license.maxusers);
   fprintf (pf, "deposit 0%lx=%09lx\n", address+16, license.function_mask);
   put16   (pf, address+20, license.serial);
   address += 36;
   for (i=0;   i<16;   ++i) {
      put16 (pf, address, license.sysid[i]);
      address += 16;
      if (NULLSTR (license.sysid[i]))  break;
   }
   fprintf (pf, "update\n");
   fprintf (pf, "exit\n");
   fclose  (pf);
#endif

/*=====Section 5: report license information. ======== */
   if (PLAIN(license.expires) != 0L)  sysdate (str, PLAIN(license.expires));
   else                               strcpy  (str, " (never)");
   printf ("%4ld %9s %s\n", PLAIN(license.maxusers), str,
                         strplain(license.serial));
   printf ("%5ld %5ld %5ld\n", confmax, itemmax, respmax);
   for (i=0;   NOT NULLSTR(license.sysid[i]);   ++i)
      printf ("%s\n", strplain(license.sysid[i]));
}


FUNCTION long searchfile (fd, start)
   int   fd;
   long  start;
{
   char  buf[520];
   long  addr;
   short i, j;

   for (addr=start;   lseek (fd, addr, 0), read (fd, buf, 520) == 520;
        addr += 512) {

      for (i=0;   i<512;   ++i) {
         for (j=0;   j<8;   ++j)   if (license.locator[j] != buf[i+j])   break;
         if  (j >= 8)  return (addr + i);
      }
   }

   return (-1L);
}

FUNCTION put16 (fp, base, str)
   FILE *fp;
   long  base;
   char  str[];
{
   union {char c[4];   long l; }  u;
   int    i;

   for (i=0;   i<16;   i+=4) {
      u.c[0] = str[i];
      u.c[1] = str[i+1];
      u.c[2] = str[i+2];
      u.c[3] = str[i+3];
      fprintf (fp, "deposit 0%lx=%09lx\n", base+i, u.l);
   }
}
