/*** LICEPASS.   Compute licensing password.
/
/    Invoked as:
/       licepass   ship  code  options
/
/    Arguments:
/       ship       first part of serial code, e.g. "TY01".
/       code       numeric part of serial code, e.g. "023".
/       options    licensing options.
/
/    Purpose:
/       LICEPASS computes the licensing password needed by program
/       LICE2.  It contains no user interface code, and is meant to 
/       be run from a shell script that asks all of the questions 
/       about how Caucus should be licensed for a particular customer.
/
/    How it works:
/       Any or all of the following "options" may be specified.  The
/       "-s" option may be specified more than once to allow multiple
/       sysids.
/
/          -e days         license expires in this many days from now.
/          -s id           license runs on this system id.
/          -f mask         specify function mask value as integer
/          -u users        specify maximum number of simultaneous users
/          -c confmax      max number of conferences
/          -i itemmax      max number of items
/          -r respmax      max number of responses
/
/       Licepass writes the computed password on stdout.
/
/    Exit status:
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xlice2/licepass.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: CR  1/24/93 12:03 1st carvmark */
/*: CK 10/13/93 17:33 Add confmax, itemmax, respmax. */
/*: JZ 10/18/93 11:00 Declare strcpy() */

#include <stdio.h>
#include "lice2.h"

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   char   result[80], *strcpy();
   long   expires_in, mask, users, confmax, itemmax, respmax;
   int    i;

   if (argc < 4) {
      fprintf (stderr,
         "Usage: licepass ship code [-e days] [-s id] [-f mask] [-u users]\n");
      exit (1);
   }

   /*** Start with "abcdefg".  Permute by SHIP and CODE. */
   strcpy      (result, "abcdefg");
   lstrpermute (result, argv[1]);
   lstrpermute (result, argv[2]);

   /*** Permute by expiration, mask, and users. */
   expires_in = mask = users = 0;
   for (i=3;   i<argc;   ++i) {
      if (streq (argv[i], "-e"))  sscanf (argv[++i], "%ld", &expires_in);
      if (streq (argv[i], "-f"))  sscanf (argv[++i], "%ld", &mask);
      if (streq (argv[i], "-u"))  sscanf (argv[++i], "%ld", &users);
      if (streq (argv[i], "-c"))  sscanf (argv[++i], "%ld", &confmax);
      if (streq (argv[i], "-i"))  sscanf (argv[++i], "%ld", &itemmax);
      if (streq (argv[i], "-r"))  sscanf (argv[++i], "%ld", &respmax);
   }
   lnumpermute (result, expires_in / 30);
   lnumpermute (result, expires_in % 30);
   lnumpermute (result, mask);
   lnumpermute (result, users);
   lnumpermute (result, confmax);
   lnumpermute (result, itemmax);
   lnumpermute (result, respmax);

   /*** Permute by each sysid. */
   for (i=3;   i<argc;   ++i)
      if (streq (argv[i], "-s"))
         lstrpermute (result, argv[++i]);

   printf ("%s\n", result);
}
