/*** BUILD_TRANS_LIST.   Build linked list of received transport chunks.
/
/    list = build_trans_list (shipmeth, error);
/
/    Parameters:
/       Trans     list;        (returned list of transport chunks)
/       int       shipmeth;    (shipping method)
/       int      *error;       (returned error code)
/
/    Purpose:
/       Build_trans_list() scans the all of the files in a particular 
/       shipping method's RECVnnn/CHUNK bay.  It builds a linked list
/       of the transport files the chunks represent, including which
/       of the chunk files for each transport file have been received.
/
/    How it works:
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions
/       NOERR      success.
/  
/    Side effects:
/
/    Related functions:  free_trans()
/
/    Called by: 
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/buildtra.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/14/91 18:33 New function. */
/*: CR  9/18/91 14:33 Allocate space for pdir, & free when done.*/
/*: CR  9/19/91 13:07 Cast arg to sysfree. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JV 12/16/91 16:53 Change pdir.lnum to long.*/
/*: JV  1/03/92 15:24 Declare strcpy(). */
/*: JX  7/29/92 16:09 Chixify. */
/*: JX  9/17/92 09:39 Initialize LAST. */
/*: CR 12/14/92 16:00 Sysmem arg is long. */

#include <stdio.h>
#include "entryex.h"
#include "common.h"
#include "handicap.h"
#include "unitcode.h"
#include "derr.h"
#include "recv.h"

FUNCTION Trans build_trans_list (shipmeth, error)
   int   shipmeth;
   int  *error;
{
   struct pdir_t  pdir;
   Trans          head, p, make_trans(), find_trans();
   int            i, part, last=0, acktype, parse_err;
   Chix           host;
   long           xfile, seq, ackseq;
   long           systime();
   char          *sysmem();

   ENTRY ("build_trans_list", "");

   host = chxalloc (60L, THIN, "buildtr host");

   /*** Create an empty Trans list. */
   head = make_trans();

   /*** Get the array of all of the chunk files in the RECVrrr/CHUNK
   /    directory. */
   pdir.type  = -1;
   pdir.parts =  0;
   pdir.resp  = (short *) sysmem (sizeof(short) * L(PDIRMAX + 3));
   pdir.lnum  = (long  *) sysmem (sizeof(long)  * L(PDIRMAX + 3));
   syspdir (&pdir, XRXC, shipmeth, 0, nullchix);

   /*** Examine each chunk file... */
   for (i=0;   i<pdir.parts;   ++i) {

      /*** Read the chunk header of a chunk file. */
      xfile = pdir.lnum[i];
      parse_chunk (shipmeth, xfile, host, &seq, &part, &last, 
                             &acktype, &ackseq, &parse_err);

      /*** For now, report errors. */
      if (parse_err > 0) {
         recverror (parse_err, shipmeth, xfile);
         continue;
      }

      /*** This chunk references a transport file.  Is that file already in
      /    the Trans list? */
      p = find_trans (head, seq, host);

      /*** If not, allocate a new one, hook it into the HEAD list, and
      /    initialize it. */
      if (p == NULL) {
         p = make_trans();
         p->next = head->next;
         head->next = p;
         p->seq  = seq;
         chxcpy (p->chn, host);
         p->first_receipt = systime();
         p->acktype = acktype;
         p->ackseq  = ackseq;
      }

      /*** In either case, update this node to include this chunk file. 
      /    If this is a duplicate of a chunk we've already received,
      /    delete it. */
      expand_trans (p, part);
      if      (p->xnums[part] < 0)   p->xnums[part] = xfile;
      else if (unit_lock (XRXC, WRITE, L(shipmeth),  xfile, 0L, 0L, nullchix)){
         unit_kill (XRXC);
         unit_unlk (XRXC);
      }
      if (last)  p->gotlast = 1;
   }

   chxfree (host);
   sysfree ( (char *) pdir.resp);
   sysfree ( (char *) pdir.lnum);
   *error = NOERR;
   RETURN (head);
}
