/*** DO_ACKS.  Handle received ACK files.
/
/    do_acks (list);
/
/    Parameters:
/       Trans     list;      (list of transport files)
/
/    Purpose:
/       Process received ACK files.  Remove them from LIST.
/
/    How it works:
/       Scan through the LIST of received files, and examine the
/       ACK files.  Remove them from LIST as we go.
/
/       ACK's delete the relevant transport/header file in the RESHIP bay.
/       The shipper will eventually notice this fact, and increment its
/       table of ACK'd transport files.
/
/       NAK's rewrite the relevant transport header file in the RESHIP bay,
/       so that its original-shipment time appears to be now.  This ensures
/       that the transport file will remain around long enough to be
/       reshipped.
/
/    Returns: 1
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/doacks.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/20/91 15:51 New function. */
/*: JV  8/28/91 18:11 Check value from hnumber before using */
/*: CR  9/04/91 11:59 Fix linked list traversal. */
/*: CR 10/04/91 12:36 Change arg list, get proper shipmeth #. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR  1/06/92 15:33 Add shiplast arg to store_xheader() call. */
/*: JV  1/27/92 11:08 Log receipt of ACK. */
/*: JV  8/10/92 09:16 Fix minor problems in definitions. */
/*: CR  8/13/92 14:27 Remove double ";;". */
/*: JX  7/29/92 16:13 Chixify. */
/*: JX  9/15/92 15:38 Allocate HEAD. */
/*: CI 10/07/92 11:20 Chix Integration. */
/*: CP  2/09/94 13:45 Use ascquick() when preparing bug_msg. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "chixuse.h"
#include "xaction.h"
#include "recv.h"
#include "xfp.h"

FUNCTION  do_acks (list)
   Trans  list;
{
   Trans     p, q;
   int       hnum, shipmeth;
   XPORTHDR *head, *make_xporthdr();
   HOST     *host, *neighbor();
   long      shiporig, shiplast;
   long      systime();
   char      bug_msg[80], buf[12], *strito36();

   ENTRY ("do_acks", "");

   head = make_xporthdr();

   /*** Examine each entry in LIST, ignoring non-ACK entries.   Q is the
   /    node under consideration, P is its predecessor. */
   for (p=list;   p->next != NULL;  ) {
      q = p->next;
      if (q->acktype == NOTACK)  { p = q;   continue; }

      /*** Get the number of the host to whom we originally sent the
      /    ACK/NAK'd transport file.   If a bad host, complain, ignore
      /    the ACK, and keep going. */
      if ((hnum = host_number (q->chn)) < 0) {
         sprintf (bug_msg, "DO_ACKS: bad hnumber on host: %s.", 
                           ascquick(q->chn));
         bug (bug_msg, 0);
         q->acktype = NOTACK;
      }

      /*** Get the shipping method used to originally ship the file that
      /    is being ACK'd or NAK'd.   If bad host, ignore ACK or NAK
      /    and continue. */
      host = neighbor (hnum);
      shipmeth = host->info->shipmeth;
      if (EMPTYCHX(host->chn))  q->acktype = NOTACK;

      /*** ACK means delete the transport text and header files in the
      /    RESHIP (or CANTSHIP) bay. */
      if (q->acktype == ACK) {
         /* Report it. */
         strito36 (buf, q->ackseq, 5);
         printf ("Received ACK for SHIP%03d/RESHIP/J%03d%4s.\n",
                  shipmeth, hnum, buf);
         if (unit_lock (XPRT, WRITE, L(shipmeth), L(hnum),  q->ackseq, 0L,
                              nullchix)) {
            unit_kill  (XPRT);
            unit_unlk  (XPRT);
         }
         if (unit_lock (XPRH, WRITE, L(shipmeth), L(hnum),  q->ackseq, 0L,
                              nullchix)) {
            unit_kill  (XPRH);
            unit_unlk  (XPRH);
         }
         if (unit_lock (XPNT, WRITE, L(shipmeth), L(hnum),  q->ackseq, 0L,
                              nullchix)) {
            unit_kill  (XPNT);
            unit_unlk  (XPNT);
         }
         if (unit_lock (XPNH, WRITE, L(shipmeth), L(hnum),  q->ackseq, 0L,
                              nullchix)) {
            unit_kill  (XPNH);
            unit_unlk  (XPNH);
         }
      }

      /*** NAK means reset the ORIGSHIP time in the transport header in
      /    the RESHIP bay.  This ensures that the NAK'd file will stay
      /    around long enough to be reshipped.  (We may later be able
      /    to figure out how to get it back to the SHIP bay for quick
      /    shipment.)  */
      if (q->acktype == NAK) {
         if (load_xheader (shipmeth, hnum, q->ackseq, 1, head,
                                 &shiporig, &shiplast)) {
            shiporig = systime();
            store_xheader (shipmeth, hnum, q->ackseq, head, shiporig,
                                                             shiplast);
         }
      }

      /*** Remove node Q from the linked list. */
      p->next = q->next;
      q->next = NULL;
      free_trans (q);
   }

   RETURN (1);
}
