/*** DO_PARTIALS.  Handle partially-received transport files.
/
/    do_partials (list, shipmeth);
/
/    Parameters:
/       Trans     list;      (list of transport files)
/       int       shipmeth;  (shipping method)
/
/    Purpose:
/       Scan a list of of received chunks of transport files.  Remove
/       entries that are only partially received.
/
/    How it works:
/       Do_partials() scans a list of chunks of transport files,
/       as created by build_trans_list(), and looks for sets of chunks
/       that do NOT form a complete transport file.
/
/       If a set of chunks has been around for time > KEEPMAX, the entire set
/       of chunk files is deleted.  Otherwise, the set is kept, in the hope
/       that the missing chunks will be received later.
/
/       In either case, these sets are removed from LIST, so that only
/       sets of chunk files that make up complete transport files remain.
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions: can't open or lock XRCH chunk history file.
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  recvmeth (program)
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/dopartia.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/14/91 18:33 New function. */
/*: CR  9/04/91 12:02 Add comments. */
/*: CR  9/11/91 16:49 Do not fail if can't open XRCH; it's empty. */
/*: CR 10/07/91 18:11 Call neighbor() to get cache'd host info. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JX  7/29/92 16:26 Chixify. */
/*: JX  9/17/92 11:30 Fixify. */
/*: CR 12/14/92 15:20 Long chxlong 2nd arg. */

#include <stdio.h>
#include "entryex.h"
#include "common.h"
#include "handicap.h"
#include "unitcode.h"
#include "chixuse.h"
#include "derr.h"
#include "recv.h"
#include "xfp.h"

#define   FAIL  { success = 0;   goto done; }

FUNCTION  do_partials (list, shipmeth)
   Trans  list;
   int    shipmeth;
{
   Trans  partial, p, q;
   int    complete, i, success, hnum;
   long   seqnum, receipt, now, keepmax, index;
   Chix   s, temp, hostname, seqstr;
   HOST  *host, *neighbor();
   Trans  make_trans(), find_trans();
   long   systime();

   ENTRY ("do_partials", "");

   s        = chxalloc (20L, THIN, "dopartial s");
   temp     = chxalloc (100L, THIN, "dopartial temp");
   seqstr   = chxalloc (100L, THIN, "dopartial seqstr");
   hostname = chxalloc (100L, THIN, "dopartial hostname");

   /*** Separate LIST into two lists: completely received files (LIST),
   /    and partially received files (PARTIAL). */
   success = 1;
   partial = make_trans();

   /*** Scan LIST for sets of chunks that do not make a COMPLETE 
   /    transport file.  Move those entries to PARTIAL.   Q is the
   /    node under consideration, P is its predecessor. */
   for (p=list;   p->next != NULL;   ) {
      q = p->next;
      
      /*** A COMPLETE transport file must have the CHUNKLAST flag set, and
      /    have no gaps in the list of chunk files. */
      complete = 0;
      if (q->gotlast) {
         for (i=1;   i <=q->highchunk  &&  q->xnums[i] >= 0;   ++i) ;
         complete = (i > q->highchunk);
      }
      if (complete)  { p = p->next;   continue; }

      /*** Unhook an incomplete set from LIST, and add it to PARTIAL. */
      p->next = q->next;
      q->next = partial->next;
      partial->next = q;
   }

   /*** Read the chunk history file, and use it to update the first_receipt
   /    time of the partial chunk sets. */
   if (NOT unit_lock (XRCH, WRITE, L(shipmeth), 0L, 0L, 0L, nullchix))  FAIL;
   if (unit_view (XRCH)) {
      while  (unit_read (XRCH, temp, 0L)) {
         index = 0;
         chxtoken (hostname, nullchix, 1, temp);
         chxtoken (seqstr, s, 2, temp);
         seqnum  = chxlong (s, &index);
         receipt = chxlong (s, &index);
   
         /*** If this chunk set is still around, set the proper first_receipt
         /    time. */
         if ( (p = find_trans (partial, seqnum, hostname)) == NULL) 
          continue;
         p->first_receipt = receipt;
      }
      unit_close (XRCH);
   }


   /*** Scan the PARTIAL list.   Delete any chunk sets that have been
   /    around longer than KEEPMAX.  The rest are written back to the
   /    chunk history file. */
   if (NOT unit_make (XRCH))  success = 0;
   else {
      now = systime();
      for (p=partial->next;   p!=NULL;   p = p->next) {

         /*** Get the KEEPMAX for this host.  If it fails, use a value
         /    of 0, which will force the partial chunk set to be deleted. */
         keepmax = 0;
         if ( (hnum = host_number (q->chn)) >= 0) {
            host = neighbor (hnum);
            if (NOT EMPTYCHX(host->chn))  keepmax = host->info->keep_chunks;
         }

         if (now - p->first_receipt < keepmax) {
            chxclear  (temp);
            chxformat (temp, CQ("%s %d %d\n"), p->first_receipt, p->seq,
                                               p->chn, nullchix);
            unit_write (XRCH, temp);
            continue;
         }

         for (i=1;   i<=p->highchunk;   ++i)  if (p->xnums[i] >= 0) {
            if (unit_lock (XRXC, WRITE, L(shipmeth),  p->xnums[i],
                           0L, 0L, nullchix)) {
               unit_kill  (XRXC);
               unit_unlk  (XRXC);
            }
         }
      }
      unit_close (XRCH);
   }
   unit_unlk  (XRCH);

done:
   chxfree (s);
   chxfree (temp);
   chxfree (hostname);
   free_trans (partial);

   RETURN (success);
}
