/*** DO_TRANSPORT.  Find the earliest transport file in LIST, and process it.
/
/    do_transport (list, recmeth, parseseq, error);
/
/    Parameters:
/       Trans     list;      (list of transport files)
/       int       recmeth;   (receiving bay number)
/       long     *parseseq;  (parse bay sequence number for transport file)
/       int      *error;     (returned error code)
/
/    Purpose:
/       Find the lowest sequence-numbered transport file in the a Trans
/       LIST, process it, and remove it from the LIST.
/
/    How it works:
/       Do_transport() scans the entire LIST for the lowest sequence number.
/       It then performs one of the following actions on the chunks that
/       make up the transport file, depending on the sequence number.
/       FROM is the XportFileSeq#From for the host that sent the transport
/       file.
/
/          (a) If seq# < FROM, ACK seq#, and delete the chunks.  (This
/              is a repeat of a file we've already received.)
/
/          (b) If seq# == FROM, ACK seq#, assemble the chunks, and put
/              it in the PARSEBAY.  (Scan the REREC bay for transport
/              files that may now be processed.)
/
/          (c) If seq# >  FROM, assemble the chunks and put it in the
/              REREC bay.  NAK the files between FROM and seq#-1.
/
/    Returns: 1 if there was a transport file remaining in the LIST.
/             0 otherwise.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: 
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/dotransp.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/20/91 15:51 New function. */
/*: JV  8/28/91 18:15 Check value from hnumber. */
/*: CR  9/04/91 11:48 "Increment" P in linked list loop! */
/*: CR  9/09/91 16:59 ACK "early-bird" file. */
/*: JV  9/09/91 11:01 Use seqnum macro ops */
/*: CR 10/04/91 17:07 Delete old chunks using recmeth, not shipmeth. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR  1/31/92 14:06 Delete chunks from unknown host. */
/*: CR  2/03/92 11:39 ACK after assembly; NAK on "impossible" errors. */
/*: JX  7/29/92 17:07 Chixify. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "done.h"
#include "xfp.h"
#include "unitcode.h"
#include "xaction.h"
#include "derr.h"
#include "recv.h"
#include "sequence.h"

FUNCTION  do_transport (list, recmeth, parseseq, error)
   Trans  list;
   int    recmeth;
   long  *parseseq;
   int   *error;
{
   HOSTPTR   host, make_host();
   Hinfoptr  make_hinfo();
   Trans     p, q;
   long      ackseq, from, seq;
   int       hnum, ok, changed, success;
   char      bug_msg[80];

   ENTRY ("do_transport", "");

   *error = NOERR;
   host = make_host();
   host->info = make_hinfo();

   /*** Input validation. */
   if (list==NULL  ||  list->next == NULL)  FAIL;

   /*** Find P, the lowest numbered entry in LIST.  Q is it's predecessor. */
   for (q=list, p=list->next;   p->next != NULL;  p = p->next)
      if (GT( (q->next)->seq, (p->next)->seq, SEQ_FROM_MOD))  q = p;
   p = q->next;

   /*** Get the host number HNUM for P's transport file, and the 
   /    information about that host.   If that host does not appear
   /    to exist, delete the chunks that make up the transport file,
   /    remove node P, and return. */
   if ( (hnum = hnumber (p->chn)) < 0) {
      sprintf (bug_msg, "RECVMETH: DO_TRANSPORT: bad host name: '%s'.", 
               ascquick (p->chn));
      bug (bug_msg, 0);
      kill_chunks (p, recmeth);
      q->next = p->next;
      p->next = NULL;
      free_trans (p);
      SUCCEED;
   }

   if (ok = unit_lock (XHI1, READ, L(hnum), 0L, 0L, 0L, nullchix)) {
      ok = get_neighbor (hnum, host, XHI1);
      unit_unlk (XHI1);
   }
   if (NOT ok) {
      bug ("Recvmeth: do_transport: get_neighbor failed: ", hnum);
      *error = DR_GETNEI;
      FAIL;
   }

   /*** We need, and may change, two sets of sequence numbers:
   /    ACKSEQ: seq #s to build ACK/NAK files;
   /    FROM:   seq # of transport file next expected from host HNUM. */
   ackseq = host->info->ackseq;
   from   = host->info->seqnum_from;
      
   /*** Now that we are ready to assemble the chunks into a transport
   /    file, there are three possible cases:
   /      (a) we've already seen this file, toss it.
   /      (b) this is the file we've been waiting for!
   /      (c) this is a later file, squirrel it away.  */

   /*** Received a transport file we've already seen.  ACK it, and delete
   /    the chunk files. */
   if (GT (from, p->seq, SEQ_FROM_MOD)) {
      send_ack (hnum, host->info->shipmeth, p->seq, &ackseq, ACK);
      kill_chunks (p, recmeth);
   }

   /*** Received exactly the file we want.  Assemble the transport file in 
   /    the PARSEBAY, then ACK it.  (The chunks are deleted by assemble_chunks.)
   /    Look in the RERECV bay for "early-bird" files that were previously
   /    received and can now be put in the PARSEBAY.  */
   if (p->seq == from) {
      if (ok = open_transport (XSPB, 0, 0, parseseq)) {
         ok = assemble_chunks (XSPB, recmeth, p);
         unit_close (XSPB);
         if (ok)  {
            *parseseq = INC (*parseseq, SEQ_PARS_MOD);
            from      = INC (from,      SEQ_FROM_MOD);
         }
         else  unit_kill (XSPB);
         unit_unlk  (XSPB);
      }

      /*** ACK it.  (NAK if "impossible" error.) */
      send_ack (hnum, host->info->shipmeth, p->seq, &ackseq, (ok ? ACK : NAK));

      /*** Now that we got the transport file we were expecting, process
      /    any that are waiting in the re-receive bay. */
      if (ok) do_rereceive (&from, parseseq, hnum, recmeth);
   }

   /*** Received a "early-bird" file, with seq # higher than we were expecting.
   /    Assemble it in the RERECV bay for use later, and NAK everything before
   /    it that we were hoping to see.  Then ACK it.  (NAK on "impossible"
   /    error). */
   if (GT (p->seq, from, SEQ_FROM_MOD)) {
      if (ok = open_transport (XRRX, recmeth, hnum, &p->seq)) {
         ok = assemble_chunks (XRRX, recmeth, p);
         unit_close (XRRX);
         if (NOT ok)  unit_kill (XRRX);
         unit_unlk  (XRRX);
      }
      for (seq = from;   seq != p->seq;   seq = INC (seq, SEQ_FROM_MOD))
         send_ack (hnum, host->info->shipmeth, seq, &ackseq, NAK);
      send_ack (hnum, host->info->shipmeth, p->seq, &ackseq, (ok ? ACK : NAK));
   }

   /*** We're done with node P, remove it from LIST. */
   q->next = p->next;
   p->next = NULL;
   free_trans (p);

   /*** Rewrite the hostinfo file, ackseq and from may have changed... */
   if (ok = unit_lock (XHI1, WRITE, L(hnum), 0L, 0L, 0L, nullchix)) {
      if (ok = get_neighbor (hnum, host, XHI1)) {
         changed = 0;
         if (GT (ackseq, host->info->ackseq,      SEQ_FROM_MOD))
            { host->info->ackseq      = ackseq;   changed = 1; }
         if (GT (from,   host->info->seqnum_from, SEQ_FROM_MOD))
            { host->info->seqnum_from = from;     changed = 1; }
   
         if (changed)  ok = set_neighbor (host, XHI1);
      }
      unit_unlk (XHI1);
   }
   if (NOT ok) {
      bug ("Recvmeth: do_transport: cannot rewrite hostinfo: ", hnum);
      *error = DR_WRINEI;
   }

   success = (*error == NOERR);

done:
   free_host (host);
   RETURN (success);
}


FUNCTION  kill_chunks (p, recmeth)
   Trans  p;
   int    recmeth;
{
   int    i;

   for (i=1;   i<=p->highchunk;   ++i)  if (p->xnums[i] >= 0) {
      if (unit_lock (XRXC, WRITE, L(recmeth),  p->xnums[i], 0L, 0L, nullchix)) {
         unit_kill  (XRXC);
         unit_unlk  (XRXC);
      }
   }
}
