/*** OPEN_ENCRYPT.   Open the "crypt envelope".
/
/    ok = open_encrypt (recvmeth, xfile, error);
/
/    Parameters:
/       int   ok,       (success?)
/             recvmeth, (number of receiving method)
/            *error;    (returned error code)
/       long  xfile,    (sequence number of chunk)
/
/    Purpose:
/       As explained in recvopen.c, a chunk can have the following envelopes:
/       [ uuencode ]
/          [ crypt ]
/             chunk header
/                xport file header
/
/       Both the uuencode and crypt headers are optional; however, the crypt
/       header requires the uuencode header.
/
/       open_encrypt() runs a particular chunk (specified by RECVMETH & XFILE)
/       through crypt, using the key stored in line 1 of Hnnn/recvv001,
/       and leaves the unencrypted data in a file having the same
/       name as the original encrypted chunk.
/
/    How it works:
/       A simple function, open_encrypt checks for an appropriate header,
/       feeds the file to crypt, along with the key, and replaces the
/       original encrypted chunk with the unencrypted text.
/
/       To get the key, it must know which neighbor the chunk is coming from.
/       The encrypted data is prepended with a header of the form:
/          crypt CHN
/       where CHN is the CHN of the neighbor which shipped this chunk.
/       This allows open_encrypt() to look up the key.
/
/    Exit status:
/       1 on success
/       0 on failure
/
/    Error Conditions:
/      Most problems are a sign of trouble in the database.
/      Problems detected here:
/  
/    Side effects:
/
/    Related functions: open_uuencode()
/
/    Called by: xrec/recvopen.c (main)
/
/    Operating system dependencies:
/
/    Known bugs:
/       Building the command line that runs the decryptor is heavily
/       Unix dependent.
/
/    Home:  xrec/openencr.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: JV 10/26/92 13:09 Create this. */
/*: JV 11/30/92 13:30 Change XFILE arg to long. */
/*: CR 12/14/92 14:53 ascofchx long arg, replace ALLCHARS. */
/*: CR 12/14/92 15:08 Long chxindex 2nd arg. */
/*: CR  2/22/93 13:21 Correct comments wrt XHRV; put nl's in rewritten XRXC. */
/*: CR  2/22/93 17:17 Substitute in caudecry for decryption method. */
/*: CP  2/09/94 13:49 Having created key_str, use it! */

#include <stdio.h>
#include "caucus.h"
#include "xaction.h"
#include "recv.h"
#include "xfp.h"
#include "sequence.h"
#include "derr.h"

extern int  debug;
extern Chix confid;

FUNCTION  open_encrypt (recvmeth, xfile, error)
   int    recvmeth, *error;
   long   xfile;
{
   int  hnum, success;
   char key_str[100], command[100], file[100];
   Chix temp, chn, key;
   Chix nl;

   chn  = chxalloc (40L, THIN, "recvopen chnp");
   key  = chxalloc (40L, THIN, "recvopen key");
   temp = chxalloc (80L, THIN, "recvopen temp");

   /*** Lock and open the individual XRXC chunk file. */
   if (NOT unit_lock (XRXC, WRITE, L(recvmeth),  xfile, 0L, 0L, nullchix)) {
      recverror (DR_NOXRXC, recvmeth, xfile); /* For now, report errors. */
      FAIL;
   }

   if (NOT unit_view (XRXC)) {
      recverror (DR_NOXRXC, recvmeth, xfile); /* For now, report errors. */
      unit_unlk (XRXC);
      FAIL;
   }
   
   /*** Read the first line.  If it is not  "crypt", we have a bug.
   /    Report it and fail. */
   unit_read (XRXC, temp, 0L);
   chxtrim   (temp);
   
   if (chxindex (temp, 0L, CQ("crypt")) != 0 || 
       chxtoken (chn, nullchix, 2, temp) == -1 ||
       (hnum = hnumber (chn)) < 1) {
      recverror (DR_BADCPT, recvmeth, xfile);
      unit_close (XRXC);
      unit_unlk  (XRXC);
      FAIL;
   }

   /*** Get the key. */
   unit_lock  (XHRV, READ, L(hnum), L(recvmeth), 0L, 0L, nullchix);
   unit_view  (XHRV);
   unit_read  (XHRV, key, 0L);
   unit_close (XHRV);
   unit_unlk  (XHRV);

   /*** Get rid of the first line (the envelope label). */
   unit_lock  (XIRE, WRITE, 0L, 0L, 0L, 0L, CQ("tmpchunk")); 
   unit_make  (XIRE); 
   nl = chxquick ("\n", 8);
   while (unit_read (XRXC, temp, 0L)) {
      unit_write (XIRE, temp);
      unit_write (XIRE, nl);
   }

   unit_close (XIRE);
   unit_close (XRXC);
   unit_move  (XIRE, XRXC);    /* Put back into the chunk file. */
   unit_unlk  (XIRE);

   /*** Now decrypt. */
   ascofchx  (key_str, key, 0L, 100L);
   unit_name (file, XRXC, L(recvmeth), xfile, 0L, 0L, nullchix);
   sprintf   (command, "%s/SCRIPTS/caudecry %s %s cptchunk", 
                 ascquick(confid), key_str, file);
   sysscript (command);

   /*** Now replace the original chunk with the part we want. */
   unit_lock (XIRE, WRITE, 0L, 0L, 0L, 0L, CQ("cptchunk")); 
   unit_move (XIRE, XRXC); 
   unit_unlk (XIRE);
   unit_unlk (XRXC);

   SUCCEED;

done:
   chxfree (chn);
   chxfree (key);
   chxfree (temp);
   RETURN  (success);
}
