/*** OPEN_UUENCODE.   Open the "uuencode envelope".
/
/    ok = open_uuencode (recvmeth, xfile, error);
/
/    Parameters:
/       int   ok,       (success?)
/             recvmeth, (number of receiving method)
/            *error;    (returned error code)
/       long  xfile,    (sequence number of chunk)
/
/    Purpose:
/       As explained in recvopen.c, a chunk can have the following envelopes:
/       [ uuencode ]
/          [ crypt ]
/             chunk header
/                xport file header
/
/       Both the uuencode and crypt headers are optional; however, the crypt
/       header requires the uuencode header.
/
/       open_uuencode() runs a particular chunk (specified by XFILE)
/       through uudecode, and leaves the result in a file having the same
/       name as the original uuencoded chunk.
/
/    How it works:
/       A simple function, open_uuencode checks for an appropriate header,
/       feeds the file to uudecode, and replaces the original uuencoded
/       chunk with the result of uudecode.
/
/       The following checks are performed as data integrity checks:
/       * The header should specify the uudecoded file as "uuchunk"
/       * The header should specify the perm's as 600
/
/    Exit status:
/
/    Error Conditions:
/      Most always a problem is an indication of a bad DB.
/  
/    Side effects:
/
/    Related functions: open_encrypt()
/
/    Called by: xrec/recvopen.c (main)
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/openuuen.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: JV 10/26/92 11:09 Create this. */
/*: JV 11/30/92 14:00 Change XFILE to long. */
/*: CR 12/14/92 15:08 Long chxindex 2nd arg. */
/*: CR  2/19/93 15:31 Replace "perm!=600" with "perm < 600". */
/*: CR  2/19/93 15:39 Prettify. */

#include <stdio.h>
#include "caucus.h"
#include "xaction.h"
#include "recv.h"
#include "xfp.h"
#include "sequence.h"
#include "derr.h"

FUNCTION  open_uuencode (recvmeth, xfile, error)
   int    recvmeth, *error;
   long   xfile;
{
   int   success;
   long  perm;
   char  command[100], chunk[80];
   Chix  temp, perm_chx, file;

   file     = chxalloc (40L, THIN, "recvopen file");
   temp     = chxalloc (80L, THIN, "recvopen temp");
   perm_chx = chxalloc ( 8L, THIN, "recvopen perm_chx");

   /*** Lock and open the individual XRXC chunk file. */
   if (NOT unit_lock (XRXC, WRITE, L(recvmeth),  xfile, 0L, 0L, nullchix)) {
      recverror (DR_NOXRXC, recvmeth, xfile); /* For now, report errors. */
      FAIL;
   }

   if (NOT unit_view (XRXC)) {
      recverror (DR_NOXRXC, recvmeth, xfile); /* For now, report errors. */
      unit_unlk (XRXC);
      FAIL;
   }
   
   /*** Read the first line.  If it is not  "begin", we have a bug.
   /    Report it and fail. */
   unit_read (XRXC, temp, 0L);
   chxtrim   (temp);
   if (chxindex (temp, 0L, CQ("begin")) != 0) {
      recverror (DR_BADUU, recvmeth, xfile);
      unit_close (XRXC);
      unit_unlk  (XRXC);
      FAIL;
   }
   chxtoken  (perm_chx, nullchix, 2, temp);
   chxtoken  (file,     nullchix, 3, temp);

   /*** The following permission should be 600(decimal), not 0600(octal)
   /    because of the way chxnum interprets the uuencode header! */
   if (NOT chxnum (perm_chx, &perm) || perm < 600 || 
                                       NOT chxeq (file, CQ("uuchunk"))) {
      recverror  (DR_BADUU, recvmeth, xfile);
      unit_close (XRXC);
      unit_unlk  (XRXC);
      FAIL;
   }

   /*** Alright, we have a valid uuencoded chunk.  Let's get on with it. */
   unit_close (XRXC);
   unit_name  (chunk, XRXC, L(recvmeth), xfile, 0L, 0L, nullchix);
   sprintf (command, "uudecode %s", chunk);
   printf  ("uudecoding %s...\n", chunk);
   sysscript (command);

   /*** Now replace the original chunk with the part we want. */
   unit_lock (XIRE, WRITE, 0L, 0L, 0L, 0L, CQ("uuchunk"));
   unit_move (XIRE, XRXC);
   unit_unlk (XIRE);
   unit_unlk (XRXC);

   SUCCEED;

done:
   chxfree (file);
   chxfree (temp);
   chxfree (perm_chx);
   RETURN  (success);
}
