
/*** PARSE_CHUNK.  Parse a received-chunk file chunk header.
/
/    ok = parse_chunk (shipmeth, xfile, host, seq, part, last, acktype, 
/                        ackseq, error)
/
/    Parameters:
/       int       recmeth;     (receiving method)
/       long      xfile;       (chunk file number in receive bay)
/       Chix      host;        (returned host name)
/       long     *seq;         (returned sequence number)
/       int      *part;        (returned chunk part number)
/       int      *last;        (returned 'last-chunk' indicator)
/       int      *acktype;     (returned acktype of chunk)
/       long     *ackseq;      (returned ack seq number, if any)
/       int      *error;       (returned error code)
/
/    Purpose:
/       Parse the chunk header of a chunk file, and return the results.
/
/    How it works:
/       Lock and open the chunk file specified by receiving method RECMETH
/       and number XFILE.  Scan the chunk header and put the relevant
/       information into HOST, SEQ, PART, LAST.
/
/       Scan further and see if this is an ACK file.  If so, put the
/       relevant information into ACKTYPE and ACKSEQ.  Otherwise, set
/       ACKTYPE to NOTACK.  Delete all ACK files.
/
/       If this chunk file does not have a chunk header at all, delete 
/       the impostor!
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions -- sets ERROR to one of:
/       NOERR       success.
/       DR_NOXRXC   can't lock or open chunk file XFILE
/       DR_BADCHK   chunk file doesn't have a chunk header!
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: build_trans_list()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/parsechu.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/16/91 12:38 New function. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JV  1/03/92 15:25 Declare strcpy(). */
/*: JX  7/30/92 14:44 Chixify. */
/*: CR 12/14/92 14:54 Long ascofchx arg, replace ALLCHARS. */
/*: CR 12/14/92 15:05 Long chxcatsub args. */
/*: CR 12/14/92 15:08 Long chxindex 2nd arg. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "chixuse.h"
#include "done.h"
#include "derr.h"
#include "recv.h"

#define UNIT_READ_STR(a,c) unit_read (a,temp, 0L); ascofchx(c,temp,0L,200L);

FUNCTION  parse_chunk (shipmeth, xfile, host, seq, part, last, acktype, 
                       ackseq, error)
   int    shipmeth;
   long   xfile;
   Chix   host;
   long  *seq, *ackseq;
   int   *part, *last, *acktype, *error;
{
   Chix   temp;
   char   tempstr[200];
   int    success;
   char  *s;
   long   strscan();

   ENTRY ("parse_chunk", "");

   temp = chxalloc (100L, THIN, "parsech temp");

   /*** Lock and open the individual XRXC chunk file. */
   if (NOT unit_lock (XRXC, WRITE, L(shipmeth),  xfile, 0L, 0L, nullchix))
      { *error = DR_NOXRXC;                       FAIL; }

   if (NOT unit_view (XRXC))
      { *error = DR_NOXRXC;   unit_unlk (XRXC);   FAIL; }

   /*** Read the first line.  If it is not CHUNKHOST, we have an
   /    impostor chunk file.  Delete it and return an error code. */
   unit_read (XRXC, temp, 0L);
   chxtrim   (temp);
   if (chxindex (temp, 0L, CQ("CHUNKHOST=")) != 0) {
      unit_close (XRXC);
      unit_kill  (XRXC);
      *error = DR_BADCHK;
      unit_unlk  (XRXC);
      FAIL;
   }

   /*** Get the host name, sequence #, part #, and last marker. */
   chxclear  (host);
   chxcatsub (host, temp, 10L, ALLCHARS);
   UNIT_READ_STR (XRXC, tempstr); sscanf (tempstr, "CHUNKSEQN=%ld", seq);
   UNIT_READ_STR (XRXC, tempstr); sscanf (tempstr, "CHUNKPART=%d",  part);
   UNIT_READ_STR (XRXC, tempstr); sscanf (tempstr, "CHUNKLAST=%d",  last);

   /*** Look beyond the chunk header: is this an ACK file? */
   UNIT_READ_STR  (XRXC, tempstr);
   if (strindex (tempstr, "ACKTYPE=") == 0) {
      *acktype = (strindex (tempstr+8, "ACK") == 0);
      UNIT_READ_STR  (XRXC, tempstr);
      s = tempstr + 8;
      *ackseq  = strscan (&s);
   }
   else { *acktype = NOTACK;   *ackseq = 0L; }

   unit_close (XRXC);
   if (*acktype != NOTACK)  unit_kill (XRXC);
   unit_unlk  (XRXC);

   *error = NOERR;
   SUCCEED;

done:
   chxfree (temp);
   RETURN (success);
}
