
/*** RECSPLIT.   Program to split "raw" receipt file into chunks.
/
/    Invoked as:
/       recsplit  confid  raw  recmeth  [-d]
/
/    Arguments:
/       confid     full path name of Caucus home directory.
/       raw        full path name of "raw" receipt file
/       recmeth    receiving method number
/       [-d]       option debug flag
/
/    Purpose:
/       This program is used by the CaucusNet RECVM001 receiver-method
/       script, to split a single file produced by the Unix mail utility
/       into a set of transport "chunk" files.
/
/    How it works:
/       This should work for files produced by both Unix and Vax/VMS mailers.
/       It (unfortunately) depends on seeing one of the "CHUNKHOST" or
/       "begin" keywords at the beginning of the data to be able to parse
/       the message.
/
/    Exit status: 0 on success, 1 on failure.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/recsplit.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/18/91 18:48 New program. */
/*: JV  9/06/91 12:45 Change def of confid to decl. */
/*: JV  9/19/91 11:27 Use seqnum macro ops. */
/*: CR  9/16/91 20:49 Requires mainglobals(). */
/*: CR 10/01/91 14:07 Use "CHUNKHOST" to find start of message text. */
/*: CR 10/14/91 11:48 Sysavail: files == -1 means "can't tell". */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR  1/13/92 17:56 Remove completely empty lines. */
/*: JX  7/31/92 11:35 Chixify, utilglob before mainglob. */
/*: JV 10/29/92 14:18 Allow "begin" for uuencode. */
/*: CR 12/14/92 15:15 Long chxvalue arg. */
/*: CR 12/14/92 15:20 Long chxlong 2nd arg. */
/*: CP  3/04/93 23:58 Make 'debug' extern. */
/*: CP  3/28/93 23:51 Add ^A testing for Coherent. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "xaction.h"
#include "recv.h"
#include "sequence.h"

#define  IN_LIMBO   0
#define  IN_HDR     1
#define  IN_TXT     2

extern struct screen_template  used, screen;
extern Chix                    confid;
extern int                     debug;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   int    state, ok, syserr, recmeth;
   long   size, freedisk, files, chunk, index;
   Chix   presline, temp;

   screen.lines =  0;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   temp     = chxalloc (100L, THIN, "recsplit temp");
   presline = chxalloc (100L, THIN, "recsplit presline");

   /*** Check arguments and initialize CONFID. */
   if (argc < 4  ||  argc > 5) {
      printf ("Usage: recsplit  confid  raw  recmeth  [-d]\n");
      exit   (1);
   }
   if (argc == 5)  debug = -1;
   sscanf (argv[3], "%d", &recmeth);

   confid = chxalloc (40L, THIN, "recsplit confid");
   chxofascii (confid, argv[1]);
   utilglobals();
   mainglobals();

   /*** Get the size of the raw file.  We'll use this later
   /    to ensure there's enough free space for the chunk files. */
   sysfsize  (argv[2], &size, &syserr);
   if (syserr != 0) {
      printf ("RECSPLIT: error in sysfsize, code=%d\n", syserr);
      exit   (1);
   }
   if (size < 10000L)  size = 10000L;
   size = 2 * size;

   /*** Get the current chunk counter. */
   if (ok = unit_lock (XRCC, READ, L(recmeth), 0L, 0L, 0L, nullchix)) {
      if (ok = unit_view (XRCC)) {
         if (ok = unit_read (XRCC, temp, 0L)) {
            index = 0;   chunk = chxlong (temp, &index); }
         unit_close (XRCC);
      }
      unit_unlk (XRCC);
   }
   if (NOT ok)  chunk = 0L;

   /*** Lock and open the raw file. */
   if (NOT unit_lock (XIRE, WRITE, 0L, 0L, 0L, 0L, CQ(argv[2])))  exit (1);
   if (NOT unit_view (XIRE))       { unit_unlk (XIRE);    exit (1); }

   /*** Most of RECSPLIT is a big loop that parses the text in the raw file
   /    one line at a time.  The initial state is IN_LIMBO, i.e. not in a
   /    message.  A <FF> (VV) or a line starting with "From" (UNIX) 
   /    starts a new message, changing state to IN_HDR.  The header
   /    is read and ignored, until the "CHUNKHOST" keyword is found,
   /    which starts the text of the message and changes the state
   /    to IN_TXT.  The text is copied into its own XRXC "chunk" 
   /    file, until the next message starts.  */
   state = IN_LIMBO;
   while (unit_read (XIRE, presline, 0L)) {

#if UNIX
      /*** Line starting with ^A, *or* "From" followed by at least two 
      /    words starts a new message. */
      if (chxvalue (presline, 0L) == '\001'  ||
           (chxindex (presline, 0L, CQ("From")) == 0  &&
            chxtoken (temp, nullchix, 3, presline) != -1) ) {
#endif

#if VV
      /*** Formfeed starts a new message. */
      if (chxvalue (presline, 0L) == '\014') {
#endif

         /*** If we were already in a message, close the old chunk file. */
         if (state == IN_TXT) {
            unit_close (XRXC);
            unit_unlk  (XRXC);
         }

         state = IN_HDR;
         continue;
      }

      /*** Skip through the header until we see CHUNKHOST or a uuencode hdr.
      /    Then try to create the next chunk file. */
      if (state == IN_HDR) {
         if (chxindex (presline, 0L, CQ("CHUNKHOST")) != 0 && 
             chxindex (presline, 0L, CQ("begin")) != 0 )  continue;

         /*** Is there enough free disk space for another chunk file?
         /    (This is kind of clumsy... do we have to check every time?) */
         sysavail (argv[2], &freedisk, &files, &syserr);
         if (syserr != 0)  
            printf ("RECSPLIT: error in sysavail, code=%d\n", syserr);
         if (syserr != 0  ||  freedisk < size  || 
                             (files >= 0  &&  files < 20))  goto failed;

         /*** Get the next unused chunk number. */
         while (1) {
            chunk = INC (chunk, SEQ_CHNK_MOD);
            if (NOT unit_lock (XRXC, WRITE, L(recmeth),  chunk, 0L, 0L,
                               nullchix))
               goto failed;
            if (NOT unit_check (XRXC))  break;
            unit_unlk (XRXC);
         }

         /*** And create that chunk file. */
         if (NOT unit_make (XRXC))  { unit_unlk (XRXC);   goto failed; }
         unit_write (XRXC, presline);
         unit_write (XRXC, CQ("\n"));
         state = IN_TXT;
         continue;
      }

      /*** Copy regular message data lines to chunk file. */
      if (state == IN_TXT) {
         if (EMPTYCHX (presline))  continue;
         unit_write (XRXC, presline);
         unit_write (XRXC, CQ("\n"));
      }
   }

   /*** After successful processing, close the last chunk file... */
   if (state == IN_TXT) {
      unit_close (XRXC);
      unit_unlk  (XRXC);
   }

   /*** ... close and delete the raw file... */
   unit_close (XIRE);
   unit_kill  (XIRE);
   unit_unlk  (XIRE);

   /*** ... and rewrite the chunk counter. */
   if (unit_lock (XRCC, WRITE, L(recmeth), 0L, 0L, 0L, nullchix)) {
      if (unit_make (XRCC)) {
         chxclear  (temp);
         chxformat (temp, CQ("%d\n"), chunk, 0L, nullchix, nullchix);
         unit_write (XRCC, temp);
         unit_close (XRCC);
      }
      unit_unlk (XRCC);
   }
   exit (0);

failed:
   unit_close (XIRE);
   unit_unlk  (XIRE);
   chxfree (temp);
   chxfree (presline);
   exit (1);
}
