
/*** RECVALL.   Program to receive all files across all receive methods.
/
/    Invoked as:
/       recvall  confid  free  [-d]
/
/    Arguments:
/       confid     full path name of Caucus home directory.
/       free       require at least FREE bytes of disk space free
/       [-d]       optional debugging flag
/
/    Purpose:
/       Recvall examines each receiving method it knows about, and tries
/       to receive any files that might be outstanding for that method.
/       It does this once, and then exits.  It is up to the Caucus manager
/       to make sure this happens on a regular basis.
/
/    How it works:
/       Recvall looks in XSRM for the list of shipping methods.  For 
/       each method, it:
/         (a) Checks to see there is at least FREE available bytes on disk;
/         (b) Runs the RECVMnnn script for method "nnn".  That script
/             in turn receives the chunk files, and runs the program
/             recvmeth_x, which assembles the chunks into transport
/             files in the parse bay.
/
/    Exit status:
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  xrec/recvall.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/21/91 16:59 New program. */
/*: JV  8/25/90 19:41 Warn somebody about no disk space. */
/*: JV  9/06/91 12:45 Change def of confid to decl. */
/*: CR  9/12/91 16:28 Replace recvmeth with XSRS recvmnnn script. */
/*: CR  9/16/91 20:41 Requires mainglobals(). */
/*: CR  9/17/91 14:55 Fix lint (long/int) errors. */
/*: CR 10/14/91 11:49 Sysavail: files == -1 means "can't tell". */
/*: CR 10/29/91 13:11 Fix syntax error! */
/*: UL 12/11/91 18:45 Make unit_lock/name args 3-6 long.*/
/*: JV 12/31/91 15:28 Rerun sysscript() if fails on tmp resource limit */
/*: JX  7/31/92 11:38 Chixify, utilglob before mainglob. */
/*: CR 12/14/92 15:15 Long chxvalue arg. */
/*: CP  3/28/93 20:05 Make debug extern. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

#define MAXLOOP 4

extern struct screen_template  used, screen;
extern Chix                    confid;
extern union  null_t           null;
extern int                     debug;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   Chix   recmethods, cline, word, ctext, ctitle, chxalloc();
   int    ok, syserr, loop, scrret;
   char   dometh[200], recfile[160], recscript[160];
   long   m, free, freedisk, files, pos;

   ENTRY ("main", "");

   screen.lines =  0;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   /*** Check arguments and initialize CONFID. */
   if (argc < 3) {
      printf ("Usage: recvall confid free [-d]\n");
      exit   (1);
   }
   if (argc == 4)  debug = -1;

   confid = chxalloc (40L, THIN, "recvall confid");
   chxofascii (confid, argv[1]);
   sscanf (argv[2], "%ld", &free);

   utilglobals();
   mainglobals();

   recmethods = chxalloc (500L, THIN, "recvall recmethods");
   cline      = chxalloc (100L, THIN, "recvall cline");
   word       = chxalloc ( 50L, THIN, "recvall word");

   /*** Read in the entire list of receiving methods. */
   if (ok = unit_lock (XSRM, READ, 0L, 0L, 0L, 0L, nullchix))  {
       ok = chixfile (XSRM, recmethods, "");
       unit_unlk (XSRM);
   }
   if (NOT ok) 
      { bug ("Recvall: cannot lock/read XSRM file.\n", 0);   exit (1); }
   
   /*** Pick up each line of the receiving method.  Parse off the method
   /    number, and invoke the RECVMnnn script for that method. */
   unit_name (recfile, XSRM, 0L, 0L, 0L, 0L, nullchix);
   for (pos=0;   chxnextline (cline, recmethods, &pos);   ) {
      if (chxvalue (cline, 0L) == '#')               continue;
      if (chxtoken (word, nullchix, 1, cline) == -1) continue;
      if (NOT chxnum   (word, &m))                   continue;

      if (debug)  printf ("Recvall: method %ld.\n", m);

      /*** Make sure there is at least FREE available disk space. */
      sysavail (recfile, &freedisk, &files, &syserr);
      if (syserr != 0)  {
         bug ("Recvall: sysavail failed.\n", syserr);
         break;
      }
      if (freedisk < free  ||  (files >= 0  &&  files < 20)) {
         bug ("Not enough free disk space or file entries left.", 0);
         if (debug)
            printf ("There are %ld bytes left, %ld was requested\n",
                     freedisk, free);
         break;
      }

      unit_name (recscript, XSRS,  m, 0L, 0L, 0L, nullchix);
      sprintf   (dometh, "%s %s %03ld", recscript, ascquick (confid), m);
      if (debug) printf ("recvall: running: %s...\n", dometh);

      for (loop=0; (scrret = sysscript(dometh)) < 0 && loop < MAXLOOP;
           loop++) {
         switch (scrret) {
            case -1:
            case -2: sleep (10); continue;    /* Try again */
            case  1:                          /* Success */
            case  0:                          /* Unknwon Failure */
            default: loop = MAXLOOP; break;   /* Terminate loop */
         }
      }
      if (scrret != 1) {
         ctitle = chxalloc (100L, THIN, "recvall ctitle");
         mdstr (ctitle, "net_Ttelltitle", null.md);
         ctext = chxalloc (100L, THIN, "recvall ctext");
         mdstr (ctext, "net_Ebadscript", null.md);
         sendmgr (ctext, ctitle);
         chxfree (ctext);
         chxfree (ctitle);
      }
   }

   exit (0);
}
