/*** RECVMETH.   Program to receive transport files for a given ship method.
/
/    Invoked as:
/       recvmeth  confid  shipmeth  [-d]
/
/    Arguments:
/       confid     full path name of Caucus home directory.
/       shipmeth   shipping method number
/       [-d]       option debug flag
/
/    Purpose:
/       Receiving transport files occurs in N steps:
/
/         1. A "recvmNNN" script receives the raw files, and places 
/            them, in chunk form, in the RECVnnn receiving bay.
/
/         2. The RECVMETH program scans these chunk files, and assembles
/            them and processes them, hopefully putting transport files
/            into the parse bay.  (From where the XFU will pick them up.)
/
/    How it works:
/
/    Exit status:
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/recvmeth.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/14/91 22:37 New program. */
/*: JV  8/25/91 12:43 Change seqnum_to_low to seqnum_from. */
/*: JV  9/05/91 08:59 Change seqnum_from to ackseq. */
/*: JV  9/06/91 12:45 Change def of confid to decl. */
/*: CR  9/09/91 15:38 Change "bug" comments. */
/*: JV  9/11/91 12:25 Put bug comments back in...we ain't done yet. */
/*: JV  9/19/91 11:11 Use seqnum macro ops */
/*: CR  9/16/91 20:40 Requires mainglobals(). */
/*: CR 10/04/91 12:22 Change do_acks() calling sequence. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JX  7/30/92 14:49 Chixify, utilglob before mainglob. */
/*: CP  3/04/93 23:54 Make 'debug' extern. */

#include <stdio.h>
#include "caucus.h"
#include "xaction.h"
#include "recv.h"
#include "xfp.h"
#include "sequence.h"

extern struct screen_template  used, screen;
extern Chix                    confid;
extern int                     debug;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   int       shipmeth, error, ok;
   long      parseseq;
   HOSTPTR   host, make_host();
   Hinfoptr  make_hinfo();
   Trans     list, build_trans_list();

   screen.lines =  0;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   host = make_host();
   host->info = make_hinfo();

   /*** Check arguments and initialize CONFID. */
   if (argc < 3  ||  argc > 4) {
      printf ("Usage: recvmeth confid shipmeth [-d]\n");
      exit   (1);
   }
   sscanf (argv[2], "%d", &shipmeth);
   if (argc == 4)  debug = -1;

   confid = chxalloc (40L, THIN, "recvmeth confid");
   chxofascii (confid, argv[1]);

   utilglobals();
   mainglobals();

   /*** Lock only one receiver per method running at a time! */
   if (NOT unit_lock (XRRL, TLOCK, L(shipmeth), 0L, 0L, 0L, nullchix)) exit (1);

   /*** Read the XSNF "sysinfo" file to get information about *this* host. */
   if (ok = unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, nullchix)) {
      ok = get_neighbor (0, host, XSNF);
      unit_unlk (XSNF);
   }
   if (NOT ok) {
      bug  ("Recvmeth: get_neighbor failed on XSNF", 0);
      unit_unlk (XRRL);
      exit (1);
   }

   /*** Sysinfo's ackseq field contains next seq # to be put in
   /    parsebay.  Do_transport() uses it and may change it. */
   parseseq = host->info->ackseq;

   /*** Build the list of received chunk files, organized by incoming
   /    transport file sequence number. */
   list = build_trans_list (shipmeth, &error);

   /*** Remove "partial" entries from list, i.e. ones that have not 
   /    yet received all their chunks. */
   do_partials (list, shipmeth);

   /*** Process all received ACK/NAK files, remove from LIST. */
   do_acks (list);

   /*** Finally, process all complete transport files. */
   while (do_transport (list, shipmeth, &parseseq, &error)) ;
   free_trans (list);

   /*** If PARSESEQ has changed, rewrite the sysinfo file. */
   if (parseseq != host->info->ackseq) {
      if (ok = unit_lock (XSNF, WRITE, 0L, 0L, 0L, 0L, nullchix)) {
         if (ok = get_neighbor (0, host, XSNF)) {
            if (GT (parseseq, host->info->ackseq, SEQ_ACK_MOD)) {
               host->info->ackseq = parseseq;
               ok = set_neighbor (host, XSNF);
            }
         }
         unit_unlk (XSNF);
      }
      if (NOT ok)  bug ("Recvmeth: cannot rewrite sysinfo.", 0);
   }

   free_host (host);
   unit_unlk (XRRL);
   exit (0);
}
