/*** RECVOPEN.   Program to "open the envelopes" of transport file chunks.
/
/    Invoked as:
/       recvopen  confid  recvmeth  [-d]
/
/    Arguments:
/       confid     full path name of Caucus home directory.
/       recvmeth   receiving method number
/       [-d]       option debug flag
/
/    Purpose:
/       Receiving transport files occurs in 3 steps:
/
/         1. A "recvmNNN" script receives the raw files, and places 
/            them, in chunk form, in the RECVnnn receiving bay.
/
/         2. RECVOPEN looks at each chunk file and determines which
/            envelopes the chunk is packaged in.  It opens all except the
/            chunk envelope (and the envelopes within).  (This means 
/            removing the envelope headers and decrypting and or
/            uudecoding the files as necessary.)  The chunk files are
/            rewritten as plain text.  The receiving method
/            determines the set of extra-CHUNK envelope types.
/
/         3. The RECVMETH program scans these chunk files, and assembles
/            them into transport files in the parse bay (From where the 
/            XFU will pick them up.)
/
/    How it works:
/       There is a pre-determined set of envelopes and order for these envelopes.
/       The innermost (which we're concerned with) is the chunk envelope.  This
/       envelope is indicated by the word CHUNKHOST starting the first line.
/       RECVOPEN does not open this envelope.
/
/       For receiving method 1, the chunk envelope may be enclosed in one
/       or both of these envelopes:
/       *  uuencode envelope
/       *  encryption envelope
/
/       An encryption envelope *must* be enclosed by a uuencode envelope.
/
/       There is one rule: the outermost envelope is either the chunk envelope
/       or the uuencode envelope.  In other words:
/
/       [ uuencode [chunk] ]      or
/       [ uuencode [crypt [chunk] ] ]
/
/    Exit status:
/
/    Error Conditions:
/       Problems are almost always an indication of a problem with the DB.
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/recvopen.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: JV 10/23/92 16:01 Create this. */
/*: JV 11/30/92 14:00 Arg chg in open_encrypt. */
/*: CR 12/14/92 15:08 Long chxindex 2nd arg. */
/*: CR 12/14/92 16:00 Sysmem arg is long. */
/*: CR  2/18/93 15:22 Expand upon the header comments. */
/*: CP  3/25/93 19:43 Make debug extern. */

#include <stdio.h>
#include "caucus.h"
#include "xaction.h"
#include "recv.h"
#include "xfp.h"
#include "sequence.h"
#include "derr.h"

extern struct screen_template  used, screen;
extern Chix                    confid;
extern int                     debug;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   struct pdir_t  pdir;
   int       recvmeth, error, ok;
   long      xfile, i;
   HOSTPTR   host, make_host();
   Hinfoptr  make_hinfo();
   Chix      temp;
   char     *sysmem();

   screen.lines =  0;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   temp = chxalloc (80L, THIN, "recvopen temp");
   host = make_host();
   host->info = make_hinfo();

   /*** Check arguments and initialize CONFID. */
   if (argc < 3  ||  argc > 4) {
      printf ("Usage: recvopen confid recvmeth [-d]\n");
      exit   (1);
   }
   sscanf (argv[2], "%d", &recvmeth);
   if (argc == 4)  debug = -1;

   confid = chxalloc (40L, THIN, "recvmeth confid");
   chxofascii (confid, argv[1]);

   utilglobals();
   mainglobals();

   /*** Lock only one receiver per method running at a time! */
   if (NOT unit_lock (XRRL, TLOCK, L(recvmeth), 0L, 0L, 0L, nullchix)) exit (1);

   /*** Read the XSNF "sysinfo" file to get information about *this* host. */
   if (ok = unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, nullchix)) {
      ok = get_neighbor (0, host, XSNF);
      unit_unlk (XSNF);
   }
   if (NOT ok) {
      bug  ("Recvmeth: get_neighbor failed on XSNF", 0);
      unit_unlk (XRRL);
      exit (1);
   }

   /*** Get the array of all of the chunk files in the RECVrrr/CHUNK
   /    directory. */
   pdir.type  = -1;
   pdir.parts =  0;
   pdir.resp  = (short *) sysmem (sizeof(short) * L(PDIRMAX + 3));
   pdir.lnum  = (long  *) sysmem (sizeof(long)  * L(PDIRMAX + 3));
   syspdir (&pdir, XRXC, recvmeth, 0, nullchix);

   /*** Process each chunk independently. */
   for (i=0;   i<pdir.parts;   ++i) {

      /*** Read the chunk header of a chunk file. */
      xfile = pdir.lnum[i];

      /*** Lock and open the individual XRXC chunk file. */
      if (NOT unit_lock (XRXC, WRITE, L(recvmeth),  xfile, 0L, 0L, nullchix)) {
         recverror (DR_NOXRXC, recvmeth, xfile); /* For now, report errors. */
         continue;
      }
   
      if (NOT unit_view (XRXC)) {
         recverror (DR_NOXRXC, recvmeth, xfile); /* For now, report errors. */
         unit_unlk (XRXC);
         continue;
      }
   
      /*** Read the first line.  If it is not CHUNKHOST or "begin", we have an
      /    impostor chunk file.  Delete it and return an error code. 
      /    Open any envelopes around the chunk envelope. */
      unit_read (XRXC, temp, 0L);
      chxtrim   (temp);
      if (chxindex (temp, 0L, CQ("CHUNKHOST=")) == 0) { /* No "extra" envelopes */
         unit_close (XRXC);
         unit_unlk  (XRXC);
         continue;
      } else
      if (chxindex (temp, 0L, CQ("begin")) == 0) {  /* uuencode envelope */
         unit_close (XRXC);
         unit_unlk  (XRXC);
         open_uuencode (recvmeth, xfile, &error); /* Open uuencode envelope */
         unit_lock (XRXC, WRITE, L(recvmeth),  xfile, 0L, 0L, nullchix);
         unit_view (XRXC);
         unit_read (XRXC, temp, 0L);          /* Check next envelope */
         chxtrim   (temp);
         if (chxindex (temp, 0L, CQ("CHUNKHOST=")) == 0) {
            unit_close (XRXC);
            unit_unlk  (XRXC);
            continue;
         } else
            if (chxindex (temp, 0L, CQ("crypt")) == 0) {
            unit_close (XRXC);
            unit_unlk  (XRXC);
            open_encrypt (recvmeth, xfile, &error); /* Open encryption envlp */
         } else
            recverror (DR_BADCHK, recvmeth, xfile);     /* report errors. */

      } else {
         unit_close (XRXC);
         unit_kill  (XRXC);
         unit_unlk  (XRXC);
         recverror (DR_BADCHK, recvmeth, xfile); /* For now, report errors. */
      }
   }

   sysfree ( (char *) pdir.resp);
   sysfree ( (char *) pdir.lnum);
   free_host (host);
   unit_unlk (XRRL);
   exit (0);
}
