
/*** ENOUGH_FOR.   Is there enough disk space for copy of file NAME?
/
/    ok = enough_for (name, maxbytes, error);
/
/    Parameters:
/       int    ok;          (Success?)
/       char  *name;        (file name)
/       long   maxbytes;    (size of chunks)
/       int    error;       (returned error code)
/
/    Purpose:
/       The CaucusNet shipper and reshipper need to know if there is enough
/       free disk space to make a copy of a transaction file NAME.  (If
/       the file is going to be broken up into chunks, they also need to
/       know if there are enough free file slots to accomodate the chunks.)
/
/    How it works:
/       Enough_for() checks the size of file NAME, and makes sure there
/       is at least twice that space remaining on the disk.  It also uses
/       the MAXBYTES argument (max size of a chunk) to determine the number
/       chunks needed, and thus how many free file slots are needed.
/
/    Returns: 1 on success, 0 on failure.
/
/    Error Conditions -- Enough_for() always sets ERROR to one of:
/       NOERR      success.
/       DS_NOXPXT  file NAME does not exist
/       DS_NOTIMP  sysavail() or sysfsize() not implemented.
/       DS_NOTDSK  can't discover amount of free disk space
/       DS_DSKLOW  insufficient free disk space to ship transport file. 
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:  uses sysavail() and sysfsize().
/
/    Known bugs:
/
/    Home:  xship/enoughfo.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/12/91 12:29 New function. */
/*: CR  6/29/92 12:22 Add debug stuff. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "derr.h"
#include "syserror.h"

extern int debug;

FUNCTION  enough_for (name, maxbytes, error)
   char  *name;
   long   maxbytes;
   int   *error;
{
   long   size, freedisk, files;
   int    syserr, needs;

   ENTRY ("enough_for", "");

   *error = NOERR;

   /*** Get the size of the transport text file, and the free disk 
   /    space and file slots. */
   sysfsize  (name, &size, &syserr);
   if (syserr == SE_NOTIMP)            { *error = DS_NOTIMP;   RETURN (0); }
   if (syserr >  0)                    { *error = DS_NOXPXT;   RETURN (0); }
   sysavail  (name, &freedisk, &files, &syserr);
   if (debug) {
      printf ("freedisk=%ld\n", freedisk);
      printf ("size    =%ld\n", size);
      printf ("files   =%ld\n", files);
      printf ("syserr  =%d\n", syserr);
   }
   if (syserr == SE_NOTIMP)            { *error = DS_NOTIMP;   RETURN (0); }
   if (syserr >  0)                    { *error = DS_NOTDSK;   RETURN (0); }

   /*** Is there enough room to write a copy of file NAME? */
   if (size     < 10000L)  size = 10000L;
   if (size * 2 > freedisk)            { *error = DS_DSKLOW;   RETURN (0); }
   needs = (maxbytes ? (size / maxbytes) + 4 : 0);
   if (needs < 20)  needs = 20;
   if (files >= 0  &&  files < needs)  { *error = DS_DSKLOW;   RETURN (0); }

   RETURN (1);
}
