/*** RESHIP_IT.   Re-ship a transport file.
/
/    ok = reship_it (hostnum, shipmeth, seqnum, maxbytes, maxlines, maxwidth,
/                    tosstime, reshiptime, ack, error);
/
/    Parameters:
/       int    ok;          (Success?)
/       int    hostnum;     (ship file to this host)
/       int    shipmeth;    (shipping method number)
/       long   seqnum;      (sequence number of transport file)
/       long   maxbytes;    (max size of chunk to ship)
/       long   maxlines;    (max lines in chunk to ship)
/       long   maxwidth;    (max line width of chunk to ship)
/       long   tosstime;    (toss reshipment files after this many seconds)
/       long   reshiptime;  (reship files after this many seconds)
/       int   *ack;         (returned ACK status)
/       int   *error;       (returned error code)
/
/    Purpose:
/       Consider reshipping a transport file that is in the RESHIP bay.
/       Either reship it, leave it around to ripen further, or toss it
/       (it's been around RESHIP too long, move to CANTSHIP).
/
/    How it works: Reship_it() goes through the following steps in the
/       process of shipping the contents of the transport header and text
/       files:
/    
/       (A) Load the transport header.
/       (B) Decide if the file should be shipped, tossed, or left to ripen.
/       (C) Make sure there is enough free disk space for step D.
/       (D) Ship it via shipfile().
/       (E) Write a reship header file in the RESHIP bay that begins with a
/           shipment date/time header, and is followed by the transport
/           header.
/
/       If the transport file does not exist at all (even in CANTSHIP), it
/       receiver got an ACK and deleted it.  Set ACK to 1.
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions - sets ERROR variable to:
/       NOERR      success.
/       DS_NOXPXH  can't get at (i.e., lock or open) transport header file.
/       DS_NOXPXT  can't get at transport text file.
/       DS_NOXPXJ  can't create a joint chunk shipment file.
/       DS_NOXPRH  can't put header file in RESHIP bay.
/       DS_XTMOVE  can't move transport text file to RESHIP bay.
/       DS_NOTIMP  sysavail() or sysfsize() not implemented.
/       DS_NOTDSK  can't discover amount of free disk space
/       DS_DSKLOW  insufficient free disk space to ship transport file. 
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:
/       MAXWIDTH is ignored in this version.  It may always be ignored.
/
/    Home:  xship/reshipit.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/12/91 14:43 New function. */
/*: JV  8/25/91 12:10 Add to docs. */
/*: UL 12/11/91 18:45 Make unit_lock/name args 3-6 long.*/
/*: CR  1/06/92 15:37 Add shiplast arg to store_xheader() call. */
/*: JX  7/30/92 15:36 Chixify. */
/*: JX  9/16/92 15:44 Allocate HEAD. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "done.h"
#include "unitcode.h"
#include "xaction.h"
#include "derr.h"

FUNCTION  reship_it (hostnum, shipmeth, seqnum, maxbytes, maxlines, maxwidth,
                     tosstime, reshiptime, ack, error)
   int    hostnum, shipmeth;
   long   seqnum, maxbytes, maxlines, maxwidth, tosstime, reshiptime;
   int   *error, *ack;
{
   XPORTHDR *head, *make_xporthdr();
   int       ok, saved, success;
   long      now, shiporig, shiplast;
   char      temp[202];
   long      systime();

   ENTRY ("reship_it", "");

   *error = NOERR;
   *ack   = 0;

   head = make_xporthdr();

   /*** Attempt to load the transport file header.   If it fails, see
   /    if it has been SAVED in the CANTSHIP bay.  If the file is completely
   /    gone, it was deleted by the receiver--ACK the file and return. */
   if (NOT load_xheader (shipmeth, hostnum, seqnum, 1, head,
                             &shiporig, &shiplast)) {
      if (saved = unit_lock  (XPNH, READ, L(shipmeth), L(hostnum),  seqnum,
          0L, nullchix)) {
          saved = unit_check (XPNH);
          unit_unlk (XPNH);
      }
      
      *ack = NOT saved;
      SUCCEED;
   }
   
   /*** If this transport file has been in the RESHIP bay too long,
   /    toss it into CANTSHIP and complain to the manager. */
   now = systime();
   if (now - shiporig > tosstime) {
      tossfile (hostnum, shipmeth, seqnum, now - shiporig);
      SUCCEED;
   }

   /*** If this transport file was shipped (too) recently, just leave
   /    it here to ripen some more. */
   if (now - shiplast < reshiptime)   SUCCEED;


   /*** Make sure there is enough room for the transport file chunks. */
   unit_name (temp, XPRT, L(shipmeth), L(hostnum),  seqnum, 0L, nullchix);
   if (NOT enough_for (temp, maxbytes, error))   FAIL;


   /*** Open the transport text file. */
   if (ok = unit_lock (XPRT, READ, L(shipmeth), L(hostnum),  seqnum, 0L,
          nullchix)) {
      if (NOT (ok = unit_view (XPRT)))  unit_unlk (XPRT); 
   }
   if (NOT ok) { *error = DS_NOXPXT;   FAIL; }

   /*** Ship it! */
   if (NOT shipfile (XPRT, head, hostnum, shipmeth, seqnum, 
                       maxbytes, maxlines, maxwidth, error))  FAIL;
   unit_close (XPRT);
   unit_unlk  (XPRT);

   /*** Write out the header file in the RESHIP bay with a new time
   /    last-shipped (i.e, now.) */
   if (NOT store_xheader (shipmeth, hostnum, seqnum, head, shiporig, now))
      *error = DS_NOXPRH;

   DONE (*error == NOERR);

done:
   free_xporthdr (head);
   RETURN (success);
}
