/*** SHIPFILE.   Ship a transport file.
/
/    ok = shipfile (utext, head, hostnum, shipmeth, seqnum, 
/                     maxbytes, maxlines, maxwidth, error);
/
/    Parameters:
/       int       ok;          (Success?)
/       int       utext;       (unit number of transport text file to ship)
/       XPORTHDR *head;        (transport header information)
/       int       hostnum;     (ship file to this host)
/       int       shipmeth;    (shipping method number)
/       long      seqnum;      (sequence number of transport file)
/       long      maxbytes;    (max size of chunk to ship)
/       long      maxlines;    (max lines in chunk to ship)
/       long      maxwidth;    (max line width of chunk to ship)
/       int       error;       (returned error code)
/
/    Purpose:
/       Ship the transport file open on unit UTEXT to destination
/       HOSTNUM.
/
/    How it works:
/       The transport text file to be shipped is open on unit UTEXT.  The
/       transport header information is in HEAD.  The sequence number of
/       this file is SEQNUM.
/
/       Ship the file to host HOSTNUM according to shipping method SHIPMETH.
/       This means breaking the file into chunks, according to the limits
/       specified in MAXBYTES, MAXLINES, and MAXWIDTH, putting a chunk
/       header on each chunk, and shipping the individual chunks.  (A value
/       of 0 for any MAX... limit means there is no limit for that
          parameter.)
/    
/       (The first chunk contains the transport header, followed by a piece
/        of the transport text; subsequent chunks contain the remaining
/        pieces of the transport text.  There may be a final chunk that
/        contains no data, just a chunk header saying that this is the last
/        chunk.)
/
/       Once the chunks are shipped, they are deleted.
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions
/       NOERR      success.
/       DS_NOXPXJ  can't create a joint chunk shipment file.
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: ship_it(), reship_it()
/
/    Operating system dependencies:
/
/    Known bugs:
/       MAXWIDTH is ignored in this version.  It may always be ignored.
/
/    Home:  xship/shipfile.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  7/31/91 18:18 New function. */
/*: CR  8/08/91 14:09 Add disk space checking. */
/*: CR  8/09/91 14:24 New, trimmed-down incarnation. */
/*: CR  8/12/91 11:07 Minor changes. */
/*: CR 10/14/91 10:08 Chunk file gets CHUNKHOST=`current host`. */
/*: UL 12/11/91 18:45 Make unit_lock/name args 3-6 long.*/
/*: JV 12/31/91 14:55 Rerun sysscript() if fails on tmp resource limit */
/*: JX  7/30/92 15:53 Chixify. */
/*: JX  9/04/92 15:57 Fixify. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "xaction.h"
#include "derr.h"
#include "syserror.h"
#include "done.h"
#include "chixuse.h"
#include "null.h"

#define MAXLOOP 4

extern union null_t null;

FUNCTION  shipfile (utext, head, hostnum, shipmeth, seqnum, 
                      maxbytes, maxlines, maxwidth, error)
   int       utext, hostnum, shipmeth;
   XPORTHDR *head;
   long      seqnum, maxbytes, maxlines, maxwidth;
   int      *error;
{
   int       chunk, no_max, more, i, loop, scrret;
   long      bytes, lines;
   Chix      temp;
   char      shipscript[120], varsfile[120], chunkfile[120], runscript[400];
   Chix      ctext, ctitle, chxalloc();

   ENTRY ("shipfile", "");

   temp = chxalloc (100L, THIN, "shipfile temp");

   /*** Input validation on max parameters. */
   *error = NOERR;
   if (maxbytes != 0  &&  maxbytes < 500)  maxbytes = 500;
   if (maxlines != 0  &&  maxlines <  10)  maxlines =  10;

   /*** Read through the transport text file, creating multiple chunks 
   /    as needed. */
   no_max = (maxbytes==0  &&  maxlines==0  /* &&  maxwidth==0 */);
   for (more=chunk=1;   more;   ++chunk) {

      /*** Create an XPXJ chunk, and write its chunk header. */
      if (NOT make_xpxj (shipmeth, hostnum, seqnum, chunk, head, no_max))
         { *error = DS_NOXPXJ;    goto delete_chunk; }
      bytes = 160L;
      lines =   4L;

      /*** The first chunk also gets the transport header.   Estimate 7 lines
      /    of 40 bytes each for the header. */
      if (chunk == 1) {
         set_xpheader (XPXJ, head);
         bytes += 7 * 40L;
         lines += 7;
      }

      /*** And finally, the very inside envelope is the data in the
      /    transport text file.  Read lines from it and write to XPXJ
      /    until we hit a max limit or the end of the transport text file. */
      for (;   (maxbytes == 0L  ||  bytes+200 < maxbytes)  &&
               (maxlines == 0L  ||  lines+ 50 < maxlines)  &&
               (more = unit_read (utext, temp, 0L));
           ++lines, bytes += chxlen (temp) + 2) {
         unit_write (XPXJ, temp);
         unit_write (XPXJ, CQ("\n"));
      }
      unit_close (XPXJ);
      unit_unlk  (XPXJ);
   }

   /*** If there were maximums, we need to write one more XPXJ chunk that
   /    just has a header saying "th..th..th..that's all, folks!" */
   if (NOT no_max) {
      if (NOT make_xpxj (shipmeth, hostnum, seqnum, chunk++, head, 1)) 
         { *error = DS_NOXPXJ;    goto delete_chunk; }
      unit_close (XPXJ);
      unit_unlk  (XPXJ);
   }

   /*** Actually ship (hurrah!) the individual XPXJ chunks.  */
   unit_name (shipscript, XSSS, L(shipmeth), 0L, 0L, 0L, nullchix);
   unit_name (varsfile,   XHSV, L(hostnum), L(shipmeth), 0L, 0L, nullchix);
   for (i=1;   i<chunk;   ++i) {
      unit_name (chunkfile, XPXJ, L(shipmeth), L(hostnum), seqnum,
                 L(i), nullchix);
      sprintf   (runscript, "%s %s %s", shipscript, varsfile, chunkfile);

      for (loop=0; (scrret = sysscript(runscript)) < 0 && loop < MAXLOOP;
           loop++) {
         switch (scrret) {
            case -1:
            case -2: sleep (10); continue;    /* Try again */
            case  1:                          /* Success */
            case  0:                          /* Unknwon Failure */
            default: loop = MAXLOOP; break;   /* Terminate loop */
         }
      }
      if (scrret != 1) {
         ctitle = chxalloc (100L, THIN, "recvall ctitle");
         mdstr (ctitle, "net_Ttelltitle", null.md);
         ctext = chxalloc (100L, THIN, "recvall ctext");
         mdstr (ctext, "net_Ebadscript", null.md);
         sendmgr (ctext, ctitle);
         chxfree (ctitle);
         chxfree (ctext);
      }
   }


delete_chunk:
   /*** Delete all of the chunk files this call created. */
   for (i=1;   i<chunk;   ++i) {
      unit_name (chunkfile, XPXJ, L(shipmeth), L(hostnum), seqnum,
                 L(i), nullchix);
      if (unit_lock (XPXJ, WRITE, L(shipmeth), L(hostnum), seqnum,
                 L(i), nullchix)) {
         unit_kill  (XPXJ);
         unit_unlk  (XPXJ);
      }
   }

   RETURN (*error == NOERR);
}


/*** Make_xpxj is an internal function.  It creates an individual
/    XPXJ chunk file, and writes the chunk header for it. */

FUNCTION  make_xpxj (shipmeth, hostnum, seqnum, chunk, head, last)
   int       shipmeth, hostnum, chunk, last;
   long      seqnum;
   XPORTHDR *head;
{
   Chix host, temp, seq;
   int  success;

   ENTRY ("make_xpxj", "");

   host = chxalloc (100L, THIN, "makexpxj host");
   temp = chxalloc (100L, THIN, "makexpxj temp");
   seq  = chxalloc (20L,  THIN, "makexpxj seq");

   /*** Create an XPXJ chunk. */
   if (NOT unit_lock (XPXJ, WRITE, L(shipmeth), L(hostnum),  seqnum,
                              L(chunk), nullchix)) FAIL;
   if (NOT unit_make (XPXJ)) { unit_unlk (XPXJ);   FAIL; }

   /*** Start it with the shipment envelope. */
   getmyhost (host);
   chxformat (temp, CQ("CHUNKHOST=%s\n"), 0L, 0L, host, nullchix);
   unit_write (XPXJ, temp);
   chxclear  (temp);
   chxcpy (seq, CQ(head->nayseq));
   chxformat (temp, CQ("CHUNKSEQN=%s\n"), 0L, 0L, seq, nullchix);
   unit_write (XPXJ, temp);
   chxclear  (temp);
   chxformat (temp, CQ("CHUNKPART=%d\n"), L(chunk), 0L, nullchix, nullchix);
   unit_write (XPXJ, temp);
   chxclear  (temp);
   chxformat (temp, CQ("CHUNKLAST=%d\n"), L(last), 0L, nullchix, nullchix);
   unit_write (XPXJ, temp);
   SUCCEED;

done:
   chxfree (host);
   chxfree (temp);
   chxfree (seq);

   RETURN  (success);
}
