/*** SHIPHOST.   Program to ship transport files to a given neighbor.
/
/    Invoked as:
/       shiphost  confid  hostn  type  [-d]
/
/    Arguments:
/       confid     full path name of Caucus home directory.
/       hostn      local host number
/       type       the word "ship" or "reship"
/       [-d]       option debug flag
/
/    Purpose:
/       Shiphost is the general purpose transport file shipper.  It is
/       used for two purposes:
/          (a) shipping new transport files to their destination.
/          (b) re-shipping old transport files when appropriate.
/
/    How it works:
/       If the TYPE argument is "ship", Shiphost scans all the new transport
/       files and "ACK" files destined for host HOSTN, and ships them.
/
/       If TYPE is "reship", Shiphost scans all of the "old" transport files
/       in the reship bay for host HOSTN.  It applies two rules:
/
/          (a) If the file is older than the "tosstime" parameter, it is moved
/              to the "CANTSHIP" bay, and the Caucus manager is notified.
/
/          (b) If the file hasn't been (re)shipped in time > the "reshiptime"
/              parameter, it is shipped again.
/
/       Note that both of these rules use "time" expressed in seconds.
/
/    Exit status: 0
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xship/shiphost.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  7/30/91 23:51 New program. */
/*: CR  8/02/91 14:13 Call get_neighbor() properly. */
/*: CR  8/08/91 12:31 Use TLOCK, ship_ack(). */
/*: CR  8/08/91 20:19 Use for shipping and reshipping. */
/*: CR  8/13/91  9:47 Clean up. */
/*: JV  8/25/91 12:12 Add to docs. */
/*: JV  9/19/91 09:05 Move sequence # macros to sequence.h, use them. */
/*: CR  9/16/91 19:49 Must explicitly call mainglobals(). */
/*: CR  9/19/91 11:26 Set new_to_low/new_to_shipd right after loading XHI1.*/
/*: CR 10/28/91 11:01 Make confid[] extern. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR  6/26/92 19:00 Return 0 so shell thinks we succeeded! */
/*: JX  7/30/92 15:37 Chixify. */
/*: CR 10/08/92 23:33 Remove mdglobals(). */
/*: CR 12/14/92 15:20 Long chxlong 2nd arg. */
/*: CP  3/04/93 23:16 Make 'debug' extern. */

#include <stdio.h>
#include "caucus.h"
#include "xfp.h"
#include "xaction.h"

#include "sequence.h"

extern struct screen_template  used, screen;
extern Chix                    confid;
extern int                     debug;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   HOSTPTR   host, make_host();
   Hinfoptr  make_hinfo();
   int       hostnum, ok, new_hostinfo, ack, reship, error;
   char      temp[200];
   long      maxbytes, maxlines, maxwidth, seq, last_ack, index;
   long      new_to_low, new_to_shipd;
   Chix      maxparms;

   screen.lines =  0;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   host = make_host();
   host->info = make_hinfo();

   maxparms = chxalloc (40L, THIN, "shiphost");

   /*** Check arguments and initialize CONFID. */
   if (argc < 4  ||  argc > 5) {
      printf ("Usage: shiphost confid host_number type [-d]\n");
      exit   (1);
   }
   sscanf (argv[2], "%d", &hostnum);
   reship = streq (argv[3], "reship");
   if (argc == 5)  debug = -1;

   confid = chxalloc (40L, THIN, "shiphost confid");
   chxofascii (confid, argv[1]);
   utilglobals();
   mainglobals();

   /*** Lock XHSL for this host to ensure only one shiphost program at
   /    a time is shipping to this host. */
   if (NOT unit_lock (XHSL, TLOCK, L(hostnum), 0L, 0L, 0L, nullchix)) {
      printf ("Shiphost: already in use for host %d.\n", hostnum);
      return (0);
   }

   /*** Get this host's "hostinfo" file to find out which transport files
   /    (by a range of sequence numbers) we should actually ship. */
   if (ok = unit_lock (XHI1, READ, L(hostnum), 0L, 0L, 0L, nullchix)) {
      ok = get_neighbor (hostnum, host, XHI1);
      if (NOT ok) bug ("Shiphost: get_neighbor failed on host: ", hostnum);
      unit_unlk (XHI1);
   }
   else bug ("Shiphost: cannot lock hostinfo file for host: ", hostnum);
   if (NOT ok) goto unlock;
   new_to_shipd = host->info->seqnum_to_shipd;


   /*** Read host ship variables file to get maximum chunk size parameters.  */
   if (ok = unit_lock (XHSV, READ, L(hostnum), L(host->info->shipmeth), 0L, 0L,
          nullchix)) {
      if (ok = unit_view (XHSV)) {
         unit_read (XHSV, maxparms, 0L);
         unit_close(XHSV);
      }
      unit_unlk (XHSV);
   }
   if (NOT ok) {
      sprintf (temp, "Shiphost: %s\n   shipmeth=%d, host=%d\n",
         "Cannot lock or open XHSV host shipping-variables file.",
          host->info->shipmeth, hostnum);
      bug (temp, 0);
      goto unlock;
   }
   index=0;
   maxbytes = chxlong (maxparms, &index);
   maxlines = chxlong (maxparms, &index);
   maxwidth = chxlong (maxparms, &index);
   new_hostinfo = 0;

   /*** Reshipper: look at a range of transport files in the reship bay:
   /    from 1 + last seq num ACK'd, to last seq num shipped. */
   if (reship) {
      last_ack = host->info->seqnum_to_low;
      for (seq = INC  (host->info->seqnum_to_low, SEQ_TO_MOD);
                 LTEQ (seq, host->info->seqnum_to_shipd, SEQ_TO_MOD);
                 seq = INC (seq, SEQ_TO_MOD) ) {
         reship_it (hostnum, host->info->shipmeth, seq,
                    maxbytes, maxlines, maxwidth,
                    host->info->tosstime, host->info->reshiptime, &ack,
          &error);
         if (ack  &&  seq == INC (last_ack, SEQ_TO_MOD))  last_ack = seq;
  
         shiperror (error, hostnum, seq);
      }

      if (GT (last_ack, host->info->seqnum_to_low, SEQ_TO_MOD)) {
         new_to_low   = last_ack;
         new_hostinfo = 1;
      }
   }

   /*** Shipper: try to ship each of the transport files in the regular
   /    shipping bay: from 1 + last seq # shipped, to (but not including)
   /    next seq num to be used by XFP. */
   else {
      for (seq = INC (host->info->seqnum_to_shipd, SEQ_TO_MOD);
                 LT  (seq, host->info->seqnum_to_hi, SEQ_TO_MOD);
                 seq = INC (seq, SEQ_TO_MOD)) {
         ship_it (hostnum, host->info->shipmeth, seq,
                     maxbytes, maxlines, maxwidth, &error);
  
         if (shiperror (error, hostnum, seq))  break;
      }
  
      if (GT (seq, INC (host->info->seqnum_to_shipd, SEQ_TO_MOD),
          SEQ_TO_MOD)) {
         new_to_shipd = DEC (seq, SEQ_TO_MOD);
         new_hostinfo = 1;
      }

      /*** Try to ship all the outstanding ACKnowledgements. */
      ship_ack (hostnum, host->info->shipmeth);
   }

   /*** Update the hostinfo file if seqnum_to_shipd or seqnum_to_low
   /    changed.  (I.e., if a file was first-time shipped OK or if
   /    the reshipper saw an implicit ACK.) */
   if (new_hostinfo) {
      if (ok = unit_lock (XHI1, WRITE, L(hostnum), 0L, 0L, 0L, nullchix)) {
         ok = get_neighbor (hostnum, host, XHI1);
         if (GT (new_to_shipd, host->info->seqnum_to_shipd, SEQ_TO_MOD))
            host->info->seqnum_to_shipd = new_to_shipd;
         if (GT (new_to_low,   host->info->seqnum_to_low, SEQ_TO_MOD))
            host->info->seqnum_to_low   = new_to_low;
         if (ok)  ok = set_neighbor (host, XHI1);
         unit_unlk (XHI1);
      }
      if (NOT ok) bug ("Shiphost: could not rewrite hostinfo file", hostnum);
   }
  
unlock:
   unit_unlk (XHSL);

   free_host (host);
   chxfree (maxparms);
   return (0);
}
