/*** SHIP_IT.   Ship a transport file (the first time).
/
/    ok = ship_it (hostnum, shipmeth, seqnum, maxbytes, maxlines, maxwidth,
/                   error);
/
/    Parameters:
/       int    ok;          (Success?)
/       int    hostnum;     (ship file to this host)
/       int    shipmeth;    (shipping method number)
/       long   seqnum;      (sequence number of transport file)
/       long   maxbytes;    (max size of chunk to ship)
/       long   maxlines;    (max lines in chunk to ship)
/       long   maxwidth;    (max line width of chunk to ship)
/       int   *error;       (returned error code)
/
/    Purpose:
/       Ship a transport file selected by SEQNUM, HOSTNUM (destination)
/       and SHIPMETH (shipping method).  After shipping, put it in the
/       RESHIP bay.
/
/    How it works: Ship_it() goes through the following steps in the
/       process of shipping the contents of the transport header and text
/       files:
/    
/       (A) Load the transport header.
/       (B) Make sure there is enough free disk space for step C.
/       (C) Ship the transport header/text file via shipfile().
/       (D) Write a reship header file in the RESHIP bay that begins with a
/           shipment date/time header, and is followed by the transport
/           header.
/       (E) Move the transport text file to the RESHIP bay, and delete 
/           the original transport header file.
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions - sets ERROR variable to:
/       NOERR      success.
/       DS_NOXPXH  can't get at (i.e., lock or open) transport header file.
/       DS_NOXPXT  can't get at transport text file.
/       DS_NOXPXJ  can't create a joint chunk shipment file.
/       DS_NOXPRH  can't put header file in RESHIP bay.
/       DS_XTMOVE  can't move transport text file to RESHIP bay.
/       DS_NOTIMP  sysavail() or sysfsize() not implemented.
/       DS_NOTDSK  can't discover amount of free disk space
/       DS_DSKLOW  insufficient free disk space to ship transport file. 
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  SHIPHOST program.
/
/    Operating system dependencies:
/
/    Known bugs:
/       MAXWIDTH is ignored in this version.  It may always be ignored.
/
/    Home:  xship/shipit.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/12/91 12:58 New function. */
/*: UL 12/11/91 18:45 Make unit_lock/name args 3-6 long.*/
/*: CR  1/06/92 15:35 Add shiplast arg to store_xheader() call. */
/*: JX  7/30/92 16:17 Chixify. */
/*: JX  9/16/92 15:47 Allocate HEAD. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "done.h"
#include "unitcode.h"
#include "xaction.h"
#include "derr.h"

FUNCTION  ship_it (hostnum, shipmeth, seqnum, maxbytes, maxlines, maxwidth,
                    error)
   int    hostnum, shipmeth;
   long   seqnum, maxbytes, maxlines, maxwidth;
   int   *error;
{
   XPORTHDR *head, *make_xporthdr();
   int       ok, success;
   long      shiporig, shiplast;
   char      temp[202];
   long      now;
   long      systime();

   ENTRY ("shipit", "");

   *error = NOERR;

   head = make_xporthdr();

   /*** Attempt to load the transport file header.  */
   if (NOT load_xheader (shipmeth, hostnum, seqnum, 0, head,
                             &shiporig, &shiplast))
      { *error = DS_NOXPXH;   FAIL; }
   
   /*** Make sure there is enough room for the transport file chunks. */
   unit_name (temp, XPXT, L(shipmeth), L(hostnum),  seqnum, 0L, nullchix);
   if (NOT enough_for (temp, maxbytes, error))   FAIL;

   /*** Open the transport text file. */
   if (ok = unit_lock (XPXT, READ, L(shipmeth), L(hostnum),  seqnum, 0L,
          nullchix)) {
      if (NOT (ok = unit_view (XPXT)))  unit_unlk (XPXT); 
   }
   if (NOT ok) { *error = DS_NOXPXT;   FAIL; }


   /*** Ship it! */
   if (NOT shipfile (XPXT, head, hostnum, shipmeth, seqnum, 
                       maxbytes, maxlines, maxwidth, error))  FAIL;
   unit_close (XPXT);
   unit_unlk  (XPXT);


   /*** Write out the header file in the RESHIP bay with original and
   /    last-shipped time of now. */
   now = systime();
   if (NOT store_xheader (shipmeth, hostnum, seqnum, head, now, now))
      *error = DS_NOXPRH;

   /*** Delete the XPXH header file, and move the transport text file
   /    to the RESHIP bay. */
   if (unit_lock (XPXH, WRITE, L(shipmeth), L(hostnum),  seqnum, 0L,
                  nullchix)) {
      unit_kill  (XPXH);
      unit_unlk  (XPXH);
   }
   if (NOT movetrans (XPXT, XPRT, hostnum, shipmeth, seqnum)) 
      *error = DS_XTMOVE;

   DONE (*error == NOERR);

done:
   free_xporthdr (head);
   RETURN (success);
}
