/*** A_GET_NAMEX.   Fetch list of item/resp's indexed by names.
/
/    ok = a_get_namex (namex, mask, wait);
/
/    Parameters:
/       int       ok;       (return status)
/       Namex     namex;    (object to fill with name index info)
/       long      mask;     (mask of properties to get)
/       int       wait;     (synchronicity flag)
/
/    Purpose:
/       Given the partial name of a name index, fetch the list of
/       item/resp numbers that are indexed under that name.
/
/    How it works:
/       MASK selects what kind of name index to scan:
/          P_TITLE for  item titles
/          P_NAME  for item authors
/          P_SUBJECT for subject categories
/
/       Caller must fill in members NAMEPART, CNUM, and OWNER (the latter
/       only for psubjects) to select the particular named index.  If 
/       there is more than one title/author/subject match, a_get_namex()
/       fills the RLIST member with the item/resp list for all matches.
/
/       The exact meaning of the contents of NAMEPART vary according to MASK:
/           P_TITLE   any part of an item title
/           P_SUBJECT any part of a subject category name
/           P_NAME    an initial substring of any *userid* of a participant.
/
/    Returns: A_OK       on success
/             A_BADARG   bad arguments
/             A_NOCONF   no such conference
/             A_NOREAD   user not allowed in this conf
/             A_NOTOWNER trying to access other user's psubjects
/             A_NONE     no name indices matched
/             A_DBERR    database error
/
/    Error Conditions:
/
/    Side effects: 
/
/    Related functions:
/
/    Called by:  UI
/
/    Operating system dependencies:
/
/    Known bugs:
/       It is not clear what should happen when there is a name index
/       match, but no items are actually found.  (E.g., an empty subject
/       category).  Right now a_get_namex() returns A_OK.
/
/    Home:  api/agetname.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: CR  2/15/93 21:31 New function. */
/*: CR  2/23/93 22:23 Clear PREV at the end of each rlist block. */
/*: CR  2/23/93 22:36 Clarify comments about NAMEPART. */
/*: CR  2/28/93 22:39 Get tbuf from a_cch_tbuf(). */
/*: CK  3/01/93 16:51 sysmem requires long arg. */
/*: CN  5/15/93 17:11 Use new item selection codes. */

#include <stdio.h>
#include "chixuse.h"
#include "api.h"
#include "unitcode.h"
#include "maxparms.h"
#include "textbuf.h"
#include "handicap.h"
#include "done.h"

FUNCTION  a_get_namex (namex, mask, wait)
   Namex  namex;
   long   mask;
   int    wait;
{
   Namelist thisname, matches, this, nlnames();
   Rlist    rthis;
   int      usd, usf, utd, utf;
   int      ok, access, additem, chgresp, chgsubj, success, is_psub, lno;
   int      prev, item;
   long     tag;
   short   *select;
   Chix     cname, temp, userid;
   Chix     line, getline();
   Textbuf  tbuf, a_cch_tbuf();
   char    *sysmem();

   /*** Check argument validity, clear list of matching namexect names. */
   if (namex == (Namex) NULL  ||  namex->tag != T_NAMEX)  return (A_BADARG);
   if (EMPTYCHX(namex->namepart))                         return (A_BADARG);
   if (namex->cnum <= 0)                                  return (A_NOCONF);

   cname  = chxalloc (30L, THIN, "agetsu cname");
   temp   = chxalloc (20L, THIN, "agetsu temp");
   userid = chxalloc (20L, THIN, "agetnamex userid");

   /*** This user can only work with their own psubjects. */
   is_psub = (NOT EMPTYCHX (namex->owner))  &&  mask==P_SUBJECT;
   sysuserid (userid);
   if (is_psub  &&  NOT chxeq (userid, namex->owner))  DONE (A_NOTOWNER);

   /*** Make sure this user can read this conference. */
   if (NOT a_cache (V_GET, namex->cnum, &access, &additem, &chgresp, 
                    &chgsubj, cname))        DONE (A_NOCONF);
   if (access < 1)                           DONE (A_NOREAD);

   /*** Select unit numbers of namelist partfiles to scan, depending
   /    on MASK and the OWNER member. */
   if      (is_psub)         { usd=XUND;  usf=XUNN;  utd=XUSD;  utf=XUSF; }
   else if (mask==P_SUBJECT) { usd=XCND;  usf=XCNN;  utd=XCSD;  utf=XCSF; }
   else if (mask==P_TITLE)   { usd=XCTD;  usf=XCTN;  }
   else if (mask==P_NAME)    { usd=XCAD;  usf=XCAN;  }
   else                      DONE (A_BADARG);

   /*** Actually scan the relevant namelist partfiles for matches. */
   thisname = nlnames (namex->namepart, nullchix);
   ok = matchnames (usd, usf, namex->cnum, namex->owner, thisname, 
                    1, &matches, NOPUNCT);
   nlfree (thisname);
   if (NOT ok)                { nlfree (matches);   DONE (A_DBERR);  }
   if (nlsize (matches) == 0) { nlfree (matches);   DONE (A_NONE); }

   /*** Scan the matches.  Add the appropriate items to RLIST. */
   select = (short *) sysmem (sizeof(short) * L(MAXITEMS+1));
   tbuf  = (Textbuf) NULL;
   for (this=matches->next;   this!=(Namelist) NULL;   this=this->next) {

      /*** Find the tag for each match. */
      chxtoken (temp, nullchix, 2, this->str);
      chxnum   (temp, &tag);

      /*** For author and title indices, the tag *is* the item number. */
      if (mask==P_NAME  ||  mask==P_TITLE) {
         select[tag] = 0;
         continue;
      }

      /*** The remainder of the loop handles subjects, where the TAG is
      /    the subject number. */

      /*** See if that subject is in this conference. */
      if (tbuf == (Textbuf) NULL)
         tbuf = a_cch_tbuf (V_GET, utd, utf, namex->cnum, 0, (int) tag, 
                                   namex->owner);
      line = getline (utd, utf, namex->cnum, namex->owner, 0, 
                          (int) tag, 0, tbuf);
      if (line == nullchix)         continue;
      chxtoken (temp, nullchix, 2, line);
      if (NOT chxeq (temp, cname))  continue;

      for (lno=2;   (line = getline (utd, utf, namex->cnum, namex->owner,
                        0, (int) tag, lno, tbuf)) != nullchix;
           lno++) {
         parsenum (line, select, MAXITEMS, 0);
      }
   }
   nlfree (matches);

   /*** Empty old contents of NAMEX's rlist. */
   a_fre_rlist (namex->rlist);
   namex->rlist = rthis = a_mak_rlist();

   /*** Convert "select" marked list of items into RLIST. */
   for (prev=0, item=1;   item<=MAXITEMS;   ++item) {
      if      (prev==0  &&  select[item]==  0)  prev = item;
      else if (prev!=0  &&  select[item]== -1) {
         rthis->next = a_mak_rlist();
         rthis = rthis->next;
         rthis->i0 = prev;
         rthis->i1 = item-1;
         rthis->r0 =  0;
         rthis->r1 = -1;
         prev = 0;
      }
   }
   sysfree ( (char *) select);

   namex->ready = 1;
   DONE (A_OK);

done:
   chxfree (cname);
   chxfree (temp);
   chxfree (userid);
   return  (success);
}
