/*** SYSAVAIL.   Return the amount of available space on the disk.
/
/    ok = sysavail (path, bytes, files, error);
/
/    Parameters:
/       int    ok;          (did sysavail succeed in getting # bytes?)
/       char  *path;        (path name on a disk)
/       long  *bytes;       (returned number of free disk bytes)
/       long  *files;       (returned number of available files)
/       int   *error;       (error code if sysavail failed)
/
/    Purpose:
/       Many programs need to know if there is enough disk available
/       before proceeding to write a disk file.  Sysavail() returns
/       the best possible guess at the amount of free disk space
/       and number of files that can be created ("file slots") on a
/       particular disk or partition.
/
/    How it works:
/       Sysavail first looks to see if a quota is enabled for this
/       user.  If so, it gets the remaining space & # of files
/       allowed for by the quota, and returns those values.
/
/       If there is no quota implemented on this system, or enabled
/       for this user, it examines the file system or partition that
/       contains the file whose full pathname is PATH.  If it succeeds in
/       finding the number of free bytes and file slots, it returns 1,
/       sets BYTES to the number of free bytes, sets FILES to the
/       number of free file slots, and sets ERROR to 0.
/
/       Since checking free disk space is very expensive on some OS's,
/       we don't really check each time.  In some cases, e.g. the CaucusLink
/       XFP, we need to check disk space frequently.  To avoid unnecessary
/       delays, sysavail() caches the results, and uses the cached data
/       if the most recent call was "recent enough".  sysavail() remembers
/       the time of the last call, and compares it to the present call.
/       After a certain amount of time has elapsed (CACHELIFE), it ditches
/       the cache.
/
/    Implementation Notes:
/       Some systems may not be able to find the number of free file
/       slots, in which case FILES is set to -1.
/
/       Some systems do not provide the block size to programs.  In this
/       case we assume 512 bytes.
/
/       On all Unix, NFS version 2 does not provide free inodes across
/       the network.  If we can't find out if this partition is NFS-
/       mounted, set FILES to -1.
/
/       Otherwise, it returns 0, and puts an error code in ERROR.
/
/    Returns: 1 on success, 0 on failure.
/
/    Error Conditions:  sets one of the following error codes from syserror.h:
/       SE_BADDIR       Bad directory or filename in PATH.
/       SE_BADDEV       Bad device name derived from PATH.
/       SE_NOTIMP       Function not implemented for this system.
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by:  application.
/
/    Operating system dependencies: considerable!
/
/    Known bugs:
/
/    Home:  sys/sysavail.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/07/91 15:07 New function. */
/*: CR  8/08/91 14:37 Add missing systems to NONE. */
/*: JV  8/25/91 19:38 Add bug warning. */
/*: CR  6/23/92 13:58 Make RO a SYSV system.  Add quota stuff. */
/*: JV  7/16/92 15:21 Add U4, move include files. */
/*: JV  9/04/92 15:10 Move DYP to SYSV. */
/*: JV 10/14/92 15:40 Move HU to SYSV. */
/*: CV  5/10/93 13:11 Add ']' checking to VV code. */
/*: CP  3/28/93 18:59 Add MV code. */
/*: CP  7/08/93 16:00 Integrate 2.5 changes. */
/*: JV  8/06/93 16:36 Absorb diskfree() code, fix minor bugs, remove AOS code. */
/*: JV  9/26/93 13:08 Add ENTRY. */

#include <stdio.h>
#include "handicap.h"
#include "systype.h"
#include "syserror.h"
#include "entryex.h"

#define SYSV A3 | SX | TX | U3 | RO | U4 | DYP | HU

#if CA
#include <packets/filestatus.h>
#endif

#if SYSV
#include <sys/types.h>
#include <sys/stat.h>
#include <ustat.h>
#include <errno.h>
#endif

#if SU
#include <sys/types.h>
#include <sys/vfs.h>
#include <sys/stat.h>
#include <ufs/quota.h>
#include <errno.h>
#endif

#if RO
#include <sys/param.h>
#include <sys/quota.h>
#endif

#if MV
#include <sys/types.h>
#include <sys/statfs.h>
#endif

#define CACHELIFE 120

FUNCTION  sysavail (path, bytes, files, error)
   char  *path;
   long  *bytes, *files;
   int   *error;
{
   long   now, systime();
   static int lasterror, lastret;
   static long lastcall=0L, lastbytes, lastfiles;

/*--------------------------SunOS & Coherent (statfs() systems)----------*/
#if MV | SU
   struct statfs buffer;
   int    length;
   ENTRY ("sysavail", "");
 
   now = systime();               /* If we made a recent call, use that data. */
   if (now - lastcall < CACHELIFE) {
      *error = lasterror;
      *bytes = lastbytes;
      *files = lastfiles;
      RETURN (lastret);
   }

#if SU
   {
/*  Unfinished code for quota checking on SunOS.
      struct dqblk qblk;
      int          uid;
      uid = getuid();
      if (quotactl (Q_GETQUOTA, "", uid, &qblk) >= 0) {
         *bytes = (qblk.dqb_bsoftlimit - qblk.dqb_curblocks) * 512L;
         *files = (qblk.dqb_fsoftlimit - qblk.dqb_curfiles);
         *error = 0;
         RETURN (1);
      }
      if (errno != ESRCH) printf ("sysavail: quotactl: errno=%d\n", errno);
*/
   }
#endif

   lastcall = now;

#if MV
   length = sizeof(buffer);
   if (statfs (path, &buffer, length, 0) < 0) {
#endif
#if SU
   if (statfs (path, &buffer)) {
#endif
      *error = lasterror = SE_BADDEV;
      lastret = 0;
      RETURN(0);
   }

   *bytes = lastbytes = buffer.f_bsize * buffer.f_bfree;
   *files = lastfiles = buffer.f_ffree;
   *error = lasterror = 0;
   lastret = 1;
   RETURN (1);
#endif

/*--------------------------System V Unix--------------------------------*/
#if SYSV
   struct stat  statbuf;
   struct ustat ustatbuf;
   int          device;
   ENTRY  ("sysavail", "");
 
   now = systime();               /* If we made a recent call, use that data. */
   if (now - lastcall < CACHELIFE) {
      *error = lasterror;
      *bytes = lastbytes;
      *files = lastfiles;
      RETURN (lastret);
   }

   /*** Get the DEVICE number for the disk containing PATH. */
   lastcall = now;
   if (stat  (path, &statbuf))   {
      *error = lasterror = SE_BADDIR;
      lastret = 0;
      RETURN (0);
   }
   device = statbuf.st_dev;

#if RO
   {  /* Ultrix, really... */
/* This code is fucked!  quota seems to return whatever it feels like.
      struct dqblk qblk;
      int          uid, code;
      uid = getuid();
      printf ("path  =%s\n", path);
      printf ("uid   =%d\n", uid);
      printf ("device=%d\n", device);
      code = quota (Q_GETDLIM, uid, device, &qblk);
      if (code >= 0  &&  qblk.dqb_bsoftlimit > 0) {
         printf ("bsoftlimit=%d\n", qblk.dqb_bsoftlimit);
         printf ("curblocks =%d\n", qblk.dqb_curblocks);
         *bytes = (qblk.dqb_bsoftlimit - qblk.dqb_curblocks) * 512L;
         *files = (qblk.dqb_isoftlimit - qblk.dqb_curinodes);
         *error = 0;
         RETURN (1);
      }
      if (code < 0  &&  errno != ESRCH  &&  errno != EINVAL)
         printf ("sysavail: quotactl: errno=%d\n", errno);
*/
   }
#endif

   /*** Get the disk info for DEVICE. */
   if (ustat (device, &ustatbuf)) { 
      *error = lasterror = SE_BADDEV;
      lastret = 0;
      RETURN (0);
   }

   /*** 512 bytes per block: verified on SU only so far. */
   *bytes = lastbytes = ustatbuf.f_tfree * 512L;
   *files = lastfiles = ustatbuf.f_tinode;
   *error = lasterror = 0;
   if (*files <= 0)  *files = -1L;
   lastret = 1;
   RETURN (1);
#endif
  
/*---------------------------------------PRIME---------------------------*/
#define NONE1  PR  | MU | PU | LA | SV | IX | CX
#define NONE2  CVM | DU | PS | AU | AX | UL | BU | DY3 | FU | PX
#if NONE1 | NONE2
   ENTRY  ("sysavail", "");
   *error = lasterror = SE_NOTIMP;
   lastret = 0;
   RETURN (0);
#endif

/*---------------------------------------IBM PC--------------------------*/
#if IM | PN
#include <dos.h>
   struct   diskfree_t dfree;
   long     total_bytes;
   unsigned disk;
 
   ENTRY ("sysavail", "");

   now = systime();               /* If we made a recent call, use that data. */
   if (now - lastcall < CACHELIFE) {
      *error = lasterror;
      *bytes = lastbytes;
      *files = lastfiles;
      RETURN (lastret);
   }

   /*** Figure out which DISK we're checking. */
   if (path[1] == ':') {
      disk = path[0];
      if (disk >= 'A'  &&  disk <= 'Z')  disk += ('a' - 'A');
      disk = disk - 'a' + 1;
   }
   else      disk = 0;

   lastcall = now;
   if (_dos_getdiskfree (disk, &dfree) != 0) {
      *error = lasterror = SE_BADDEV;
      lastret = 0;
      RETURN (0);
   }

   /*** Figure total bytes remaining.
   /    (I'm not quite sure that avail_clusters is the proper test for
   /    # of files that can be created, but...?) */
   total_bytes =   ((long) dfree.bytes_per_sector)
                 * ((long) dfree.sectors_per_cluster)
                 * ((long) dfree.avail_clusters);
   *bytes = lastbytes = total_bytes;
   *files = lastfiles = dfree.avail_clusters;
   *error = lasterror = 0;
   lastret = 1;
   RETURN (1);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
#include <rmsdef>
#include <ssdef>
#include <descrip>
#include <iodef>

#define  EFN          0
#define  CHAN         0
#define  IOSB         0
#define  ASTADR       0
#define  ASTPRM       0
#define  THRESHOLD  100
#define  DVI$_FREEBLOCKS   42

   struct dsc$descriptor_s devnam, f_desc, r_desc;
   long   freeblocks, RETURN_length, context;
   int    found, rc;
   char   length, filename[160], result[255];

   struct {
      short buffer_length;
      short item_code;
      long *buffer_ptr;
      long *RETURN_ptr;
      long  zero;
   } itemlist;

   ENTRY ("sysavail", "");

   now = systime();               /* If we made a recent call, use that data. */
   if (now - lastcall < CACHELIFE) {
      *error = lasterror;
      *bytes = lastbytes;
      *files = lastfiles;
      RETURN (lastret);
   }

   /*** Turn PATH into a filespec.  If no ']', add ']*.dir. */
   strcpy (filename, path);
   if (strindex (filename, "]") < 0)  strcat (filename, "]*.dir");

   /*** Using FILENAME, get the device. */
   f_desc.dsc$w_length  = strlen (filename);
   f_desc.dsc$a_pointer = filename;
   f_desc.dsc$b_class   = DSC$K_CLASS_S;
   f_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
   r_desc.dsc$w_length  = 254;
   r_desc.dsc$a_pointer = result;
   r_desc.dsc$b_class   = DSC$K_CLASS_S;
   r_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
   context = 0L;
   found   = lib$find_file (&f_desc, &r_desc, &context, 0, 0, 0, 0);
   lib$find_file_end (&context);
   lastcall = now;
   if (found != RMS$_NORMAL)  {
      *error = lasterror = SE_BADDIR;
      lastret = 0;
      RETURN (0);
   }

   /*** Set up the string descriptor DEVNAM for the DISKNAME. */
   devnam.dsc$w_length  = r_desc.dsc$w_length;
   devnam.dsc$a_pointer = result;
   devnam.dsc$b_class   = DSC$K_CLASS_S;
   devnam.dsc$b_dtype   = DSC$K_DTYPE_T;

   /*** Set up the ITEMLIST to receive the # of free blocks into. */
   itemlist.buffer_length = 4;
   itemlist.item_code     = DVI$_FREEBLOCKS;
   itemlist.buffer_ptr    = &freeblocks;
   itemlist.RETURN_ptr    = &RETURN_length;
   itemlist.zero          = 0L;

   rc = sys$getdviw (EFN, CHAN, &devnam, &itemlist, IOSB, ASTADR, ASTPRM, 0);
   *bytes = lastblocks = freeblocks * 512L;
   *files = lastfiles = -1L;
   *error = lasterror = 0;
   lastret = 1;
   RETURN (1);
#endif

}
