/*** SYSDISKFULL.  Is the disk containing PATH too full?
/
/    int n = sysdiskfull (path)
/       Chix path;
/
/  Purpose:
/    Sometimes Caucus needs to know if there's "enough" room on the
/    disk (or disk partition) containing PATH.  Further, since most
/    of the Distributed Caucus programs have the potential to 
/    temporarily use *alot* of disk, and because they run unattended
/    they need to frequently check disk space.
/
/  How it works:
/
/     The AOS/VS version uses the ?FSTAT system call to get the number of
/     disk blocks left in both the root directory and the home directory
/     of Caucus.  If either is less than THRESHOLD (50) we fail (return 1).
/
/     Primos always succeeds (returns 0).
/
/     Unix System V uses the stat(2) and ustat(2) system calls to get the
/     number of disk blocks left on the device (or partition) containing
          PATH.
/     Then we compare this against THRESHOLD (50).
/
/  NOTE: Probably many more Unix systems can move to the SysV code described
/     above.  If you find one, move it to the SYSV #define below.
/
/  Returns:
/    SYSDISKFULL returns 1 if the disk containing PATH is
/    too full, i.e. if the number of free blocks is less
/    than THRESHOLD.  Otherwise, it returns 0.
/
/  Error Conditions:
/
/  Related Functions:
/
/  Called by:
/
/  Home: sys/sysdiskf.c
/*/

/*** Copyright (C) 1985 Camber-Roth.  All rights reserved. */

/*** Note that the PRIME version always returns 0. */

/*: AA  7/29/88 17:44 Source code master starting point */
/*: CR  8/23/88 14:50 Vax/VMS change to use lib$find_file */
/*: CR  9/21/88 15:25 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 13:00 Add "IX" system type. */
/*: CR 10/06/88 15:22 Change IM to use QuickC. */
/*: CR 11/02/88 17:28 Add CX, CA code. */
/*: CR  1/13/89 12:36 Fix bug in VV code. */
/*: JV 11/22/88 14:12 Fixed AOS code to use confid instead of :udd:caucus.*/
/*: CR  3/02/89 11:32 Add CVM code. */
/*: CR  6/09/89 12:16 Add "SV" system type code. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JV 10/04/89 18:01 Stripped out AOS non-server code. */
/*: CR 10/12/89 15:35 Add DU system type code. */
/*: JV  7/06/90 20:36 Add MIPS RISC/os (RO). */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  9/27/90 12:19 Add U3, AU system types. */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: CR  1/28/91 20:48 Detect hangup while wait()ing for child. */
/*: CR  2/01/91 16:23 Replace hangup with syshngtest(). */
/*: CR  2/09/91 21:44 Split long #if line in two (NONE1, NONE2). */
/*: SB 11/29/90 14:04 Add DY system code*/
/*: CR  3/11/91 16:20 Add AX & UL system type codes. */
/*: CR  3/20/91 18:04 Use NONE for UL. */
/*: CR  4/08/91 19:18 Use new confnum argument to unit_name(). */
/*: JV  6/24/91 17:00 Add header, change Unix SysV to call ustat(). */
/*: CR  9/16/91 13:22 Since SX is in SYSV, remove it elsewhere! */
/*: UL 12/11/91 20:12 Make unit_name args 3-6 long.*/
/*: CR  1/06/92 18:35 Add DP system type. */
/*: DE  5/28/92 15:40 Chixified */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#if CA
#include <packets/filestatus.h>
extern Chix confid;
#endif

#define SYSV A3 | SX | TX
#if SYSV
#include <sys/types.h>
#include <sys/stat.h>
#include <ustat.h>
#endif

extern int  debug;

FUNCTION  sysdiskfull (path)
   Chix   path;
{

 /*-----------------------------------AOS, MV/UX--------------------------*/
 /*** In AOSLand there are two types of directories, those without size
 /    restrictions (DIR) and those with (CPD=Control Point Directory).  
 /    Here, both the root directory (a CPD) and the Caucus Directory 
 /    (also a CPD) get checked. This must be done because it's possible
 /    for a CPD below the root to think it has more disk space left
 /    (SMSH-SCSH) than is left on the disk!
 */
#if CA
#define THRESHOLD 50
    long rootleft, caucleft;
    P_FSTAT_DIR fstat_pack;
 
    ENTRY ("sysdiskfill", "");
    fstat_pack.stim = -1;
    fstat_pack.sacp = -1;
    fstat_pack.slau = 0;
    fstat_pack.ssts = 0;
    fstat_pack.sefw = 0;
    fstat_pack.sefh = 0;
    sys_fstat (":", 0, &fstat_pack);
    rootleft = fstat_pack.smsh - fstat_pack.scsh;
    sys_fstat (confid, 0, &fstat_pack);
    caucleft = fstat_pack.smsh - fstat_pack.scsh;
    RETURN (rootleft < THRESHOLD || caucleft < THRESHOLD);
#endif

/*--------------------------System V Unix--------------------------------*/
#if SYSV
#define THRESHOLD 50
   char   bugstr[80];
   struct stat  statbuf;
   struct ustat ustatbuf;
   ENTRY  ("sysdiskfull", "");

   if (stat  (path, &statbuf)) { /* Succeed and complain if bad directory */
      sprintf (bugstr, "Sysdiskfull: bad directory: %s.\n", ascquick(path));
      bugstr [79]='\0'; buglist (bugstr);
      RETURN (0);
   }
   if (ustat (statbuf.st_dev, &ustatbuf)) {      /* Succeed and complain if
          */
      sprintf (bugstr, "Sysdiskfull: bad device: %s.\n", ascquick(path));  
          /* confusd     */
      bugstr [79]='\0'; buglist (bugstr);
      RETURN (0);
   }
   RETURN (ustatbuf.f_tfree < THRESHOLD);
#endif
   
/*---------------------------------------PRIME---------------------------*/
#define NONE1  PR  | MU | PU | MV | SU | HU | LA | SV | IX | CX
#define NONE2  CVM | DU | RO | PS | AU | AX | UL | DP
#if NONE1 | NONE2
   ENTRY  ("sysdiskfull", "");
   RETURN (0);
#endif

/*----------------------------------FORTUNE & MASSCOMP--------------*/
#if FU | BU | PX | U3 | DY
   static int  was_full  = 1;
   static long last_time = 0;
   long   systime(), strscan();
   long   freeblocks, blocks;
   int    fd[2], pid, status, freebytes, paren, colon, maxlen, len;
   char   str[200], freestr[80], *s;
   char  *strtoken();

   ENTRY ("sysdiskfull", "");

   /*** SYSDISKFREE works by spawning off a task running the Unix
   /    Disk Free program (DF) that communicates over a pipe to
   /    the current task.   SYSDISKFREE only checks the free blocks
   /    if more than 3 minutes has elapsed since the last check. */
   if (NOT was_full  &&  systime() < last_time + 180)  RETURN (0);

   /*** Make sure the disk referenced by PATH really exists. */
   if ( (status = open (ascquick(path), 0)) < 0)  RETURN (1);
   close (status);

   /*** Open the pipe. */
   if (pipe (fd) < 0)          RETURN (1);

   /*** Spawn off a task to run DF. */
   if ( (pid = fork()) == -1)  RETURN (1);

   /*** The child process only writes to the pipe.  It closes off the
   /    read, and replaces it's STDOUT with the pipe write fd. */
   if (pid == 0) {    /* Child */
      close (fd[0]);
      close (1);
      dup   (fd[1]);
      close (fd[1]);
#if FU | BU | DY
      execl ("/bin/df", "df", ascquick(path), NULL);
#endif
#if PX | U3
      execl ("/bin/df", "df", NULL);
#endif
      exit  (1);
   }

   /*** The parent only reads from the pipe. */
   close (fd[1]);
#endif

/*----------------------------------FORTUNE-------------------------*/
#if FU | BU | DY
#define  THRESHOLD  50
   /*** Pluck out the number of kbytes free. */
   read     (fd[0], str, 200);
#if FU
   strtoken (freestr, 5, str + strindex (str, "/dev/"));
#endif
#if BU | DY
   strtoken (freestr, 4, str + strindex (str, "\n"));
#endif
   strnum   (freestr, &freebytes);

   /*** Close the pipe entirely, and wait for the DF task to die. */
   close (fd[0]);
   while (wait (&status) != pid)  { if (syshngtest())  break; }

   /*** So?  Is the disk too full? */
   last_time = systime();
   was_full  = freebytes < THRESHOLD;
   RETURN (was_full);
#endif

/*---------------------------------------SCO Xenix-----------------------*/
#if PX | U3
#define  THRESHOLD  100
   /*** Prepare to find which disk PATH is on, and how many
   /    FREEBLOCKS it has. */
   s          = str;
   maxlen     = 0;
   freeblocks = 0;

   /*** Read each line from DF, one character at a time.  When we've got
   /    a whole line, pluck out the mounted disk name and # free blocks. */
   while (read (fd[0], s, 1) > 0) {
      if (*s != '\n') { ++s;   continue; }

      /*** Get the number of free blocks. */
      *s = '\0';
      colon = strindex (str, ":");
      s = str + colon + 1;
      blocks = strscan (&s);

      /*** Pluck out the mounted disk name, and trim it. */
      paren  = strindex (str, "(");
      str[paren] = '\0';
      strtrim (str);
      len = strlen (str);

      /*** If this disk name is the longest sub string of PATH, then
      /    this is the number of free blocks we want. */
      if (strindex (ascquick(path), str) >= 0  &&  len > maxlen) {
         maxlen     = len;
         freeblocks = blocks;
      }

      s = str;
   }

   /*** Close the pipe entirely, and wait for the DF task to die. */
   close (fd[0]);
   while (wait (&status) != pid)  { if (syshngtest())  break; }

   /*** So, are there too few blocks? */
   last_time = systime();
   was_full  = freeblocks < THRESHOLD;
   RETURN (was_full);
#endif

/*---------------------------------------IBM PC--------------------------*/
#if IM | JM | PN
#include <dos.h>
#define  MINBYTES   9000
#define  MINFILES      8

   static int  was_full  = 1;
   static long last_time = 0;
   struct diskfree_t dfree;
   long   now, total_bytes, systime();
   char   str[80];
   unsigned disk;

   ENTRY ("sysdiskfull", "");

   /*** Check if the disk is full no more than once per minute.
   /    (Unless it was full the last time we checked.) */
   now = systime();
   if (NOT was_full  &&  now < last_time + 60)  RETURN (0);
   
   /*** Figure out which DISK we're checking. */
   ascofchx   (str, path, 0, ALLCHARS);
   strlower (str);
   if (str[1] == ':')  disk = str[0] - 'a' + 1;
   else                disk = 0;

   if (_dos_getdiskfree (disk, &dfree) != 0)  RETURN (1);  /* bad disk */

   /*** Is there enough room for MINFILES files and MINBYTES bytes? */
   total_bytes =   ((long) dfree.bytes_per_sector)
                 * ((long) dfree.sectors_per_cluster) 
                 * ((long) dfree.avail_clusters);
   was_full    = dfree.avail_clusters < MINFILES  ||  total_bytes < MINBYTES;
   last_time   = now;
   RETURN (was_full);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
#include <rmsdef>
#include <ssdef>
#include <descrip>
#include <iodef>

#define  EFN          0
#define  CHAN         0
#define  IOSB         0
#define  ASTADR       0
#define  ASTPRM       0
#define  THRESHOLD  100
#define  DVI$_FREEBLOCKS   42

   struct dsc$descriptor_s devnam, f_desc, r_desc;
   long   freeblocks, RETURN_length, context;
   int    found, rc;
   char   length, master[100], result[255];

   struct {
      short buffer_length;
      short item_code;
      long *buffer_ptr;
      long *RETURN_ptr;
      long  zero;
   } itemlist;

   ENTRY ("sysdiskfull", "");

   /*** Make up the name of the "master" file for this conference.  Use
   /    lib$find_file to get the full path name of this file, including
   /    the disk name. */
   unit_name (master, XCMA, L(confnum), 0L, 0L, 0L, null.chx);
   
   f_desc.dsc$w_length  = strlen (master);
   f_desc.dsc$a_pointer = master;
   f_desc.dsc$b_class   = DSC$K_CLASS_S;
   f_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
   r_desc.dsc$w_length  = 254;
   r_desc.dsc$a_pointer = result;
   r_desc.dsc$b_class   = DSC$K_CLASS_S;
   r_desc.dsc$b_dtype   = DSC$K_DTYPE_T;

   context = 0L;
   found   = lib$find_file (&f_desc, &r_desc, &context, 0, 0, 0, 0);
   lib$find_file_end (&context);
   if (found != RMS$_NORMAL)  {
      if (debug) bugtell ("sysdiskfull err: ", found, 0L, master);
      RETURN (0);
   }

   /*** Set up the string descriptor DEVNAM for the DISKNAME. */
   devnam.dsc$w_length  = r_desc.dsc$w_length;
   devnam.dsc$a_pointer = result;
   devnam.dsc$b_class   = DSC$K_CLASS_S;
   devnam.dsc$b_dtype   = DSC$K_DTYPE_T;

   /*** Set up the ITEMLIST to receive the # of free blocks into. */
   itemlist.buffer_length = 4;
   itemlist.item_code     = DVI$_FREEBLOCKS;
   itemlist.buffer_ptr    = &freeblocks;
   itemlist.RETURN_ptr    = &RETURN_length;
   itemlist.zero          = 0L;

   rc = sys$getdviw (EFN, CHAN, &devnam, &itemlist, IOSB, ASTADR, ASTPRM, 0);

   if (debug) bugtell ("sysdiskfull: ", rc, freeblocks, "");

   /*** So?  Is the disk too full? */
   RETURN (freeblocks < THRESHOLD);
#endif

}
